<?php

namespace App\Console\Commands;


use App\Enum\ConversationTypes;
use App\Enum\NotificationTypes;
use App\Facades\Context;
use App\Jobs\SendNotification;
use App\Mail\BirthdayNotification;
use App\Mail\WorkAnniversaryNotification;
use App\Models\User\User;
use App\Services\Message\CreateConversation;
use App\Services\Message\CreateMessage;
use Carbon\Carbon;
use Illuminate\Console\Command;
use App\Enum\AccessPermissions;
use App\Services\UserAccess\GetAccessByRole;
use App\Enum\Roles;
use App\Jobs\SendNotificationConsole;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;

class AdminDailyNotification extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'admin-daily:notification';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Create daily notification for manager';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        //Get all user have birthday today
        $userBirthdays = User::active()->whereHas('employee', function ($query) {
            $query->whereDay('date_of_birth', Carbon::today()->day);
            $query->whereMonth('date_of_birth', Carbon::today()->month);
        })->with('employee')->get()->groupBy('employee.client_id')->all();

        //Get all user have working anniversary today
        $userAnniversary = User::active()->whereHas('employee', function ($query) {
            $query->whereDay('start_date', Carbon::today()->day);
            $query->whereMonth('start_date', Carbon::today()->month);
        })->with('employee')->get()->groupBy('employee.client_id')->all();

        $clientIds = collect(array_merge(array_keys($userBirthdays), array_keys($userAnniversary)))->unique();

        //Get all admins
        $admins = User::active()->where('is_demo', 0)->whereHas('employee', function ($query) use ($clientIds) {
            $query->whereIn('client_id', $clientIds->toArray());
        })->with('employee')->role(Roles::ADMIN)->get();

        foreach ($admins as $admin) {
            Context::user()->set($admin);
            Context::client()->set($admin->employee->client);
            $adminAccess = app(GetAccessByRole::class)->execute(['role' => Roles::ADMIN]);
            $accessAdminPermissions = $adminAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];
            // Send at 9AM
            $sendAfter = Carbon::today()->setTimezone($admin->employee->timezone)->hour(9);
            if ($accessAdminPermissions[AccessPermissions::NOTIFICATION_EMAIL_BIRTHDAY] && isset($userBirthdays[$admin->employee->client->getKey()])) {
                foreach ($userBirthdays[$admin->employee->client->getKey()] as $user) {
                    Mail::to($admin->email)->send(new BirthdayNotification($admin, $user));
                }
            }
            if ($accessAdminPermissions[AccessPermissions::NOTIFICATION_CONSOLE_BIRTHDAY] && isset($userBirthdays[$admin->employee->client->getKey()])) {
                foreach ($userBirthdays[$admin->employee->client->getKey()] as $user) {
                    SendNotificationConsole::dispatch($user, $admin, 'Employee birthday');
                }
            }
            if ($accessAdminPermissions[AccessPermissions::NOTIFICATION_APP_BIRTHDAY] && isset($userBirthdays[$admin->employee->client->getKey()])) {
                foreach ($userBirthdays[$admin->employee->client->getKey()] as $user) {
                    $conversation = app(CreateConversation::class)
                        ->execute(['type' => ConversationTypes::DIRECT_MESSAGE, 'receiver_code' => $user->getBusinessKey()]);
                    app(CreateMessage::class)->execute(['content' => __(
                        "notify.employee_birthday",
                        ["first_name" => $user->first_name, "surname" => $user->surname]
                    ), 'conversation_id' => $conversation->id]);
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => array($admin->getBusinessKey()),
                        "headings"   => __(
                            "notify.employee_birthday",
                            ["first_name" => $user->first_name, "surname" => $user->surname]
                        ),
                        "contents"   => __('Tap here to open message'),
                        "data"       => [
                            "type"            => NotificationTypes::MESSAGE_RECEIVED,
                            "conversation_id" => $conversation->id
                        ],
                        'send_after' => $sendAfter
                    );
                    SendNotification::dispatch($arguments);
                }
            }
            if ($accessAdminPermissions[AccessPermissions::NOTIFICATION_EMAIL_WORK_ANNIVERSARY] && isset($userAnniversary[$admin->employee->client->getKey()])) {
                foreach ($userAnniversary[$admin->employee->client->getKey()] as $user) {
                    Mail::to($admin->email)->send(new WorkAnniversaryNotification($admin, $user));
                }
            }
            if ($accessAdminPermissions[AccessPermissions::NOTIFICATION_CONSOLE_WORK_ANNIVERSARY] && isset($userAnniversary[$admin->employee->client->getKey()])) {
                foreach ($userAnniversary[$admin->employee->client->getKey()] as $user) {
                    SendNotificationConsole::dispatch($user, $admin, 'Work anniversary');
                }
            }
            if ($accessAdminPermissions[AccessPermissions::NOTIFICATION_APP_WORK_ANNIVERSARY] && isset($userAnniversary[$admin->employee->client->getKey()])) {
                foreach ($userAnniversary[$admin->employee->client->getKey()] as $user) {
                    $yearNumber = Carbon::today()->diffInYears($user->employee->start_date);
                    $conversation = app(CreateConversation::class)
                        ->execute(['type' => ConversationTypes::DIRECT_MESSAGE, 'receiver_code' => $user->getBusinessKey()]);
                    app(CreateMessage::class)->execute(['content' => __("notify.employee_anniversary",
                        ["first_name"  => $user->first_name,
                            "surname"     => $user->surname,
                            "year_number" => $yearNumber,
                            "year"        => $yearNumber > 1 ? "years" : "year"
                        ]), 'conversation_id' => $conversation->id]);
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => array($admin->getBusinessKey()),
                        "headings"   => __(
                            "notify.employee_anniversary",
                            [
                                "first_name"  => $user->first_name,
                                "surname"     => $user->surname,
                                "year_number" => $yearNumber,
                                "year"        => $yearNumber > 1 ? "years" : "year"
                            ]
                        ),
                        "contents"   => __('Tap here to open message'),
                        "data"       => [
                            "type"            => NotificationTypes::MESSAGE_RECEIVED,
                            "conversation_id" => $conversation->id
                        ],
                        'send_after' => $sendAfter
                    );
                    SendNotification::dispatch($arguments);
                }
            }
        }

        return 0;
    }
}
