<?php

namespace App\Console\Commands;

use App\Enum\Roles;
use App\Facades\Context;
use App\Models\Client\Client;
use App\Models\Client\ClientValue;
use App\Models\DataControl\DataControl;
use App\Models\Driver\Driver;
use App\Models\Initiative\Initiative;
use App\Models\Questionnaire\Questionnaire;
use App\Models\Questionnaire\QuestionnaireClientSetting;
use App\Models\Schedule\ScheduleFrequency;
use App\Models\Schedule\ScheduleFrequencyRound;
use App\Models\Schedule\ScheduleRoundQuestionnaire;
use App\Models\Segment\Segment;
use App\Models\Segment\SegmentType;
use App\Models\Segment\SegmentUser;
use App\Models\User\Employee;
use App\Models\User\User;
use App\Services\Mobile\Questionnaire\AnswerQuestions;
use App\Services\Mobile\Questionnaire\HandleAnswerQuestions;
use App\Services\Questionnaire\GetPublishedQuestionnaire;
use App\Services\Schedule\CreateSchedule;
use App\Services\Schedule\CreateScheduleRoundsAndDistributes;
use App\Services\Schedule\CreateSegments;
use App\Services\Segment\CreateSegment;
use App\Services\Segment\Employee\AddSegmentEmployee;
use Carbon\Carbon;
use Faker\Factory as Faker;
use Illuminate\Console\Command;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Hash;
use DateTime;
use Illuminate\Support\Str;

class EngagementDemoData extends Command
{
    protected $clientDemo;
    protected $userDemo;
    protected $adminDemo;
    protected $survey;
    protected $publishedQuestions;
    protected $roundOneQuestions;
    protected $roundOneDriver = [
        [
            'driver_name' => 'Accomplishment',
            'point' => 6,
            'comment' => "I'm proud of the work I deliver",
        ],
        [
            'driver_name' => 'Challenging',
            'point' => 4,
            'comment' => "I'm definitely ready for new challenges with my team",
        ],
        [
            'driver_name' => 'Autonomy',
            'point' => 8,
            'comment' => "I can always talk to my manager about how I work",
        ],
        [
            'driver_name' => 'Equipment',
            'point' => 5,
            'comment' => "We could be more productive with new computers",
        ],
        [
            'driver_name' => 'Alignment',
            'point' => 6,
            'comment' => NULL,
        ],
        [
            'driver_name' => 'Growth',
            'point' => 5,
            'comment' => "I could be challenged more",
        ],
        [
            'driver_name' => 'Management Support',
            'point' => 3,
            'comment' => "I'm usually left to myself in finishing work",
        ],
        [
            'driver_name' => NULL,
            'point' => 8,
            'comment' => NULL,
        ]
    ];

    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'engagement:create-client-demo-data';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Create data for client demo';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        // Get client demo and user demo
        $clientDemo = Client::where('is_demo', true)->firstOrFail();
        $userDemo = $clientDemo->users()->role(Roles::FRONT_USER)->where('is_demo',true)->firstOrFail();
        $adminDemo = $clientDemo->users()->role(Roles::ADMIN)->where('is_demo',true)->firstOrFail();

        $this->clientDemo = $clientDemo;
        $this->userDemo = $userDemo;
        $this->adminDemo = $adminDemo;
        Context::user()->set($this->adminDemo);
        Context::client()->set($this->clientDemo);

        // Create client value
        $this->createClientValues();

        // Publish only 10 question for client demo
        $this->publishQuestions();

        // Create initiative data
        $this->createInitiative();

        // Setup data control
        $this->createDataControl();

        // Create new segment demo
        $this->createSegment();

        // Add new employee to able pass data control
        $this->createNewEmployee();

        // Create schedule with frequency is monthly
        $this->createSchedule();

        // Create answer questions
        $this->createAnswerQuestions();
    }

    /**
     * Publish question
     *
     * @return bool
     */
    private function publishQuestions()
    {
        $questionsLowPriority = Questionnaire::whereHas('driver', function ($query) {
            return $query->where('priority', 0);
        })->limit(2)->get();

        $questionsMediumPriority = Questionnaire::whereHas('driver', function ($query) {
            return $query->where('priority', 1);
        })->limit(4)->get();

        $questionsHighPriority = Questionnaire::whereHas('driver', function ($query) {
            return $query->where('priority', 2);
        })->limit(4)->get();

        $roundOneDriversName = array_filter(Arr::pluck($this->roundOneDriver, 'driver_name'));
        $roundOneQuestions = Questionnaire::select('questionnaires.*')
            ->join('drivers', 'drivers.id', '=', 'questionnaires.driver_id')
            ->whereIn('drivers.name', $roundOneDriversName)
            ->get();
        $this->roundOneQuestions = $roundOneQuestions;

        $questions = $questionsLowPriority->merge($questionsMediumPriority)->merge($questionsHighPriority)->merge($roundOneQuestions);

        // publish questions
        foreach($questions as $question) {
            QuestionnaireClientSetting::updateOrCreate(
                ['client_id' => $this->clientDemo->id, 'questionnaire_id' => $question->id],
                ['status' => 1]
            );
        }

        $questionsId = $questions->pluck('id')->toArray();
        $otherQuestions = Questionnaire::whereNotIn('id', $questionsId)->get();
        // Unpublished other questions
        foreach($otherQuestions as $otherQuestion) {
            QuestionnaireClientSetting::updateOrCreate(
                ['client_id' => $this->clientDemo->id, 'questionnaire_id' => $otherQuestion->id],
                ['status' => 0]
            );
        }
        $this->publishedQuestions = app(GetPublishedQuestionnaire::class)->execute();

        return true;
    }

    /**
     * Create initiative
     *
     * @return bool
     */
    private function createInitiative()
    {
        $clientId = $this->clientDemo->id;

        $driverIds = !empty($this->publishedQuestions) ? $this->publishedQuestions->pluck('driver_id')->toArray() : [];

        $initiativeData = [
            [
                'responsible_id' => $this->adminDemo->id,
                'title' => 'Buy everyone in IT new screens',
                'description' => 'Purchase larger monitors for everyone in the IT department',
                'due_by' => Carbon::now()->addMonths(2),
                'created_by' => $this->adminDemo->id,
                'status' => 0
            ],
            [
                'responsible_id' => $this->adminDemo->id,
                'title' => 'Draft a flexible work policy',
                'description' => 'Work on a new flexible work policy that will give employees more control over their time',
                'due_by' => Carbon::now()->addMonths(3),
                'created_by' => $this->adminDemo->id,
                'status' => 0
            ],
            [
                'responsible_id' => $this->adminDemo->id,
                'title' => 'Bring your own device',
                'description' => 'Work with IT to support employees',
                'due_by' => Carbon::now()->addMonths(1),
                'created_by' => $this->adminDemo->id,
                'status' => 1
            ],
        ];
        if (!empty($driverIds)) {
            for ($i = 0; $i < 3; $i++) {
                if(!empty($driverIds[$i])) {
                    $initiativeData[$i]['driver_id'] = $driverIds[$i];
                }
            }
        }

        foreach ($initiativeData as $data) {
            if (!empty($data['driver_id'])) {
                Initiative::updateOrCreate(
                    ['client_id' => $clientId, 'driver_id' => $data['driver_id']],
                    $data
                );
            }
        }

        return true;
    }

    /**
     * Create values for client demo
     *
     * @return bool
     */
    private function createClientValues()
    {
        $values = [
            [
                'client_id' => $this->clientDemo->id,
                'name' => 'Integrity',
                'description' => "We always do what's right"
            ],
            [
                'client_id' => $this->clientDemo->id,
                'name' => 'Collaboration',
                'description' => 'Leverage collective genius'
            ]
        ];

        foreach ($values as $value) {
            $clientValue = ClientValue::where([
                'client_id' => $this->clientDemo->id,
                'name' => $value['name']
            ])->first();

            if (empty($clientValue)) {
                $clientValue = ClientValue::create($value);
            }
        }

        return true;
    }

    /**
     * Create Data Control
     *
     * @return mixed
     */
    private function createDataControl()
    {
        $data = [
            'segment_identification' => 3,
            'data_expiry' => 60,
            'anonymity_difference' => 3,
            'data_availability' => 3,
            'data_referencing' => 12,
            'engagement_score_default' => 0,
            'driver_score_default' => 0,
            'driver_sorting' => 0,
        ];

        $status = DataControl::updateOrCreate(
            ['client_id' => $this->clientDemo->id],
            $data
        );
        return $status;
    }

    private function createSegment()
    {
        $segmentsData = [
            [
                'type' => 'Location',
                'segments' => [
                    'Singapore',
                    'Manila',
                    'Bangkok'
                ]
            ],
            [
                'type' => 'Department',
                'segments' => [
                    'Sale',
                    'Help Desk'
                ]
            ],
            [
                'type' => 'Gender',
                'segments' => [
                    'Male',
                    'Female'
                ]
            ]
        ];
        foreach ($segmentsData as $segment) {
            $segmentType = SegmentType::where('name', $segment)->first();

            if(empty($segmentType)) continue;

            foreach ($segment['segments'] as $segmentName) {
                $findSegment = Segment::where([
                    'segment_type_id' => $segmentType->id,
                    'name' => $segmentName,
                    'client_id' => $this->clientDemo->id
                ])->first();

                if (empty($findSegment)) {
                    $params = ['name' => $segmentName, 'segment_type_id' => $segmentType->id, 'client_id' => $this->clientDemo->id];
                    Segment::create($params);
                }
            }
        }

        return true;
    }

    /**
     * Create new employees
     *
     * @return bool
     */
    private function createNewEmployee()
    {
        $employees = $this->clientDemo->users()->role(Roles::FRONT_USER)->where('is_demo',true)->get();
        $totalEmployees = 9;
        $faker = Faker::create();

        if (count($employees) < $totalEmployees) {
            $userAdded = [];
            for ($i = 1; $i <= $totalEmployees - count($employees); $i++) {
                $email = Str::slug(strtolower($faker->unique()->firstName)) . '@qualee.com';
                $findUs = User::where('email', $email)->first();

                if (!empty($findUs)) {
                    $i--;
                } else {
                    $front_user = User::create([
                        'name' => $faker->name,
                        'first_name' => $faker->firstName,
                        'surname' => $faker->lastName,
                        'email' => $email,
                        'password' => Hash::make('test'),
                        'status' => 1,
                        'is_subscription' => 0,
                        'phone' => $faker->phoneNumber,
                        'profile_pic' => '',
                        'is_default' => 1,
                        'remaining' => 0,
                        'complete' => 1,
                        'email_verified_at'=> new DateTime(),
                        'trial_ends_at' => new DateTime('2222-03-15 11:15:00'),
                        'is_demo' => true
                    ]);
                    $userAdded[] = $front_user->toArray();
                    $front_user->assignRole(Roles::FRONT_USER);
                    Employee::create([
                        "user_id" => $front_user->id,
                        "department_id" => '34',
                        "city_id" => '1',
                        "start_date" => new DateTime(),
                        "document_profile_id" => '12',
                        "client_id" => $this->clientDemo->id,
                        "job_title" => 'Demo',
                    ]);
                }
            }
            $this->addEmployeeToSegment();
        }
        return true;
    }

    /**
     * Add Employee To Segment
     *
     * @param $employees
     * @return bool
     */
    private function addEmployeeToSegment()
    {
        $employees = $this->clientDemo->users()->role(Roles::FRONT_USER)->where('is_demo',true)->get();

        $segmentTypes = SegmentType::all();
        $employeesChunk = $employees->chunk(3);
        $index = 0;
        foreach ($segmentTypes as $segmentType) {
            if (empty($employeesChunk[$index])) break;

            $segment = $segmentType->segments()->where('client_id', $this->clientDemo->id)->first();
            if (empty($segment)) continue;
            
            $employeeCodes = [];
            foreach ($employeesChunk[$index] as $item) {
                $checkUsSegment = SegmentUser::where(['user_id' => $item->id, 'segment_id' => $segment->id])->first();
                if (empty($checkUsSegment)) {
                    $employeeCodes[] = $item->code;
                }
            }
            if (!empty($employeeCodes)) {
                $params = [
                    'segment_code' => $segment->code,
                    'users' => $employeeCodes,
                    'segment_type_code' => $segmentType->code
                ];
                app(AddSegmentEmployee::class)->execute($params);
            }

            $index++;
        }
        return true;
    }

    /**
     * Create Schedule
     *
     * @return bool
     */
    private function createSchedule()
    {
        $frequencyWeekly = ScheduleFrequency::where('name', 'Weekly')->first();
        $frequencyWeeklySetting = $frequencyWeekly->frequencySetting()->first();

        $segments = Segment::where('client_id', $this->clientDemo->id)->get();
        $segmentCodes = $segments->pluck('code')->toArray();

        if (empty($frequencyWeekly) || empty($segmentCodes)) return;

        $weekComplete = !empty($frequencyWeeklySetting) ? $frequencyWeeklySetting->week_complete : 0;
        $startDate = Carbon::now()->subMonths(2);
        $renewDate = (clone $startDate)->addDays($weekComplete * 7 + 1);

        $params = [
            'name' => 'Demo schedule',
            'description' => 'Demo schedule',
            'schedule_frequency_id' => $frequencyWeekly->id,
            'send_to' => implode(",", $segmentCodes),
            'start_date' => $startDate->format('Y-m-d'),
            'renewal_date' => $renewDate->format('Y-m-d')
        ];

        // -------------------------------------------------------------------------------//
        // Schedule Survey
        $params['created_at'] = Carbon::now()->subMonths(2);
        $params['updated_at'] = Carbon::now()->subMonths(2);
        $survey = app(CreateSchedule::class)->execute($params);
        $survey->save();

        $this->survey = $survey;

        // Schedule Survey Segment
        $surveySegmentParams = array_merge($params, ['schedule_survey_id' => $survey->id]);
        app(CreateSegments::class)->execute($surveySegmentParams);

        // Schedule Frequency Round (2 round weekly for 12 questions)
        $firstRound = ScheduleFrequencyRound::create([
            'schedule_survey_id' => $survey->id,
            'start_date' => $startDate,
            'expire_date' => (clone $startDate)->addWeek(),
            'created_by' => $this->adminDemo->id,
            'status' => 1,
            'created_at' => $startDate,
            'updated_at' => $startDate
        ]);

        $startDateSecondRound = (clone($firstRound->expire_date))->addDay();
        $secondRound = ScheduleFrequencyRound::create([
            'schedule_survey_id' => $survey->id,
            'start_date' => $startDateSecondRound,
            'expire_date' => (clone $startDateSecondRound)->addWeek(),
            'created_by' => $this->adminDemo->id,
            'status' => 1,
            'created_at' => $startDate,
            'updated_at' => $startDate
        ]);

        // Schedule Round Questionnaire
        $publishedQuestionIds = $this->publishedQuestions->pluck('id')->toArray();
        $roundOneQuestionIds = $this->roundOneQuestions->pluck('id')->toArray();
        $sortedQuestionIds = array_unique(array_merge($roundOneQuestionIds, $publishedQuestionIds));

        $publishedQuestionChunk = array_chunk($sortedQuestionIds, 7);
        $employees = $this->clientDemo->users()->role(Roles::FRONT_USER)->where('is_demo',true)->get();
        $clientValues = ClientValue::client($this->clientDemo->id)->get();
        $questionValue = Questionnaire::questionValue()->first();

        foreach ($employees as $employee) {
            $publishedQuestionChunkFirst = $publishedQuestionChunk[0];
            $publishedQuestionChunkSecond = $publishedQuestionChunk[1];

            $roundQuestions = [
                $firstRound->id => array_merge($publishedQuestionChunkFirst, [$questionValue->id]),
                $secondRound->id => array_merge($publishedQuestionChunkSecond, [$questionValue->id])
            ];
            $publishedQuestionChunkFirstClone = array_merge($publishedQuestionChunkFirst, []);
            $publishedQuestionChunkSecondClone = array_merge($publishedQuestionChunkSecond, []);
            $roundQuestionsValue = [
                $firstRound->id => array_merge(data_set($publishedQuestionChunkFirstClone, '*', null), [$clientValues[0]->id]),
                $secondRound->id => array_merge(data_set($publishedQuestionChunkSecondClone, '*', null), [$clientValues[1]->id])
            ];

            $roundQuestionData = [
                'schedule_survey_id' => $this->survey->id,
                'questionnaire_id' => json_encode($roundQuestions),
                'user_id' => $employee->id,
                'status' => 1,
                'client_value' => json_encode($roundQuestionsValue),
                'created_at' => $startDate,
                'updated_at' => $startDate
            ];
            ScheduleRoundQuestionnaire::create($roundQuestionData);
        }

        return $survey;
    }

    /**
     * Create answers
     *
     * @return bool
     */
    private function createAnswerQuestions()
    {
        if (empty($this->survey)) return false;

        $scheduleRoundQuestions = ScheduleRoundQuestionnaire::where([
            'schedule_survey_id' => $this->survey->id
        ])->get();
        $drivers = Driver::with('questionnaire')->get();
        $driversMap = $drivers->mapWithKeys(function($item) {
            return [$item['questionnaire']['id'] => $item];
        });
        $questionValue = Questionnaire::questionValue()->first();

        foreach ($scheduleRoundQuestions as $indexUsRound => $userRound) {
            $roundsQuestion = $userRound->questionnaire_id;
            $clientValue = $userRound->client_value;

            $countRound = 1;
            $answersData = [];
            foreach ($roundsQuestion as $roundId => $questionIds) {
                if ($countRound > 1)  break;

                $answersData['round_id'] = $roundId;
                $answers = [];
                $answerQuestionCreatedAt = $indexUsRound <= 1
                    ? Carbon::now()->subMonths(2)
                    : ($indexUsRound > 1 && $indexUsRound <= 3
                        ? Carbon::now()->subMonths(1)
                        : Carbon::now());
                foreach ($questionIds as $qIndex => $questionId) {
                    $answerQuestionScore = $indexUsRound <= 1 ? rand(0, 6) : ($indexUsRound > 1 && $indexUsRound <= 3 ? rand(7,8) : rand(9,10));
                    $comment = 'U.S. COVID DEATH TOLL NEARS 500,000';

                    if ($this->userDemo->id == $userRound->user_id) {
                        if ($questionId == $questionValue->id) {
                            $findClientDemoData = Arr::first(Arr::where($this->roundOneDriver, function ($item) {
                                return $item['driver_name'] == null;
                            }));
                        } else {
                            $getDriver = $driversMap[$questionId];
                            $findClientDemoData = Arr::first(Arr::where($this->roundOneDriver, function ($item) use ($getDriver) {
                                return $item['driver_name'] == $getDriver['name'];
                            }));
                        }

                        if (!empty($findClientDemoData)) {
                            $answerQuestionScore = $findClientDemoData['point'];
                            $comment = $findClientDemoData['comment'];
                        }
                    }

                    $answers[] = [
                        'questionnaire_id' => $questionId,
                        'point' => $answerQuestionScore,
                        'comment' => $comment,
                        'skip_question_id' => null,
                        'client_value_id' => !empty($clientValue[$roundId]) ? $clientValue[$roundId][$qIndex] : null,
                        'created_at' => $answerQuestionCreatedAt,
                        'updated_at' => $answerQuestionCreatedAt
                    ];
                }
                $answersData['answers'] = $answers;
                $countRound++;
            }
            $clientMobileDemo = User::find($userRound->user_id);

            if (empty($clientMobileDemo)) continue;

            Context::user()->set($clientMobileDemo);
            app(HandleAnswerQuestions::class)->execute($answersData);
        }

        Context::user()->set($this->adminDemo);
        return true;
    }
}
