<?php

namespace App\Console\Commands;

use App\Enum\AccessPermissions;
use App\Enum\EmailNotifications;
use App\Enum\NotificationTypes;
use App\Enum\Roles;
use App\Helpers\EventHelper;
use App\Jobs\SendNotification;
use App\Jobs\SendNotificationConsole;
use App\Jobs\SendNotificationEmail;
use App\Models\Event\Event;
use App\Models\User\User;
use App\Services\Timezone\GetTimezoneByHour;
use App\Services\UserAccess\GetAccessByRole;
use Illuminate\Console\Command;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Auth;

class EventDailyNotification extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'event-daily:notification';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Create notification for scheduled events';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        $timezones = app(GetTimezoneByHour::class)->execute(['hour' => 9]);

        if ($timezones->count() == 0)
            return true;
        $startTime = (new Carbon())->setTimezone($timezones->first())->startOfDay()->setTimezone('UTC')->toDateTimeString();
        $endTime = (new Carbon())->setTimezone($timezones->first())->endOfDay()->setTimezone('UTC')->toDateTimeString();
        $events = Event::where('is_published', 1)
            ->whereBetween('start_date', [$startTime, $endTime])
            ->get();
        if ($events->count() == 0)
            return true;

        foreach ($events as $event) {
            $sendTime = Carbon::now()->setMinute(0);
            $userCodes = EventHelper::getUserCodesByEvent($event, $timezones)->toArray();
            if (empty($userCodes))
                continue;
            $arguments = array(
                'sender' => Auth::user()->id,
                "userIds" => $userCodes,
                "headings" => __("Upcoming event"),
                "contents" => __($event->name . ", " . Carbon::instance($event->start_date)->diffForHumans($sendTime)),
                "data" => [
                    "type" => NotificationTypes::EVENT_UPCOMING,
                    'code' => $event->code
                ],
                'send_after' => null
            );
            SendNotification::dispatch($arguments);


            // Send Invite Notifications to employees
            $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
            $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

            $userArr = User::whereIn('code', $userCodes)->get();
            $owner = $event->creator;
            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_CONSOLE_UPCOMING]) {
                foreach ($userArr as $user) {
                    SendNotificationConsole::dispatch($owner, $user, 'Upcoming Event reminder');
                }
            }
            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_UPCOMING]) {
                foreach ($userArr as $user) {
                    SendNotificationEmail::dispatch($user, EmailNotifications::EVENT_UPCOMING, ['event' => $event->name, 'date' => $event->start_date . '  ~  ' . $event->end_date]);
                }
            }
        }

        return 0;
    }
}