<?php

namespace App\Console\Commands;

use App\Enum\AccessPermissions;
use App\Enum\AlertMessage as AlertMessageEnum;
use App\Enum\Roles;
use App\Models\AlertMessage\AlertMessage;
use App\Models\Event\Event;
use Illuminate\Console\Command;
use Illuminate\Support\Carbon;
use Spatie\Permission\Models\Permission;
use Spatie\Permission\Models\Role;

class EventUpcomingAlert extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'event:upcoming';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Create alert for upcoming event ';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        $role = Role::firstWhere('name', Roles::ADMIN);
        $permission = Permission::firstWhere('name', AccessPermissions::NOTIFICATION_CONSOLE_UPCOMING);
        $events = Event::join("client_access", function($join) use ($role, $permission) {
            $join->on("client_access.client_id", "events.client_id");
            $join->where("client_access.permission_id", $permission->getKey());
            $join->where("client_access.role_id", $role->getKey());
        })->where('is_published', 1)
            ->where('is_full_day', 0)
            ->where(function($subQuery) {
                $subQuery->where(function($sub){
                    $sub->where('end_date', ">=", Carbon::now())
                        ->where('start_date', "<=", Carbon::now()->addHours(2))
                        ->whereNull('alert_at');
                });
                $subQuery->orWhere(function($sub){
                    $sub->where('end_date', ">=", Carbon::now())
                        ->where('alert_at', "<=", Carbon::now());
                });
            })->get();

        if ($events->count() == 0) return true;
        foreach ($events as $event) {
            $client = $event->client;
            if (!$client['NOTIFICATION_CONSOLE_UPCOMING'] && !$event['alert_at']) {
                break;
            }
            $parts = explode(" ", $client['NOTIFICATION_CONSOLE_UPCOMING']);
            switch ($parts[1]) {
                case 'days':
                    $limit = Carbon::now()->addDays($parts[0]);
                    break;
                case 'hours':
                    $limit = Carbon::now()->addHours($parts[0]);
                    break;
                case 'minutes':
                    $limit = Carbon::now()->addMinutes($parts[0]);
                    break;
                default:
                    $limit = Carbon::now();
                    break;
            }
            if ($event['start_date'] > $limit) {
                break;
            }
            $alertData = [
                'message' => AlertMessageEnum::UPCOMING_EVENT,
                'type' => AlertMessageEnum::AUTOMATIC_TYPE,
                'payload' => json_encode([
                    'type' => 'event',
                    'id' => $event->getKey()
                ])
            ];
            AlertMessage::updateOrCreate($alertData, [
                'expire_on' => $event->end_date,
                'display_on' => Carbon::now()
            ]);

        }
    }
}
