<?php

namespace App\Console\Commands;

use App\Enum\JourneyUserPendingStatuses;
use App\Enum\NotificationTypes;
use App\Facades\Context;
use App\Http\Resources\Mobile\Journey\JourneyUserPendingStatus;
use App\Jobs\SendNotification;
use App\Models\Journey\Journey;
use App\Models\User\User;
use App\Services\Mobile\Journey\GetJourneyUserCurrentStep;
use App\Services\Mobile\Journey\IsFirstStep;
use App\Services\Mobile\Journey\IsStepAvailable;
use App\Services\Timezone\GetTimezoneByHour;
use Illuminate\Console\Command;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Auth;

class JourneyDailyNotification extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'journey-daily:notification';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Create notification for scheduled journeys';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {

        // Sent pending journeys
        $journeys = Journey::where('is_published', 1)
            ->get();

        $timezones = app(GetTimezoneByHour::class)->execute(['hour' => 9]);
        if ($journeys->count() == 0 || $timezones->count() == 0) return true;
        foreach ($journeys as $journey) {
            $users = $journey->users()->wherePivot('is_completed', 0)->byTimezone($timezones)->get();
            $results = [];
            foreach ($users as $user) {
                Context::user()->set($user);
                $currentStep = $this->getJourneyUserCurrentStep($journey, $user);
                if ($currentStep) {
                    $isAvailable = app(IsStepAvailable::class)->execute(['step' => $currentStep]);
                    $isFirstStep = app(IsFirstStep::class)->execute(['step' => $currentStep]);
                    //If current step is available
                    $status = '';
                    if ($isAvailable['status']) {
                        $availableDay = (new Carbon($isAvailable['availableDay']))->startOfDay();
                        if ($availableDay->lessThan(Carbon::today())) {
                            $status = JourneyUserPendingStatuses::PENDING;
                        } else if ($availableDay->equalTo(Carbon::today())) {
                            $status = $isFirstStep ? JourneyUserPendingStatuses::ONBOARDING_DAY : JourneyUserPendingStatuses::AVAILABLE_STEP;
                        }
                    } else {
                        if ($isFirstStep && isset($isAvailable['days']) && $isAvailable['days'] === 1) {
                            $status = JourneyUserPendingStatuses::BEFORE_ONBOARDING_DAY;
                        } else {
                            $status = JourneyUserPendingStatuses::WAITING;
                        }
                    }
                    $results[$status][] = $user->code;
                }
            }
            $this->sendNotificationByStatus($journey, $results);
        }
    }

    public function getJourneyUserCurrentStep(Journey $journey, User $user)
    {
        return app(GetJourneyUserCurrentStep::class)->execute($journey, $user);
    }

    public function sendNotificationByStatus($journey, $journeyUsers)
    {
        foreach ($journeyUsers as $key => $userIds) {
            if (sizeof($userIds) === 0)
                continue;
            $arguments = array();
            switch ($key) {
                case JourneyUserPendingStatuses::PENDING:
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => $userIds,
                        "headings"   => __("A journey is pending"),
                        "contents"   => __("Tap to get started"),
                        "data"       => [
                            "type" => NotificationTypes::JOURNEY_PENDING,
                            'code' => $journey->code,
                            'id'   => $journey->id
                        ],
                        'send_after' => null
                    );
                    break;
                case JourneyUserPendingStatuses::AVAILABLE_STEP:
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => $userIds,
                        "headings"   => __("New journey step available"),
                        "contents"   => __("You can view it right now"),
                        "data"       => [
                            "type" => NotificationTypes::JOURNEY_STEP_AVAILABLE,
                            "code" => $journey->code,
                            "id"   => $journey->id
                        ],
                        'send_after' => null
                    );
                    break;
                case JourneyUserPendingStatuses::ONBOARDING_DAY:
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => $userIds,
                        "headings"   => __("You're starting today"),
                        "contents"   => __("We hope you have an amazing day!"),
                        "data"       => [
                            "type" => NotificationTypes::JOURNEY_STARTED,
                            "code" => $journey->code,
                            "id"   => $journey->id
                        ],
                        'send_after' => null
                    );
                    break;
                case JourneyUserPendingStatuses::BEFORE_ONBOARDING_DAY:
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => $userIds,
                        "headings"   => __("You're starting tomorrow"),
                        "contents"   => __("You're almost there!"),
                        "data"       => [
                            "type" => NotificationTypes::JOURNEY_UPCOMING,
                            'code' => $journey->code,
                            'id'   => $journey->id
                        ],
                        'send_after' => null
                    );
                    break;
            }
            if (!empty($arguments)) {
                SendNotification::dispatch($arguments);
            }
        }
    }
}
