<?php


namespace App\Console\Commands;


use App\Enum\ConversationTypes;
use App\Enum\NotificationTypes;
use App\Facades\Context;
use App\Jobs\SendNotification;
use App\Mail\BirthdayNotification;
use App\Mail\WorkAnniversaryNotification;
use App\Models\User\User;
use App\Services\Message\CreateConversation;
use App\Services\Message\CreateMessage;
use Carbon\Carbon;
use Illuminate\Console\Command;
use App\Enum\AccessPermissions;
use App\Services\UserAccess\GetAccessByRole;
use App\Enum\Roles;
use App\Jobs\SendNotificationConsole;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;

class ManagerDailyNotification extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'manager-daily:notification';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Create daily notification for manager';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     */
    public function handle()
    {
        //Get all user have birthday today
        $userBirthdays = User::active()->whereHas('employee', function($query) {
            $query->whereDay('date_of_birth', Carbon::today()->day);
            $query->whereMonth('date_of_birth', Carbon::today()->month);
            $query->where('manager_id', '>', 0);
        })->with('employee')->get()->groupBy('employee.manager_id')->all();

        //Get all user have working anniversary today
        $userAnniversary = User::active()->whereHas('employee', function($query) {
            $query->whereDay('start_date', Carbon::today()->day);
            $query->whereMonth('start_date', Carbon::today()->month);
            $query->where('manager_id', '>', 0);
        })->with('employee')->get()->groupBy('employee.manager_id')->all();

        $managersIds = collect(array_merge(array_keys($userBirthdays), array_keys($userAnniversary)))->unique();

        //Get all managers
        $managers = User::active()->where('is_demo', 0)->with('employee')->whereIn('id', $managersIds->toArray())->get();

        foreach ($managers as $manager) {
            Context::user()->set($manager);
            Context::client()->set($manager->employee->client);
            $managerAccess = app(GetAccessByRole::class)->execute(['role' => Roles::MANAGER]);
            $accessManagerPermissions = $managerAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];
            // Send at 9AM
            $sendAfter = Carbon::today()->setTimezone($manager->employee->timezone)->hour(9);
            if ($accessManagerPermissions[AccessPermissions::NOTIFICATION_EMAIL_BIRTHDAY] && isset($userBirthdays[$manager->getKey()])) {
                foreach ($userBirthdays[$manager->getKey()] as $user) {
                    Mail::to($manager->email)->send(new BirthdayNotification($manager, $user));
                }
            }
            if ($accessManagerPermissions[AccessPermissions::NOTIFICATION_CONSOLE_BIRTHDAY] && isset($userBirthdays[$manager->getKey()])) {
                foreach ($userBirthdays[$manager->getKey()] as $user) {
                    SendNotificationConsole::dispatch($user, $manager, 'Employee birthday');
                }
            }
            if ($accessManagerPermissions[AccessPermissions::NOTIFICATION_APP_BIRTHDAY] && isset($userBirthdays[$manager->getKey()])) {
                foreach ($userBirthdays[$manager->getKey()] as $user) {
                    $conversation = app(CreateConversation::class)
                        ->execute(['type' => ConversationTypes::DIRECT_MESSAGE, 'receiver_code' => $user->getBusinessKey()]);
                    app(CreateMessage::class)->execute(['content' => __(
                        "notify.employee_birthday",
                        ["first_name" => $user->first_name, "surname" => $user->surname]
                    ), 'conversation_id' => $conversation->id]);
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => array($manager->getBusinessKey()),
                        "headings"   => __("notify.employee_birthday",
                            ["first_name" => $user->first_name, "surname" => $user->surname]),
                        "contents"   => __('Tap here to open message'),
                        "data"       => [
                            "type"            => NotificationTypes::MESSAGE_RECEIVED,
                            "conversation_id" => $conversation->id
                        ],
                        'send_after' => $sendAfter
                    );
                    SendNotification::dispatch($arguments);
                }
            }
            if ($accessManagerPermissions[AccessPermissions::NOTIFICATION_EMAIL_WORK_ANNIVERSARY] && isset($userAnniversary[$manager->getKey()])) {
                foreach ($userAnniversary[$manager->getKey()] as $user) {
                    Mail::to($manager->email)->send(new WorkAnniversaryNotification($manager, $user));
                }
            }
            if ($accessManagerPermissions[AccessPermissions::NOTIFICATION_CONSOLE_WORK_ANNIVERSARY] && isset($userAnniversary[$manager->getKey()])) {
                foreach ($userAnniversary[$manager->getKey()] as $user) {
                    SendNotificationConsole::dispatch($user, $manager, 'Work anniversary');
                }
            }
            if ($accessManagerPermissions[AccessPermissions::NOTIFICATION_APP_WORK_ANNIVERSARY] && isset($userAnniversary[$manager->getKey()])) {
                foreach ($userAnniversary[$manager->getKey()] as $user) {
                    $yearNumber = Carbon::today()->diffInYears($user->employee->start_date);
                    $conversation = app(CreateConversation::class)
                        ->execute(['type' => ConversationTypes::DIRECT_MESSAGE, 'receiver_code' => $user->getBusinessKey()]);
                    app(CreateMessage::class)->execute(['content' => __("notify.employee_anniversary",
                        ["first_name"  => $user->first_name,
                            "surname"     => $user->surname,
                            "year_number" => $yearNumber,
                            "year"        => $yearNumber > 1 ? "years" : "year"
                        ]), 'conversation_id' => $conversation->id]);
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => array($manager->getBusinessKey()),
                        "headings"   => __("notify.employee_anniversary",
                            ["first_name"  => $user->first_name,
                             "surname"     => $user->surname,
                             "year_number" => $yearNumber,
                             "year"        => $yearNumber > 1 ? "years" : "year"
                            ]),
                        "contents"   => __('Tap here to open message'),
                        "data"       => [
                            "type"            => NotificationTypes::MESSAGE_RECEIVED,
                            "conversation_id" => $conversation->id
                        ],
                        'send_after' => $sendAfter
                    );
                    SendNotification::dispatch($arguments);
                }
            }
        }

        return 0;
    }
}