<?php

namespace App\Console\Commands;

use App\Enum\ActionRequestTypes;
use App\Enum\LibraryItemType;
use App\Enum\QualeeStatus;
use App\Enum\QuestionTypes;
use App\Enum\Roles;
use App\Facades\Context;
use App\Helpers\ResourceLocationBuilder;
use App\Models\ActionRequest\ActionRequest;
use App\Models\Event\Event;
use App\Models\Journey\JourneyStep;
use App\Models\Library\DocumentProfile;
use App\Models\Library\LibraryItem;
use App\Models\Quiz\Quiz;
use App\Models\Quiz\QuizOption;
use App\Models\Quiz\QuizQuestion;
use App\Models\Survey\Survey;
use App\Models\Survey\SurveyOption;
use App\Models\Survey\SurveyQuestion;
use App\Models\User\Employee;
use App\Models\User\User;
use App\Models\Client\Client;
use App\Services\Attachment\CreateAttachment;
use App\Services\Library\CreateLibraryAttachment;
use App\Services\Mobile\Journey\SubmitStepDetails;
use App\Services\Mobile\Setting\UpdateAvatar;
use App\Services\Subscription\CreateSubscriptionPlan;
use App\Services\Subscription\UpdateSubscriptionPlan;
use App\Services\Tool\Journey\AddEmployees;
use App\Services\Tool\Journey\CreateJourney;
use App\Services\Tool\Journey\PublishJourney;
use App\Services\Tool\Journey\SaveJourneyStep;
use App\Services\Tool\Quiz\CreateQuiz;
use App\Services\Tool\Quiz\SaveQuizQuestion;
use App\Services\Tool\Survey\AttachQuestion;
use App\Services\Tool\Survey\CreateSurvey;
use Exception;
use Illuminate\Console\Command;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;

/**
 * Class MigrateDataV2ToV3
 * @package App\Console\Commands
 */
class MigrateDataV2ToV3 extends Command
{
    /**
     * @var array
     */
    protected $listQuizzesV2 = array();
    /**
     * @var array
     */
    protected $listSurveysV2 = array();
    /**
     * @var array
     */
    protected $listLibraryItemV2 = array();
    /**
     *
     */
    const SURVEY_TABLES = [
        'survey_names' => 'survey_names_bck20201102',
        'survey_questions' => 'survey_questions_bck20201102',
        'survey_answers' => 'survey_answers_bck20201102',
    ];

    /**
     *
     */
    const QUIZ_TABLES = [
        'quiz_names' => 'quiz_names_bck',
        'questions' => 'questions_bck',
        'question_answers' => 'question_answers_bck'
    ];

    /**
     *
     */
    const JOURNEY_TABLES = [
        'configure_clients' => 'configure_clients_bck20201106',
        'configure_client_steps' => 'configure_client_steps_bck20201106',
        'configure_client_content_steps' => 'configure_client_content_steps_bck20201106'
    ];

    /**
     *
     */
    const CLIENT_CONFIGURATION_TABLES = [
        'client_configurations' => 'client_configurations_bak',
        'client_configuration_detail' => 'client_configuration_detail_bak'
    ];

    /**
     *
     */
    const PREMIUM_PLAN = [
        "name" => "Basic",
        "description" => "Subscription for Basic",
        "cost" => 1000000,
        "price_per_employee" => 150,
        "stripe_price" => "price_stater",
        "stripe_prod" => "price_stater",
        "is_public" => 1,
        "status" => "active",
        "config" => [
            "package_employee" => 100000,
            "package_journey" => 99,
            "package_document" => 99,
            "package_quiz" => 99,
            "package_survey" => 99,
            "package_administrator" => 9,
            "package_support_type" => "email",
            "package_dedicated_engagement_specialist" => 1,
            "package_engagement_module" => 1,
            "package_ad_free" => 1,
            "package_custom_colour" => 1,
            "package_message" => 1,
            "package_form" => 1,
            "package_checklist" => 1,
            "package_ai_document_search" => 1,
            "package_id_verification" => 1,
            "package_stock_surveys" => 1,
        ]
    ];


    /**
     *
     */
    const STARTER_PLAN = [
        "name" => "Basic",
        "description" => "Subscription for Basic",
        "cost" => 10000,
        "price_per_employee" => 1.5,
        "stripe_price" => "price_stater",
        "stripe_prod" => "price_stater",
        "is_public" => 1,
        "status" => "active",
        "config" => [
            "package_employee" => 10,
            "package_journey" => 3,
            "package_document" => 10,
            "package_quiz" => 3,
            "package_survey" => 3,
            "package_administrator" => 1,
            "package_support_type" => "email",
            "package_dedicated_engagement_specialist" => 0,
            "package_engagement_module" => 0,
            "package_ad_free" => 0,
            "package_custom_colour" => 0,
            "package_message" => 0,
            "package_form" => 0,
            "package_checklist" => 0,
            "package_ai_document_search" => 0,
            "package_id_verification" => 0,
            "package_stock_surveys" => 0,
        ]
    ];

    private $currentUser;

    const MODELS_BUSINESS_KEY = [Client::class, Event::class, User::class];
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'qualee:migratev2tov3';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command will migrate data from v2 to v3';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     * @throws Exception
     */
    public function handle()
    {
        set_time_limit(0);
        DB::transaction(function() {
            $superAdmin = User::role(Roles::SUPER_ADMIN)->first();
            if (!$superAdmin) {
                $this->info(
                    "Does not have super admin, please run db:seed first"
                );
                return;
            }

            //Create default plan
            $this->setUserContext($superAdmin->id);
            $this->currentUser =  $superAdmin;

            $subscriptionPlan = app(CreateSubscriptionPlan::class)->execute(self::PREMIUM_PLAN);

            $clients = Client::all();
            $clientConfigurationId = DB::table(self::CLIENT_CONFIGURATION_TABLES['client_configurations'])
                ->where('name', 'company_logo')
                ->whereNull('deleted_at')
                ->first()->id;
            $this->deleteErrorEmployee();
            $this->migrateBusinessKey();
            $this->migrateUserData();
            foreach ($clients as $client) {
                $admin = $client->adminUsers()->first();
                if ($admin) {
                   $this->currentUser =  $admin;
                } else {
                    $this->currentUser =  $superAdmin;
                }
                Context::client()->set($client);
                $this->migrateClientData($client, $subscriptionPlan->getKey(), $clientConfigurationId);
            }
            app(UpdateSubscriptionPlan::class)->execute(array_merge(self::STARTER_PLAN, ['code' => $subscriptionPlan->code]));
            Artisan::call('qualee:createdemodata');
            Artisan::call('db:seed --class=LanguageSeeder --force');
            Artisan::call('db:seed --class=QuestionnaireSeeder --force');
            Artisan::call('db:seed --class=QuestionnaireSkipSeeder --force');
            Artisan::call('db:seed --class=QuestionValueTranslateSeeder --force');
            Artisan::call('db:seed --class=ScheduleFrequencySeeder --force');
            Artisan::call('engagement:create-client-demo-data');
            Artisan::call('reindex:powersearch');
        });

    }

    /**
     * @param $client
     * @param $planId
     * @param $clientConfigurationId
     */
    public function migrateClientData($client, $planId, $clientConfigurationId)
    {
        $client->plan_id = $planId;
        $client->status = $client->status == 1 ? QualeeStatus::ACTIVE : QualeeStatus::INACTIVE;
        Log::stack(['stdout'])->info("Start migrate data for client: " . $client->name);
        //Get company logo configuration id
        $companyLogo = DB::table(self::CLIENT_CONFIGURATION_TABLES['client_configuration_detail'])
            ->where('configuration_id', $clientConfigurationId)
            ->where('client_id', $client->id)
            ->first()->value;
        $url = "https://console.qualee.com/images/client/company_logo/" . rawurlencode($companyLogo);
        $info = pathinfo($url);

        try {
            $contents = file_get_contents($url);
            $file = 'tmp/' . $info['basename'];
            Storage::disk('local')->put($file, $contents);
            $uploadedFile = new UploadedFile(storage_path("app/" . $file), $info['basename']);

            $fileDir = (new ResourceLocationBuilder())->setClient($client->code)->getLogoPath();
            $logo = app(CreateAttachment::class)->execute($uploadedFile, $fileDir, Context::user()->get()->getKey());
            $client->logo()->associate($logo);
            Storage::disk('local')->delete($file);

        } catch (Exception $e) {

        }

        $this->migrateUserDataByClient($client);
        $this->migrateQuizDataByClient($client);
        $this->migrateSurveyDataByClient($client);
        $this->migrateDocumentDataByClient($client);
        $this->migrateJourneyDataByClient($client);
        $client->save();
        Log::stack(['stdout'])->info("Finish migrate data for client: " . $client->name);

    }

    public function migrateUserData()
    {
        User::query()->whereNull('first_name')->update(['first_name' => '']);
        User::query()->whereNull('surname')->update(['surname' => '']);
        $users = User::all();
        foreach ($users as $user) {
            if ($user->profile_pic) {

            }
            if (!empty($user->first_name)) {
                continue;
            }
            if (!empty($user->name)) {
                $user->first_name = $user->name;
                $user->save();
            } else if (!empty($user->email)) {
                $user->first_name = $user->email;
                $user->save();
            }
        }
        //Migrate user avatar
    }

    /**
     *
     */
    public function deleteErrorEmployee()
    {
        $userDeleted = Employee::whereNotIn('client_id', Client::all()->pluck('id'))->get()->pluck('user_id');
        $userDeleted = $userDeleted->concat(User::where('is_demo', 1)->get()->pluck('id'));
        User::whereIn('id', $userDeleted)->forceDelete();
        Employee::whereIn('user_id', $userDeleted)->forceDelete();
    }

    /**
     *
     */
    public function migrateBusinessKey()
    {
        foreach (self::MODELS_BUSINESS_KEY as $model) {
            $model::genarateMissingBusinessKey();
        }
    }

    /**
     * @param $client
     */
    public function migrateUserDataByClient($client)
    {
        $users = User::query()->whereHas('employee', function($query) use ($client) {
            $query->where('client_id', $client->getKey());
        })->where('is_demo', 0)->get();
        foreach ($users as $user) {
            if ($user->is_default == 1 && !!$user->stripe_id) {
                $client->stripe_cus_id = $user->stripe_id;
                $client->save();
            }


            $employee = $user->employee;
            $employee->status = QualeeStatus::ACTIVE;

            if ($employee->profile_pic) {
                if ($user->hasRole(Roles::ADMIN)) {
                    $url = "https://console.qualee.com/images/admin/profile/" . rawurlencode($employee->profile_pic);
                } else {
                    $url = "https://console.qualee.com/images/user/profile_pic/" . rawurlencode($employee->profile_pic);
                }
                $info = pathinfo($url);

                try {
                    $contents = file_get_contents($url);
                    $file = 'tmp/' . $info['basename'];
                    Storage::disk('local')->put($file, $contents);
                    $uploadedFile = new UploadedFile(storage_path("app/" . $file), $info['basename'], null, null, true);
                    $this->setUserContext($user->id);
                    app(UpdateAvatar::class)->execute(['avatar' => $uploadedFile]);
                    Storage::disk('local')->delete($file);
                } catch (Exception $e) {

                }
            }

            $employee->save();
        }
    }

    /**
     * @param $client
     */
    public function migrateQuizDataByClient($client)
    {
        $rawQuizzes = DB::table(self::QUIZ_TABLES['quiz_names'])
            ->where("client_id", $client->id)
            ->whereNull('deleted_at')
            ->get();
        foreach ($rawQuizzes as $rawQuiz) {
            $this->setUserContext($rawQuiz->added_by);
            $quiz = app(CreateQuiz::class)->execute(['name' => $rawQuiz->name]);
            $this->listQuizzesV2[$rawQuiz->id] = $quiz->id;
            $quizResult = array();
            $quizResult['code'] = $quiz->code;
            $quizQuestions = DB::table(self::QUIZ_TABLES['questions'])->whereNull('deleted_at')->where("quiz_name_id", $rawQuiz->id)->get();
            foreach ($quizQuestions as $quizQuestion) {
                $quizQuestionResult = array();
                $quizQuestionResult['title'] = $quizQuestion->title;
                $quizAnswer = DB::table(self::QUIZ_TABLES['question_answers'])
                    ->whereNull('deleted_at')
                    ->where("question_id", $quizQuestion->id)->first();
                $options = array();
                $options[] = ["content" => $quizAnswer->option_one];
                $options[] = ["content" => $quizAnswer->option_two];
                $options[] = ["content" => $quizAnswer->option_three];
                $options[] = ["content" => $quizAnswer->option_four];
                $quizQuestionResult['correct_option'] = 0;
                foreach ($options as $key => $option) {
                    if ($option['content'] === $quizAnswer->correct) {
                        $quizQuestionResult['correct_option'] = $key;
                        break;
                    }
                }
                $quizQuestionResult['options'] = $options;
                $quizResult['questions'][] = $quizQuestionResult;
            }
            if (isset($quizResult['questions'])) {
                $quiz = app(SaveQuizQuestion::class)->execute($quizResult);
                $quiz->is_published = true;
                $quiz->save();
            }
        }
    }

    /**
     * @param $client
     */
    public function migrateSurveyDataByClient($client)
    {
        $rawSurveys = DB::table(self::SURVEY_TABLES['survey_names'])
            ->where("client_id", $client->id)
            ->whereNull('deleted_at')->get();
        foreach ($rawSurveys as $rawSurvey) {
            $this->setUserContext($rawSurvey->added_by);
            $survey = app(CreateSurvey::class)->execute(['name' => $rawSurvey->name]);
            $this->listSurveysV2[$rawSurvey->id] = $survey->id;
            $surveyResult = array();
            $surveyResult['code'] = $survey->code;
            $surveyQuestions = DB::table(self::SURVEY_TABLES['survey_questions'])
                ->whereNull('deleted_at')
                ->where("survey_name_id", $rawSurvey->id)->get();
            foreach ($surveyQuestions as $surveyQuestion) {
                $surveyQuestionResult = array();
                switch ($surveyQuestion->survey_question_type_id) {
                    case 1:
                        $surveyQuestionResult['type'] = QuestionTypes::TYPE_MULTI;
                        break;
                    case 2:
                        $surveyQuestionResult['type'] = QuestionTypes::TYPE_TEXT;
                        break;
                    case 3:
                        $surveyQuestionResult['type'] = QuestionTypes::TYPE_RATING;
                        break;
                }
                $surveyQuestionResult['title'] = $surveyQuestion->title;
                if ($surveyQuestionResult['type'] === QuestionTypes::TYPE_MULTI) {
                    $surveyAnswer = DB::table(self::SURVEY_TABLES['survey_answers'])
                        ->whereNull('deleted_at')
                        ->where("survey_question_id", $surveyQuestion->id)->first();
                    $options = array();
                    $options[] = ["content" => $surveyAnswer->option_one];
                    $options[] = ["content" => $surveyAnswer->option_two];
                    $options[] = ["content" => $surveyAnswer->option_three];
                    $options[] = ["content" => $surveyAnswer->option_four];
                    $surveyQuestionResult['options'] = $options;
                }

                $surveyResult['questions'][] = $surveyQuestionResult;
            }
            if (isset($surveyResult['questions'])) {
                $survey = app(AttachQuestion::class)->execute($surveyResult);
                $survey->is_published = true;
                $survey->save();
            }
        }
    }

    /**
     * @param $client
     */
    public function migrateDocumentDataByClient($client)
    {
        $rawLibraryItems = DB::table('documents')
            ->where("client_id", $client->id)
            ->whereNull('deleted_at')
            ->get();
        foreach ($rawLibraryItems as $rawLibraryItem) {
            $this->setUserContext($rawLibraryItem->added_by);
            $documentProfile = DocumentProfile::find($rawLibraryItem->document_profile_id);
            if ($documentProfile) {
                $libraryItem = array();
                $url = "https://console.qualee.com//storage/app/public/documents/" . rawurlencode($rawLibraryItem->file_name);

                $info = pathinfo($url);
                $contents = file_get_contents($url);

                $info['basename'] = rawurldecode($info['basename']);

                $file = 'tmp/' . $info['basename'];
                Storage::disk('local')->put($file, $contents);
                $uploadedFile = new UploadedFile(storage_path("app/" . $file), $info['basename'], null, null, true);

                $libraryItem['name'] = $rawLibraryItem->name;
                $libraryItem['document_profile_id'] = $rawLibraryItem->document_profile_id;
                $libraryItem['type'] = LibraryItemType::FILE;
                $libraryItem['file'] = $uploadedFile;
                $libraryItem = app(CreateLibraryAttachment::class)->execute($libraryItem);
                $this->listLibraryItemV2[$rawLibraryItem->id] = $libraryItem->id;
                Storage::disk('local')->delete($file);
            }
        }
    }

    /**
     * @param $client
     */
    public function migrateJourneyDataByClient($client)
    {
        $rawJourneyItems = DB::table(self::JOURNEY_TABLES['configure_clients'])
            ->where("client_id", $client->id)
            ->whereNull('deleted_at')
            ->get();
        foreach ($rawJourneyItems as $rawJourneyItem) {
            $this->setUserContext($rawJourneyItem->added_by);
            $journey = app(CreateJourney::class)->execute(['name' => $rawJourneyItem->name_journey]);
            $journeyRawSteps = DB::table(self::JOURNEY_TABLES['configure_client_content_steps'] . ' as ct')
                ->join(self::JOURNEY_TABLES['configure_client_steps'] . ' as c', 'ct.configure_client_step_id', '=', 'c.id')
                ->where('id_configure_client', $rawJourneyItem->id)
                ->select('*', DB::raw('ct.id as configure_client_content_step_id'))->get();
            $journeySteps = array();
            $journeySteps['code'] = $journey->code;
            foreach ($journeyRawSteps as $journeyRawStep) {
                $journeyStep = array();
                $isFalse = false;
                switch ($journeyRawStep->content_type) {
                    case "1":
                        $journeyStep['type'] = ActionRequestTypes::TEXT;
                        break;
                    case "2":
                        $journeyStep['type'] = ActionRequestTypes::FILE;
                        if (!$journeyRawStep->content_type_id || !isset($this->listLibraryItemV2[$journeyRawStep->content_type_id])) {
                            $isFalse = true;
                            break;
                        }
                        $journeyStep['content_id'] = LibraryItem::find($this->listLibraryItemV2[$journeyRawStep->content_type_id])->id;
                        break;
                    case "3":
                        $journeyStep['type'] = ActionRequestTypes::QUIZ;
                        if (!$journeyRawStep->content_type_id || !isset($this->listQuizzesV2[$journeyRawStep->content_type_id])) {
                            $isFalse = true;
                            break;
                        }
                        $journeyStep['content_id'] = Quiz::find($this->listQuizzesV2[$journeyRawStep->content_type_id])->getBusinessKey();
                        break;
                    case "4":
                        $journeyStep['type'] = ActionRequestTypes::SURVEY;
                        if (!$journeyRawStep->content_type_id || !isset($this->listSurveysV2[$journeyRawStep->content_type_id])) {
                            $isFalse = true;
                            break;
                        }
                        $journeyStep['content_id'] = Survey::find($this->listSurveysV2[$journeyRawStep->content_type_id])->getBusinessKey();
                        break;
                    case "5":
                        $isFalse = true;
                        break;
                }
                if ($isFalse)
                    continue;
                $journeyStep['message'] = empty($journeyRawStep->step_text) ? "No message" : $journeyRawStep->step_text;
                $journeyStep['name'] = $journeyRawStep->title;
                $journeyStep['is_available_immediately'] = 1;
                $journeyStep['configure_client_content_step_id'] = $journeyRawStep->configure_client_content_step_id;
                $journeyStep['content_type_id'] = $journeyRawStep->content_type_id;
                $journeySteps['steps'][] = $journeyStep;

            }
            if (isset($journeySteps['steps'])) {
                app(SaveJourneyStep::class)->execute($journeySteps);
                app(PublishJourney::class)->execute(['code' => $journey->code]);
            }
            //Invite user to journey
            $users = User::whereHas('employee', function($query) use ($rawJourneyItem) {
                $query->where('id_config_step', $rawJourneyItem->id);
            })->get();
            $usersCode = $users->pluck('code')->toArray();
            if (sizeof($users) > 0) {
                $dataEmployees['users'] = $usersCode;
                $dataEmployees['code'] = $journey->code;
                app(AddEmployees::class)->execute($dataEmployees);
            }
            foreach ($journey->steps as $journeyKey => $step) {
                //Migrate submission by step
                $actionRequest = ActionRequest::query()
                    ->where('actionable_type', JourneyStep::class)
                    ->where('actionable_id', $step->id)
                    ->first();
                $result = array();
                $result['id'] = $actionRequest->id;

                foreach ($users as $user) {
                    $this->setUserContext($user->id);
                    switch ($actionRequest->contentable_type) {
                        case Quiz::class:
                            $answersJourney = DB::table('user_quiz_and_survey_results_bk')
                                ->where('content_type_id', $journeySteps['steps'][$journeyKey]['content_type_id'])
                                ->where('user_id', $user->id)
                                ->get();
                            if ($answersJourney->count() > 0) {
                                foreach ($answersJourney as $answerJourney) {
                                    $oldQuiz = DB::table(self::QUIZ_TABLES['questions'])
                                        ->whereNull('deleted_at')
                                        ->where('title', $answerJourney->question)
                                        ->where('quiz_name_id', $journeySteps['steps'][$journeyKey]['content_type_id'])
                                        ->first();
                                    if ($oldQuiz && isset($this->listQuizzesV2[$oldQuiz->quiz_name_id])) {
                                        $questionId = QuizQuestion::where('title', $answerJourney->question)
                                            ->where('quiz_id', $actionRequest->contentable_id)
                                            ->first()
                                            ->id;
                                        $resultQuestion = array();
                                        $resultQuestion['question_id'] = $questionId;
                                        $resultQuestion['option_id'] = QuizOption::where('content', $answerJourney->user_answer)->where('quiz_question_id', $questionId)->first()->id;
                                        $result['results'][] = $resultQuestion;

                                    }
                                }
                                if (isset($result['results']) && sizeof($result['results']) > 0) {
                                    app(SubmitStepDetails::class)->execute($result);
                                }
                            }
                            break;
                        case Survey::class:
                            $answersJourney = DB::table('user_quiz_and_survey_results_bk')
                                ->where('content_type_id', $journeySteps['steps'][$journeyKey]['content_type_id'])
                                ->where('user_id', $user->id)
                                ->get();
                            if ($answersJourney->count() > 0) {
                                foreach ($answersJourney as $answerJourney) {
                                    $resultQuestion = array();
                                    $oldSurvey = DB::table(self::SURVEY_TABLES['survey_questions'])
                                        ->where('title', $answerJourney->question)
                                        ->where('survey_name_id', $journeySteps['steps'][$journeyKey]['content_type_id'])
                                        ->first();
                                    if ($oldSurvey && isset($this->listSurveysV2[$oldSurvey->survey_name_id])) {
                                        $questionId = SurveyQuestion::where('title', $answerJourney->question)
                                            ->where('survey_id', $actionRequest->contentable_id)
                                            ->first()->id;
                                        $resultQuestion['question_id'] = $questionId;
                                        switch ($answerJourney->question_type) {
                                            case 'multi-choice':
                                                $resultQuestion['options'][0]['id'] = SurveyOption::where('content', $answerJourney->user_answer)->where('survey_question_id', $questionId)->first()->id;
                                                break;
                                            case 'star-rating':
                                                $resultQuestion['content'] = $answerJourney->user_answer;
                                                break;
                                            case 'text':
                                                $resultQuestion['text'] = $answerJourney->user_answer;
                                                break;
                                        }
                                        $result['results'][] = $resultQuestion;
                                    }
                                }
                                if (isset($result['results']) && sizeof($result['results']) > 0) {
                                    app(SubmitStepDetails::class)->execute($result);
                                }
                            }
                            break;
                        default:
                            $isRead = $answersJourney = DB::table('configure_client_user_step_progresses')
                                ->where('configure_client_content_step_id', $journeySteps['steps'][$journeyKey]['configure_client_content_step_id'])
                                ->where('front_user_id', $user->id)
                                ->where('is_complete', 1)
                                ->get();
                            if ($isRead->count() > 0) {
                                app(SubmitStepDetails::class)->execute($result);
                            }
                            break;
                    }
                }
            }
        }
    }

    /**
     * Set user context
     * @param $id
     */
    public function setUserContext($id)
    {
        $user = $this->currentUser;
        if($id){
            $user = User::find($id) ?? $this->currentUser ;
        }
        Context::user()->set($user);
    }
}