<?php

namespace App\Console\Commands;

use App\Enum\NotificationTypes;
use App\Helpers\EventHelper;
use App\Jobs\SendNotification;
use App\Models\Event\Event;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Questionnaire\QuestionnaireUserCompleted;
use App\Models\Schedule\ScheduleFrequencyRound;
use App\Models\Schedule\ScheduleRoundQuestionnaire;
use App\Models\User\User;
use App\Services\Timezone\GetTimezoneByHour;
use Illuminate\Console\Command;
use Illuminate\Support\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class QuestionnaireNotification extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'questionnaire:notification';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Create notification for scheduled events';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        $now = Carbon::now();

        $activeRound = ScheduleFrequencyRound::whereDate('start_date', '<=', $now)
            ->whereHas('survey', function (Builder $query) {
                $query->where('status', 1);
            })
            ->get();

        $activeRoundMap = $activeRound->mapWithKeys(function($item){
            return [$item['id'] => $item];
        });
        $activeSurveyIds = $activeRound->pluck('schedule_survey_id')->unique()->toArray();
        $activeRoundIds = $activeRound->pluck('id')->toArray();

        // Get completed round
        $completedRoundUser = QuestionnaireUserCompleted::whereIn('round_id', $activeRoundIds)->get();
        $completedRoundIdUser = $completedRoundUser->mapToGroups(function ($item) {
            return [$item['user_id'] => $item['round_id']];
        })->toArray();

        // Round user config
        $surveyRoundUsers = ScheduleRoundQuestionnaire::whereIn('schedule_survey_id', $activeSurveyIds)->get();
        //Get timezones currently equal to 8AM
        $timezones = app(GetTimezoneByHour::class)->execute(['hour' => 8])->toArray();
        foreach ($surveyRoundUsers as $roundUser) {
            $user = User::find($roundUser->user_id);

            if (empty($user) || !in_array($user->employee->timezone,$timezones)) continue;
            $roundQuestionInfo =  $roundUser->questionnaire_id;

            $completedRoundId = !empty($completedRoundIdUser[$roundUser->user_id])
                ? $completedRoundIdUser[$roundUser->user_id]
                : [];
            $allRoundOfUser = array_keys($roundQuestionInfo);
            $activeRoundOfUs = array_intersect($allRoundOfUser, $activeRoundIds);
            $pendingRoundOfUs = array_diff($activeRoundOfUs, $completedRoundId);

            foreach ($pendingRoundOfUs as $pendingId) {
                $roundDetail = !empty($activeRoundMap[$pendingId]) ? $activeRoundMap[$pendingId] : [];
                $totalQuestion = count($roundQuestionInfo[$pendingId]);
                $totalAnswered = QuestionnaireAnswer::where([
                    'user_id' => $user->id,
                    'round_id' => $pendingId
                ])->count();
                $remainingQuestions =  ($totalQuestion - $totalAnswered) > 0 ? ($totalQuestion - $totalAnswered) : 0;

                if (!empty($roundDetail)) {
                    $nowDate = Carbon::parse($now->format('Y-m-d'));
                    $startDate = Carbon::parse(Carbon::parse($roundDetail->start_date)->format('Y-m-d'));
                    $expireDate = Carbon::parse(Carbon::parse($roundDetail->expire_date)->format('Y-m-d'));

                    $headings = '';
                    $contents = '';
                    if ($startDate == $nowDate) {
                        // Send notification today
                        $headings = __('New engagement questions');
                        $contents = __('Tap to provide your feedback');
                    } elseif($now->lt($expireDate)) {
                        // Send pending notification
                        $headings = __('Engagement questions pending');
                        $contents = __('Tap to provide your feedback');
                    } elseif($now->eq($expireDate)) {
                        // Send due date notification
                        $headings = __('Engagement feedback due today');
                        $contents = __('Tap to provide your feedback');
                    } elseif($now->gt($expireDate)) {
                        // Send over due date notification
                        $headings = __('Your feedback is overdue');
                        $contents = __('Tap to provide your feedback');
                    }

                    // Send at 9AM
                    $sendAfter = Carbon::now()->addHour()->setMinute(0)->setSecond(0);

                    if (!empty($headings)) {
                        $params = [
                            'sender' => Auth::user()->id,
                            "userIds" => $user->getBusinessKey(),
                            "headings" => $headings,
                            "contents" => $contents,
                            "data"=> [
                                "type" => NotificationTypes::QUESTIONNAIRE,
                                "id" => $roundDetail->id,
                                'total_questions' => $totalQuestion,
                                'remaining_questions' => $remainingQuestions,
                            ],
                            "send_after" => $sendAfter
                        ];

                        SendNotification::dispatch($params);
                    }
                }
            }
        }
        return true;
    }
}
