<?php
namespace App\Database\Migrations;

use iamcal\SQLParser;
use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Log;
use Illuminate\Database\Migrations\Migration;
use Illuminate\Support\Facades\DB;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;

abstract class AppMigration extends Migration
{

    const TEMPORARY_TABLE = '_tmp';

    const LONG_TEXT = 1023;
    const MEDIUM_TEXT = 255;
    const SHORT_TEXT = 63;

    /**
     * @var SQLParser
     */
    private $parser;

    public function __construct()
    {
        $this->parser = new SQLParser();
        DB::getDoctrineSchemaManager()->getDatabasePlatform()->registerDoctrineTypeMapping('enum', 'string');
    }

    /**
     * Clone an existing table
     *
     * @param $table_from
     * @param $table_to
     * @param bool $copyData Copy data from $table_from and insert into $table_to.
     * 
     * @return void
     */
    protected function cloneTable($table_from, $table_to, bool $copyData = false) {

        // get the create sql for current table
        $table_create = DB::select("SHOW CREATE TABLE `$table_from`");
        $create_from_table_sql = ((array)current($table_create))['Create Table'];

        // replace the table name to get the create table sql for the `to` table
        $create_to_table_sql = str_replace("CREATE TABLE `$table_from`", "CREATE TABLE `$table_to`", $create_from_table_sql);

        // execute the create to table sql to create the cloned table
        DB::statement($create_to_table_sql);

        if ($copyData) {
            $result = DB::select(DB::raw("SHOW KEYS FROM {$table_from} WHERE Key_name = 'PRIMARY'"));
            $primaryKey = $result[0]->Column_name;  
            $this->copyDataWithProgress($table_from, $table_to, $primaryKey);
        }
    }

    /**
     * Return an array of indexed column names based on a given table name
     *
     * @param $table_name
     * @return array
     */
    protected function getIndexColumnNamesFromTable($table_name):array {

        // parse the create statement into an array based structure
        $table_create = DB::select("SHOW CREATE TABLE `$table_name`");
        $create_from_table_sql = ((array)current($table_create))['Create Table'];
        $parsed = $this->parser->parse($create_from_table_sql);

        // extract the indexes
        $indexed_cols = [];
        foreach ($parsed[$table_name]['indexes'] as $index) {
            // ignore compound indexes
            // TODO: add support for compound indexes
            if (count($index['cols']) > 1) {
                continue;
            }
            foreach($index['cols'] as $c) {
                $indexed_cols[] = $c['name'];
            }
        }

        // remove duplicates and return
        return array_flip(array_flip($indexed_cols));
    }

    /**
     * Given a table name and an array of column names checks if any column names don't have an index
     *
     * @param $table_name
     * @param $column_names
     * @return bool
     */
    protected function hasMissingIndexesOnColumns($table_name, $column_names):bool {
        // check if any columns have missing indexes
        return count($this->getMissingIndexesOnColumns($table_name, $column_names)) > 0;
    }

    /**
     * Given the the table name and array of column names returns an array of column names that have missing index
     *
     * @param $table_name
     * @param $column_names
     * @return array
     */
    protected function getMissingIndexesOnColumns($table_name, $column_names):array {
        // get columns that have missing indexes
        $column_names = (array)$column_names;
        $indexed_column_names = $this->getIndexColumnNamesFromTable($table_name);
        $missing_indexes_on_column_names = [];
        foreach ($column_names as $column_name) {
            if (!in_array($column_name, $indexed_column_names, true)) {
                $missing_indexes_on_column_names[] = $column_name;
            }
        }
        return $missing_indexes_on_column_names;
    }

    /**
     * Given a table name and array of column names creates indexes on any columns that don't have an index
     *
     * @param $table_name
     * @param $column_names
     */
    protected function addSimpleIndexesToTable($table_name, $column_names):void {
        Log::stack(['stdout'])->info('Attempt to add missing indexes for columns: '.implode(', ', $column_names)." to table: $table_name");

        // add the missing index to the cloned table if it doesn't already have it
        if (count($column_names_without_indexes = $this->getMissingIndexesOnColumns($table_name, $column_names))) {

            // clone the table
            $table_clone_for_alter = "{$table_name}_".date('YmdHis');
            $table_backup = "{$table_name}_bak".date('YmdHis');

            Log::stack(['stdout'])->info("Cloning table $table_name => $table_clone_for_alter");
            $this->cloneTable($table_name, $table_clone_for_alter);

            // add the missing index
            Log::stack(['stdout'])->info('Adding missing indexes for columns: '.implode(', ', $column_names_without_indexes)." to table: $table_name...");
            foreach ($column_names_without_indexes as $column_name) {
                Schema::table($table_clone_for_alter, function(Blueprint $table) use ($column_name) {
                    $table->index($column_name);
                });
            }
            Log::stack(['stdout'])->info('Adding missing indexes for columns: '.implode(', ', $column_names_without_indexes)." to table: $table_name... complete!");

            // migrate the data to the cloned table
            Log::stack(['stdout'])->info("Copying data $table_name => $table_clone_for_alter");
            DB::statement("INSERT INTO $table_clone_for_alter SELECT * FROM $table_name");

            // rename old table to backup with a timestamp and the new table to the original table's name
            Log::stack(['stdout'])->info("Renaming tables $table_name => $table_backup");
            DB::statement("RENAME TABLE $table_name TO $table_backup");
            Log::stack(['stdout'])->info("Renaming tables $table_clone_for_alter => $table_name");
            DB::statement("RENAME TABLE $table_clone_for_alter TO $table_name");

        } else {

            // index already exists, nothing to do
            Log::stack(['stdout'])->info("Don't need to do anything indexes for columns: ".implode(', ', $column_names)." on table: $table_name already exists!");
        }
    }

    /**
     * Given a data structure representing an array of table names with column names to index
     *
     *  e.g. [table1 =>  [col1, col2, ....], table2 => [col1, col2, ...], ...]
     *
     * Processes the given data to ensure that all column names against relevant tables have indexes
     *
     * @param $table_index_data
     */
    protected function addSimpleIndexes($table_index_data):void {
        $table_index_data = (array)$table_index_data;
        foreach ($table_index_data as $table_name => $column_names) {
            $this->addSimpleIndexesToTable($table_name, $column_names);
        }
    }

    /**
     * Copy data from one table and insert it into another table.
     *
     * @param string $tableFrom Table to copy data from.
     * @param string $tableTo Table to insert data into.
     * @return void
     */
    protected function copyData(string $tableFrom, string $tableTo)
    {
        $chunks = DB::table($tableFrom)->get()->chunk(100)->toArray();
        foreach ($chunks AS $chunk) {
            DB::table($tableTo)->insert(array_map('get_object_vars', $chunk));
        }
    }

    /**
     * Copy data from one table and insert it into another table.
     *
     * @param string $source Table to copy data from.
     * @param string $destination Table to insert data into.
     * @param string $primaryKey PrimaryKey of $source
     * @param int $size Size of chunk
     * @return void
     */
    protected function copyDataWithProgress(
        string $source,
        string $destination,
        string $primaryKey,
        int $size = 100
    ) {
        $count = 0;
        $total = DB::table($source)->count($primaryKey);

        DB::table($source)
            ->orderBy($primaryKey)
            ->chunk($size, function ($chunk) use ($destination, $size, &$count, $total) {
                DB::table($destination)->insert(array_map('get_object_vars', $chunk->toArray()));
                $count += $size;
                $this->showProgress($destination, $count, $total);
            });

        $this->showProgress($destination, $count, $total, true);
    }

    /**
     * Swap table names.
     *
     * @param string $leftTableName Existing table name.
     * @param string $rightTableName Existing table name.
     * 
     * @return void
     */
    protected function swapTableName(string $leftTableName, string $rightTableName)
    {
        $tempTableName = $leftTableName . self::TEMPORARY_TABLE;
        Schema::rename($leftTableName, $tempTableName);
        Schema::rename($rightTableName, $leftTableName);
        Schema::rename($tempTableName, $rightTableName);
    }

    /**
     * Console output of status every progression of XX records.
     *
     * @param string $table
     * @param int $i
     * @param int $total
     * @param bool $force
     *
     * @return void
     */
    public function showProgress(string $table, int $i, int $total, bool $force = false)
    {
        $outputEvery = 5000;
        if ($i % $outputEvery === 0 || $force) {
            $percentage = $total > 0
                ? round(($i / $total) * 100)
                : 0;

            Log::stack(['stdout'])->info(
                "$table table $i of $total records processed ($percentage% Complete)"
            );
        }
    }

    /**
     * @param string $name
     * @param array $permissions
     */
    public function givePermissionTo(string $roleName, string $model, array $permissions = ['edit', 'delete', 'read'])
    {
        $role = Role::where('name', $roleName)->first();
        foreach ($permissions as $permission) {
            $permissionName = $model . "." . $permission;
            if (!Permission::where(['name' => $permissionName])->first()) {
                Permission::create(['name' => $permissionName]);
            }
            $role->givePermissionTo($permissionName);
        }
    }

    /**
     * @param string $name
     * @param array $permissions
     */
    public function givePermissionToRole(string $roleName, array $permissions)
    {
        $role = Role::where('name', $roleName)->first();
        foreach ($permissions as $permissionName) {
            Permission::firstOrCreate(['name' => $permissionName], ['name' => $permissionName]);
            $role->givePermissionTo($permissionName);
        }
    }

    /**
     * @param string $name
     * @param array $permissions
     */
    public function revokePermissionTo(string $roleName, string $model, array $permissions = ['edit', 'delete', 'read'])
    {
        $role = Role::where('name',$roleName)->first();
        foreach($permissions as $permission){
            $permissionName = $model . "." . $permission;
            if(Permission::where(['name' => $permissionName])->first()){
                $role->revokePermissionTo($permissionName);
            }
        }
    }

    /**
     * @param string $roleName
     * @param array $permissions
     */
    public function revokePermissionToRole(string $roleName, array $permissions)
    {
        $role = Role::where('name', $roleName)->first();
        foreach ($permissions as $permission) {
            if (Permission::where(['name' => $permission])->first()) {
                $role->revokePermissionTo($permission);
            }
        }
    }
}