<?php

namespace App\Helpers;

use App\Enum\ActionRequestTypes;
use App\Http\Resources\ContentFile\ContentFile as ContentFileResource;
use App\Http\Resources\ContentFile\ContentFileResult as ContentFileResultResource;
use App\Http\Resources\ContentPhoto\ContentPhoto as ContentPhotoResource;
use App\Http\Resources\ContentPhoto\ContentPhotoResult as ContentPhotoResultResource;
use App\Http\Resources\ContentText\ContentText as ContentTextResource;
use App\Http\Resources\IDVerification\IDVerification as IDVerificationResource;
use App\Http\Resources\IDVerification\IDVerificationResult as IDVerificationResultResource;
use App\Http\Resources\Library\LibraryItem as LibraryItemResource;
use App\Http\Resources\Mobile\ActionRequestSubmission\Library\LibraryItem as LibraryItemMobileResource;
use App\Http\Resources\Mobile\ActionRequestSubmission\Library\LibraryItemResult as LibraryItemResultMobileResource;
use App\Http\Resources\Mobile\ContentPhoto\ContentPhoto as ContentPhotoMobileResource;
use App\Http\Resources\Mobile\ContentPhoto\ContentPhotoResult as ContentPhotoMobileResultResource;
use App\Http\Resources\Mobile\ContentText\ContentText as ContentTextMobileResource;
use App\Http\Resources\Mobile\ContentText\ContentTextResult as ContentTextResultMobileResource;
use App\Http\Resources\Mobile\IDVerification\IDVerification as IDVerificationMobileResource;
use App\Http\Resources\Mobile\IDVerification\IDVerificationResult as IDVerificationResultMobileResource;
use App\Http\Resources\Mobile\Tool\Form\Form as FormDetailMobileResource;
use App\Http\Resources\Mobile\Tool\Quiz\QuizDetail as QuizDetailMobileResource;
use App\Http\Resources\Mobile\Tool\Quiz\QuizResult as QuizDetailResultMobileResource;
use App\Http\Resources\Mobile\Tool\Survey\SurveyDetail as SurveyDetailMobileResource;
use App\Http\Resources\Mobile\Tool\Survey\SurveyResult as SurveyDetailResultMobileResource;
use App\Http\Resources\Mobile\ContentFile\ContentFile as ContentFileMobileResource;
use App\Http\Resources\Mobile\ContentFile\ContentFileResult as ContentFileResultMobileResource;
use App\Http\Resources\Tool\Form\Form as FormResource;
use App\Http\Resources\Tool\Form\FormResult as FormResultResource;
use App\Http\Resources\Tool\Quiz\QuizDetail as QuizDetailResource;
use App\Http\Resources\Tool\Quiz\QuizResult as QuizResultResource;
use App\Http\Resources\Tool\Survey\SurveyDetail as SurveyDetailResource;
use App\Http\Resources\Tool\Survey\SurveyResult as SurveyResultResource;
use App\Models\Action\Action;
use App\Models\Action\ActionSequenceStep;
use App\Models\ActionRequest\ActionRequestSubmission;
use App\Models\ContentFile\ContentFile;
use App\Models\ContentPhoto\ContentPhoto;
use App\Models\ContentText\ContentText;
use App\Models\Form\FormTemplate;
use App\Models\IDVerification\IDVerification;
use App\Models\Journey\JourneyStep;
use App\Models\Library\LibraryItem;
use App\Models\Quiz\Quiz;
use App\Models\Survey\Survey;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Resources\Json\JsonResource;

class ActionRequestHelper
{
    /**
     * Get content model by type and id
     *
     * @param  string  $type
     * @param $id
     * @return Model|null
     */
    public static function getContentableModel(string $type, $id)
    {
        switch ($type) {
            case ActionRequestTypes::TEXT:
                $model = ContentText::findByBusinessKey(ActionRequestTypes::TEXT);
                break;
            case ActionRequestTypes::QUIZ:
                $model = $id !== null ? Quiz::findByBusinessKey($id) : null;
                break;
            case ActionRequestTypes::SURVEY:
                $model = $id !== null ? Survey::findByBusinessKey($id) : null;
                break;
            case ActionRequestTypes::FILE:
                $model = $id !== null ? LibraryItem::findOrFail($id) : null;
                break;
            case ActionRequestTypes::FORM:
                $model = $id !== null ? FormTemplate::findOrFail($id) : null;
                break;
            case ActionRequestTypes::ID_VERIFICATION:
                $model = $id !== null ? IDVerification::findByBusinessKey($id) : null;
                break;
            case ActionRequestTypes::HEADSHOT_PHOTO:
                $model = ContentPhoto::findByBusinessKey(ActionRequestTypes::HEADSHOT_PHOTO);
                break;
            case ActionRequestTypes::UPLOAD_CONTENT:
                $model = ContentFile::findByBusinessKey(ActionRequestTypes::UPLOAD_CONTENT);
                break;
            default:
                $model = null;
        }

        return $model;
    }

    /**
     * Get action request type by given content class
     *
     * @param  string  $content
     * @return string|null
     */
    public static function getActionRequestTypeByContent(string $content)
    {
        switch ($content) {
            case ContentText::class:
                $type = ActionRequestTypes::TEXT;
                break;
            case Quiz::class:
                $type = ActionRequestTypes::QUIZ;
                break;
            case Survey::class:
                $type = ActionRequestTypes::SURVEY;
                break;
            case LibraryItem::class:
                $type = ActionRequestTypes::FILE;
                break;
            case FormTemplate::class:
                $type = ActionRequestTypes::FORM;
                break;
            case IDVerification::class:
                $type = ActionRequestTypes::ID_VERIFICATION;
                break;
            case ContentPhoto::class:
                $type = ActionRequestTypes::HEADSHOT_PHOTO;
                break;
            case ContentFile::class:
                $type = ActionRequestTypes::UPLOAD_CONTENT;
                break;
            default:
                $type = null;
        }

        return $type;
    }

    /**
     * Get contentable resource
     *
     * @param  string  $content
     * @param  Model  $contentable
     * @return JsonResource|null
     */
    public static function getContentableResource(string $content, Model $contentable)
    {
        switch ($content) {
            case ContentText::class:
                $type = new ContentTextResource($contentable);
                break;
            case Quiz::class:
                $type = new QuizDetailResource($contentable);
                break;
            case Survey::class:
                $type = new SurveyDetailResource($contentable);
                break;
            case LibraryItem::class:
                $type = new LibraryItemResource($contentable);
                break;
            case FormTemplate::class:
                $type = new FormResource($contentable);
                break;
            case IDVerification::class:
                $type = new IDVerificationResource($contentable);
                break;
            case ContentPhoto::class:
                $type = new ContentPhotoResource($contentable);
                break;
            case ContentFile::class:
                $type = new ContentFileResource($contentable);
                break;
            default:
                $type = null;
        }

        return $type;
    }

    /**
     * Get contentable id
     *
     * @param  string  $content
     * @param  Model  $contentable
     * @return mixed|null
     */
    public static function getContentableId(string $content, Model $contentable)
    {
        switch ($content) {
            case Quiz::class:
            case Survey::class:
            case ContentText::class:
            case IDVerification::class:
            case ContentPhoto::class:
            case ContentFile::class:
                $id = $contentable->getBusinessKey();
                break;
            case FormTemplate::class:
            case LibraryItem::class:
                $id = $contentable->id;
                break;
            default:
                $id = null;
        }

        return $id;
    }

    /**
     * Get submission result
     *
     * @param  string  $content
     * @param  ActionRequestSubmission  $submission
     * @return JsonResource|null
     */
    public static function getSubmissionResource(string $content, ActionRequestSubmission $submission)
    {
        switch ($content) {
            case Quiz::class:
                $type = QuizResultResource::collection($submission->quizResults);
                break;
            case Survey::class:
                $type = SurveyResultResource::collection($submission->surveyResults);
                break;
            case FormTemplate::class:
                $type = FormResultResource::collection($submission->formResults);
                break;
            case IDVerification::class:
                $type = IDVerificationResultResource::collection($submission->idVerificationResults);
                break;
            case ContentPhoto::class:
                $type = ContentPhotoResultResource::collection($submission->contentPhotoResults);
                break;
            case ContentFile::class:
                $type = ContentFileResultResource::collection($submission->contentFileResults);
                break;
            default:
                $type = null;
        }

        return $type;
    }

    /**
     * Get actionable name
     *
     * @param  string  $type
     * @param  Model  $actionable
     * @return string|null
     */
    public static function getActionableName(string $type, Model $actionable)
    {
        switch ($type) {
            case JourneyStep::class:
                $name = $actionable->journey->name;
                break;
            case ActionSequenceStep::class:
                $name = $actionable->actionSequence->name;
                break;
            case Action::class:
                $name = $actionable->name;
                break;
            default:
                $name = null;
        }

        return $name;
    }

    /**
     * Get actionable type name
     *
     * @param  string  $type
     * @return string|null
     */
    public static function getActionableType(string $type)
    {
        switch ($type) {
            case JourneyStep::class:
                $typeName = ActionRequestTypes::JOURNEY;
                break;
            case ActionSequenceStep::class:
                $typeName = ActionRequestTypes::ACTION_SEQUENCE;
                break;
            case Action::class:
                $typeName = ActionRequestTypes::ACTION;
                break;
            default:
                $typeName = null;
        }

        return $typeName;
    }

    /**
     * Get action request subtitle type by given content class
     *
     * @param  string  $content
     * @return string|null
     */
    public static function getActionRequestSubTitleByContent(string $content)
    {
        switch ($content) {
            case ContentText::class:
                $title = ActionRequestTypes::SUB_TITLE_TEXT;
                break;
            case Quiz::class:
                $title = ActionRequestTypes::SUB_TITLE_QUIZ;
                break;
            case Survey::class:
                $title = ActionRequestTypes::SUB_TITLE_SURVEY;
                break;
            case LibraryItem::class:
                $title = ActionRequestTypes::SUB_TITLE_FILE;
                break;
            case FormTemplate::class:
                $title = ActionRequestTypes::SUB_TITLE_FORM;
                break;
            case IDVerification::class:
                $title = ActionRequestTypes::SUB_TITLE_ID_VERIFICATION;
                break;
            case ContentPhoto::class:
                $title = ActionRequestTypes::SUB_TITLE_HEADSHOT_PHOTO;
                break;
            case ContentFile::class:
                $title = ActionRequestTypes::SUB_TITLE_UPLOAD_CONTENT;
                break;
            default:
                $title = null;
        }

        return $title;
    }

    /**
     * Get content resource for mobile APIs
     *
     * @param  Model  $actionRequest
     * @return mixed
     */
    public static function getContentableMobileResource(Model $actionRequest)
    {
        switch ($actionRequest->contentable_type) {
            case ContentText::class:
                $resource = new ContentTextMobileResource($actionRequest);
                break;
            case Quiz::class:
                $resource = new QuizDetailMobileResource($actionRequest);
                break;
            case Survey::class:
                $resource = new SurveyDetailMobileResource($actionRequest);
                break;
            case LibraryItem::class:
                $resource = new LibraryItemMobileResource($actionRequest);
                break;
            case FormTemplate::class:
                $resource = new FormDetailMobileResource($actionRequest);
                break;
            case IDVerification::class:
                $resource = new IDVerificationMobileResource($actionRequest);
                break;
            case ContentPhoto::class:
                $resource = new ContentPhotoMobileResource($actionRequest);
                break;
            case ContentFile::class:
                $resource = new ContentFileMobileResource($actionRequest);
                break;
            default:
                $resource = null;
        }

        return $resource;
    }

    /**
     * Get result for mobile APIs
     *
     * @param  Model  $actionRequest
     * @return mixed
     */
    public static function getResultMobileResource(Model $actionRequest)
    {
        switch ($actionRequest->contentable_type) {
            case ContentText::class:
                $resource = new ContentTextResultMobileResource($actionRequest);
                break;
            case Quiz::class:
                $resource = new QuizDetailResultMobileResource($actionRequest);
                break;
            case Survey::class:
                $resource = new SurveyDetailResultMobileResource($actionRequest);
                break;
            case LibraryItem::class:
                $resource = new LibraryItemResultMobileResource($actionRequest);
                break;
            case FormTemplate::class:
                $resource = new FormDetailMobileResource($actionRequest);
                break;
            case IDVerification::class:
                $resource = new IDVerificationResultMobileResource($actionRequest);
                break;
            case ContentPhoto::class:
                $resource = new ContentPhotoMobileResultResource($actionRequest);
                break;
            case ContentFile::class:
                $resource = new ContentFileResultMobileResource($actionRequest);
                break;
            default:
                $resource = null;
        }

        return $resource;
    }

    /**
     * Get type name by type
     *
     * @param  string  $type
     * @return string|null
     */
    public static function getTypeName(string $type)
    {
        switch ($type) {
            case ActionRequestTypes::TEXT:
                $name = ActionRequestTypes::TEXT_NAME;
                break;
            case ActionRequestTypes::QUIZ:
                $name = ActionRequestTypes::QUIZ_NAME;
                break;
            case ActionRequestTypes::SURVEY:
                $name = ActionRequestTypes::SURVEY_NAME;
                break;
            case ActionRequestTypes::HEADSHOT_PHOTO:
                $name = ActionRequestTypes::HEADSHOT_PHOTO_NAME;
                break;
            case ActionRequestTypes::ID_VERIFICATION:
                $name = ActionRequestTypes::ID_VERIFICATION_NAME;
                break;
            case ActionRequestTypes::FILE:
                $name = ActionRequestTypes::FILE_NAME;
                break;
            case ActionRequestTypes::FORM:
                $name = ActionRequestTypes::FORM_NAME;
                break;
            case ActionRequestTypes::UPLOAD_CONTENT:
                $name = ActionRequestTypes::UPLOAD_CONTENT_NAME;
                break;
            default:
                $name = null;
        }

        return $name;
    }

    /**
     * Get type name by content type
     *
     * @param  string  $contentType
     * @return string|null
     */
    public static function getTypeNameByContentType(string $contentType)
    {
        $type = self::getActionRequestTypeByContent($contentType);

        return self::getTypeName($type);
    }

    /**
     * Get action type
     *
     * @param $actionRequest
     * @return string
     * TODO replace hardcode contentable type by enum
     */
    public static function getActionTypeForNotification($actionRequest)
    {
        $actionType = null;
        if ($actionRequest) {
            switch ($actionRequest->contentable_type) {
                case IDVerification::class:
                    $actionType = ActionRequestTypes::ID_VERIFICATION;
                    break;
                case ContentPhoto::class:
                    $actionType = ActionRequestTypes::HEADSHOT_PHOTO;
                    break;
                case Quiz::class:
                    $actionType = ActionRequestTypes::QUIZ;
                    break;
                case Survey::class:
                    $actionType = ActionRequestTypes::SURVEY;
                    break;
                case FormTemplate::class:
                    $actionType = ActionRequestTypes::FORM;
                    break;
                case ContentFile::class:
                    $actionType = ActionRequestTypes::UPLOAD_CONTENT;
                    break;
            }
        }
        return $actionType;
    }
}