<?php


namespace App\Helpers;

use Aws\QuickSight\Exception\QuickSightException;
use Aws\Sts\StsClient;
use Aws\QuickSight\QuickSightClient;
use Illuminate\Support\Facades\Log;

class AwsQuickSightHelper
{
    CONST LATEST_VERSION = 'latest';
    CONST REGION_AP_SOUTHEAST_1 = 'ap-southeast-1';
    CONST REGION_US_EAST_1 = 'us-east-1';

    /**
     * @param string $version
     * @param string $region
     * @param array $credentials
     * @return QuickSightClient
     */
    public function initQuickSightClient(string $version, string $region, array $credentials)
    {
        return new QuickSightClient([
            'version' => $version,
            'region' => $region,
            'credentials' => $credentials
        ]);
    }
    
    /**
     * @param string $role
     * @return mixed|null
     */
    public function getStsCredentials(string $role)
    {
        $sts = new StsClient([
            'version' => self::LATEST_VERSION,
            'region' => self::REGION_AP_SOUTHEAST_1,
            'credentials' => [
                'key' => config('aws.quicksight.access_key'),
                'secret' => config('aws.quicksight.secret_key'),
            ],
        ]);
        
        $result = $sts->assumeRole([
            'RoleArn' => "arn:aws:iam::541696823175:role/quicksight-embedded-role",
            'RoleSessionName' => $role
        ]);

        return $result->get('Credentials');
    }

    /**
     * @param string $role
     * @return mixed|null
     */
    public function getDashboardEmbedUrl(string $role)
    {
        try {
            $credentials = $this->getStsCredentials($role);
            
            $quickSightCredentials = [
                'key' => $credentials["AccessKeyId"],
                'secret' => $credentials["SecretAccessKey"],
                'token' => $credentials['SessionToken'],
            ];

            $quickSight = $this->initQuickSightClient(
                self::LATEST_VERSION, self::REGION_AP_SOUTHEAST_1, $quickSightCredentials
            );

            $result = $quickSight->GetDashboardEmbedUrl([
                'AwsAccountId' => config('aws.quicksight.account_id'),
                'DashboardId' => config('aws.quicksight.dashboard_id'),
                'IdentityType' => 'QUICKSIGHT',
                'UserArn' => 'arn:aws:quicksight:ap-southeast-1:541696823175:user/default/' . $role,
                'ResetDisabled' => true,
                'SessionLifetimeInMinutes' => 600,
                'UndoRedoDisabled' => false
            ]);

            $embedUrl = $result->get('EmbedUrl');
        } catch (QuickSightException $exception) {
            Log::error($exception);
            $embedUrl = null;
        }
        
        return $embedUrl;
    }

    /**
     * @param string $userName
     */
    public function registerUserMembership(string $userName)
    {
        $credentials = [
            'key' => config('aws.quicksight.access_key'),
            'secret' => config('aws.quicksight.secret_key'),
        ];

        $quickSight = $this->initQuickSightClient(
            self::LATEST_VERSION, self::REGION_US_EAST_1, $credentials
            );
        
        $quickSight->registerUser([
            'AwsAccountId' => config('aws.quicksight.account_id'),
            'Email' => $userName,
            'IdentityType' => 'QUICKSIGHT',
            'Namespace' => 'default',
            'UserName' => $userName,
            'UserRole' => 'READER',
        ]);
        
        $quickSight->createGroupMembership([
            'AwsAccountId' => config('aws.quicksight.account_id'),
            'GroupName' => config('aws.quicksight.group_name'),
            'MemberName' => $userName,
            'Namespace' => 'default',
        ]);
    }
}