<?php

namespace App\Helpers;

use Closure;
use App\Enum\FilterTypes;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;


/**
 * Class FilterFluent
 * @package App\Helpers
 */
class FilterFluent
{
    /**
     * @var array
     */
    private $filterTypes = FilterTypes::ALLOW_FILTERS;

    /**
     * @var array
     */
    private $filters = array();
    /**
     * @var array
     */
    private $originalFilters = array();
    /**
     * @var null
     */
    private $allowedFilters = null;
    /**
     * @var array
     */
    private $customFilters = array();
    /**
     * @var null
     */
    private $query = null;

    /**
     * @var null
     */
    private $table = '';

        /**
     *
     */
    private function constructor() {}

    /**
     * @return FilterFluent
     */
    static function init(): FilterFluent {
        $filterFluent = new FilterFluent();
        $filterFluent->filters = collect();
        $filterFluent->originalFilters = collect();
        $filterFluent->customFilters = collect();
        return $filterFluent;
    }

    /**
     * @param $query
     * @return $this
     */
    public function setQuery($query)
    {
        $this->query = $query;
        return $this;
    }

    /**
     * @param $query
     * @return $this
     */
    public function setTable($table)
    {
        $this->table = $table;
        return $this;
    }

    /**
     * @param array $options
     * @return $this
     */
    public function setFilterOptions(array $options) : FilterFluent
    {
        if (isset($options['filter'])) {
            $this->filters = collect($options['filter']);
            $this->originalFilters = collect($options['filter']);
        }
        return $this;
    }

    /**
     * @param array $filters
     * @return $this
     */
    public function setAllowedFilters(array $filters): FilterFluent
    {
        $this->allowedFilters = $filters;
        return $this;
    }

    /**
     * @param $columnFilter
     * @return $this
     */
    public function removeFilter($columnFilter): FilterFluent
    {
        $this->filters = $this->filters->reject(function ($value) use ($columnFilter) {
            return is_array($columnFilter) ?
                in_array($value['column'], $columnFilter) :
                $value['column'] === $columnFilter;
        });
        return $this;
    }

    /**
     * @param string $filterColumn
     * @param Closure $callback
     * @return $this
     */
    public function setCustomFilter(string $filterColumn, Closure $callback): FilterFluent
    {
        $this->customFilters->push([$filterColumn, $callback]);
        return $this;
    }

    /**
     * @return $this
     * @throws ValidationException
     */
    public function validate(): FilterFluent
    {
        foreach ($this->filters as $filter) {
            if (!isset($filter['value'])) {
                throw ValidationException::withMessages(['filter[value]' => __('validation.required', ['attribute' => 'filter[value]'])]);
            }
            if (!empty($this->allowFilters) && !in_array($filter['column'], $this->allowFilters)) {
                throw ValidationException::withMessages(['filter[column]' => __('validation.invalid', ['attribute' => 'filter[column]'])]);
            }
    
            if (!isset($filter['column'])) {
                throw ValidationException::withMessages(['filter[column]' => __('validation.required', ['attribute' => 'filter[column]'])]);
            }
            if (isset($filter['type'])) {
                if (!in_array($filter['type'], $this->filterTypes)) {
                    throw ValidationException::withMessages(['filter[type]' => __('validation.invalid', ['attribute' => 'filter[type]'])]);
                }
            }
        }
        return $this;
    }

    /**
     * @param string $column
     * @throws ValidationException
     */
    public function validateColumn(string $column){
        if (!in_array($column, $this->allowedFilters))
        {
            throw ValidationException::withMessages(['filter[column]' => __('validation.invalid', ['attribute' => 'filter[column]'])]);
        }
    }

    /**
     *
     */
    public function applyFilter() {
        foreach ($this->filters as $filter) {
            $this->validateColumn($filter['column']);
            self::applyBasicFilter($this->query, $filter['type'], $filter['column'], $filter['value'], $this->table);
        }

        foreach ($this->customFilters as [$column, $callback]) {
            $filters = $this->getFilters($column);
            if (!empty($filters)) {
                foreach ($filters as $filter) {
                    $this->validateColumn($filter['column']);
                    $callback($this->query, $filter['type'], $filter['column'], $filter['value']);
                }
            }
        }
    }

    /**
     * @param $query
     * @param $type
     * @param $column
     * @param $value
     * @param string $tableName
     */
    public static function applyBasicFilter($query, $type, $column, $value,string $tableName = '') {
        $column = $tableName === '' ? $column : $tableName . "." . $column;
        switch ($type) {
            case FilterTypes::IS_NOT:
                $query->where($column, '!=', $value)->orWhereNull($column);
                break;
            case FilterTypes::CONTAIN:
                $query->where($column, 'LIKE', "%" . $value . "%");
                break;
            case FilterTypes::NOT_CONTAIN:
                $query->where($column, 'NOT LIKE', "%" . $value . "%");
                break;
            case FilterTypes::IS:
                $query->where($column, '=', $value);
                break;
            case FilterTypes::LESS_THAN:
                $query->where($column, '<', $value);
                break;
            case FilterTypes::GREATER_THAN:
                $query->where($column, '>', $value);
                break;
            case FilterTypes::LESS_THAN_OR_EQUAL:
                $query->where($column, '<=', $value);
                break;
            case FilterTypes::GREATER_THAN_OR_EQUAL:
                $query->where($column, '>=', $value);
                break;
            case FilterTypes::IN:
                $arrayValue = array_map('trim', explode(',', $value));
                $query->whereIn($column, array_filter($arrayValue));
                break;
            default:
        }
    }

    /**
     * @param $query
     * @param $type
     * @param $column1
     * @param $column2
     * @param $value
     * @param string $tableName
     */
    public static function applyCombinedColumnFilter($query, $type, $column1, $column2, $value, string $tableName = '')
    {
        $column1 = $tableName === '' ? $column1 : $tableName . "." . $column1;
        $column2 = $tableName === '' ? $column2 : $tableName . "." . $column2;
        
        // Remove commas and multi spaces
        $value = preg_replace('/,/', '', $value);
        $value = preg_replace ('/\s+/', ' ', $value);
       
        switch ($type) {
            case FilterTypes::IS_NOT:
                $query->where(DB::raw("CONCAT($column1, ' ', $column2)") , '!=', $value);
                break;
            case FilterTypes::CONTAIN:
                $query->where(DB::raw("CONCAT($column1, ' ', $column2)"), 'LIKE', "%" . $value . "%");
                break;
            case FilterTypes::IS:
                $query->where(DB::raw("CONCAT($column1, ' ', $column2)"), '=', $value);
                break;
            default:
        }
    }

    /**
     * @param $query
     * @param $type
     * @param $column
     * @param $value
     * @param string $tableName
     */
    public static function applyFileExtensionFilter($query, $type, $column, $value, string $tableName = '')
    {
        $column = $tableName === '' ? $column : $tableName . "." . $column;
        $extensionColumn = DB::raw("SUBSTRING_INDEX($column, '.', -1)");
        switch ($type) {
            case FilterTypes::IS_NOT:
                $query->where($extensionColumn, '!=', $value);
                break;
            case FilterTypes::CONTAIN:
                $query->where($extensionColumn, 'LIKE', "%" . $value . "%");
                break;
            case FilterTypes::IS:
                $query->where($extensionColumn, '=', $value);
                break;
            case FilterTypes::IN:
                $arrayValue = array_map('trim', explode(',', $value));
                $query->whereIn($extensionColumn, array_filter($arrayValue));
                break;
            default:
        }
    }

    /**
     * @param $filterColumn
     * @return array
     */
    private function getFilters($filterColumn)
    {
        return $this->originalFilters->filter(function ($item) use ($filterColumn) {
            return $item['column'] === $filterColumn;
        })->toArray();
    }
}
