<?php


namespace App\Helpers;


use App\Enum\EmploymentTypes;
use App\Enum\SegmentGenderDefault;
use App\Models\User\User;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;

/**
 * Class IntegrationHelper
 * @package App\Helpers
 */
class IntegrationHelper
{
    private $apiKey;

    const MERGE_API_URL = 'https://api.merge.dev/api';
    const INTEGRATION_URL = self::MERGE_API_URL . '/integrations';
    const LINK_TOKEN_URL = self::INTEGRATION_URL . '/create-link-token';
    const ACCOUNT_TOKEN_URL = self::INTEGRATION_URL . '/account-token';

    const HRIS_CATEGORY = 'hris';
    const HRIS_API_URL = self::MERGE_API_URL . '/hris/v1';
    const HRIS_EMPLOYEE_API_URL = self::HRIS_API_URL . '/employees';
    const DELETE_LINKED_ACCOUNT_URL = self::HRIS_API_URL . '/delete-account';
    const HRIS_LOCALTION_API_URL = self::HRIS_API_URL . '/locations';
    const GET_HRIS_LINKED_ACCOUNT_URL = self::HRIS_API_URL . '/linked-accounts?category=hris&status=COMPLETE';

    const ATS_CATEGORY = 'ats';
    const ATS_API_URL = self::MERGE_API_URL . '/ats/v1';
    const ATS_EMPLOYEE_API_URL = self::ATS_API_URL . '/candidates';
    const DELETE_LINKED_ACCOUNT_ATS_URL = self::ATS_API_URL . '/delete-account';
    const GET_ATS_LINKED_ACCOUNT_URL = self::ATS_API_URL . '/linked-accounts?category=ats&status=COMPLETE';

    const CRM_CATEGORY = 'crm';
    const CRM_API_URL = self::MERGE_API_URL . '/crm/v1';
    const CRM_EMPLOYEE_API_URL = self::CRM_API_URL . '/candidates';
    const DELETE_LINKED_ACCOUNT_CRM_URL = self::CRM_API_URL . '/delete-account';
    const GET_CRM_LINKED_ACCOUNT_URL = self::CRM_API_URL . '/linked-accounts?category=crm&status=COMPLETE';

    const HEADER_ACCOUNT_TOKEN = 'X-Account-Token';

    const PAGE_SIZE = 100;

    const DEFAULT_EMPLOYEE_RELATIONSHIPS = 'employments,team';

    const INACTIVE_EMPLOYMENT_STATUS = ['PENDING', 'INACTIVE'];

    /**
     * StripeHelper constructor.
     */
    public function __construct()
    {
        $this->apiKey = config('integration.merge.api_key');
    }

    /**
     * Create link token
     *
     * @param User $user
     * @param array $options
     * @return bool|array
     */
    public function createLinkToken(User $user, array $options = [])
    {
        $client = $user->getClient();
        $params = [
            'end_user_origin_id' => $client->getBusinessKey(), // Unique entity ID
            'end_user_organization_name' => $client->name,  // User's organization name
            'end_user_email_address' => $user->email, // User's email address
            'categories' => [self::HRIS_CATEGORY, self::CRM_CATEGORY, self::ATS_CATEGORY], // Integration category
        ];

        $response = Http::withToken($this->apiKey)
            ->post(self::LINK_TOKEN_URL, array_merge($params, $options));

        $linkToken = $response->json();

        if (!$response->successful()) {
            Log::error('Error occurred when creating Integration link token', $linkToken);

            return false;
        }

        // Return 'link_token' and 'integration_name' as keys
        return $linkToken;
    }

    /**
     * Retrieve account token
     *
     * @param string $publicToken
     * @return bool|array
     */
    public function retrieveAccountToken(string $publicToken)
    {
        $accountTokenURL = self::ACCOUNT_TOKEN_URL . '/' . $publicToken;
        $response = Http::withToken($this->apiKey)
            ->get($accountTokenURL);

        $accountToken = $response->json();

        if (!$response->successful()) {
            return false;
        }

        // Return 'account_token' and 'integration' as keys
        return $accountToken;
    }

    /**
     * Get all integration employees
     *
     * @param string $accountToken
     * @return array
     */
    public function getEmployees(string $accountToken)
    {
        $allEmployees = [];
        $cursor = null;
        $time = 0;
        do {
            $query = [
                'expand' => self::DEFAULT_EMPLOYEE_RELATIONSHIPS,
                'page_size' => self::PAGE_SIZE,
            ];
            if ($cursor) {
                $query =  array_merge($query, ['cursor' => $cursor]);
            }
            $response = Http::withToken($this->apiKey)
                ->withHeaders([self::HEADER_ACCOUNT_TOKEN => $accountToken])
                ->get(self::HRIS_EMPLOYEE_API_URL, $query);

            $data = $response->json();
            $cursor = $data['next'] ?? null;
            $allEmployees = array_merge($allEmployees, $data['results'] ?? []);
            $time++;
        } while (!empty($cursor));

        return $allEmployees;
    }

    /**
     *
     */
    public function getLocations(string $accountToken, string $id)
    {
        $response = Http::withToken($this->apiKey)
            ->withHeaders([self::HEADER_ACCOUNT_TOKEN => $accountToken])
            ->get(self::HRIS_LOCALTION_API_URL . '/' . $id);
        $data = $response->json();
        return $data;
    }

    /**
     * Get all integration candidates
     *
     * @param string $accountToken
     * @return array
     */
    public function getCandidates(string $accountToken)
    {
        $allEmployees = [];
        $cursor = null;
        $time = 0;
        do {
            $query = [
                'expand' => self::DEFAULT_EMPLOYEE_RELATIONSHIPS,
                'page_size' => self::PAGE_SIZE,
            ];
            if ($cursor) {
                $query =  array_merge($query, ['cursor' => $cursor]);
            }
            $response = Http::withToken($this->apiKey)
                ->withHeaders([self::HEADER_ACCOUNT_TOKEN => $accountToken])
                ->get(self::ATS_EMPLOYEE_API_URL, $query);

            $data = $response->json();
            $cursor = $data['next'] ?? null;
            $allEmployees = array_merge($allEmployees, $data['results'] ?? []);
            $time++;
        } while (!empty($cursor));

        return $allEmployees;
    }

    /**
     * Get all integration contacts
     *
     * @param string $accountToken
     * @return array
     */
    public function getContacts(string $accountToken)
    {
        $allEmployees = [];
        $cursor = null;
        $time = 0;
        do {
            $query = [
                'expand' => self::DEFAULT_EMPLOYEE_RELATIONSHIPS,
                'page_size' => self::PAGE_SIZE,
            ];
            if ($cursor) {
                $query =  array_merge($query, ['cursor' => $cursor]);
            }
            $response = Http::withToken($this->apiKey)
                ->withHeaders([self::HEADER_ACCOUNT_TOKEN => $accountToken])
                ->get(self::CRM_EMPLOYEE_API_URL, $query);

            $data = $response->json();
            $cursor = $data['next'] ?? null;
            $allEmployees = array_merge($allEmployees, $data['results'] ?? []);
            $time++;
        } while (!empty($cursor));

        return $allEmployees;
    }

    /**
     * Get all integration employees
     *
     * @param string $accountToken
     * @param $employeeId
     * @return boolean|array
     */
    public function getEmployeeDetail(string $accountToken, $employeeId)
    {
        $response = Http::withToken($this->apiKey)
            ->withHeaders([self::HEADER_ACCOUNT_TOKEN => $accountToken])
            ->get(self::HRIS_EMPLOYEE_API_URL . '/' . $employeeId, [
                'expand' => self::DEFAULT_EMPLOYEE_RELATIONSHIPS
            ]);

        if (!$response->successful()) {
            return false;
        }

        return $response->json();
    }

    /**
     * Get integrations
     *
     * @return bool|array
     */
    public function getIntegrations()
    {
        $response = Http::withToken($this->apiKey)->get(self::INTEGRATION_URL);

        if (!$response->successful()) {
            return false;
        }

        $integrations = $response->json();
        $integrations = Arr::where($integrations, function ($integration){
            return in_array(self::HRIS_CATEGORY, $integration['categories']) ||
                in_array(self::ATS_CATEGORY, $integration['categories']) ||
                in_array(self::CRM_CATEGORY, $integration['categories']);
        });

        return $integrations;
    }

    /**
     * Get linked account
     *
     * @param string $URL
     * @return bool
     */

    public function getLinkedAccount(string $URL)
    {
        $response = Http::withToken($this->apiKey)->get($URL);

        if (!$response->successful()) {
            return false;
        }

        $integrations = $response->json();
        
        if (count($integrations['results']) == 0) {
            return false;
        }

        return true;
    }

    /**
     * Delete linked account
     *
     * @param string $accountToken
     * @return bool
     */
    public function deleteLinkedAccount(string $accountToken)
    {
        $clientIntegration = $this->getLinkedAccount(self::GET_HRIS_LINKED_ACCOUNT_URL);

        if (!$clientIntegration) {
            return true;
        }

        $response = Http::withToken($this->apiKey)
            ->withHeaders([self::HEADER_ACCOUNT_TOKEN => $accountToken])
            ->post(self::DELETE_LINKED_ACCOUNT_URL);

        if (!$response->successful()) {
            return false;
        }

        return true;
    }

    /**
     * Delete linked account
     *
     * @param string $accountToken
     * @return bool
     */
    public function deleteCandidateLinkedAccount(string $accountToken)
    {
        $clientIntegration = $this->getLinkedAccount(self::GET_ATS_LINKED_ACCOUNT_URL);

        if (!$clientIntegration) {
            return true;
        }

        $response = Http::withToken($this->apiKey)
            ->withHeaders([self::HEADER_ACCOUNT_TOKEN => $accountToken])
            ->post(self::DELETE_LINKED_ACCOUNT_ATS_URL);

        if (!$response->successful()) {
            return false;
        }

        return true;
    }

    public static function convertEmploymentType($employment)
    {
        if ($employment) {
            switch ($employment['employment_type']) {
                case 'FULL_TIME':
                    return EmploymentTypes::FULL_TIME;
                case 'PART_TIME':
                    return EmploymentTypes::PART_TIME;
                case 'CONTRACTOR':
                    return EmploymentTypes::CONTRACTOR;
                case 'PARTNER':
                    return EmploymentTypes::PARTNER;
                case 'VENDOR':
                    return EmploymentTypes::VENDOR;
                case 'CANDIDATE':
                    return EmploymentTypes::CANDIDATE;
                case 'CONTACT':
                    return EmploymentTypes::CONTACT;
                case 'CUSTOMER':
                    return EmploymentTypes::CUSTOMER;
                case 'AGENT':
                    return EmploymentTypes::AGENT;
                case 'INTERN':
                    return EmploymentTypes::INTERN;
                case 'OTHERS':
                    return EmploymentTypes::OTHERS;
            }
        }

        return null;
    }

    public static function convertGenderSegment($gender)
    {
        switch ($gender) {
            case 'MALE':
                return SegmentGenderDefault::MALE;
            case 'FEMALE':
                return SegmentGenderDefault::FEMALE;
            default:
                return null;
        }
    }
}