<?php


namespace App\Helpers;

use App\Enum\EmployeeTokens;
use App\Facades\Context;
use App\Models\Client\Client;
use App\Models\User\User;
use Exception;
use App\Enum\Roles;
use App\Services\UserAccess\GetAccessByRole;
use App\Enum\AccessPermissions;
use Illuminate\Support\Facades\DB;

class ModelHelper
{
    const REPLACEMENT_KEYS = [
        EmployeeTokens::FIRST_NAME,
        EmployeeTokens::LAST_NAME,
        EmployeeTokens::DATE_OF_BIRTH,
        EmployeeTokens::EMAIL_ADDRESS,
        EmployeeTokens::START_DATE,
        EmployeeTokens::JOB_TITLE,
        EmployeeTokens::GENDER,
        EmployeeTokens::DEPARTMENT,
        EmployeeTokens::TEAM,
        EmployeeTokens::LOCATION,
        EmployeeTokens::MANAGER_NAME,
        EmployeeTokens::SECONDARY_MANAGER_NAME,
    ];

    const LOCATION_ACCESS_PERMISSIONS = [
        AccessPermissions::ACCESS_EMPLOYEE_MESSAGE_BY_LOCATION,
        AccessPermissions::ACCESS_EMPLOYEE_DIRECTORY_BY_LOCATION,
    ];

    const DEPARTMENT_ACCESS_PERMISSIONS = [
        AccessPermissions::ACCESS_EMPLOYEE_MESSAGE_BY_DEPARTMENT,
        AccessPermissions::ACCESS_EMPLOYEE_DIRECTORY_BY_DEPARTMENT,
    ];

    const TEAM_ACCESS_PERMISSIONS = [
        AccessPermissions::ACCESS_EMPLOYEE_MESSAGE_BY_TEAM,
        AccessPermissions::ACCESS_EMPLOYEE_DIRECTORY_BY_TEAM,
    ];

    /**
     * Convert business key to id
     *
     * @param string $model
     * @param string|array $code
     * @return string|array
     * @throws Exception
     */
    static function convertBusinessKeyToId(string $model, $code)
    {
        $model = app($model);
        if (!method_exists($model, 'findByBusinessKey')) {
            throw new Exception('Model must have business key');
        }
        $isArrayCode = is_array($code);
        $convertArray = ObjectHelper::toArray($code);
        $resultArray = array();
        foreach ($convertArray as $item) {
            if (!is_null($item)) {
                $resultArray[] = $model::findByBusinessKey($item)->getKey();
            }
        }
        return $isArrayCode ? $resultArray : $resultArray[0];
    }

    /**
     * Convert id to business key
     *
     * @param string $model
     * @param int|array $ids
     * @return string|array
     * @throws Exception
     */
    static function convertIdToBusinessKey(string $model, $ids)
    {
        $model = app($model);
        if (!method_exists($model, 'findByBusinessKey')) {
            throw new Exception('Model must have business key');
        }
        $isArrayCode = is_array($ids);
        $convertArray = ObjectHelper::toArray($ids);
        $resultArray = array();
        foreach ($convertArray as $item) {
            if (!is_null($item)) {
                $resultArray[] = $model::find($item)->getBusinessKey();
            }
        }
        return $isArrayCode ? $resultArray : $resultArray[0];
    }

    static function setConditionFindByClient($query, $table = "", $columnName = "client_id")
    {
        $client = Context::client()->get();
        $columnName = $table === "" ? $columnName : $table . "." . $columnName;
        if ($client) {
            $query->where($columnName, $client->getKey());
        } else {
            //If admin, get all except demo data
            $clientDemoIds = Client::where('is_demo', true)->get()->pluck('id');
            if (sizeof($clientDemoIds) > 0) {
                $query->whereNotIn($columnName, $clientDemoIds);
            }
        }
    }

    static function setConditionFindByClientForEmployee($query, $table = "", $columnName = "client_id")
    {
        $client = Context::client()->get();
        $columnName = $table === "" ? $columnName : $table . "." . $columnName;

        if ($client) {
            $query->whereHas($table, function ($query) use ($columnName, $client) {
                $query->where($columnName, $client->getKey());
            });
        } else {
            //If admin, get all except demo data
            $clientDemoIds = Client::where('is_demo', true)->get()->pluck('id');
            if (sizeof($clientDemoIds) > 0) {
                $query->whereNotIn($columnName, $clientDemoIds);
            }
        }
    }


    static function convertTokenKeyToValueInJouneySteps($message)
    {
        $user = Context::user()->get();
        $employee = $user->employee;

        $replacementValues = [
            $user->first_name,
            $user->surname,
            !empty($employee->date_of_birth) ? DateHelper::formatYearMonthDay($employee->date_of_birth) : null,
            $user->email,
            !empty($employee->start_date) ? DateHelper::formatYearMonthDay($employee->start_date) : null,
            $employee->job_title,
            !empty($employee->gender_id) ? $employee->gender->name : null,
            !empty($employee->department_id) ? $employee->department->name : null,
            !empty($employee->team_id) ? $employee->team->name : null,
            !empty($employee->location_id) ? $employee->location->name : null,
            !empty($employee->manager_id) && isset($employee->manager) ? $employee->manager->first_name . ' ' . $employee->manager->surname : null,
            !empty($employee->secondary_manager_id) && isset($employee->secondaryManager) ? $employee->secondaryManager->first_name . ' ' . $employee->secondaryManager->surname : null
        ];

        $newMessage = str_replace(self::REPLACEMENT_KEYS, $replacementValues, $message);

        return $newMessage;
    }

    static function setConditionFindByEmployee($query, $accessPermission, $userIds = [])
    {
        $user = Context::user()->get();
        $employee = $user->employee;
        $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
        $accessPermissions = $employeeAccess[$accessPermission];

        if (empty($userIds)) {
            foreach ($accessPermissions as $permission => $value) {
                switch (true) {
                    case in_array($permission, self::LOCATION_ACCESS_PERMISSIONS) && $value:
                        $query->where('employees.location_id', $employee->location_id);
                        break;
                    case in_array($permission, self::DEPARTMENT_ACCESS_PERMISSIONS) && $value:
                        $query->where('employees.department_id', $employee->department_id);
                        break;
                    case in_array($permission, self::TEAM_ACCESS_PERMISSIONS) && $value:
                        $query->where('employees.team_id', $employee->team_id);
                        break;
                    default:
                        break;
                }
            }
        } else {
            foreach ($accessPermissions as $permission => $value) {
                switch (true) {
                    case in_array($permission, self::LOCATION_ACCESS_PERMISSIONS) && $value:
                        $query->where('employees.location_id', $employee->location_id)->orWhereIn('employees.user_id', $userIds);
                        break;
                    case in_array($permission, self::DEPARTMENT_ACCESS_PERMISSIONS) && $value:
                        $query->where('employees.department_id', $employee->department_id)->orWhereIn('employees.user_id', $userIds);
                        break;
                    case in_array($permission, self::TEAM_ACCESS_PERMISSIONS) && $value:
                        $query->where('employees.team_id', $employee->team_id)->orWhereIn('employees.user_id', $userIds);
                        break;
                    default:
                        break;
                }
            }
        }
    }
}