<?php

namespace App\Helpers;

use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;

class PandaDocHelper {
    
    private $token;
    private $logging = true;
    
    const TEMPLATE_URL = 'https://api.pandadoc.com/public/v1/templates';
    const DOCUMENT_URL = 'https://api.pandadoc.com/public/v1/documents';
    const FOLDER_URL = 'https://api.pandadoc.com/public/v1/documents/folders';
    const DOCUMENT_LINK_URL = 'https://app.pandadoc.com/s';
    const ACCESS_TOKEN_URL = 'https://api.pandadoc.com/oauth2/access_token';
    
    const SIGNER_ROLE = 'signer';
    
    const DOCUMENT_SENT_STATUS = 'document.sent';
    
    const PERMISSION_SCOPE = 'read+write';

    const AUTH_LINKED_STATUS = 'linked';
    const AUTH_OPEN_STATUS = 'open';
    
    const MAX_LIMIT = 100;
    const TEMPLATE_VERSION = '2';

    /**
     * Set authorization token
     *
     * @param $token
     * @return PandaDocHelper
     */
    public function setToken(string $token)
    {
        $this->token = $token;
        
        return $this;
    }

    /**
     * Set error logging
     *
     * @param bool $isLogging
     * @return PandaDocHelper
     */
    public function setLogging(bool $isLogging)
    {
        $this->logging = $isLogging;

        return $this;
    }

    /**
     * Get authorization token
     * 
     * @return string
     */
    public function getToken()
    {
        return $this->token;
    }

    public function getSignerRole()
    {
        return self::SIGNER_ROLE;
    }

    /**
     * Get templates
     *
     * @param array $query
     * @return array|null
     */
    public function getTemplates(array $query = [])
    {
        $allTemplates = [];
        $page = 1;
        do {
            $response = Http::withToken($this->getToken())
                ->get(self::TEMPLATE_URL, array_merge($query, ['page' => $page, 'count' => self::MAX_LIMIT]));
            
            $templates = $response->json();
            
            if (!$response->successful()) {
                Log::error('Error occurred when getting form templates', $templates);
                break;
            }
            
            $allTemplates = array_merge($allTemplates, $templates['results']);
            $count = count($templates['results']);
            $page ++;
        } while ($count === self::MAX_LIMIT);

        $allTemplates = Arr::where($allTemplates, function ($template) {
            return $template['version'] == self::TEMPLATE_VERSION;
        });

        return $allTemplates;
    }

    /**
     * Get template detail
     *
     * @param string $templateId
     * @return array|null
     */
    public function getTemplateDetail(string $templateId)
    {
        $templateDetailURL = self::TEMPLATE_URL . '/' . $templateId . '/details';

        $response = Http::withToken($this->getToken())
            ->get($templateDetailURL);

        $template = $response->json();

        if (!$response->successful()) {
            Log::error('Error occurred when getting template detail', $template);

            return null;
        }

        return $template;
    }

    /**
     * Create document
     * 
     * @param string $templateId
     * @param string $name
     * @param array $recipients
     * @param array $options
     * @return bool|mixed
     */
    public function createDocumentFromTemplate(string $templateId, string $name, array $recipients, array $options = [])
    {
        $params = [
            'name' => $name,
            'template_uuid' => $templateId,
            'recipients' => $recipients
        ];
        
        $response = Http::withToken($this->getToken())
            ->post(self::DOCUMENT_URL, array_merge($params, $options));

        $document = $response->json();

        if (!$response->successful()) {
            Log::error('Error occurred when creating document', $document);

            return false;
        }

        return $document;
    }

    /**
     * Create folder
     *
     * @param string $name
     * @param array $options
     * @return bool|mixed
     */
    public function createFolder(string $name, array $options = [])
    {
        $params = [
            'name' => $name
        ];
        
        $response = Http::withToken($this->getToken())
            ->post(self::FOLDER_URL, array_merge($params, $options));

        $folder = $response->json();

        if (!$response->successful()) {
            Log::error('Error occurred when creating folder', $folder);

            return false;
        }

        return $folder;
    }

    /**
     * Send document
     * 
     * @param string $documentId
     * @param array $options
     * @return bool|mixed
     */
    public function sendDocument(string $documentId, array $options = [])
    {
        $sendDocumentURL = self::DOCUMENT_URL . '/' . $documentId . '/send';

        $options = array_merge(['silent' => true], $options);

        $response = Http::withToken($this->getToken())
            ->post($sendDocumentURL, $options);

        $document = $response->json();

        if ($this->logging && !$response->successful()) {
            Log::error('Error occurred when sending document', $document);
            return false;
        }

        return $document;
    }

    /**
     * Create document Link
     *
     * @param string $documentId
     * @param string $recipient
     * @param array $options
     * @return bool|mixed
     */
    public function createDocumentLink(string $documentId, string $recipient, array $options = [])
    {
        $createDocumentLink = self::DOCUMENT_URL . '/' . $documentId . '/session';
        
        $params = [
            'recipient' => $recipient
        ];

        $response = Http::withToken($this->getToken())
            ->post($createDocumentLink, array_merge($params, $options));

        $document = $response->json();

        if (!$response->successful()) {
            Log::error('Error occurred when creating document link', $document);

            return false;
        }

        return $document;
    }

    /**
     * Get document Link
     *
     * @param string $documentId
     * @param string $recipient
     * @param array $options
     * @return string|null
     */
    public function getDocumentLink(string $documentId, string $recipient, array $options = [])
    {
        $document = $this->createDocumentLink($documentId, $recipient, $options);
        if ($document) {
            return self::DOCUMENT_LINK_URL . '/' .  $document['id'];
        }
        
       return null;
    }

    public function getDocumentStatus(string $documentId) {
        $sendDocumentURL = self::DOCUMENT_URL . '/' . $documentId;

        $response = Http::withToken($this->getToken())
            ->get($sendDocumentURL);

        $document = $response->json();

        if (!$response->successful()) {
            Log::error('Error occurred when getting document status', $document);

            return null;
        }

        return $document;
    }

    public function getDocumentDetail(string $documentId) {
        $sendDocumentURL = self::DOCUMENT_URL . '/' . $documentId . '/details';

        $response = Http::withToken($this->getToken())
            ->get($sendDocumentURL);

        $document = $response->json();

        if (!$response->successful()) {
            Log::error('Error occurred when getting document detail', $document);

            return false;
        }

        return $document;
    }

    /**
     * Delete document
     * 
     * @param string $documentId
     * @return bool
     */
    public function deleteDocument(string $documentId)
    {
        $sendDocumentURL = self::DOCUMENT_URL . '/' . $documentId;

        $response = Http::withToken($this->getToken())
            ->delete($sendDocumentURL);

        $document = $response->json();

        if (!$response->successful()) {
            Log::error('Error occurred when deleting document', $document);

            return false;
        }

        return true;
    }

    /**
     * Download document and return file content
     * 
     * @param string $documentId
     * @return string
     */
    public function downloadDocument(string $documentId)
    {
        $sendDocumentURL = self::DOCUMENT_URL . '/' . $documentId . '/download';

        $response = Http::withToken($this->getToken())
            ->get($sendDocumentURL);

        if (!$response->successful()) {
            Log::error('Error occurred when downloading document');

            return false;
        }

        return $response->getBody()->getContents();
    }

    /**
     * Get authorization url
     * 
     * @return string
     */
    public function getAuthorizationUrl()
    {
        $pandaDocClientId = config('pandadoc.client_id');
        $redirectUrl = config('pandadoc.redirect_url');
        $scope = self::PERMISSION_SCOPE;
        
        return "https://app.pandadoc.com/oauth2/authorize?client_id=$pandaDocClientId&redirect_uri=$redirectUrl&scope=$scope&response_type=code";
    }

    /**
     * Create access token
     * 
     * @param string $code
     * @return mixed|bool
     */
    public function createAccessToken(string $code)
    {
        $params = [
            'grant_type' => 'authorization_code',
            'client_id' => config('pandadoc.client_id'),
            'client_secret' => config('pandadoc.client_secret'),
            'code' => $code,
            'scope' => self::PERMISSION_SCOPE,
            'redirect_uri' => config('pandadoc.redirect_url')
        ];

        $response = Http::asForm()->post(self::ACCESS_TOKEN_URL, $params);

        $token = $response->json();
        
        if (!$response->successful()) {
            Log::error('Error occurred when creating access token', $token);

            return false;
        }

        return $token;
    }

    /**
     * Check template has signer role
     *
     * @param $templateDetail
     * @return bool
     */
    public function checkTemplateHasSignerRole($templateDetail)
    {
        if (!empty($templateDetail)) {
            foreach ($templateDetail['roles'] as $role) {
                if ($role['name'] === self::SIGNER_ROLE) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Check template has all fields that are assigned to specific role
     *
     * @param $templateDetail
     * @return bool
     */
    public function checkTemplateHasAllFieldsAssigned($templateDetail)
    {
        if (empty($templateDetail)) {
            return false;
        }

        foreach ($templateDetail['fields'] as $field) {
            if (!isset($field['assigned_to']) || empty($field['assigned_to']['id'])) {
                return false;
            }
        }

        return true;
    }
}