<?php

namespace App\Helpers;

use Closure;
use App\Enum\SortTypes;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;


/**
 * Class SortFluent
 * @package App\Helpers
 */
class SortFluent
{
    /**
     * @var array
     */
    private $sortTypes = [SortTypes::ASC, SortTypes::DESC];

    /**
     * @var
     */
    private $sorts;
    /**
     * @var
     */
    private $originalSorts;
    /**
     * @var null
     */
    private $allowedSorts = null;
    /**
     * @var
     */
    private $customSorts;
    /**
     * @var null
     */
    private $query = null;

    /**
     * @var null
     */
    private $tableName = '';

    /**
     *
     */
    private function constructor()
    {
    }

    /**
     * @return SortFluent
     */
    static function init(): SortFluent
    {
        $sortFluent = new SortFluent();
        $sortFluent->sorts = collect();
        $sortFluent->originalSorts = collect();
        $sortFluent->customSorts = collect();
        return $sortFluent;
    }

    /**
     * @param $query
     * @return $this
     */
    public function setQuery($query)
    {
        $this->query = $query;
        return $this;
    }

    /**
     * @param string $tableName
     * @return SortFluent
     */
    public function setTable(string $tableName)
    {
        $this->tableName = $tableName;
        return $this;
    }

    /**
     * @param array $options
     * @return $this
     */
    public function setSortOptions(array $options): SortFluent
    {
        if (isset($options['sort'])) {
            $this->sorts = collect($options['sort']);
            $this->originalSorts = collect($options['sort']);
        }
        return $this;
    }

    /**
     * @param array $sorts
     * @return $this
     */
    public function setAllowedSorts(array $sorts): SortFluent
    {
        $this->allowedSorts = collect($sorts);
        return $this;
    }

    /**
     * @param $columnSort
     * @return $this
     */
    public function removeSort($columnSort): SortFluent
    {
        $this->sorts = $this->sorts->reject(function ($value) use ($columnSort) {
            return is_array($columnSort) ?
                in_array($value['column'], $columnSort) :
                $value['column'] === $columnSort;
        });
        return $this;
    }

    /**
     * @param string $sortColumn
     * @param Closure $callback
     * @return $this
     */
    public function setCustomSort(string $sortColumn, Closure $callback): SortFluent
    {
        $this->customSorts->push([$sortColumn, $callback]);
        return $this;
    }

    /**
     * @return $this
     * @throws ValidationException
     */
    public function validate(): SortFluent
    {
        foreach ($this->sorts as $sort) {
            if (!isset($sort['column'])) {
                throw ValidationException::withMessages(['sort[column]' => __('validation.required', ['attribute' => 'sort[column]'])]);
            }
            if (!empty($this->allowedSorts) && !in_array($sort['column'], $this->allowedSorts->toArray())) {
                throw ValidationException::withMessages(['sort[column]' => __('validation.invalid', ['attribute' => 'sort[column]'])]);
            }
            if (!isset($sort['column'])) {
                throw ValidationException::withMessages(['sort[column]' => __('validation.required', ['attribute' => 'sort[column]'])]);
            }
            if (isset($sort['direction'])) {
                if (!in_array($sort['direction'], $this->sortTypes)) {
                    throw ValidationException::withMessages(['sort[direction]' => __('validation.invalid', ['attribute' => 'sort[direction]'])]);
                }
            }
        }
        return $this;
    }

    /**
     * @param string $column
     * @throws ValidationException
     */
    public function validateColumn(string $column): void
    {
        if (!$this->allowedSorts->contains($column)) {
            throw ValidationException::withMessages(['sort[column]' => __('validation.invalid', ['attribute' => 'sort[column]'])]);
        }
    }

    /**
     *
     */
    public function applySort()
    {
        foreach ($this->sorts as $sort) {
            $this->validateColumn($sort['column']);
            self::applyBasicSort($this->query, $sort['column'], $sort['direction'], $this->tableName);
        }

        foreach ($this->customSorts as [$column, $callback]) {
            $sort = $this->getSort($column);
            if (!empty($sort)) {
                $this->validateColumn($sort['column']);
                $callback($this->query, $sort['column'], $sort['direction']);
            }
        }
    }


    /**
     * @param $query
     * @param $column
     * @param $direction
     * @param string $table
     */
    public static function applyBasicSort($query, $column, $direction, string $table = '')
    {
        $column = $table === '' ? $column : $table . "." . $column;
        $query->orderBy($column, $direction);
    }

    /**
     * @param $query
     * @param $column
     * @param $direction
     * @param string $table
     */
    public static function applyFileExtensionSort($query, $column, $direction, string $table = '')
    {
        $column = $table === '' ? $column : $table . "." . $column;
        $extensionColumn = DB::raw("SUBSTRING_INDEX($column, '.', -1)");
        $query->orderBy($extensionColumn, $direction);
    }

    /**
     * @param $sortColumn
     * @return |null
     */
    private function getSort($sortColumn)
    {
        foreach ($this->originalSorts as $sort) {
            if ($sort['column'] === $sortColumn) {
                return $sort;
            }
        }
        return null;
    }
}
