<?php


namespace App\Helpers;


use Stripe\Card;
use Stripe\StripeClient;

/**
 * Class StripeHelper
 * @package App\Helpers
 */
class StripeHelper
{
    const MAX_LIMIT = 100;
    
    /**
     * @var StripeClient
     */
    protected $stripe;

    /**
     * StripeHelper constructor.
     */
    public function __construct()
    {
        $secretKey = config('stripe.secret_key');
        $this->stripe = new StripeClient($secretKey);
    }

    /**
     * @param array $options
     * @return \Stripe\StripeObject[]
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getListPrices(array $options = [])
    {
        return $this->stripe->prices->all($options)->data;
    }

    /**
     * @param array $options
     * @return \Stripe\StripeObject[]
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getListProducts(array $options = [])
    {
        return $this->stripe->products->all($options)->data;
    }

    /**
     * @param string $id
     * @return \Stripe\Product
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getProductDetail(string $id)
    {
        return $this->stripe->products->retrieve($id);
    }

    /**
     * @param int $limit
     * @return \Stripe\StripeObject[]
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getAllCustomers(int $limit = self::MAX_LIMIT)
    {
        $allCustomers = [];
        
        $customers =  $this->stripe->customers->all(['limit' => $limit, 'expand' => ['data.subscriptions']]);
        foreach ($customers->autoPagingIterator() as $customersInPage) {
            $allCustomers[] = $customersInPage;
        }
        
        return $allCustomers;
    }

    public function getAllCustomersWithSubscriptions(int $limit = self::MAX_LIMIT)
    {
        $customersWithSubscriptions = [];
        $subscriptions = $this->stripe->subscriptions->all(["limit" => $limit, 'expand' => ['data.customer']]);

        foreach ($subscriptions->autoPagingIterator() as $subscription) {
            $customer = $subscription->customer;

            // Check if the customer has an active subscription
            if ($subscription->status === 'active' && !empty($subscription->items->data)) {
                // Add the customer to the list
                $customersWithSubscriptions[] = $customer;
            }
        }
        return $customersWithSubscriptions;
    }

    /**
     * @param string $id
     * @return \Stripe\Customer
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getCustomer(string $id)
    {
        return $this->stripe->customers->retrieve($id);
    }

    /**
     * @param string $customerId
     * @return \Stripe\StripeObject
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getCustomerSubscription(string $customerId)
    {
        return $this->stripe->subscriptions->all(['customer' => $customerId])->first();
    }

    /**
     * @return \Stripe\Collection
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getAllSubscriptions()
    {
        $subscriptions = $this->stripe->subscriptions->all(["limit" => self::MAX_LIMIT]);
        foreach ($subscriptions->autoPagingIterator() as $subscription){
            $result[] = $subscription;
        }
        return $result;
    }

    /**
     * @param string $customerId
     * @param int $limit
     * @return \Stripe\StripeObject[]
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getCustomerInvoices(string $customerId, int $limit = self::MAX_LIMIT)
    {
        return $this->stripe->invoices->all(['customer' => $customerId, 'limit' => $limit])->data;
    }

    /**
     * @param string $customerId
     * @return \Stripe\Invoice
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getCustomerUpcomingInvoice(string $customerId)
    {
        return $this->stripe->invoices->upcoming(['customer' => $customerId]);
    }

    /**
     * @param string $customerId
     * @param int $limit
     * @return \Stripe\StripeObject[]
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getAllCustomerCards(string $customerId, int $limit = self::MAX_LIMIT)
    {
        return $this->stripe->customers->allSources($customerId, [
            'object' => 'card', 
            'limit' => $limit
        ])->data;
    }
    
    /**
     * @param string $customerId
     * @param string $sourceId
     * @return \Stripe\AlipayAccount|\Stripe\BankAccount|\Stripe\BitcoinReceiver|\Stripe\Card|\Stripe\Source
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getSource(string $customerId, string $sourceId)
    {
        return $this->stripe->customers->retrieveSource($customerId, $sourceId);
    }

    /**
     * @param string $customerId
     * @return \Stripe\AlipayAccount|\Stripe\BankAccount|\Stripe\BitcoinReceiver|\Stripe\Card|\Stripe\Source
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getCustomerDefaultSource(string $customerId)
    {
        $customer = $this->getCustomer($customerId);

        return $customer->default_source ? $this->getSource($customerId, $customer->default_source) : null;
    }
    
    /**
     * @param string $customerId
     * @return \Stripe\AlipayAccount|\Stripe\BankAccount|\Stripe\BitcoinReceiver|\Stripe\Card|\Stripe\Source
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getCustomerDefaultCard(string $customerId)
    {
        $source = $this->getCustomerDefaultSource($customerId);
        
        if ($source instanceof Card) {
            return $source;
        }
        
        return null;
    }

    /**
     * @param string $customerId
     * @param string $sourceId
     * @return \Stripe\AlipayAccount|\Stripe\BankAccount|\Stripe\BitcoinReceiver|\Stripe\Card|\Stripe\Source
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function deleteCustomerSource(string $customerId, string $sourceId)
    {
        return $this->stripe->customers->deleteSource($customerId, $sourceId);
    }

    /**
     * @param string $customerId
     * @param string $token
     * @return \Stripe\AlipayAccount|\Stripe\BankAccount|\Stripe\BitcoinReceiver|\Stripe\Card|\Stripe\Source
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function createCustomerSource(string $customerId, string $token)
    {
        return $this->stripe->customers->createSource($customerId, ['source' => $token]);
    }

    /**
     * @param string $subscriptionId
     * @param array $params
     * @return \Stripe\Subscription
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function updateSubscription(string $subscriptionId, array $params)
    {
        return $this->stripe->subscriptions->update($subscriptionId, $params);
    }
}