<?php


namespace App\Helpers;

use App\Contracts\CreatorProperty;
use App\Enum\AccessPermissions;
use App\Enum\Roles;
use App\Exceptions\UnauthenticatedHttpException;
use App\Exceptions\UnauthorizedHttpException;
use App\Facades\Context;
use App\Models\User\User;

class UserAccessHelper
{
    /**
     * Get all access permissions
     *
     * @return array
     */
    public static function getAllUserAccess(): array
    {
        $accessPermissions = [];
        foreach (AccessPermissions::ALL_ACCESS_GROUP_PERMISSIONS as $groupPermission) {
            foreach ($groupPermission as $permission) {
                $accessPermissions[] = $permission;
            }
        }

        return $accessPermissions;
    }

    /**
     * Check if the current user has given access
     *
     * @param string $access
     * @param bool $throwError
     * @return bool
     */
    public static function hasAccess(string $access, bool $throwError = false): bool
    {
        $user = Context::user()->get();
        if (!$user) {
            throw new UnauthenticatedHttpException();
        }

        if ($user->hasRole([Roles::SUPER_ADMIN])) {
            return true;
        }

        if (!$user->can($access)) {
            if ($throwError) {
                throw new UnauthorizedHttpException();
            }
            return false;
        }

        return true;
    }

    /**
     * Check if the current user has access on own object
     *
     * @param $object
     * @return bool
     */
    public static function hasAccessAsOwner($object): bool
    {
        $user = Context::user()->get();
        if (!$user) {
            throw new UnauthenticatedHttpException();
        }

        if ($user->hasRole([Roles::SUPER_ADMIN, Roles::ADMIN])) {
            return true;
        }

        if ($object instanceof CreatorProperty && $user->getKey() === $object->getCreatorId()){
            return true;
        }

        throw new UnauthorizedHttpException();
    }

    /**
     * Get manager staff users
     *
     * @param User $manager
     * @return mixed
     */
    public static function getManagerStaffUsers(User $manager)
    {
        return User::whereHas('employee', function ($q) use ($manager) {
            $q->where('manager_id', $manager->getKey());
        })->get();
    }

    /**
     * Check if the current user is manager
     *
     * @param User $user
     * @return bool
     */
    public static function isManager(User $user): bool
    {
        if ($user->hasRole([Roles::SUPER_ADMIN, Roles::ADMIN])) {
            return false;
        }

        if ($user->hasRole([Roles::MANAGER])) {
            return true;
        }

        return false;
    }
}