<?php


namespace App\Http\Controllers\Api;

use App\AppContext\RequestContext;
use App\Facades\Context;
use Illuminate\Http\JsonResponse;
use function Safe\json_decode;
use App\Models\Account\ApiUsage;
use App\Http\Controllers\Controller;
use App\Traits\JsonRespondController;
use Dingo\Api\Routing\UrlGenerator;

class ApiController extends Controller
{
    use JsonRespondController;

    const STATUS_SUCCESS = true;
    const STATUS_FAIL = false;

    /**
     * @var string
     */
    protected $sort = 'created_at';

    /**
     * @var string
     */
    protected $withParameter = null;

    /**
     * @var string
     */
    protected $sortDirection = 'asc';

    protected $options = array();

    public function __construct()
    {
        parent::__construct();
        $this->middleware(function($request, $next) {
            (new ApiUsage)->log($request);

            $requestContext = Context::request();

            $requestContext->fromRequest($request);

            // TODO Remove this after all services use request context.
            $this->options = $requestContext->getOptions();

            // make sure the JSON is well formatted if the call sends a JSON
            // if the call contains a JSON, the call must not be a GET or
            // a DELETE
            // TODO: there is probably a much better way to do that
            try {
                if ($request->method() != 'GET' && $request->method() != 'DELETE'
                    && is_null(json_decode($request->getContent()))) {
                    return $this->setHTTPStatusCode(400)
                        ->setErrorCode(37)
                        ->respondWithError();
                }
            } catch (\Safe\Exceptions\JsonException $e) {
                // no error
            }

            return $next($request);
        });
    }

    /**
     * Default request to the API.
     * @return JsonResponse
     */
    public function success()
    {
        $version = config('api.version');
        return $this->respond([
            'success' => [
                'message' => 'Welcome to ' . env('APP_NAME', 'Laravel'),
            ],
            'version' => $version,
            'links' => [
                'api_url' => app(UrlGenerator::class)->version($version)->route('apiSuccess'),
            ],
        ]);
    }

    /**
     * Sends a response with success
     *
     * @param string $message
     * @return JsonResponse
     */
    public function responseWithSuccessMessage(string $message)
    {
        $responseData = [
            'message' => $message,
        ];

        return $this->respond($responseData);
    }

    /**
     * Sends a response with custom status code
     *
     * @param string $message
     * @param array $data
     * @return JsonResponse
     */
    public function responseWithStatusCode(int $code, string $message = '', array $data = [])
    {
        $responseData = [
            'status' => $code == 200 ? self::STATUS_SUCCESS : self::STATUS_FAIL,
            'message' => $message,
            'data' => $data,
        ];
        return $this->setHTTPStatusCode($code)->respond($responseData);
    }

    public function responseData($data = [], int $code = 200)
    {
        $responseData = [
            'data' => $data,
        ];
        return $this->setHTTPStatusCode($code)->respond($responseData);
    }
}
