<?php

namespace App\Http\Controllers\Api\Mobile_V1\Auth;

use App\Enum\ErrorCode;
use App\Enum\Roles;
use App\Exceptions\UnauthenticatedHttpException;
use App\Guards\GuardNames;
use App\Http\Controllers\Api\ApiController;
use App\Services\Mobile\Auth\GetLoginMagicLinkForDemoClient;
use App\Services\Mobile\Auth\SyncSignalId;
use App\Services\Mobile\Auth\UserDemoLogin;
use App\Services\Mobile\Auth\UserLogin;
use App\Services\Mobile\Auth\UserLogout;
use App\Traits\Guard;
use App\Traits\JsonRespondController;
use App\Traits\MobileJsonResponseController;
use Illuminate\Auth\Events\Verified;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth as Auth;
use Laravel\Passport\HasApiTokens;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

class AuthController extends ApiController
{
    use HasApiTokens, MobileJsonResponseController;
    use Guard;

    /**
     * Login action
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function login(Request $request)
    {
        try {
            // This method will throw exception if authentication failed
            Auth::basic();

            $currentUser = Auth::user();
            $guards = [GuardNames::UserIsActive, GuardNames::ClientIsActive, GuardNames::ClientSubscriptionIsExpired];
            if ($currentUser->hasRole(Roles::CLIENT_CONSOLE_ROLES)) {
                $guards[] = GuardNames::UserEmailVerified;
            }
            $this->guard($guards, ['user' => $currentUser, 'client' => $currentUser->getClient()]);

            $result = app(UserLogin::class)->execute($request->all());

        } catch (UnauthorizedHttpException $e) {
            throw new UnauthenticatedHttpException('Invalid email or password.', ErrorCode::INVALID_CREDENTIALS);
        } catch (\Exception $e) {
            return $this->errorResponse($e->getMessage(), $e->getCode());
        }
        return $this->successResponse(__('You are successfully logged in.'), $result );
    }


    /**
     * Logout action
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function logout(Request $request)
    {
        try {
            $data = $request->all();
            $result = app(UserLogout::class)->execute($data);
        } catch (Exception $e) {
            return $this->errorResponse($e->getMessage(), $e->getCode());
        }

        return $this->successResponse(__('You are successfully logged out.'), $result);
    }

    /**
     *  Send password reset link
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function sync(Request $request)
    {
        try {
            $data = $request->all();
            $result = app(SyncSignalId::class)->execute($data);
        } catch (Exception $e) {
            return $this->errorResponse($e->getMessage(), $e->getCode());
        }

        return $this->successResponse(__('Sync signal_id successfully.'), $result);
    }
}
