<?php

namespace App\Http\Controllers\Api\v1\Action;

use App\Http\Controllers\Api\ApiController;
use App\Http\Resources\Action\Action as ActionResource;
use App\Http\Resources\Action\ActionSequence as ActionSequenceResource;
use App\Http\Resources\Action\ActionSequenceStep as ActionSequenceStepResource;
use App\Http\Resources\User\UserAction as UserActionResource;
use App\Http\Resources\Action\ActionSequenceStepSubmission as ActionSequenceStepSubmissionResource;
use App\Http\Resources\Action\ActionSequenceWithCompletedEmployeeCount as ActionSequenceWithCompletedEmployeeCountResource;
use App\Http\Resources\ActionRequestSubmission\SubmissionWithUser as SubmissionWithUserResource;
use App\Http\Resources\User\UserActionProgress as UserActionProgressResource;
use App\Http\Resources\Action\ActionSubmission as ActionSubmissionResource;
use App\Services\Action\DownloadEmployeeActionSubmission;
use App\Services\Action\GetEmployeeActionSubmission;
use App\Services\Action\CancelUserAction;
use App\Services\Action\CreateAction;
use App\Services\Action\GetPendingSubmission;
use App\Services\Action\GetActionEmployees;
use App\Services\Action\SendUserActionReminder;
use App\Services\Action\ActionSequence\CreateActionSequence;
use App\Services\Action\ActionSequence\DeleteActionSequence;
use App\Services\Action\ActionSequence\DownloadEmployeeSubmission;
use App\Services\Action\ActionSequence\GetActionSequence;
use App\Services\Action\ActionSequence\GetActionSequenceDetail;
use App\Services\Action\ActionSequence\GetEmployeeSubmission;
use App\Services\Action\ActionSequence\SaveActionSequenceStep;
use App\Services\Action\ActionSequence\AddEmployees;
use App\Services\Action\ActionSequence\SendUserActionSequenceReminder;
use App\Services\Action\ActionSequence\UpdateActionSequence;
use App\Services\Action\ActionSequence\CancelUserActionSequence;
use App\Services\ActionRequest\ApproveSubmission;
use App\Services\ActionRequest\RejectSubmission;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;

/**
 * Class ActionController
 * @package App\Http\Controllers\Api\v1\Action
 */
class ActionController extends ApiController
{
    /**
     * Create action
     * 
     * @param Request $request
     * @return ActionResource
     */
    public function create(Request $request)
    {
        $action = $this->transaction(function() use ($request) {
            return app(CreateAction::class)->execute($request->all());
        });

        return new ActionResource($action);
    }

    /**
     * Create action sequence
     *
     * @param Request $request
     * @return ActionSequenceResource
     */
    public function createActionSequence(Request $request)
    {
        $actionSequence = app(CreateActionSequence::class)->execute($request->all());

        return new ActionSequenceResource($actionSequence);
    }

    /**
     * Get action sequence
     * 
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function getActionSequence(Request $request)
    {
        $actionSequences = app(GetActionSequence::class)->execute($this->options);

        return ActionSequenceWithCompletedEmployeeCountResource::collection($actionSequences);
    }

    /**
     * Get action sequence detail
     * 
     * @param $code
     * @return ActionSequenceResource
     */
    public function getActionSequenceDetail($code)
    {
        $actionSequence = app(GetActionSequenceDetail::class)->execute(['code' => $code]);

        return new ActionSequenceResource($actionSequence);
    }
    
    /**
     * Delete action sequence
     *
     * @param $code
     * @return mixed
     */
    public function deleteActionSequence($code)
    {
        $actionSequence = app(DeleteActionSequence::class)->execute(['code' => $code]);

        return $this->respondObjectDeleted($code);
    }

    /**
     * Update action sequence
     * 
     * @param Request $request
     * @param $code
     * @return ActionSequenceResource
     */
    public function updateActionSequence(Request $request, $code)
    {
        $input = array_merge($request->all(), ['code' => $code]);
        $actionSequence = app(UpdateActionSequence::class)->execute($input);

        return new ActionSequenceResource($actionSequence);
    }
    
    /**
     * Save action sequence steps
     *
     * @param Request $request
     * @param $code
     * @return Mixed
     */
    public function saveStep(Request $request, $code)
    {
        $input = array_merge($request->all(), ['code' => $code]);
        $actionSequence = $this->transaction(function() use ($input) {
            return app(SaveActionSequenceStep::class)->execute($input);
        });

        return ActionSequenceStepResource::collection($actionSequence->steps->load('actionRequest'));
    }

    /**
     * Get action sequence steps
     *
     * @param $code
     * @return mixed
     */
    public function getSteps($code)
    {
        $actionSequence = app(GetActionSequenceDetail::class)->execute(['code' => $code]);

        return ActionSequenceStepResource::collection($actionSequence->steps);
    }

    /**
     * Add employees to an action sequence
     *
     * @param Request $request
     * @param $code
     * @return mixed
     */
    public function addEmployees(Request $request, $code)
    {
        $input = array_merge($request->all(), ['code' => $code]);
        $employees = app(AddEmployees::class)->execute($input);

        return UserActionResource::collection($employees->load('employee'));
    }

    /**
     * Get employee submission
     *
     * @param $code
     *
     * @param $employeeCode
     * @return Mixed
     */
    public function getEmployeeSubmission($code, $employeeCode)
    {
        $stepSubmission = app(GetEmployeeSubmission::class)->execute(
            ['code' => $code, 'employee_code' => $employeeCode]
        );

        return ActionSequenceStepSubmissionResource::collection($stepSubmission);
    }

    /**
     * Download employee submission
     *
     * @param $code
     * @param $employeeCode
     * @return mixed
     */
    public function downloadEmployeeSubmission($code, $employeeCode)
    {
        return app(DownloadEmployeeSubmission::class)->execute(
            ['code' => $code, 'employee_code' => $employeeCode]
        );
    }

    /**
     * Get pending reviews
     * 
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function getPendingReviews(Request $request)
    {
        $pendingSubmissions = app(GetPendingSubmission::class)->execute($this->options);
        $pendingSubmissions->load(['actionRequest', 'user.employee.client', 'user.avatar', 'contentPhotoResults.attachment']);

        return SubmissionWithUserResource::collection($pendingSubmissions);
    }

    /**
     * Approve submission
     *
     * @param $submissionId
     * @return mixed
     */
    public function approveSubmission($submissionId)
    {
        $submission = app(ApproveSubmission::class)->execute(['submission_id' => $submissionId]);

        return new SubmissionWithUserResource($submission);
    }

    /**
     * Reject submission
     *
     * @param Request $request
     * @param $submissionId
     * @return mixed
     */
    public function rejectSubmission(Request $request, $submissionId)
    {
        $input = array_merge($request->all(), ['submission_id' => $submissionId]);
        $submission = app(RejectSubmission::class)->execute($input);

        return new SubmissionWithUserResource($submission);
    }
    
    /**
     * Get action submissions
     *
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function getEmployees(Request $request)
    {
        $actionEmployees = app(GetActionEmployees::class)->execute($this->options);
        
        return UserActionProgressResource::collection($actionEmployees);
    }

    /**
     * Send user action reminder
     *
     * @param $code
     * @param $employeeCode
     * @return JsonResponse
     */
    public function sendUserActionReminder($code, $employeeCode)
    {
        app(SendUserActionReminder::class)->execute(
            ['code' => $code, 'employee_code' => $employeeCode]
        );

        return $this->responseWithSuccessMessage('Sent action reminder successfully');
    }

    /**
     * Send user Action sequence reminder
     *
     * @param $code
     * @param $employeeCode
     * @return JsonResponse
     */
    public function sendUserActionSequenceReminder($code, $employeeCode)
    {
        app(SendUserActionSequenceReminder::class)->execute(
            ['code' => $code, 'employee_code' => $employeeCode]
        );

        return $this->responseWithSuccessMessage('Sent action sequence reminder successfully');
    }
    
    /**
     * Cancel user action
     *
     * @param $code
     * @param $employeeCode
     * @return JsonResponse
     */
    public function cancelUserAction($code, $employeeCode)
    {
        app(CancelUserAction::class)->execute(
            ['code' => $code, 'employee_code' => $employeeCode]
        );

        return $this->responseWithSuccessMessage('Cancelled action successfully');
    }
    
    /**
     * Cancel user action sequence
     * 
     * @param $code
     * @param $employeeCode
     * @return JsonResponse
     */
    public function cancelUserActionSequence($code, $employeeCode)
    {
        app(CancelUserActionSequence::class)->execute(
            ['code' => $code, 'employee_code' => $employeeCode]
        );

        return $this->responseWithSuccessMessage('Cancelled action sequence successfully');
    }

    /**
     * Get employee single action submission
     * 
     * @param $code
     * @param $employeeCode
     * @return ActionSubmissionResource
     */
    public function getEmployeeActionSubmission($code, $employeeCode)
    {
        $submission = app(GetEmployeeActionSubmission::class)->execute(
            ['code' => $code, 'employee_code' => $employeeCode]
        );
        
        return new ActionSubmissionResource($submission);
    }

    /**
     * Download employee action submission
     *
     * @param $code
     * @param $employeeCode
     * @return mixed
     */
    public function downloadEmployeeActionSubmission($code, $employeeCode)
    {
        return app(DownloadEmployeeActionSubmission::class)->execute(
            ['code' => $code, 'employee_code' => $employeeCode]
        );
    }
}
