<?php

namespace App\Http\Controllers\Api\v1\Auth;

use App\Enum\ClientConfigurations;
use App\Enum\ErrorCode;
use App\Enum\RequireSSOOptions;
use App\Enum\Roles;
use App\Exceptions\InvalidRequestException;
use App\Guards\GuardNames;
use App\Helpers\CookieHelper;
use App\Http\Controllers\Controller;
use App\Http\Resources\User\MagicLogin as MagicLoginResource;
use App\Http\Resources\User\AuthUser as AuthUserResource;
use App\Services\Auth\UserMagicLinkLogin;
use App\Services\Auth\VerifyReCaptcha;
use App\Services\Auth\AuthWebsocket;
use App\Services\Mobile\Auth\GetLoginMagicLinkForDemoClient;
use App\Services\TwoFactorAuthentication\Verify2FA;
use App\Traits\Guard;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth as Auth;
use App\Traits\JsonRespondController;
use App\Services\Auth\UserLogin;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;
use Illuminate\Support\Facades\Log;

class AuthController extends Controller
{
    use JsonRespondController;
    use Guard;

    public function login(Request $request)
    {
        try {
            app(VerifyReCaptcha::class)->execute([
                'recaptcha_token' => $request->input('recaptcha_token'),
                'remote_ip' => $request->ip()
            ]);

            // This method will throw exception if authentication failed
            Auth::basic();

            $currentUser = Auth::user();
            $this->guard([GuardNames::AuthenticatedConsole], ['user' => $currentUser]);
            if ($currentUser->hasRole(Roles::CLIENT_CONSOLE_ROLES)) {
                $currentClient = $currentUser->getClient();
                // Not allow user login normally
                // If current client enable SSO and force manager login by sso
                if($currentClient->configurations[ClientConfigurations::IS_ENABLE_SSO]
                    && $currentClient->configurations[ClientConfigurations::REQUIRE_SINGLE_SIGN_ON] === RequireSSOOptions::MANAGER
                    && ($currentUser->hasRole([Roles::MANAGER]) && !$currentUser->hasRole([Roles::ADMIN]))){
                    throw new \App\Exceptions\UnauthorizedHttpException(
                        'Your company is SSO configured. Please select the SSO log in option',
                        ErrorCode::FORCE_LOGIN_SSO
                    );
                }

                if ($currentUser->google2fa_enable) {
                    return $this->respond([
                        'message' => 'Google 2FA authentication enabled, Please login with 2FA',
                        'google2fa_enable' => true
                    ]);
                }
                
                $this->guard([GuardNames::UserIsActive, GuardNames::UserEmailVerified, GuardNames::ClientIsActive],
                    ['user' => $currentUser, 'client' => $currentUser->getClient()]);
            }

            app(UserLogin::class)->execute(['ip' => $request->ip()]);

            $accessToken = $currentUser->createToken(config('auth.access_token_name'))->accessToken;
            $user = new AuthUserResource($currentUser);
            $cookie = CookieHelper::generateJWTToken($accessToken);
            return $this->respond(['user' => $user])->cookie($cookie);
        } catch (UnauthorizedHttpException $e) {
            throw new \App\Exceptions\UnauthorizedHttpException('Invalid email or password.', ErrorCode::INVALID_CREDENTIALS);
        }
    }

    public function loginWith2FA(Request $request)
    {
        try {
            Auth::basic();
            $currentUser = Auth::user();
            if (!$currentUser->google2fa_enable) {
                throw new InvalidRequestException('Invalid 2FA request', ErrorCode::INVALID_2FA);
            }

            $timestamp = app(Verify2FA::class)->execute([
                'secret' => $currentUser->google2fa_secret,
                'one_time_password' => $request->input('one_time_password'),
                'old_timestamp' => $currentUser->google2fa_timestamp
            ]);

            $currentUser->mark2FAAsVerified($timestamp);
            
            app(UserLogin::class)->execute(['ip' => $request->ip()]);

            $accessToken = $currentUser->createToken(config('auth.access_token_name'))->accessToken;
            $cookie = CookieHelper::generateJWTToken($accessToken);;

            return $this->respond(['user' => new AuthUserResource($currentUser)])->cookie($cookie);
        } catch (UnauthorizedHttpException $e) {
            throw new \App\Exceptions\UnauthorizedHttpException('Invalid email or password.', ErrorCode::INVALID_CREDENTIALS);
        }
    }

    public function logout(Request $request)
    {
        $token = Auth::user()->token();
        $token->revoke();
        return $this->respond(true);
    }

    public function authWebsocket(Request $request)
    {
        $channelName = $request->input('channel_name');
        return app(AuthWebsocket::class)->execute(['channelName' => $channelName]);
    }

    public function getLoginMagicLink(Request $request)
    {
        $result = app(GetLoginMagicLinkForDemoClient::class)->execute($request->all());
        return new MagicLoginResource($result);
    }

    public function loginMagicLink(Request $request)
    {
        $currentUser = app(UserMagicLinkLogin::class)->execute($request->all());
        $accessToken = $currentUser->createToken(config('auth.access_token_name'))->accessToken;
        $user = new AuthUserResource($currentUser);
        $cookie = CookieHelper::generateJWTToken($accessToken);
        return $this->respond(['user' => $user])->cookie($cookie);

    }
}
