<?php

namespace App\Http\Controllers\Api\v1\Auth;

use App\Exceptions\InvalidRequestException;
use App\Http\Controllers\Api\ApiController;
use App\Rules\PasswordRule;
use App\Services\Auth\VerifyReCaptcha;
use App\Traits\JsonRespondController;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Password;
use Illuminate\Support\Str;

class ForgotPasswordController extends ApiController
{
    use JsonRespondController;
    
    /**
     * Send password reset link
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function forgot(Request $request)
    {
        $rules = [
            'email' => 'required|email'
        ];
        app(VerifyReCaptcha::class)->execute([
            'recaptcha_token' => $request->input('recaptcha_token'),
            'remote_ip' => $request->ip()
        ]);
        $credentials = $request->validate($rules);

        Password::sendResetLink($credentials);

        return $this->responseWithSuccessMessage(trans(Password::RESET_LINK_SENT));

    }

    /**
     * Reset user password
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function reset(Request $request)
    {
        $rules = [
            'token' => 'required',
            'email' => 'required|email',
            'password' => ['required', 'confirmed', 'min:8', new PasswordRule()],
        ];
        app(VerifyReCaptcha::class)->execute([
            'recaptcha_token' => $request->input('recaptcha_token'),
            'remote_ip' => $request->ip()
        ]);
        $credentials = $request->validate($rules);

        Password::reset($credentials, function ($user, $password){
            $this->resetPassword($user, $password);
        });

        return $this->responseWithSuccessMessage(trans(Password::PASSWORD_RESET));

    }

    /**
     * Set user password
     *
     * @param $user
     * @param $password
     */
    protected function resetPassword($user, $password)
    {
        $user->password = Hash::make($password);
        $user->setRememberToken(Str::random(60));
        $user->save();
        event(new PasswordReset($user));
    }
}
