<?php

namespace App\Http\Controllers\Api\v1\Auth;

use App\Facades\Context;
use App\Http\Controllers\Api\ApiController;
use App\Mail\Resend2FAMail;
use App\Models\User\User;
use App\Services\TwoFactorAuthentication\Disable2FA;
use App\Services\TwoFactorAuthentication\Enable2FA;
use App\Services\TwoFactorAuthentication\Prepare2FA;
use App\Services\TwoFactorAuthentication\Reset2FA;
use App\Services\TwoFactorAuthentication\Verify2FA;
use App\Traits\JsonRespondController;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Mail;

class TwoFactorAuthenticationController extends ApiController
{

    use JsonRespondController;
    
    /**
     * Reset google two factor authentication
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function reset(Request $request)
    {
        $rules = [
            'email' => 'required|email|exists:users,email',
        ];
        
        $credentials = $request->validate($rules);
        
        $user = User::where('email', '=', $credentials['email'])->first();
        
        [$secret, $base64QR] = app(Reset2FA::class)->execute($user);

        Mail::to($user->email)->send(new Resend2FAMail($user, $base64QR));

        return $this->responseWithSuccessMessage('Thank you. The code has been sent to your email address.');
    }

    /**
     * Prepare 2FA code
     * 
     * @return JsonResponse
     */
    public function prepare()
    {
        $user = Context::user()->get();
        $base64QR = app(Prepare2FA::class)->execute($user);
        
        return $this->respond([
            'base64_qr_code' => $base64QR,
        ]);
    }

    /**
     * Enable current user 2FA
     * 
     * @param Request $request
     * @return JsonResponse
     */
    public function enable(Request $request)
    {
        $user = Context::user()->get();
        $secret = $user->google2fa_secret;
        $currentTimestamp = $user->google2fa_timestamp;

        $newTimestamp = app(Verify2FA::class)->execute([
            'secret' => $secret,
            'one_time_password' => $request->input('one_time_password'),
            'old_timestamp' => $currentTimestamp
        ]);
        
        $user->mark2FAAsVerified($newTimestamp);
        
        app(Enable2FA::class)->execute($user, $secret);

        return $this->responseWithSuccessMessage('2FA enabled successfully');
    }

    /**
     * Disable current user 2FA
     * 
     * @param Request $request
     * @return JsonResponse
     */
    public function disable(Request $request)
    {
        $user = Context::user()->get();
        
        $timestamp = app(Verify2FA::class)->execute([
            'secret' => $user->google2fa_secret,
            'one_time_password' => $request->input('one_time_password'),
            'old_timestamp' => $user->google2fa_timestamp
        ]);

        $user->mark2FAAsVerified($timestamp);
        
        app(Disable2FA::class)->execute($user);

        return $this->responseWithSuccessMessage('2FA disabled successfully');
    }
}
