<?php

namespace App\Http\Controllers\Api\v1\Auth;

use App\Enum\AccessPermissions;
use App\Enum\ConversationTypes;
use App\Enum\ErrorCode;
use App\Enum\NotificationTypes;
use App\Enum\Roles;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\AwsComprehendHelper;
use App\Http\Controllers\Api\ApiController;
use App\Jobs\SendNewAdminEmail;
use App\Jobs\SendNotification;
use App\Models\Client\Client;
use App\Models\Survey\SurveyTextResult;
use App\Models\User\User;
use App\Services\Message\CreateConversation;
use App\Services\Message\CreateMessage;
use App\Services\UserAccess\GetAccessByRole;
use App\Traits\JsonRespondController;
use Carbon\Carbon;
use Illuminate\Auth\Events\Verified;
use Illuminate\Foundation\Auth\VerifiesEmails;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;

class VerificationController extends ApiController
{
    /*
    |--------------------------------------------------------------------------
    | Email Verification Controller
    |--------------------------------------------------------------------------
    |
    | This controller is responsible for handling email verification for any
    | user that recently registered with the application. Emails may also
    | be re-sent if the user didn't receive the original email message.
    |
    */

    use VerifiesEmails, JsonRespondController;

    /**
     * Verify email
     * 
     * @param Request $request
     * @return JsonResponse
     */
    public function verify(Request $request)
    {
        $user = User::find($request->route('id'));

        if (empty($user)) {
            return $this->respondNotFound('We couldn\'t found your information in our records.');
        }

        $hash = $request->route('hash');
        if (!hash_equals((string)$hash, sha1($user->getEmailForVerification())) &&
            !hash_equals((string)$hash, sha1($user->email_change))) {
            throw new InvalidRequestException('Your request is invalid.', ErrorCode::INVALID_VERIFICATION_REQUEST);
        }

        if (!$request->hasValidSignature(false)) {
            throw new InvalidRequestException('Your request is expired.', ErrorCode::VERIFICATION_REQUEST_EXPIRED);
        }

        // Verify changed email
        if (!empty($user->email_change) && hash_equals((string)$hash, sha1($user->email_change)) ) {
            $newEmailCount = User::where('email', $user->email_change)->count();
            if ($newEmailCount) {
                throw new InvalidRequestException('New email has already been used.', ErrorCode::EMAIL_ALREADY_USED);
            }
            // Update changed email to primary email
            $user->applyEmailChange();

            if ($user->hasVerifiedEmail()) {
                return $this->responseWithSuccessMessage('Your new email has been verified.');
            }
        }

        if ($user->hasVerifiedEmail()) {
            throw new InvalidRequestException('Your email address has already been verified.', ErrorCode::EMAIL_ALREADY_VERIFIED);
        }

        if ($user->markEmailAsVerified()) {
            event(new Verified($user));

            if (!$user->is_welcome_mail_sent) {
                // Generate a random password
                $password = Str::random(8);
                $this->setUserPassword($user, $password);

                SendNewAdminEmail::dispatch($user, $password, true);
            }

            return $this->responseWithSuccessMessage(
                'Thank you for verifying your email address. Login details have just been emailed to you.'
            );
        }
        
        throw new InvalidRequestException(
            'Error occurred when verifying your email address. Please try again',
            ErrorCode::INVALID_REQUEST
        );
    }

    public function test()
    {
        // Send at 9AM
        $sendAfter = Carbon::today()->setTimezone('Asia/Singapore')->hour(9);

        //Get all user have birthday today
        $userBirthdays = User::active()->whereHas('employee', function ($query) {
            $query->whereDay('date_of_birth', Carbon::today()->day);
            $query->whereMonth('date_of_birth', Carbon::today()->month);
        })->with('employee')->get()->groupBy('employee.client_id')->all();

        //Get all user have working anniversary today
        $userAnniversary = User::active()->whereHas('employee', function ($query) {
            $query->whereDay('start_date', Carbon::today()->day);
            $query->whereMonth('start_date', Carbon::today()->month);
        })->with('employee')->get()->groupBy('employee.client_id')->all();

        $clientIds = collect(array_merge(array_keys($userBirthdays), array_keys($userAnniversary)))->unique();

        //Get all admins
        $admins = User::active()->where('is_demo', 0)->whereHas('employee', function ($query) use ($clientIds) {
            $query->whereIn('client_id', $clientIds->toArray());
        })->with('employee')->role(Roles::ADMIN)->get();

//        return $admins;
        $count = array();
        foreach ($admins as $admin) {
            Context::user()->set($admin);
            Context::client()->set($admin->employee->client);
            $adminAccess = app(GetAccessByRole::class)->execute(['role' => Roles::ADMIN]);
            $accessAdminPermissions = $adminAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

            if ($accessAdminPermissions[AccessPermissions::NOTIFICATION_APP_BIRTHDAY] && isset($userBirthdays[$admin->employee->client->getKey()])) {
                foreach ($userBirthdays[$admin->employee->client->getKey()] as $user) {
                    $conversation = app(CreateConversation::class)
                        ->execute(['type' => ConversationTypes::DIRECT_MESSAGE, 'receiver_code' => $user->getBusinessKey()]);
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => array($admin->getBusinessKey()),
                        "headings"   => __(
                            "notify.employee_birthday",
                            ["first_name" => $user->first_name, "surname" => $user->surname]
                        ),
                        "contents"   => __('Tap here to open message'),
                        "data"       => [
                            "role"            => "Admin",
                            "type"            => NotificationTypes::MESSAGE_RECEIVED,
                            "conversation_id" => $conversation->id
                        ],
                        'send_after' => null
                    );
                    SendNotification::dispatch($arguments);
                    array_push($count, $arguments);
                }
            }

            if ($accessAdminPermissions[AccessPermissions::NOTIFICATION_APP_WORK_ANNIVERSARY] && isset($userAnniversary[$admin->employee->client->getKey()])) {
                foreach ($userAnniversary[$admin->employee->client->getKey()] as $user) {
                    $yearNumber = Carbon::today()->diffInYears($user->employee->start_date);
                    $conversation = app(CreateConversation::class)
                        ->execute(['type' => ConversationTypes::DIRECT_MESSAGE, 'receiver_code' => $user->getBusinessKey()]);
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => array($admin->getBusinessKey()),
                        "headings"   => __(
                            "notify.employee_anniversary",
                            [
                                "first_name"  => $user->first_name,
                                "surname"     => $user->surname,
                                "year_number" => $yearNumber,
                                "year"        => $yearNumber > 1 ? "years" : "year"
                            ]
                        ),
                        "contents"   => __('Tap here to open message'),
                        "data"       => [
                            "role"            => "Admin",
                            "type"            => NotificationTypes::MESSAGE_RECEIVED,
                            "conversation_id" => $conversation->id
                        ],
                        'send_after' => null
                    );
                    SendNotification::dispatch($arguments);
                    array_push($count, $arguments);
                }
            }
        }

        return $count;
    }

    public function test2()
    {
        // Send at 9AM
        $sendAfter = Carbon::today()->setTimezone('Asia/Singapore')->hour(9);
        //Get all user have birthday today
        $userBirthdays = User::active()->whereHas('employee', function($query) {
            $query->whereDay('date_of_birth', Carbon::today()->day);
            $query->whereMonth('date_of_birth', Carbon::today()->month);
            $query->where('manager_id', '>', 0);
        })->with('employee')->get()->groupBy('employee.manager_id')->all();

        //Get all user have working anniversary today
        $userAnniversary = User::active()->whereHas('employee', function($query) {
            $query->whereDay('start_date', Carbon::today()->day);
            $query->whereMonth('start_date', Carbon::today()->month);
            $query->where('manager_id', '>', 0);
        })->with('employee')->get()->groupBy('employee.manager_id')->all();

        $managersIds = collect(array_merge(array_keys($userBirthdays), array_keys($userAnniversary)))->unique();
        $count = 0;
        //Get all managers
        $managers = User::active()->where('is_demo', 0)->with('employee')->whereIn('id', $managersIds->toArray())->get();
        foreach ($managers as $manager) {
            Context::user()->set($manager);
            Context::client()->set($manager->employee->client);
            $managerAccess = app(GetAccessByRole::class)->execute(['role' => Roles::MANAGER]);
            $accessManagerPermissions = $managerAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

            if ($accessManagerPermissions[AccessPermissions::NOTIFICATION_APP_BIRTHDAY] && isset($userBirthdays[$manager->getKey()])) {
                foreach ($userBirthdays[$manager->getKey()] as $user) {
                    $conversation = app(CreateConversation::class)
                        ->execute(['type' => ConversationTypes::DIRECT_MESSAGE, 'receiver_code' => $user->getBusinessKey()]);
                    app(CreateMessage::class)->execute(['content' => __("notify.employee_birthday",
                        ["first_name" => $user->first_name, "surname" => $user->surname]), 'conversation_id' => $conversation->id]);
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => array($manager->getBusinessKey()),
                        "headings"   => __("notify.employee_birthday",
                            ["first_name" => $user->first_name, "surname" => $user->surname]),
                        "contents"   => __('Tap here to open message'),
                        "data"       => [
                            "role"            => "Manager",
                            "type"            => NotificationTypes::MESSAGE_RECEIVED,
                            "conversation_id" => $conversation->id
                        ],
                        'send_after' => null
                    );
                    SendNotification::dispatch($arguments);
                    $count++;
                }
            }

            if ($accessManagerPermissions[AccessPermissions::NOTIFICATION_APP_WORK_ANNIVERSARY] && isset($userAnniversary[$manager->getKey()])) {
                foreach ($userAnniversary[$manager->getKey()] as $user) {
                    $yearNumber = Carbon::today()->diffInYears($user->employee->start_date);
                    $conversation = app(CreateConversation::class)
                        ->execute(['type' => ConversationTypes::DIRECT_MESSAGE, 'receiver_code' => $user->getBusinessKey()]);
                    app(CreateMessage::class)->execute(['content' => __("notify.employee_anniversary",
                        ["first_name"  => $user->first_name,
                            "surname"     => $user->surname,
                            "year_number" => $yearNumber,
                            "year"        => $yearNumber > 1 ? "years" : "year"
                        ]), 'conversation_id' => $conversation->id]);
                    $arguments = array(
                        'sender' => Auth::user()->id,
                        "userIds"    => array($manager->getBusinessKey()),
                        "headings"   => __("notify.employee_anniversary",
                            ["first_name"  => $user->first_name,
                                "surname"     => $user->surname,
                                "year_number" => $yearNumber,
                                "year"        => $yearNumber > 1 ? "years" : "year"
                            ]),
                        "contents"   => __('Tap here to open message'),
                        "data"       => [
                            "role"            => "Manager",
                            "type"            => NotificationTypes::MESSAGE_RECEIVED,
                            "conversation_id" => $conversation->id
                        ],
                        'send_after' => null
                    );
                    SendNotification::dispatch($arguments);
                    $count++;
                }
            }
        }

//        $emptyUser = new User();
//        $emptyClient = new Client();
//        Context::user()->set($emptyUser);
//        Context::client()->set($emptyClient);
        return $count;
    }

    public function detectSentiment()
    {
//        $awsComprehendHelper = new AwsComprehendHelper();
//        $detectEntities = $awsComprehendHelper->detechSentiment("Hmm, I think it's also okay");
        try {
            $awsComprehendHelper = new AwsComprehendHelper();
            $detectEntities = $awsComprehendHelper->detechSentiment("I don't like it");
        } catch (\Exception $e) {

        }
        $mixed_score = isset($detectEntities) ? $detectEntities['SentimentScore']['Mixed'] : 0;
        $positive_score = isset($detectEntities) ? $detectEntities['SentimentScore']['Positive'] : 0;
        $neutral_score = isset($detectEntities) ? $detectEntities['SentimentScore']['Neutral'] : 0;
        $negative_score = isset($detectEntities) ? $detectEntities['SentimentScore']['Negative'] : 0;
        $sentiment = isset($detectEntities) ? $detectEntities['Sentiment'] : '';
        return [
            'mixed' => $mixed_score,
            'positive' => $positive_score,
            'neutral' => $neutral_score,
            'negative' => $negative_score,
            'sentiment' => $sentiment,
        ];
    }

    public function batchLanguages()
    {
        $results = SurveyTextResult::query()->whereNotNull('text')->where('text', '!=', '')->whereNull('language')->limit(25)->get();
        if (count($results) > 0) {
            $awsComprehendHelper = new AwsComprehendHelper();
            $detectEntities = $awsComprehendHelper->detectDominantLanguages(array_map(function ($item) { return $item['text']; }, $results->toArray()));
            foreach ($results as $index => $result) {
                $result->language = $detectEntities[$index];
                $result->save();
            }
        }
        return $results;
    }


    public function batchDetectSentiment($language)
    {
        $awsComprehendHelper = new AwsComprehendHelper();
        $results = SurveyTextResult::query()
            ->whereNotNull('text')
            ->where('text', '!=', '')
            ->whereNull('sentiment')
            ->where('language', $language)
            ->limit(25)->get();

        $detectEntities = $awsComprehendHelper->batchDetectSentiment(array_map(function ($item) { return $item['text']; }, $results->toArray()), $language);
        foreach ($results as $index => $result) {
            $result->mixed_score = $detectEntities[$index]['SentimentScore']['Mixed'];
            $result->positive_score = $detectEntities[$index]['SentimentScore']['Positive'];
            $result->neutral_score = $detectEntities[$index]['SentimentScore']['Neutral'];
            $result->negative_score = $detectEntities[$index]['SentimentScore']['Negative'];
            $result->sentiment = $detectEntities[$index]['Sentiment'];
            $result->save();
        }

        return $results;
    }

    /**
     * Set user password
     * 
     * @param User $user
     * @param string $password
     */
    public function setUserPassword(User $user, string $password)
    {
        $user->password = Hash::make($password);
        $user->save();
    }
}
