<?php

namespace App\Http\Controllers\Api\v1\Client;

use App\Http\Resources\Client\ClientWithSubscriptions;
use App\Http\Resources\Client\Company as CompanyResource;
use App\Http\Resources\Subscription\SubscriptionPlanDetailRemain as SubscriptionPlanDetailRemainResource;
use App\Http\Resources\User\Employee as EmployeeResource;
use App\Http\Resources\User\UserLite as UserLiteResource;
use App\Services\Client\GetClientWithSubscriptions;
use App\Services\Client\GetDeniedUserAccess;
use App\Services\Client\GetUserCurrentCompany;
use App\Services\Client\GetClientDetail;
use App\Services\Client\GetCompany;
use App\Services\Client\GrantManagerAccess;
use App\Services\Client\GrantWhistleblowerAccess;
use App\Services\Client\RemoveManagerAccess;
use App\Services\Client\RemoveWhistleblowerAccess;
use App\Services\Subscription\AddTrial;
use App\Services\Subscription\GetCompanyRemainPlan;
use App\Services\Client\GrantAdminAccess;
use App\Services\Client\RemoveAdminAccess;
use App\Services\Client\UpdateCompany;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use App\Http\Controllers\Api\ApiController;
use App\Services\Client\GetClientWithAdmins;
use App\Services\Client\CreateClient;
use App\Services\Client\UpdateClient;
use App\Services\Client\ChangeClientStatus;
use App\Services\Client\DeleteClient;
use App\Http\Resources\Client\ClientWithAdmins as ClientWithAdminsResource;
use App\Http\Resources\Client\Client as ClientResource;
use App\Enum\ClientStatus;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;
use Illuminate\Http\Resources\Json\JsonResource;

/**
 * Class ClientController
 * @package App\Http\Controllers\Api\v1\Client
 */
class ClientController extends ApiController
{
    /**
     * Get client list
     *
     * @return AnonymousResourceCollection
     */
    public function getClientWithAdmins(): AnonymousResourceCollection
    {
        $clients = app(GetClientWithAdmins::class)->execute();
        return ClientWithAdminsResource::collection($clients);
    }

    /**
     * Create client
     *
     * @param Request $request
     * @return ClientResource
     */
    public function createClient(Request $request): ClientResource
    {
        $client = app(CreateClient::class)->execute($request->all());
        return new ClientResource($client);
    }

    /**
     * Update client
     *
     * @param Request $request
     * @param $code
     * @return ClientResource
     */
    public function updateClient(Request $request, $code): ClientResource
    {
        $input = array_merge($request->all(), ['code' => $code]);
        $client = app(UpdateClient::class)->execute($input);
        return new ClientResource($client);
    }

    /**
     * Suspend client
     *
     * @param $code
     * @return ClientResource
     */
    public function suspendClient($code): ClientResource
    {
        $input = ['code' => $code, 'status' => ClientStatus::SUSPENDED];
        $client = app(ChangeClientStatus::class)->execute($input);
        return new ClientResource($client);
    }

    /**
     * Activate client
     *
     * @param $code
     * @return ClientResource
     */
    public function activateClient($code): ClientResource
    {
        $input = ['code' => $code, 'status' => ClientStatus::ACTIVE];
        $client = app(ChangeClientStatus::class)->execute($input);
        return new ClientResource($client);
    }

    /**
     * Delete client
     *
     * @param $code
     * @return JsonResponse
     */
    public function deleteClient($code): JsonResponse
    {
        $input = ['code' => $code];

        app(DeleteClient::class)->execute($input);

        return $this->respondObjectDeleted($code);
    }

    /**
     * Get client detail
     *
     * @param $code
     * @return ClientResource
     */
    public function getClientDetail($code): ClientResource
    {
        $client = app(GetClientDetail::class)->execute(['code' => $code]);

        return new ClientResource($client);
    }

    /**
     * Update client
     *
     * @param Request $request
     * @param $code
     * @return ClientResource
     */
    public function updateCompany(Request $request): CompanyResource
    {
        $client = app(UpdateCompany::class)->execute($request->all());
        return new CompanyResource($client);
    }

    /**
     * Update client
     *
     * @param Request $request
     * @param $code
     * @return CompanyResource
     */
    public function getCompany(Request $request): CompanyResource
    {
        $client = app(GetCompany::class)->execute();
        return new CompanyResource($client);
    }

    /**
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function getUserCurrentCompany(Request $request, string $type = '')
    {
        $options = array_merge(["type" => $type], $this->options);
        $admins = app(GetUserCurrentCompany::class)->execute($options);
        return UserLiteResource::collection($admins);
    }

    /**
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function grantAdminAccess(Request $request)
    {
        $admins = $this->transaction(function () use ($request) {
            return app(GrantAdminAccess::class)->execute($request->all());
        });
        return EmployeeResource::collection($admins);
    }

    /**
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function removeAdminAccess(Request $request)
    {
        $admins = app(RemoveAdminAccess::class)->execute($request->all());
        return EmployeeResource::collection($admins);
    }

    /**
     * Get Company Remain plan
     * @param Request $request
     * @return JsonResponse
     */
    public function getCompanyRemainPlan(Request $request)
    {
        $remainPlan = app(GetCompanyRemainPlan::class)->execute();
        return new SubscriptionPlanDetailRemainResource($remainPlan);
    }

    public function getClientSubscriptions(Request $request)
    {
        $clientSubscription = app(GetClientWithSubscriptions::class)->execute([]);
        return ClientWithSubscriptions::collection($clientSubscription);
    }

    /**
     * Add subscription trial period
     * 
     * @param Request $request
     * @return JsonResource|bool
     */
    public function addSubscriptionTrial(Request $request)
    {
        $clientSubscription = app(AddTrial::class)->execute($request->all());
        
        return new JsonResource($clientSubscription ? $clientSubscription : null);
    }

    /**
     * Grant manager access
     *
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function grantManagerAccess(Request $request)
    {
        $managers = app(GrantManagerAccess::class)->execute($request->all());

        return EmployeeResource::collection($managers);
    }

    /**
     * Remove manager access
     *
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function removeManagerAccess(Request $request)
    {
        $managers = app(RemoveManagerAccess::class)->execute($request->all());

        return EmployeeResource::collection($managers);
    }

    /**
     * Grant manager access
     *
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function grantWhistleblowerAccess(Request $request)
    {
        $managers = app(GrantWhistleblowerAccess::class)->execute($request->all());

        return UserLiteResource::collection($managers);
    }

    /**
     * Remove manager access
     *
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function removeWhistleblowerAccess(Request $request)
    {
        $managers = app(RemoveWhistleblowerAccess::class)->execute($request->all());

        return EmployeeResource::collection($managers);
    }

    /**
     * Get denied user access
     *
     * @param Request $request
     * @param $role
     * @return AnonymousResourceCollection
     */
    public function getDeniedUserAccess(Request $request, $role)
    {
        $managers = app(GetDeniedUserAccess::class)->execute(['role' => $role]);

        return UserLiteResource::collection($managers);
    }
}