<?php

namespace App\Http\Controllers\Api\v1\Employee;

use App\Enum\EmployeeStatus;
use App\Enum\Roles;
use App\Http\Controllers\Api\ApiController;
use App\Http\Resources\User\Employee as EmployeeResource;
use App\Http\Resources\User\EmployeeInList as EmployeeInListResource;
use App\Http\Resources\User\UserLite;
use App\Services\Employee\BulkUpdateContentProfiles;
use App\Services\Employee\ChangeEmployeeStatus;
use App\Services\Employee\CreateEmployee;
use App\Services\Employee\DeleteEmployee;
use App\Services\Employee\DownloadActivity;
use App\Services\Employee\GeneratePassword;
use App\Services\Employee\GetEmployee;
use App\Services\Employee\GetEmployeeDetail;
use App\Services\Employee\GetStartingNextEmployees;
use App\Services\Employee\ImportEmployee;
use App\Services\Employee\ResendVerifyEmail;
use App\Services\Employee\ExportEmployees;
use App\Services\Employee\UpdateEmployee;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use App\Services\Employee\GetEmployeeJourneys;
use App\Http\Resources\User\JourneyEmployee as JourneyEmployeeResource;
use App\Services\Tool\Journey\DownloadEmployeeSubmission;
use App\Services\Employee\DownloadEmployeesSubmission;
use App\Services\Employee\QuickUpdateEmployee;
use App\Models\User\User;

/**
 * Class EmployeeController
 * @package App\Http\Controllers\Api\v1\Employee
 */
class EmployeeController extends ApiController
{

    /**
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function getEmployee(Request $request)
    {
        $employees = app(GetEmployee::class)->execute();
        $employees->loadMissing([
            'employee.manager',
            'employee.client',
            'employee.location',
            'employee.department',
            'employee.gender',
            'employee.documentProfiles',
            'employee.team'
        ]);
        return EmployeeInListResource::collection($employees);
    }

    /**
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function getEmployeeLite(Request $request)
    {
        $employees = app(GetEmployee::class)->execute(['employee_lite' => true]);
        return UserLite::collection($employees);
    }


    /**
     * @param Request $request
     * @param string $code
     * @return EmployeeResource
     */
    public function getEmployeeDetail(Request $request, string $code)
    {
        $employee = app(GetEmployeeDetail::class)->execute(['code' => $code]);
        return new EmployeeResource($employee);
    }

    /**
     * Download admin activities
     *
     * @param Request $request
     * @param $code
     * @return BinaryFileResponse
     */
    public function downloadAdminActivities(Request $request, $code): BinaryFileResponse
    {
        $input = ['code' => $code];
        $file = app(DownloadActivity::class)->execute($input);
        return response()->download($file)->deleteFileAfterSend();
    }

    /**
     * @return AnonymousResourceCollection
     */
    public function getStartingNext(): AnonymousResourceCollection
    {
        $employees = app(GetStartingNextEmployees::class)->execute();
        return EmployeeResource::collection($employees);
    }

    /**
     * Create front-user employee
     *
     * @param Request $request
     * @return EmployeeResource
     */
    public function create(Request $request)
    {
        $employee = $this->transaction(function () use ($request) {
            return app(CreateEmployee::class)->execute(array_merge($request->all(), [
                'role' => Roles::FRONT_USER,
                'is_additional_field_required' => true
            ]));
        });

        return new EmployeeResource($employee);
    }

    /**
     * Create admin employee
     *
     * @param Request $request
     * @return EmployeeResource
     */
    public function createAdmin(Request $request)
    {
        $employee = $this->transaction(function () use ($request) {
            return app(CreateEmployee::class)->execute(array_merge($request->all(), ['role' => Roles::ADMIN, 'action' => 'invite']));
        });

        return new EmployeeResource($employee);
    }

    /**
     * Update employee
     *
     * @param Request $request
     * @param $code
     * @return EmployeeResource
     */
    public function update(Request $request, $code)
    {
        $input = array_merge($request->all(), ['code' => $code, 'is_additional_field_required' => true]);
        $employee = app(UpdateEmployee::class)->execute($input);

        return new EmployeeResource($employee);
    }


    /**
     * Enable employee
     *
     * @param $code
     * @return EmployeeResource
     */
    public function enable($code)
    {
        $employee = app(ChangeEmployeeStatus::class)->execute([
            'code' => $code,
            'status' => EmployeeStatus::ACTIVE
        ]);

        return new EmployeeResource($employee);
    }

    /**
     * Disable employee
     *
     * @param $code
     * @return EmployeeResource
     */
    public function disable($code)
    {
        $employee = app(ChangeEmployeeStatus::class)->execute([
            'code' => $code,
            'status' => EmployeeStatus::INACTIVE
        ]);

        return new EmployeeResource($employee);
    }

    /**
     * Enable or disable employees
     *
     * @param $code
     * @return EmployeeResource
     */
    public function bulkEnableOrDisableEmployee(Request $request, string $code)
    {
        ini_set('max_execution_time', 900);
        $data = $request->all();
        $headers = [
            'Access-Control-Allow-Origin' => config('app.front_end_url'),
            'Access-Control-Allow-Credentials' => 'true'
        ];
        response()->json([
            "status" => true,
            "message" => "Enable/disable employees successfully",
            "data" => []
        ], 200, $headers)->send();

        if ($code === 'all') {
            $employees = app(GetEmployee::class)->execute(['isNotPaging' => true]);
            $unSelectedEmployeesCode = $data['unSelectedEmployeesCode'];
            $codes = array_map(function ($employee) use ($unSelectedEmployeesCode) {
                if (!in_array($employee['code'], $unSelectedEmployeesCode)) {
                    return $employee['code'];
                }
            }, EmployeeResource::collection($employees)->jsonSerialize());
        } else {
            $codes = explode("-", $code);
        }
        $codes = array_filter($codes, function ($code) {
            return isset($code);
        });
        foreach ($codes as $employeeCode) {
            $user = User::findByBusinessKey($employeeCode);
            $employee = $user->employee;

            if ($employee->status === EmployeeStatus::ACTIVE) {
                $employeeStatus = EmployeeStatus::INACTIVE;
            } else {
                $employeeStatus = EmployeeStatus::ACTIVE;
            }
            app(ChangeEmployeeStatus::class)->execute([
                'code' => $employeeCode,
                'status' => $employeeStatus,
                'is_multiple' => true
            ]);
        }

        return $this->responseWithStatusCode(200, "Enable/disable employees successfully");
    }
    /**
     * Quick update employee
     *
     * @param $code
     * @return EmployeeResource
     */
    public function quickUpdate(Request $request, $code)
    {
        $input = array_merge($request->all(), ['code' => $code]);
        $employee = app(QuickUpdateEmployee::class)->execute($input);

        return new EmployeeResource($employee);
    }

    public function bulkUpdateContentProfiles(Request $request, string $code)
    {
        $id = now()->unix();
        $headers = [
            'Access-Control-Allow-Origin' => config('app.front_end_url'),
            'Access-Control-Allow-Credentials' => 'true'
        ];
        response()->json([
            "data" => [
                'id' => $id,
                'inProcessing' => true,
                'isCompleted' => false,
            ]
        ], 200, $headers)->send();
        $data = $request->all();
        if ($code === 'all') {
            $unSelectedEmployeesCode = $data['unSelectedEmployeesCode'] ?? [];
            app(BulkUpdateContentProfiles::class)->execute(['codes' => null, 'excludeIds' => $unSelectedEmployeesCode, 'document_profiles' => $data['document_profiles'], 'id' => $id]);
        } else {
            $codes = explode("-", $code);
            app(BulkUpdateContentProfiles::class)->execute(['codes' => $codes, 'document_profiles' => $data['document_profiles'], 'id' => $id]);
        }
    }

    /**
     * Get import employees by CSV status
     *
     * @param $file
     * @return JsonResponse
     */
    public function updateStatus(string $code)
    {
        $result = app(BulkUpdateContentProfiles::class)->getUpdateStatus($code);
        return $this->responseData($result);
    }

    /**
     * Delete employee
     *
     * @param $code
     * @return JsonResponse
     */
    public function delete($code)
    {
        $employee = app(DeleteEmployee::class)->execute(['code' => $code]);

        return $this->respondObjectDeleted($code);
    }

    public function bulkDeleteEmployee(Request $request, string $code)
    {
        ini_set('max_execution_time', 900);
        $data = $request->all();
        $headers = [
            'Access-Control-Allow-Origin' => config('app.front_end_url'),
            'Access-Control-Allow-Credentials' => 'true'
        ];
        response()->json([
            "status" => true,
            "message" => "Delete employees successfully",
            "data" => []
        ], 200, $headers)->send();

        if ($code === 'all') {
            $employees = app(GetEmployee::class)->execute(['isNotPaging' => true]);
            $unSelectedEmployeesCode = $data['unSelectedEmployeesCode'];
            $codes = array_map(function ($employee) use ($unSelectedEmployeesCode) {
                if (!in_array($employee['code'], $unSelectedEmployeesCode)) {
                    return $employee['code'];
                }
            }, EmployeeResource::collection($employees)->jsonSerialize());
        } else {
            $codes = explode("-", $code);
        }
        $codes = array_filter($codes, function ($code) {
            return isset($code);
        });
        foreach ($codes as $employeeCode) {
            app(DeleteEmployee::class)->execute(['code' => $employeeCode, 'is_multiple' => true]);
        }

        return $this->responseWithStatusCode(200, "Delete employees successfully");
    }

    /**
     * Import employees by CSV
     *
     * @param $file
     * @return JsonResponse
     */
    public function import()
    {
        $result = app(ImportEmployee::class)->execute(['file' => request()->file('file')]);
        return $this->responseData($result);
    }

    /**
     * Get import employees by CSV status
     *
     * @param $file
     * @return JsonResponse
     */
    public function importStatus(string $code)
    {
        $result = app(ImportEmployee::class)->getImportStatus($code);
        return $this->responseData($result);
    }

    public function resendVerifyEmail(string $code)
    {
        app(ResendVerifyEmail::class)->execute(['code' => $code]);
        return $this->responseWithStatusCode(200, "Resend verify email successfully");
    }

    public function bulkReVerifyEmail(Request $request, string $code)
    {
        ini_set('max_execution_time', 900);
        $data = $request->all();
        $headers = [
            'Access-Control-Allow-Origin' => config('app.front_end_url'),
            'Access-Control-Allow-Credentials' => 'true'
        ];
        response()->json([
            "status" => true,
            "message" => "Resend verify email successfully",
            "data" => []
        ], 200, $headers)->send();
        $response = [];
        if ($code === 'all') {
            $employees = app(GetEmployee::class)->execute(['isNotPaging' => true]);
            $unSelectedEmployeesCode = $data['unSelectedEmployeesCode'];
            $codes = array_map(function ($employee) use ($unSelectedEmployeesCode) {
                if (!in_array($employee['code'], $unSelectedEmployeesCode)) {
                    return $employee['code'];
                }
            }, EmployeeResource::collection($employees)->jsonSerialize());
        } else {
            $codes = explode("-", $code);
        }
        $codes = array_filter($codes, function ($code) {
            return isset($code);
        });
        foreach ($codes as $employeeCode) {
            $user = User::findByBusinessKey($employeeCode);

            $result = app(ResendVerifyEmail::class)->execute(['code' => $employeeCode, 'is_multiple' => true]);

            $item = [
                'employee_name' => $user->first_name . ' ' . $user->surname,
                'is_resend' => $result,
                'logged' => $user->email_verified_at || $user->last_login,
            ];

            array_push($response, $item);
        }

        return $this->responseWithStatusCode(200, "Resend verify email successfully", $response);
    }

    public function bulkPasswordsEmail(Request $request, string $code)
    {
        $data = $request->all();
        if ($code === 'all') {
            app(GeneratePassword::class)->execute(['codes' => null]);
            return $this->responseWithStatusCode(200, "Sent email of passwords successfully");
        }
        $unSelectedEmployeesCode = !empty($data['unSelectedEmployeesCode']) ? explode("-", $data['unSelectedEmployeesCode']) : [];
        if (isset($data['previousStateOfSelectAll']) && $data['previousStateOfSelectAll']
            && isset($unSelectedEmployeesCode) && count($unSelectedEmployeesCode) > 0) {
            app(GeneratePassword::class)->execute(['codes' => null, 'excludeIds' => $unSelectedEmployeesCode ]);
            return $this->responseWithStatusCode(200, "Sent email of passwords successfully");
        }
        $codes = explode("-", $code);
        app(GeneratePassword::class)->execute(['codes' => $codes]);
        return $this->responseWithStatusCode(200, "Sent email of passwords successfully");
    }

    public function exportEmployees(Request $request, string $code)
    {
        $data = $request->all();
        if ($code === 'all') {
            return app(ExportEmployees::class)->execute(['codes' => null, 'columns' => $data['columns']]);
        }
        $unSelectedEmployeesCode = !empty($data['unSelectedEmployeesCode']) ? explode("-", $data['unSelectedEmployeesCode']) : [];
        if (isset($data['previousStateOfSelectAll']) && $data['previousStateOfSelectAll']
            && isset($unSelectedEmployeesCode) && count($unSelectedEmployeesCode) > 0) {
            return app(ExportEmployees::class)->execute(['codes' => null, 'columns' => $data['columns'], 'excludeIds' => $unSelectedEmployeesCode ]);
        }
        $codes = explode("-", $code);
        return app(ExportEmployees::class)->execute(['codes' => $codes, 'columns' => $data['columns']]);
    }

    /**
     * Download employee upload template
     *
     * @return BinaryFileResponse
     */
    public function downloadTemplate()
    {
        return response()->download(public_path('templates/employee_upload_template.csv'));
    }

    /**
     * Get all journeys by employee
     *
     * @return BinaryFileResponse
     */
    public function getJourneys($code)
    {
        $journeys = app(GetEmployeeJourneys::class)->execute(['code' => $code]);
        return JourneyEmployeeResource::collection($journeys);
    }

    /**
     * Download employee submission
     * 
     * @param $code
     * @param $journeyCode
     * @return mixed
     */
    public function downloadEmployeeSubmission($code, $journeyCode)
    {
        return app(DownloadEmployeeSubmission::class)->execute(
            ['code' => $journeyCode, 'employee_code' => $code]
        );
    }

    /**
     * Download employees submission in all journeys
     *
     * @param $code
     * @param $journeysCode
     * @return mixed
     */
    public function downloadEmployeesSubmissionInAllJourneys($code, $journeysCode)
    {
        return app(DownloadEmployeesSubmission::class)->execute(
            ['code' => $code, 'journeys_code' => $journeysCode]
        );
    }
}