<?php

namespace App\Http\Controllers\Api\v1\Journey;

use App\Http\Controllers\Api\ApiController;
use App\Jobs\CreateJourneyEmployeeDocument;
use App\Models\Journey\Journey;
use \App\Services\Tool\Journey\CreateJourney;
use App\Http\Resources\Tool\Journey\Journey as JourneyResource;
use App\Http\Resources\Tool\Journey\JourneyStep as JourneyStepResource;
use App\Http\Resources\Tool\Journey\JourneyStepProgress as JourneyStepProgressResource;
use App\Http\Resources\User\UserJourney as UserJourneyResource;
use App\Http\Resources\User\UserJourneyCustom as UserJourneyCustomResource;
use App\Http\Resources\Tool\Journey\JourneyStepSubmission as JourneyStepSubmissionResource;
use App\Services\Tool\Journey\DeleteJourney;
use App\Services\Tool\Journey\DownloadEmployeeSubmission;
use App\Services\Tool\Journey\DownloadEmployeesSubmission;
use App\Services\Tool\Journey\DuplicateJourney;
use App\Services\Tool\Journey\GetEmployeeSubmission;
use App\Services\Tool\Journey\GetJourney;
use App\Services\Tool\Journey\GetJourneyDetail;
use App\Services\Tool\Journey\AddEmployees;
use App\Services\Tool\Journey\GetJourneyEmployees;
use App\Services\Tool\Journey\GetJourneyProgress;
use App\Services\Tool\Journey\PublishJourney;
use App\Services\Tool\Journey\RemoveEmployee;
use App\Services\Tool\Journey\RestartJourney;
use App\Services\Tool\Journey\RestartJourneyUser;
use App\Services\Tool\Journey\SaveJourneyStep;
use App\Services\Tool\Journey\SendReminderUser;
use App\Services\Tool\Journey\UnpublishJourney;
use App\Services\Tool\Journey\UpdateJourney;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use App\Exceptions\InvalidRequestException;
use App\Enum\ErrorCode;

class JourneyController extends ApiController
{

    /**
     * List all journey
     *
     * @return mixed
     */
    public function getAll()
    {
        $journeys = app(GetJourney::class)->execute();

        return JourneyResource::collection($journeys);
    }

    /**
     * Get journey detail
     * 
     * @param $code
     * @return JourneyResource
     */
    public function detail($code)
    {
        $journey = app(GetJourneyDetail::class)->execute(['code' => $code]);

        return new JourneyResource($journey);
    }

    /**
     * Get journey employees
     *
     * @param $code
     * @return mixed
     */
    public function getEmployees($code)
    {
        $employees = app(GetJourneyEmployees::class)->execute(['code' => $code]);

        return UserJourneyResource::collection($employees);
    }

    /**
     * Get journey steps
     *
     * @param $code
     * @return mixed
     */
    public function getSteps($code)
    {
        $journey = app(GetJourneyDetail::class)->execute(['code' => $code]);

        return JourneyStepResource::collection($journey->steps);
    }

    /**
     * Get journey progress
     *
     * @param $code
     * @return mixed
     */
    public function getJourneyProgress($code)
    {
        $journeySteps = app(GetJourneyProgress::class)->execute(['code' => $code]);

        return JourneyStepProgressResource::collection($journeySteps);
    }

    /**
     * Create Journey
     *
     * @param Request $request
     * @return JourneyResource
     */
    public function create(Request $request)
    {
        $journey = app(CreateJourney::class)->execute($request->all());

        return new JourneyResource($journey);
    }

    /**
     * Save journey step
     *
     * @param Request $request
     * @param $code
     * @return Mixed
     */
    public function saveStep(Request $request, $code)
    {
        $codeParam = ['code' => $code];
        $input = array_merge($request->all(), $codeParam);
        $savedJourney = app(GetJourneyDetail::class)->execute($codeParam);
        if ($savedJourney && $savedJourney->is_published && empty($input['steps'])) {
            throw new InvalidRequestException('Can\'t update published journey without steps.', ErrorCode::EMPTY_PUBLISHED_JOURNEY_STEPS);
        }
        $journey = $this->transaction(function () use ($input) {
            return app(SaveJourneyStep::class)->execute($input);
        });
        return JourneyStepResource::collection($journey->steps);
    }

    /**
     * Update journey
     *
     * @param Request $request
     * @param $code
     * @return JourneyResource
     */
    public function update(Request $request, $code)
    {
        $input = array_merge($request->all(), ['code' => $code]);
        $journey = app(UpdateJourney::class)->execute($input);

        return new JourneyResource($journey);
    }

    /**
     * Delete journey
     * 
     * @param $code
     * @return JsonResponse
     */
    public function delete($code)
    {
        app(DeleteJourney::class)->execute(['code' => $code]);

        return $this->respondObjectDeleted($code);
    }

    /**
     * Duplicate journey
     *
     * @param $code
     * @return JourneyResource
     */
    public function duplicate($code)
    {
        $journey = app(DuplicateJourney::class)->execute(['code' => $code]);

        return new JourneyResource($journey);
    }

    /**
     * Publish journey
     *
     * @param $code
     * @return JourneyResource
     */
    public function publish($code)
    {
        $journey = $this->transaction(function () use ($code) {
            return app(PublishJourney::class)->execute(['code' => $code]);
        });

        return new JourneyResource($journey);
    }

    /**
     * Unpublish journey
     *
     * @param $code
     * @return JourneyResource
     */
    public function unpublish($code)
    {
        $journey = app(UnpublishJourney::class)->execute(['code' => $code]);

        return new JourneyResource($journey);
    }

    /**
     * Add employees to journey
     *
     * @param Request $request
     * @param $code
     * @return mixed
     */
    public function addEmployees(Request $request, $code)
    {
        $input = array_merge($request->all(), ['code' => $code]);
        $employees = app(AddEmployees::class)->execute($input);

        return UserJourneyCustomResource::collection($employees);
    }

    /**
     * Remove employee from journey
     *
     * @param $code
     * @param $employeeCode
     * @return JsonResponse
     */
    public function removeEmployee($code, $employeeCode)
    {
        app(RemoveEmployee::class)->execute(['code' => $code, 'employee_code' => $employeeCode]);

        return $this->respondObjectDeleted($employeeCode);
    }

    /**
     * Remove employees from journey
     *
     * @param $code
     * @param $employeesCode
     * @return JsonResponse
     */
    public function removeEmployees($code, $employeesCode)
    {
        if ($employeesCode === 'all') {
            $employees = app(GetJourneyEmployees::class)->execute(['code' => $code, 'isNotPaging' => true]);
            $codes = array_map(function ($employee) {
                return $employee['code'];
            }, UserJourneyResource::collection($employees)->jsonSerialize());
        } else {
            $codes = explode("-", $employeesCode);
        }

        foreach ($codes as $employeeCode) {
            app(RemoveEmployee::class)->execute(['code' => $code, 'employee_code' => $employeeCode]);
        }
        return $this->respondObjectDeleted($codes);
    }

    /**
     * Send reminder to specific user
     * @param string $code
     * @param string $employeeCode
     * @return JsonResponse
     */
    public function sendReminderUser(string $code, string $employeeCode)
    {
        app(SendReminderUser::class)->execute(['journey_code' => $code, 'user' => $employeeCode]);

        return $this->responseWithStatusCode(200, "Send notification successfully");
    }

    /**
     * Send reminder to selected users
     * @param string $code
     * @param string $employeesCode
     * @return JsonResponse
     */
    public function sendReminderUsers(string $code, string $employeesCode)
    {
        if ($employeesCode === 'all') {
            $employees = app(GetJourneyEmployees::class)->execute(['code' => $code, 'isNotPaging' => true]);
            $codes = array_map(function ($employee) {
                return $employee['code'];
            }, UserJourneyResource::collection($employees)->jsonSerialize());
        } else {
            $codes = explode("-", $employeesCode);
        }

        foreach ($codes as $employeeCode) {
            app(SendReminderUser::class)->execute(['journey_code' => $code, 'user' => $employeeCode]);
        }

        return $this->responseWithStatusCode(200, "Send notification successfully");
    }

    /**
     * Get employee submission
     *
     * @param $code
     * 
     * @param $employeeCode
     * @return Mixed
     */
    public function getEmployeeSubmission($code, $employeeCode)
    {
        $stepSubmission = app(GetEmployeeSubmission::class)->execute(
            ['code' => $code, 'employee_code' => $employeeCode]
        );

        return JourneyStepSubmissionResource::collection($stepSubmission);
    }

    /**
     * @param string $code
     * @return JsonResponse
     */
    public function restartJourney(string $code)
    {
        app(RestartJourney::class)->execute(['code' => $code]);

        return $this->responseWithStatusCode(200, "Restart journey successfully");
    }

    /**
     * @param string $code
     * @param string $employeeCode
     * @return JsonResponse
     */
    public function restartJourneyUser(string $code, string $employeeCode)
    {
        app(RestartJourneyUser::class)->execute(['code' => $code, 'employee_code' => $employeeCode]);

        return $this->responseWithStatusCode(200, "Restart journey successfully");
    }

    /**
     * @param string $code
     * @param string $employeesCode
     * @return JsonResponse
     */
    public function restartJourneyUsers(string $code, string $employeesCode)
    {
        if ($employeesCode === 'all') {
            $employees = app(GetJourneyEmployees::class)->execute(['code' => $code, 'isNotPaging' => true]);
            $codes = array_map(function ($employee) {
                return $employee['code'];
            }, UserJourneyResource::collection($employees)->jsonSerialize());
        } else {
            $codes = explode("-", $employeesCode);
        }

        foreach ($codes as $employeeCode) {
            app(RestartJourneyUser::class)->execute(['code' => $code, 'employee_code' => $employeeCode]);
        }

        return $this->responseWithStatusCode(200, "Restart journey successfully");
    }

    /**
     * Download employee submission
     * 
     * @param $code
     * @param $employeeCode
     * @return mixed
     */
    public function downloadEmployeeSubmission($code, $employeeCode)
    {
        return app(DownloadEmployeeSubmission::class)->execute(
            ['code' => $code, 'employee_code' => $employeeCode]
        );
    }

    /**
     * Download employees submission
     *
     * @param $code
     * @param $employeesCode
     * @return mixed
     */
    public function downloadEmployeesSubmission($code, $employeesCode)
    {
        if ($employeesCode === 'all') {
            $employees = app(GetJourneyEmployees::class)->execute(['code' => $code, 'isNotPaging' => true]);
            $codes = array_map(function ($employee) {
                return $employee['code'];
            }, UserJourneyResource::collection($employees)->jsonSerialize());
        } else {
            $codes = explode("-", $employeesCode);
        }
        return app(DownloadEmployeesSubmission::class)->execute(
            ['code' => $code, 'employees_code' => $codes]
        );
    }

    /**
     * Create journey employee documents
     * 
     * @param $code
     * @return JsonResponse
     */
    public function createFormDocument($code)
    {
        $journey = Journey::findByBusinessKey($code);

        CreateJourneyEmployeeDocument::dispatch($journey);

        return $this->responseWithStatusCode(200, "Created journey documents successfully");
    }
}