<?php

namespace App\Http\Controllers\Api\v1\Library;

use App\Http\Controllers\Api\ApiController;
use App\Http\Resources\Library\LibraryItem as LibraryItemResource;
use App\Http\Resources\Library\DocumentProfile as DocumentProfileResource;
use App\Services\Library\CreateLibraryAttachment;
use App\Services\Library\CreateLibraryLink;
use App\Services\Library\UpdateLibraryItem;
use App\Services\Library\DeleteLibraryItem;
use App\Services\Library\DeleteMultipleLibraryItem;
use App\Services\Library\GetLibraryItems;
use App\Services\Library\CreateDocumentProfile;
use App\Services\Library\UpdateDocumentProfile;
use App\Services\Library\DeleteDocumentProfile;
use App\Services\Library\GetDocumentProfiles;
use Exception;
use Illuminate\Http\Request;
use App\Exceptions\InvalidRequestException;
use App\Enum\LibraryItemType;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;
use App\Enum\ErrorCode;

class LibraryController extends ApiController
{
    /**
     * List all library items
     *
     * @return mixed
     */
    public function getAll()
    {
        $libraryItems =  app(GetLibraryItems::class)->execute();
        return LibraryItemResource::collection($libraryItems);
    }

    /**
     * Add Library Item
     *
     * @param Request $request
     * @return mixed
     * @throws Exception
     */
    public function create(Request $request)
    {
        try {
            $data =  $request->all();

            if (empty($data['type'])) {
                throw new InvalidRequestException();
            }

            if ($data['type'] === LibraryItemType::FILE) {
                $documentProfiles = app(GetDocumentProfiles::class)->execute([]);
                $documentProfileIds = $documentProfiles->map(function ($item) {
                    return $item->id;
                })->toArray();

                if ($documentProfiles->count() <= 0) {
                    $documentProfile = app(CreateDocumentProfile::class)->execute(['name' => 'Journey Introduction']);
                    $data['document_profile_id'] = $documentProfile->id;
                } else if (!isset($data['document_profile_id']) || (isset($data['document_profile_id']) && !in_array((int)$data['document_profile_id'], $documentProfileIds))) {
                    $data['document_profile_id'] = $documentProfileIds[0];
                }

                $libraryItem = $this->transaction(function () use ($data) {
                    return app(CreateLibraryAttachment::class)->execute($data);
                });
            } elseif ($data['type'] === LibraryItemType::LINK) {
                $libraryItem = $this->transaction(function () use ($data) {
                    return app(CreateLibraryLink::class)->execute($data);
                });
            } else {
                throw new InvalidRequestException();
            }
        } catch (Exception $e) {
            throw $e;
        }
        return new LibraryItemResource($libraryItem);
    }

    /**
     * Bulk upload file to library
     *
     * @param Request request
     * @return mixed
     * @throws Exception
     */
    function bulkUploadFileToLibrary(Request $request)
    {
        $results = [];

        try {
            $data =  $request->all();
            if (count($data['filesData']) < 0) {
                throw new InvalidRequestException();
            } else {
                foreach ($data['filesData'] as $file) {
                    $libraryItem = $this->transaction(function () use ($file) {
                        return app(CreateLibraryAttachment::class)->execute($file);
                    });

                    array_push($results, new LibraryItemResource($libraryItem));
                }
            }

            return $results;
        } catch (\Throwable $e) {
            throw $e;
        }

        return $results;
    }

    /**
     * Update library item
     *
     * @param Request $request
     * @param $id
     * @return LibraryItemResource
     * @throws Exception
     */
    public function update(Request $request, $id)
    {
        try {
            $input = array_merge($request->all(), ['id' => $id]);
            $libraryItem = app(UpdateLibraryItem::class)->execute($input);
        } catch (Exception $e) {
            throw $e;
        }
        return new LibraryItemResource($libraryItem);
    }

    /**
     * Delete library item
     *
     * @param $id
     * @return \Illuminate\Http\JsonResponse
     * @throws Exception
     */
    public function delete(int $id)
    {
        try {
            app(DeleteLibraryItem::class)->execute(['id' => $id]);
        } catch (Exception $e) {
            throw $e;
        }
        return $this->respondObjectDeleted($id);
    }

    /**
     * Delete multiple library item
     *
     * @param $ids
     * @return \Illuminate\Http\JsonResponse
     * @throws Exception
     */

    public function deleteMultiple(Request $request)
    {
        $data = $request->all();
        $results = [];
        $ids = $data['ids'];
        if ($data['ids'] == "all") {
            $libraryItems =  app(GetLibraryItems::class)->execute(array_merge([$request->all(), 'isNotPaging' => true]));
            $ids = array_map(function ($libraryItem) {
                return $libraryItem['id'];
            }, LibraryItemResource::collection($libraryItems)->jsonSerialize());
        }
        foreach ($ids as $id) {
            try {
                app(DeleteMultipleLibraryItem::class)->execute(['id' => $id]);
                $obj = [
                    'deleted' => true,
                    'id' => $id,
                    'message' => null
                ];
                array_push($results, $obj);
            } catch (Exception $e) {
                $obj = [
                    'deleted' => false,
                    'id' => $id,
                    'errorCode' => $e->errorCode,
                    'message' => $e->getMessage()
                ];

                if ($e->errorCode === ErrorCode::LIBRARY_ITEM_IN_USED_JOURNEY) {
                    $obj['action'] = $e->replacements['action'];
                    $obj['type'] = $e->replacements['type'];
                    array_push($results, $obj);
                } else {
                    array_push($results, $obj);
                    break;
                }
            }
        }

        return $this->respondObjectMultipleDeleted($results);
    }

    /**
     * Get Document profiles
     *
     * @param $id
     * @return AnonymousResourceCollection
     * @throws Exception
     */
    public function getDocumentProfiles(Request $request)
    {
        $documentProfiles = app(GetDocumentProfiles::class)->execute($request->all());
        return DocumentProfileResource::collection($documentProfiles);
    }

    /**
     * Create document profile
     *
     * @param $id
     * @return \Illuminate\Http\JsonResponse
     * @throws Exception
     */
    public function createDocumentProfile(Request $request)
    {
        $data =  $request->all();
        $documentProfile = app(CreateDocumentProfile::class)->execute($data);
        return new DocumentProfileResource($documentProfile);
    }

    /**
     * Update document profile
     *
     * @param $id
     * @return \Illuminate\Http\JsonResponse
     * @throws Exception
     */
    public function updateDocumentProfile(Request $request, $id)
    {
        $data = array_merge($request->all(), ['id' => $id]);
        $documentProfile = app(UpdateDocumentProfile::class)->execute($data);
        return new DocumentProfileResource($documentProfile);
    }

    /**
     * Delete profile
     *
     * @param $id
     * @return \Illuminate\Http\JsonResponse
     * @throws Exception
     */
    public function deleteDocumentProfile(int $id)
    {
        app(DeleteDocumentProfile::class)->execute(['id' => $id]);
        return $this->respondObjectDeleted($id);
    }
}
