<?php

namespace App\Http\Controllers\Api\v1\Message;

use App\Enum\ConversationStatues;
use App\Enum\Roles;
use App\Http\Controllers\Api\ApiController;
use App\Http\Resources\Message\Conversation;
use App\Http\Resources\Message\Conversation as ConversationResource;
use App\Http\Resources\Message\ConversationMessage as ConversationMessageResource;
use App\Http\Resources\Message\Notification as NotificationResource;
use App\Http\Resources\Message\MessageLite as MessageLiteResource;
use App\Http\Resources\Message\CommentNote as CommentNoteResource;
use App\Models\Message\Notification;
use App\Services\Message\ChangeAnonymousConversationStatus;
use App\Services\Message\CreateCommentDiscussion;
use App\Services\Message\CreateCommentNote;
use App\Services\Message\CreateConversationAdmin;
use App\Services\Message\CreateMessage;
use App\Services\Message\CreateSensitiveCommentDiscussion;
use App\Services\Message\DeleteConversation;
use App\Services\Message\DownloadAnonymousConversation;
use App\Services\Message\GetCommentNote;
use App\Services\Message\GetConversations;
use App\Services\Message\GetMessageConversation;
use App\Services\Message\GetNotifications;
use App\Services\Message\GetNumberUnreadMessage;
use App\Services\Message\ManageWhistleblowingCategoryConversation;
use App\Services\Message\MarkConversationAsRead;
use App\Services\Message\TransferWhistleblower;
use App\Services\Message\TranslateMessage;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use PHPUnit\Framework\Error\Notice;

/**
 * Class MessageController
 * @package App\Http\Controllers\Api\v1\Message
 */
class MessageController extends ApiController
{

    /**
     * @param Request $request
     * @return MessageLiteResource
     */
    public function store(Request $request)
    {
        $message = app(CreateMessage::class)->execute($request->except('sender_id'));

        return new MessageLiteResource($message);
    }

    /**
     * @param Request $request
     * @return CommentNoteResource
     */
    public function sendCommentNote(Request $request)
    {
        $note = app(CreateCommentNote::class)->execute($request->except('sender_id'));

        return new CommentNoteResource($note);
    }

    /**
     * @param Request $request
     * @return CommentNoteResource
     */
    public function getCommentNotes(string $comment_id, Request $request)
    {
        $options = array_merge(['comment_id' => $comment_id], $request->all());
        $notes = app(GetCommentNote::class)->execute($options);

        return CommentNoteResource::collection($notes);
    }

    public function getNotifications(Request $request)
    {
        $notifications = app(GetNotifications::class)->execute($request->all());
        return NotificationResource::collection($notifications);
    }

    public function readNotification(Request $request, $id)
    {
        Notification::find($id)->where('receiver_id', Auth::user()->id)->update(['read' => $request->read ?? 1]);
        return response()->json(['success' => true]);
    }

    public function readAllNotifications()
    {
        Notification::where('receiver_id', Auth::user()->id)->update(['read' => 1]);
        return response()->json(['success' => true]);
    }

    /**
     * @param Request $request
     * @return ConversationResource
     */
    public function createConversationAdmin(Request $request)
    {
        $data = $request->all();
        $data['is_anonymous'] = 0;
        $conversation = app(CreateConversationAdmin::class)->execute($data);
        return new ConversationResource($conversation);
    }

    /**
     * @param Request $request
     * @return \Illuminate\Http\Resources\Json\AnonymousResourceCollection
     */
    public function getConversations(Request $request)
    {
        $conversations = app(GetConversations::class)->execute($request->all());
        return ConversationResource::collection($conversations);
    }

    /**
     * @param Request $request
     * @return \Illuminate\Http\Resources\Json\AnonymousResourceCollection
     */
    public function getConversationDetail(string $id, Request $request)
    {
        $data = array_merge(['id' => $id, $this->options]);
        $conversation = app(GetConversations::class)->execute($data)->first();
        return new ConversationResource($conversation);
    }

    /**
     * @param string $id
     * @param Request $request
     * @return ConversationMessageResource
     */
    public function getMessageConversation(string $id, Request $request)
    {
        $options = array_merge(['id' => $id], $request->all());
        $conversation = app(GetMessageConversation::class)->execute($options);
        return new ConversationMessageResource($conversation);
    }

    /**
     * @param string $id
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function markConversationAsRead(string $id, Request $request)
    {
        $data = array_merge(['id' => $id], $request->all());
        app(MarkConversationAsRead::class)->execute($data);

        return $this->responseWithStatusCode(200, "Mark conversation as read successfully");
    }

    /**
     * @param string $id
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function deleteConversation(string $id, Request $request)
    {
        $data = array_merge(['id' => $id], $request->all());
        app(DeleteConversation::class)->execute($data);

        return $this->responseWithStatusCode(200, "Delete conversation successfully");
    }

    /**
     * @param  Request  $request
     * @return ConversationResource
     */
    public function createSensitiveCommentDiscussion(Request $request)
    {
        $conversation = app(CreateSensitiveCommentDiscussion::class)->execute($request->all());
        return new Conversation($conversation);
    }

    /**
     * @param  Request  $request
     * @return ConversationResource
     */
    public function createCommentDiscussion(Request $request)
    {
        $conversation = app(CreateCommentDiscussion::class)->execute($request->all());
        return new Conversation($conversation);
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function getNumberUnreadMessage()
    {
        $count = app(GetNumberUnreadMessage::class)->execute(['type' => Roles::ADMIN]);
        return $this->responseData(['count' => $count]);
    }

    /**
     * @param  string  $id
     * @param  Request  $request
     * @return ConversationResource
     */
    public function manageWhistlegblowingCategoryConversation(string $id, Request $request)
    {
        $input = array_merge($request->all(), ['id' => $id]);
        $conversation = app(ManageWhistleblowingCategoryConversation::class)->execute($input);
        return new Conversation($conversation);
    }

    /**
     * @param  string  $id
     * @param  Request  $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function transferWhistleblower(string $id, Request $request)
    {
        $input = array_merge($request->all(), ['id' => $id]);
        app(TransferWhistleblower::class)->execute($input);
        return $this->responseWithStatusCode(200, "Transfer conversation successfully");
    }

    /**
     * @param  string  $id
     * @param  Request  $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function closeAnonymousConversation(string $id, Request $request)
    {
        $input = array_merge($request->all(), ['id' => $id, 'status' => ConversationStatues::CLOSE]);
        $conversation = app(ChangeAnonymousConversationStatus::class)->execute($input);
        return $this->responseWithStatusCode(200, "Close conversation successfully");
    }

    /**
     * @param  string  $id
     * @param  Request  $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function reopenAnonymousConversation(string $id, Request $request)
    {
        $input = array_merge($request->all(), ['id' => $id, 'status' => ConversationStatues::ACTIVE]);
        $conversation = app(ChangeAnonymousConversationStatus::class)->execute($input);
        return $this->responseWithStatusCode(200, "Reopen conversation successfully");
    }

    /**
     *
     */
    public function updateStatusAnonymousConversation(string $id, string $status, Request $request)
    {
        $input = array_merge($request->all(), ['id' => $id, 'status' => $status]);
        $conversation = app(ChangeAnonymousConversationStatus::class)->execute($input);
        return $this->responseWithStatusCode(200, "Update conversation successfully");
    }

    /**
     * @param  string  $id
     * @param  Request  $request
     */
    public function downloadAnonymousConversation(string $id, Request $request)
    {
        return app(DownloadAnonymousConversation::class)->execute(
            ['conversation_id' => $id]
        );
    }

    /**
     * @param  string  $id
     * @param  Request  $request
     */
    public function translateMessage(string $id, Request $request)
    {
        app(TranslateMessage::class)->execute(array_merge($request->all(), ['id' => $id]));
        return $this->responseWithStatusCode(200, "Create message translation successfully");
    }
}
