<?php


namespace App\Http\Controllers\Api\v1\SSO;


use App\Enum\ErrorCode;
use App\Exceptions\BaseHttpException;
use App\Helpers\Saml2AuthHelper;
use App\Helpers\URLHelper;
use App\Http\Controllers\Api\ApiController;
use App\Services\Auth\SSOLogin;
use App\Services\SSO\GetConfigUrl;
use App\Services\SSO\GetLoginUrl;
use App\Services\SSO\ProcessSaml2AssertionRequest;
use Exception;
use Illuminate\Http\Request;

class Saml2Controller extends ApiController
{
    /**
     * Get SSO login url
     * @param  Request  $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function login(Request $request)
    {
        $loginUrl = app(GetLoginUrl::class)->execute($request->all());
        return $this->responseData(['login_url' => $loginUrl]);
    }

    /**
     * Get config url by client for SSO login
     * @param  Request  $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function getConfigUrl(Request $request)
    {
        $urls = app(GetConfigUrl::class)->execute();
        return $this->responseData($urls);
    }

    /**
     * Generate local sp metadata.
     *
     * @param  Saml2AuthHelper  $saml2Auth
     * @return \Illuminate\Http\Response
     */
    public function metadata(Saml2AuthHelper $saml2Auth)
    {
        $metadata = $saml2Auth->getMetadata();

        return response($metadata, 200, ['Content-Type' => 'text/xml']);
    }

    /**
     * Process an incoming saml2 assertion request.
     * Fires 'Saml2LoginEvent' event if a valid user is found.
     *
     * @return \Illuminate\Http\Response
     */
    public function acs(Request $request, $clientCode)
    {
        try {
            $currentUser = app(ProcessSaml2AssertionRequest::class)->execute(['client_code' => $clientCode]);
            $redirectUrl = config('app.front_end_url');
            $result = app(SSOLogin::class)->execute(['user' => $currentUser]);
            return redirect($redirectUrl)->withCookie($result['cookie']);
        } catch (BaseHttpException $exception) {
            return redirect(URLHelper::url_query(config('saml2_settings.loginUrl'),
                ['errorCode' => $exception->errorCode]));
        } catch (Exception $exception) {
            return redirect(URLHelper::url_query(config('saml2_settings.loginUrl'),
                ['errorCode' => ErrorCode::SSO_ERROR]));
        }
    }

    /**
     * Process an incoming saml2 logout request.
     * Fires 'Saml2LogoutEvent' event if its valid.
     * This means the user logged out of the SSO infrastructure, you 'should' log them out locally too.
     *
     * @param  Saml2AuthHelper  $saml2Auth
     * @param $clientCode
     * @return \Illuminate\Http\Response
     */
    public function sls(Saml2AuthHelper $saml2Auth, $clientCode)
    {
        $errors = $saml2Auth->sls($clientCode, config('saml2_settings.retrieveParametersFromServer'));
        if (!empty($errors)) {
            logger()->error('Saml2 error', $errors);
            session()->flash('saml2_error', $errors);
            throw new Exception("Could not log out");
        }

        return redirect(config('saml2_settings.loginUrl')); //may be set a configurable default
    }

    /**
     * Initiate a logout request across all the SSO infrastructure.
     *
     * @param  Saml2AuthHelper  $saml2Auth
     * @param  Request  $request
     */
    public function logout(Saml2AuthHelper $saml2Auth, Request $request)
    {
        $returnTo = $request->query('returnTo');
        $sessionIndex = $request->query('sessionIndex');
        $nameId = $request->query('nameId');
        $saml2Auth->logout($returnTo, $nameId, $sessionIndex); //will actually end up in the sls endpoint
        //does not return
    }


}