<?php

namespace App\Http\Controllers\Api\v1\Subscription;

use App\Helpers\StripeHelper;
use App\Http\Controllers\Api\ApiController;
use App\Services\Subscription\ChangeCard;
use App\Services\Subscription\ChangePlan;
use App\Services\Subscription\CreateSubscriptionPlan;
use App\Services\Subscription\DeleteSubscriptionPlan;
use App\Services\Subscription\GetCurrentPlan;
use App\Services\Subscription\GetCard;
use App\Services\Subscription\GetDetailSubscriptionPlan;
use App\Services\Subscription\GetInvoice;
use App\Services\Subscription\GetStripePlans;
use App\Services\Subscription\GetUpcomingInvoice;
use App\Services\Subscription\UpdateSubscriptionPlan;
use App\Services\Subscription\DuplicateSubscriptionPlan;
use Illuminate\Http\Request;
use App\Http\Resources\Subscription\SubscriptionPlan as SubscriptionPlanResource;
use App\Http\Resources\Subscription\SubscriptionPlanDetail as SubscriptionPlanDetailResource;
use App\Http\Resources\Subscription\StripePlan as StripePlanResource;
use App\Http\Resources\Subscription\Invoice as InvoiceResource;
use App\Http\Resources\Subscription\Card as CardResource;
use App\Services\Subscription\GetSubscriptionPlans;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;
use Illuminate\Http\Resources\Json\JsonResource;


/**
 * Class SubscriptionController
 * @package App\Http\Controllers\Api\v1\Subscription
 */
class SubscriptionController extends ApiController
{
    /**
     * Get subscription plans
     *
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function getStripePlans(Request $request)
    {
        $stripePlans = app(GetStripePlans::class)->execute($this->options);
        return StripePlanResource::collection($stripePlans);
    }

    /**
     * Get Stripe customers
     * 
     * @return AnonymousResourceCollection
     * @throws \Stripe\Exception\ApiErrorException
     */
    public function getStripeCustomers()
    {
        $stripeCustomers = collect((new StripeHelper())->getAllCustomersWithSubscriptions());
        
        return JsonResource::collection($stripeCustomers);
    }

    /**
     * Get subscription plans
     *
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function getSubscriptionPlans(Request $request)
    {
        $subscriptionPlans = app(GetSubscriptionPlans::class)->execute($this->options);
        return SubscriptionPlanResource::collection($subscriptionPlans);
    }

    /**
     * Get subscription plans
     *
     * @param Request $request
     * @return AnonymousResourceCollection
     */
    public function getSubscriptionPlansWithDetail(Request $request)
    {
        $subscriptionPlans = app(GetSubscriptionPlans::class)->execute($this->options);
        return SubscriptionPlanDetailResource::collection($subscriptionPlans);
    }

    /**
     * Get subscription plans detail
     *
     * @param string $code
     * @return SubscriptionPlanDetailResource
     */
    public function GetDetailSubscriptionPlan(string $code)
    {
        $subscriptionPlan = app(GetDetailSubscriptionPlan::class)->execute(['code' => $code]);
        return new SubscriptionPlanDetailResource($subscriptionPlan);
    }

    /**
     * Create subscription plans
     *
     * @param Request $request
     * @return SubscriptionPlanDetailResource
     */
    public function createSubscriptionPlan(Request $request)
    {
        $subscriptionPlans = $this->transaction(function () use ($request) {
            return app(CreateSubscriptionPlan::class)->execute($request->all());
        });
        return new SubscriptionPlanDetailResource($subscriptionPlans);
    }

    /**
     * Update subscription plans
     *
     * @param string $code
     * @param Request $request
     * @return SubscriptionPlanDetailResource
     */
    public function updateSubscriptionPlan(string $code, Request $request)
    {
        $data = array_merge(['code' => $code], $request->all());
        $subscriptionPlans = $this->transaction(function () use ($data) {
            return app(UpdateSubscriptionPlan::class)->execute($data);
        });
        return new SubscriptionPlanDetailResource($subscriptionPlans);
    }

    /**
     * Duplicate subscription plans
     *
     * @param string $code
     * @param Request $request
     * @return SubscriptionPlanDetailResource
     */
    public function duplicateSubscriptionPlan(string $code, Request $request)
    {
        $subscriptionPlans =  app(DuplicateSubscriptionPlan::class)->execute(['code' => $code]);
        return new SubscriptionPlanDetailResource($subscriptionPlans);
    }

    /**
     * Delete subscription plans
     *
     * @param string $code
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function deleteSubscriptionPlan(string $code, Request $request)
    {
        app(DeleteSubscriptionPlan::class)->execute(['code' => $code]);
        return $this->respondObjectDeleted($code);
    }

    /**
     * Get current plan
     *
     * @return SubscriptionPlanDetailResource
     */ 
    public function getCurrentPlan()
    {
        $currentPlan = app(GetCurrentPlan::class)->execute();
        
        return new SubscriptionPlanDetailResource($currentPlan);
    }

    /**
     * Get invoices
     *
     * @return AnonymousResourceCollection
     */
    public function getInvoices()
    {
        $invoices = app(GetInvoice::class)->execute($this->options);
        
        return InvoiceResource::collection($invoices);
    }

    /**
     * Get upcoming invoice
     *
     * @return InvoiceResource
     */
    public function getUpcomingInvoice()
    {
        $upcomingInvoice = app(GetUpcomingInvoice::class)->execute();
        
        return new InvoiceResource($upcomingInvoice);
    }

    /**
     * Get cards
     * 
     * @return CardResource
     */
    public function getCard()
    {
        $card = app(GetCard::class)->execute();
        
        return new CardResource($card);
    }

    /**
     * Change card
     *
     * @param Request $request
     * @return CardResource
     */
    public function changeCard(Request $request)
    {
        $card = app(ChangeCard::class)->execute($request->all());

        return new CardResource($card);
    }

    /**
     * Change plan
     *
     * @param Request $request
     * @return SubscriptionPlanDetailResource
     */
    public function changePlan(Request $request)
    {
        $plan = app(ChangePlan::class)->execute($request->all());

        return new SubscriptionPlanDetailResource($plan);
    }
}