<?php

namespace App\Http\Controllers\Api\v1\Survey;

use App\Enum\CommentTypes;
use App\Http\Controllers\Api\ApiController;
use App\Http\Resources\Report\ReportComment as ReportCommentResource;
use App\Http\Resources\Tool\Survey\Survey as SurveyResource;
use App\Http\Resources\Tool\Survey\SurveyDetail as SurveyDetailResource;
use App\Http\Resources\Tool\Survey\SurveyReport as SurveyReportResource;
use App\Services\Report\ReactComment;
use App\Services\Tool\Survey\DownloadSurvey;
use App\Services\Tool\Survey\GetSurvey;
use App\Services\Tool\Survey\GetDetailSurvey;
use App\Services\Tool\Survey\CreateSurvey;
use App\Services\Tool\Survey\DeleteSurvey;
use App\Services\Tool\Survey\PublishSurvey;
use App\Services\Tool\Survey\UnpublishSurvey;
use App\Services\Tool\Survey\UpdateSurvey;
use App\Services\Tool\Survey\AttachQuestion;
use App\Services\Tool\Survey\DuplicateSurvey;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

/**
 * Class SurveyController
 * @package App\Http\Controllers\Api\v1\Survey
 */
class SurveyController extends ApiController
{
    /**
     * List all survey
     *
     * @param Request $request
     * @return mixed
     */
    public function getAll(Request $request)
    {
        $surveys = app(GetSurvey::class)->execute($this->options);
        return SurveyResource::collection($surveys);
    }

    /**
     * @param string $code
     * @return SurveyDetailResource
     */
    public function get(string $code)
    {
        $survey = app(GetDetailSurvey::class)->execute(['code' => $code]);
        return new SurveyDetailResource($survey);
    }

    /**
     * Add survey
     *
     * @param Request $request
     * @return mixed
     * @throws Exception
     */
    public function create(Request $request)
    {
        $survey = app(CreateSurvey::class)->execute($request->all());
        return new SurveyResource($survey);
    }

    /**
     * Update Survey
     *
     * @param Request $request
     * @param $code
     * @return SurveyDetailResource ()
     * @throws Exception
     */
    public function update(Request $request, $code)
    {

        $input = array_merge($request->all(), ['code' => $code]);
        $survey = app(UpdateSurvey::class)->execute($input);
        return new SurveyDetailResource($survey);
    }

    /**
     * Publish Survey
     *
     * @param int $id
     * @return SurveyResource
     * @throws Exception
     */
    public function publish(string $code)
    {
        $survey = $this->transaction(function() use ($code) {
            return app(PublishSurvey::class)->execute(['code' => $code]);
        });

        return new SurveyResource($survey);
    }

    /**
     * Unpublish survey
     *
     * @param string $code
     * @return SurveyResource
     * @throws Exception
     */
    public function unpublish(string $code)
    {
        $survey = app(UnpublishSurvey::class)->execute(['code' => $code]);
        return new SurveyResource($survey);
    }

    /**
     * Attach question into survey
     *
     * @param Request $request
     * @param string $code
     * @return SurveyDetailResource ()
     * @throws Exception
     */
    public function question(Request $request, string $code)
    {
        $input = array_merge($request->all(), ['code' => $code]);
        $survey = app(AttachQuestion::class)->execute($input);
        return new SurveyDetailResource($survey);
    }

    /**
     * Attach question into survey
     *
     * @param Request $request
     * @param string $code
     * @return SurveyDetailResource ()
     * @throws Exception
     */
    public function duplicate(Request $request, string $code)
    {
        $input = array_merge($request->all(), ['code' => $code]);
        $survey = app(DuplicateSurvey::class)->execute($input);
        return new SurveyDetailResource($survey);
    }

    /**
     * Delete survey
     *
     * @param $code
     * @return JsonResponse
     * @throws Exception
     */
    public function delete(string $code)
    {
        app(DeleteSurvey::class)->execute(['code' => $code]);
        return $this->respondObjectDeleted($code);
    }

    /**
     * @param string $code
     * @return array
     */
    public function report(Request $request, string $code)
    {
        $survey = app(GetDetailSurvey::class)->execute(['code' => $code]);
        return new SurveyReportResource($survey);
    }

    /**
     *
     */
    public function download(Request $request, string $code)
    {
        return app(DownloadSurvey::class)->execute(['code' => $code]);
    }

    /**
     * react to the comments
     */
    public function reactComment(Request $request, $id)
    {
        $data = $request->all();
        $data['id'] = $id;
        $data['comment_type'] = CommentTypes::NORMAL_SURVEY_COMMENT;

        $result = app(ReactComment::class)->execute($data);

        return new ReportCommentResource($result);
    }
}
