<?php

namespace App\Http\Resources\User;

use App\Helpers\DateHelper;
use App\Http\Resources\Client\ClientLite;
use App\Http\Resources\Library\DocumentProfile;
use App\Http\Resources\Segment\Segment;
use Illuminate\Http\Resources\Json\JsonResource;
use App\Models\User\User;
use App\Models\Journey\Journey;
use App\Models\ActionRequest\ActionRequestSubmission;
use App\Enum\SubmissionStatus;
use App\Models\Action\ActionSequence;

/**
 * @extends JsonResource<\App\Models\User\User>
 */
class Employee extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @param \Illuminate\Http\Request $request
     * @return array
     */
    public function toArray($request)
    {
        $employee = $this->employee;

        $managerNotifications = [];
        if ($employee->manager_notification_id) {
            $managerIds = explode(';', $employee->manager_notification_id);
            if (isset($managerIds)) {
                $managers = User::whereIn('id', $managerIds)->get();
                foreach ($managers as $manager) {
                    array_push($managerNotifications, new Manager($manager));
                }
            }
        }

        $journeys = $this->getJourneys($this->id);
        $actionSequences = $this->getActionSequence($this->id);


        return [
            'id' => $this->id,
            'code' => $this->code,
            'first_name' => $this->first_name,
            'surname' => $this->surname,
            'email' => $this->email,
            'email_change' => $this->email_change,
            'email_verified_at' => $this->email_verified_at,
            'last_login' => $this->last_login,
            'job_title' => $employee->job_title,
            'avatar' => $this->avatarUrl,
            'start_date' => $employee->start_date ? DateHelper::format($employee->start_date) : null,
            'created_at' => $employee->created_at ? DateHelper::format($this->created_at) : null,
            'date_of_birth' => $employee->date_of_birth ? DateHelper::format($employee->date_of_birth) : null,
            'employment_type' => $employee->employment_type,
            'location' => $this->when($employee->location_id && $employee->location, new Segment($employee->location), null),
            'department' => $this->when($employee->department_id && $employee->department, new Segment($employee->department), null),
            'gender' => $employee ? $this->when($employee->gender_id && $employee->gender, new Segment($employee->gender), null) : null,
            'team' => $this->when($employee->team_id && $employee->team, new Segment($employee->team), null),
            'status' => $employee->status,
            'manager' => $this->when($employee->manager_id && $employee->manager, new Manager($employee->manager), null),
            'secondary_manager' => $this->when($employee->secondary_manager_id && $employee->secondaryManager, new Manager($employee->secondaryManager), null),
            'manager_notifications' => $managerNotifications,
            'document_profiles' => DocumentProfile::collection($employee->documentProfiles),
            'client' => new ClientLite($employee->client),
            'timezone' => $employee->timezone,
            'journeys' => count($journeys) > 0 ? $journeys : null,
            'action_sequences' => count($actionSequences) > 0 ? $actionSequences : null,
            "location_id" => is_null($employee->location_id) ? null : $employee->location_id,
            "team_id" => is_null($employee->team_id) ? null : $employee->team_id,
            "department_id" => is_null($employee->department_id) ? null : $employee->department_id,
            "social_security_number" => $employee->social_security_number,
            "employee_client_id" => $employee->employee_client_id,
            "phone_number" => $employee->phone_number,
            "race" => $employee->race,
        ];
    }

    private function getJourneys($userId)
    {
        $journeys = Journey::select('journeys.*', 'journey_user.user_id')
            ->rightJoin('journey_user', 'journeys.id', 'journey_user.journey_id')
            ->where('journey_user.user_id', $userId)
            ->orderBy('journey_user.created_at', 'asc')
            ->get();

        $journeysArray = [];

        foreach ($journeys as $key => $journey) {
            $journeyItem = $journey->toArray();
            $steps = $journey->steps()->with('actionRequest')->get();
            $actionRequestIds = [];
            foreach ($steps as $step) {
                if (!empty($step->actionRequest)) {
                    $actionRequestIds[] = $step->actionRequest->getKey();
                }
            }

            $completedSubmissionCount = $this->getCompletedSubmissionCount($actionRequestIds, $journey['user_id']);
            if ($completedSubmissionCount == 0) {
                $journeyItem['status'] = "Not started";
            } else if ($completedSubmissionCount < count($steps)) {
                $journeyItem['status'] = "In Progress";
            } else {
                $journeyItem['status'] = "Completed";
            }

            array_push($journeysArray, $journeyItem);
        }

        return $journeysArray;
    }

    private function getActionSequence($userId)
    {
        $actionSequences = ActionSequence::select('action_sequences.*', 'action_sequence_user.user_id')
            ->rightJoin('action_sequence_user', 'action_sequences.id', 'action_sequence_user.action_sequence_id')
            ->where('action_sequence_user.user_id', $userId)
            ->orderBy('action_sequence_user.created_at', 'asc')
            ->get();

        $actionSequencesArray = $actionSequences->toArray();

        return $actionSequencesArray;
    }

    public function getCompletedSubmissionCount(array $actionRequestIds, $userId)
    {
        return ActionRequestSubmission::whereIn('action_request_id', $actionRequestIds)
            ->whereIn('status', SubmissionStatus::COMPLETED_STATUS)
            ->where('user_id', $userId)
            ->select('action_request_id')
            ->groupBy('action_request_id')
            ->get()
            ->count();
    }
}