<?php

namespace App\Models\Action;

use App\Contracts\ClientProperty;
use App\Enum\ActionRequestTypes;
use App\Enum\SubmissionStatus;
use App\Helpers\ActionRequestHelper;
use App\Models\ActionRequest\ActionRequestSubmission;
use App\Models\Client\Client;
use App\Models\User\User;
use App\Traits\HasBusinessKey;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Support\Facades\DB;

class ActionSequence extends Model implements ClientProperty
{
    use HasBusinessKey;
    
    protected $table = 'action_sequences';

    protected $fillable = ['client_id', 'name', 'created_by'];

    /**
     * @return BelongsToMany
     */
    public function users(): BelongsToMany
    {
        return $this->belongsToMany(User::class);
    }

    /**
     * @return HasMany
     */
    public function steps() : HasMany
    {
        return $this->hasMany(ActionSequenceStep::class)->orderBy('order');
    }

    /**
     * Count employees completed the action sequence
     *
     * @return int
     */
    public function countCompletedEmployee()
    {
        $employeeCount = 0;
        
        $actionRequestIds = [];
        foreach ($this->steps as $step) {
            if (!empty($step->actionRequest)) {
                $actionRequestIds[] = $step->actionRequest->getKey();
            }
        }

        $actionRequestSubmission = ActionRequestSubmission::whereIn('action_request_id', $actionRequestIds)
            ->whereIn('status', SubmissionStatus::COMPLETED_STATUS)
            ->select(DB::raw('count(id) as count'), 'user_id')
            ->groupBy('user_id')
            ->get();

        foreach ($actionRequestSubmission as $item) {
            if ($item['count'] === count($actionRequestIds)) {
                $employeeCount++;
            }
        }

        return $employeeCount;
    }

    public function client()
    {
        return $this->belongsTo(Client::class);
    }

    public function getClientId()
    {
        return $this->client_id;
    }

    /**
     * Get form type steps
     *
     * @return array
     */
    public function getFormTypeSteps()
    {
        $stepsWithFormType = [];

        $steps = $this->steps()->with('actionRequest')->get();
        foreach ($steps as $step) {
            if (ActionRequestHelper::getActionRequestTypeByContent($step->actionRequest->contentable_type)
                == ActionRequestTypes::FORM) {
                $stepsWithFormType[] = $step;
            }
        }

        return $stepsWithFormType;
    }
}