<?php

namespace App\Models\Client;

use App\Contracts\ClientProperty;
use App\Enum\ConfigurationTypes;
use App\Helpers\StripeHelper;
use App\Models\Integration\DisconnectedIntegration;
use App\Models\Integration\Integration;
use App\Models\Library\LibraryItem;
use App\Models\Segment\Segment;
use App\Models\User\Employee;
use App\Models\Attachment;
use App\Models\Phone;
use App\Models\MasterData\Industry;
use App\Models\Subscription\SubscriptionPlan;
use App\Models\User\User;
use Illuminate\Database\Eloquent\Model;
use App\Traits\HasBusinessKey;
use App\Enum\Roles;
use App\Models\Form\FormAuthenticate;
use App\Models\Form\FormTemplate;
use Illuminate\Database\Eloquent\Relations\HasMany;

/**
 * @property mixed id
 */
class Client extends Model implements ClientProperty
{
    use HasBusinessKey;

    protected $table = 'clients';

    protected $fillable = ['name', 'plan_id', 'industry_id', 'company_size', 'stripe_cus_id', 'is_demo'];

    public function clientConfigurations()
    {
        return $this->belongsToMany(ClientConfiguration::class,'client_configuration_detail','client_id','configuration_id')
            ->withPivot('value', 'applied_date');
    }

    public function getConfigurationsAttribute()
    {
        return $this->clientConfigurations->mapWithKeys(function($item, $key) {
            switch ($item->type) {
                case ConfigurationTypes::BOOLEAN:
                    $value = (bool) $item->pivot->value;
                    break;
                case ConfigurationTypes::NUMBER:
                    $value = (int) $item->pivot->value;
                    break;
                default:
                    $value = $item->pivot->value;
                    break;
            }
            return [$item->name => $value];
        })->toArray();
    }
    
    public function formAuthenticate()
    {
        return $this->hasOne(FormAuthenticate::class);
    }
    
    public function formTemplates()
    {
        return $this->hasMany(FormTemplate::class);
    }

    public function employees()
    {
        return $this->hasMany(Employee::class);
    }

    /**
     * @return HasMany
     */
    public function segments(): HasMany
    {
        return $this->hasMany(Segment::class);
    }

    public function admins() 
    {
        return $this->hasMany(Employee::class)->whereHas('user', function ($uq) {
            return $uq->with('roles')->whereHas("roles", function ($rq) {
                $rq->where('name', Roles::ADMIN);
            });
        });
    }

    public function frontEmployees()
    {
        return $this->hasMany(Employee::class)->whereHas('user', function ($uq) {
            $uq->with('roles')->whereHas("roles", function ($rq) {
                $rq->where('name', Roles::FRONT_USER);
            });
        });
    }

    public function managers()
    {
        return $this->hasMany(Employee::class)->whereHas('user', function ($uq) {
            $uq->with('roles')->whereHas("roles", function ($rq) {
                $rq->where('name', Roles::MANAGER);
            });
        });
    }

    /**
     * Get users query builder
     * 
     * @return mixed
     */
    public function users()
    {
        return User::whereIntegerInRaw('users.id', $this->employees()->pluck('user_id')->toArray());
    }

    /**
     * Get users query builder
     *
     * @return mixed
     */
    public function adminUsers()
    {
        return User::whereIntegerInRaw('users.id', $this->admins()->pluck('user_id')->toArray());
    }

    /**
     * Get front-users query builder
     *
     * @return mixed
     */
    public function frontUsers()
    {
        return User::whereIntegerInRaw('users.id', $this->frontEmployees()->pluck('user_id')->toArray());
    }

    /**
     * Get manager users query builder
     *
     * @return mixed
     */
    public function managerUsers()
    {
        return User::whereIntegerInRaw('users.id', $this->managers()->pluck('user_id')->toArray());
    }

    /**
     * Get all users
     * 
     * @return mixed
     */
    public function getUsers()
    {
        return $this->users()->get();
    }

    /**
     * Get admin users
     * 
     * @return mixed
     */
    public function getAdminUsers()
    {
        return $this->adminUsers()->get();
    }

    public function logo () {
        return $this->belongsTo(Attachment::class, 'logo_id');
    }

    public function phone () {
        return $this->belongsTo(Phone::class, 'phone_id');
    }

    public function industry () {
        return $this->belongsTo(Industry::class, 'industry_id');
    }

    public function subscriptionPlan () {
        return $this->belongsTo(SubscriptionPlan::class, 'plan_id');
    }

    public function getClientId()
    {
        return $this->id;
    }

    public function integration()
    {
        return $this->hasOne(Integration::class);
    }

    public function disconnectedIntegration()
    {
        return $this->hasMany(DisconnectedIntegration::class);
    }
}
