<?php


namespace App\Models\Event;


use App\Contracts\ClientProperty;
use App\Contracts\CreatorProperty;
use App\Enum\EmployeeStatus;
use App\Models\Client\Client;
use App\Models\Segment\Segment;
use App\Models\User\User;
use App\Traits\HasBusinessKey;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Support\Facades\Config;

class Event extends Model implements ClientProperty, CreatorProperty
{
    use HasBusinessKey;

    protected $table = 'events';
    
    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded =['id'];

    protected $dates = ['start_date', 'end_date', 'deleted_at', 'source_updated_at'];
    
    protected $with = ['client', 'users.avatar', 'activeUsers.avatar',  'segments.segmentType'];

    /**
     * @return BelongsToMany
     */
    public function users(): BelongsToMany
    {
        return $this->belongsToMany(User::class);
    }

    /**
     * @return BelongsToMany
     */
    public function activeUsers()
    {
        return $this->users()->active();
    }

    /**
     * @return BelongsToMany
     */
    public function segments(): BelongsToMany
    {
        return $this->belongsToMany(Segment::class);
    }

    public function client()
    {
        return $this->belongsTo(Client::class);
    }

    public function getClientId()
    {
        return $this->client_id;
    }
    
    public function scopePublish($query)
    {
        return $query->where('is_published', 1);
    }

    public function scopeOfClient($query, $clientId)
    {
        return $query->where('client_id', $clientId);
    }
    
    public function scopeAllEmployeesSegment($query)
    {
        return $query->whereHas('segments', function ($q) {
            $q->allSegmentType();
        });
    }

    public function getCountInvitedUsers()
    {
        if ($this->hasAllEmployeeSegment()) {
            $key = "total_client_" . $this->getClientId() . "_users";
            $totalClientUsers = Config::get($key);
            if (!$totalClientUsers) {
                $totalClientUsers = $this->getUsersBySegments()->count();
                Config::set($key, $totalClientUsers);
            }
            return $totalClientUsers;
        } else {
            $segmentIds = $this->segments->pluck('id')->toArray();
            if (count($segmentIds) == 0) {
                return 0;
            }
            $key = "total_segments_" . implode('_', $segmentIds) . "_users";
            $totalSegmentsUsers = Config::get($key);
            if (!$totalSegmentsUsers) {
                $totalSegmentsUsers = $this->getUsersBySegments($segmentIds)->count();
                Config::set($key, $totalSegmentsUsers);
            }
            return $totalSegmentsUsers;
        }
    }

    public function invitedUsers()
    {
        if ($this->hasAllEmployeeSegment()) {
            return $this->getUsersBySegments();
        } else {
            $segmentIds = $this->segments->pluck('id')->toArray();
            return $this->getUsersBySegments($segmentIds);
        }
    }

    public function creator()
    {
        return $this->belongsTo(User::class, 'created_by');
    }

    public function getCreatorId()
    {
        return $this->created_by;
    }

    public function getUsersBySegments($segmentIds = null)
    {
        $query = User::query()->select('users.*')->join('employees', 'users.id', '=', 'employees.user_id')
            ->where('employees.status', EmployeeStatus::ACTIVE);
        if ($segmentIds) {
            $query->join('segment_user', 'users.id', 'segment_user.user_id')
                ->whereIn('segment_user.segment_id', $segmentIds);
        } else {
            $query->where('employees.client_id', $this->getClientId());
        }
        return $query;
    }

    public function hasAllEmployeeSegment()
    {
        foreach ($this->segments as $segment) {
            if ($segment->isAllSegmentType()) {
                return true;
            }
        }
        return false;
    }
}
