<?php

namespace App\Models\Journey;

use App\Contracts\ClientProperty;
use App\Contracts\CreatorProperty;
use App\Enum\ActionRequestTypes;
use App\Enum\SubmissionStatus;
use App\Helpers\ActionRequestHelper;
use App\Models\ActionRequest\ActionRequestSubmission;
use App\Models\Client\Client;
use App\Models\User\User;
use App\Traits\HasBusinessKey;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\DB;

/**
 * Class Journey
 * @package App\Models\Journey
 */
class Journey extends Model implements ClientProperty, CreatorProperty
{
    use HasBusinessKey, SoftDeletes;

    protected $table = 'journeys';

    protected $fillable = ['client_id', 'name', 'status', 'created_by', 'start_date', 'due_date'];

    protected $dates = ['published_at'];

    /**
     * @return HasMany
     */
    public function steps(): HasMany
    {
        return $this->hasMany(JourneyStep::class)->orderBy('step_order');
    }

    /**
     * @return BelongsToMany
     */
    public function users(): BelongsToMany
    {
        return $this->belongsToMany(User::class);
    }

    /**
     * @return BelongsTo
     */
    public function client(): BelongsTo
    {
        return $this->belongsTo(Client::class);
    }

    /**
     * Get steps with type is Form
     * 
     * @return array
     */
    public function getFormTypeSteps()
    {
        $stepsWithFormType = [];

        $steps = $this->steps()->with('actionRequest')->get();
        foreach ($steps as $step) {
            if (
                ActionRequestHelper::getActionRequestTypeByContent($step->actionRequest->contentable_type)
                == ActionRequestTypes::FORM
            ) {
                $stepsWithFormType[] = $step;
            }
        }

        return $stepsWithFormType;
    }

    /**
     * Count employees completed the journey
     * 
     * @return int
     */
    public function countCompletedEmployee()
    {
        $employeeCount = 0;

        $actionRequestIds = [];
        foreach ($this->steps as $step) {
            if (!empty($step->actionRequest)) {
                $actionRequestIds[] = $step->actionRequest->getKey();
            }
        }

        $actionRequestSubmission = ActionRequestSubmission::whereIn('action_request_id', $actionRequestIds)
            ->whereIn('status', SubmissionStatus::COMPLETED_STATUS)
            ->select(DB::raw('count(id) as count'), 'user_id')
            ->groupBy('user_id')
            ->get();

        foreach ($actionRequestSubmission as $item) {
            if ($item['count'] >= count($actionRequestIds)) {
                $employeeCount++;
            }
        }

        return $employeeCount;
    }

    /**
     * Count employees completed the journey
     *
     * @return int
     */
    public function countNotStartedEmployee()
    {
        $actionRequestIds = [];
        foreach ($this->steps as $step) {
            if (!empty($step->actionRequest)) {
                $actionRequestIds[] = $step->actionRequest->getKey();
            }
        }

        return $this->users()->join('employees', 'users.id', '=', 'employees.user_id')
            ->with(['avatar', 'employee', 'actionRequestSubmissions'])
            ->whereNotIn('users.id', DB::table('action_request_submissions')
            ->select('user_id')
            ->whereIn('action_request_id', $actionRequestIds))
            ->count();
    }

    /**
     * Get users belong to journey.
     */
    public function journeyUsers()
    {
        return $this->hasMany(JourneyUser::class, 'journey_id');
    }

    public function getClientId()
    {
        return $this->client_id;
    }

    /**
     * @param $query
     * @param $clientId
     * @return mixed
     */
    public function scopeOfClient($query, $clientId)
    {
        return $query->where('client_id', $clientId);
    }

    public function creator()
    {
        return $this->belongsTo(User::class, 'created_by');
    }

    public function getCreatorId()
    {
        return $this->created_by;
    }
}
