<?php

namespace App\Models\Message;

use App\Contracts\ClientProperty;
use App\Enum\ConversationTypes;
use App\Enum\MessageableTypes;
use App\Enum\QualeeStatus;
use App\Facades\Context;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Segment\AgeSegment;
use App\Models\Segment\EmploymentPhaseSegment;
use App\Models\Segment\Segment;
use App\Models\Survey\SurveyTextResult;
use App\Models\User\User;
use DateTime;
use GoldSpecDigital\LaravelEloquentUUID\Database\Eloquent\Uuid;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;

/**
 * Class Conversation
 * @package App\Models\Message
 */
class Conversation extends Model implements ClientProperty
{
    use Uuid;

    /**
     * @var bool
     */
    public $incrementing = false;
    /**
     * @var string
     */
    protected $keyType = 'string';

    /**
     * @return BelongsTo
     */
    public function owner(): BelongsTo
    {
        return $this->belongsTo(User::class, 'owner_id');
    }

    /**
     * @return HasMany
     */
    public function messages($isMaster = false): HasMany
    {
        if ($isMaster) {
            return $this->hasMany(Message::class, 'conversation_id');
        }
        $userConversation = $this->users()->find(Context::user()->get()->getKey());
        $filterDate = now()->toDateTimeString();

        if ($userConversation) {
            $filterDate = $userConversation->pivot->last_available->toDateTimeString();
        }
        return $this->hasMany(Message::class, 'conversation_id')->where('created_at', '>', $filterDate);
    }

    /**
     * @return HasOne
     */
    public function reactionMessages(): HasOne
    {
        return $this->hasOne(Message::class, 'conversation_id')->where('content_type', 'reaction');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function users()
    {
        return $this->morphedByMany(User::class, 'messageable')
            ->using(Messageable::class)
            ->withPivot(['last_read', 'last_available', 'is_anonymous', 'status'])
            ->wherePivot('type', MessageableTypes::USER);
    }

    /**
     * Return message without checking deleted data
     * @return HasMany
     */
    public function rawMessages(): HasMany
    {
        return $this->hasMany(Message::class, 'conversation_id');
    }

    /**
     * @return mixed
     */
    public function getClientId()
    {
        return $this->client_id;
    }

    /**
     * @param $value
     * @return bool
     */
    public function getIsAnonymousAttribute($value)
    {
        return $this->type === ConversationTypes::ANONYMOUS_MESSAGE;
    }

    /**
     * @return bool
     */
    public function getIsDisplayOwnerAttribute()
    {
        return !($this->type === ConversationTypes::ANONYMOUS_MESSAGE) || $this->owner_id === Context::user()->get()->getKey();
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function basicSegments()
    {
        return $this->morphedByMany(Segment::class, 'messageable')
            ->using(Messageable::class)
            ->withPivot(['last_read', 'last_available', 'is_anonymous', 'status'])
            ->wherePivot('type', MessageableTypes::SEGMENT);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function ageSegments()
    {
        return $this->morphedByMany(AgeSegment::class, 'messageable')
            ->using(Messageable::class)
            ->withPivot(['last_read', 'last_available', 'is_anonymous', 'status'])
            ->wherePivot('type', MessageableTypes::SEGMENT);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function employmentPhaseSegments()
    {
        return $this->morphedByMany(EmploymentPhaseSegment::class, 'messageable')
            ->using(Messageable::class)
            ->withPivot(['last_read', 'last_available', 'is_anonymous', 'status'])
            ->wherePivot('type', MessageableTypes::SEGMENT);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function managerSegments()
    {
        return $this->morphedByMany(User::class, 'messageable')
            ->using(Messageable::class)
            ->withPivot(['last_read', 'last_available', 'is_anonymous', 'status'])
            ->wherePivot('type', MessageableTypes::SEGMENT);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function anonymousConversationCategories(): BelongsToMany
    {
        return $this->belongsToMany(AnonymousConversationCategory::class,
            'whistleblowing_category',
            'conversation_id',
            'category_id');
    }

    /**
     * @param $query
     */
    public function scopeActive($query)
    {
        $query->where('conversations.status', QualeeStatus::ACTIVE);
    }

    /**
     * @param $query
     */
    public function scopeWhistleblowing($query)
    {
        $query->where('conversations.type', ConversationTypes::ANONYMOUS_MESSAGE);
    }

    public function scopeOfClient($query, $clientId)
    {
        $query->where('conversations.client_id', $clientId);
    }

    /**
     * @return mixed
     */
    public function getReceiversAttribute()
    {
        $receivers = $this->users->filter(function ($item) {
            $currentUserId = Context::user()->get()->id;
            if ($item->id === $currentUserId
                || $item->pivot->status !==  QualeeStatus::ACTIVE
                || ($this->isAnonymous && $this->owner_id === $item->id)) {
                return false;
            }
            return true;
        })->map(function ($item) {
            $item->last_read = $item->pivot->last_read;
            return $item;
        });
        return $receivers;
    }

    /**
     * @return mixed
     */
    public function getSegmentsAttribute()
    {
        return $this->basicSegments
            ->merge($this->ageSegments)
            ->merge($this->employmentPhaseSegments)
            ->merge($this->managerSegments)
            ->filter();
    }

    /**
     * @return HasMany
     */
    public function anonymousConversationStatusLogs(): HasMany
    {
        return $this->hasMany(AnonymousConversationStatusLog::class);
    }

    /**
     * @return BelongsTo
     */
    public function comment(): BelongsTo
    {
        return $this->belongsTo(QuestionnaireAnswer::class, 'comment_id', 'id');
    }

    /**
     * @return BelongsTo
     */
    public function normalComment(): BelongsTo
    {
        return $this->belongsTo(SurveyTextResult::class, 'comment_id', 'id');
    }
}
