<?php

namespace App\Models\Questionnaire;

use App\Enum\QuestionnaireEnum;
use App\Facades\Context;
use App\Models\Driver\Driver;
use App\Models\Message\Conversation as MessageConversation;
use App\Models\Questionnaire\Questionnaire;
use App\Models\Schedule\ScheduleSegments;
use App\Models\User\User;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;

/**
 * @method static questionnaire($questionnaires)
 */
class QuestionnaireAnswer extends Model
{
    /**
     * @var string
     */
    protected $table = 'questionnaire_answers';

    /**
     * @var string[]
     */
    protected $fillable = [
        'client_id','user_id', 'driver_id',
        'client_id', 'questionnaire_id', 'point',
        'comment', 'round_id', 'skip_question_id',
        'client_value_id', 'survey_id', 'analyzed',
        'is_sensitive', 'created_at', 'updated_at'
    ];

    /**
     * @param $query
     * @param $id
     * @return mixed
     */
    public function scopeUser($query, $id)
    {
        return $query->where('user_id', $id);
    }

    /**
     * @param $query
     * @param $id
     * @return mixed
     */
    public function scopeClient($query, $id)
    {
        return $query->where($this->table . '.client_id', $id);
    }

    /**
     * @param $query
     * @param $id
     * @return mixed
     */
    public function scopeRound($query, $id)
    {
        return $query->where('round_id', $id);
    }

    /**
     * @param $query
     * @param $id
     * @return mixed
     */
    public function scopeQuestionnaire($query, $id)
    {
        if (is_array($id)) {
            return $query->whereIn('questionnaire_id', $id);
        }
        return $query->where('questionnaire_id', $id);
    }

    /**
     * @param $year
     * @param $month
     * @param $day
     * @return $this
     */
    public function filterDate($year, $month, $day)
    {
        if (!empty($day)) {
            $this->whereDay('created_at', $day);
        }
        if (!empty($month)) {
            $this->whereMonth('created_at', $month);
        }
        if (!empty($year)) {
            $this->whereYear('created_at', $year);
        }
        return $this;
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function question()
    {
        return $this->belongsTo(Questionnaire::class, 'questionnaire_id', 'id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function driver()
    {
        return $this->belongsTo(Driver::class, 'driver_id', 'id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function surveySegment()
    {
        return $this->belongsTo(ScheduleSegments::class, 'survey_id', 'schedule_survey_id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function questionnaireSensitiveComments()
    {
        return $this->hasMany(QuestionnaireSensitiveComment::class, 'questionnaire_answer_id', 'id');
    }

    /**
     * @param $query
     * @param $data
     * @return mixed
     */
    public function scopeReport($query, $data)
    {
        $tableName = $this->table;

        $client = $data['client'];
        $questionValue = $data['questionValue'];
        $scoreType = !empty($data['scoreType']) ? $data['scoreType'] : '';

        $query = $query->whereNotNull($tableName.'.comment')
            ->where($tableName.'.client_id', $client->id)
            ->where($tableName.'.questionnaire_id', '!=', $questionValue->id);

        if (!empty($data['fromDate']) && !empty($data['toDate'])) {
            $fromDate = $data['fromDate'];
            $toDate = $data['toDate'];
            $query = $query->where(function($q) use ($fromDate, $toDate, $tableName) {
                return $q->whereDate($tableName.'.created_at', '>=', $fromDate)
                    ->whereDate($tableName.'.created_at', '<=', $toDate);
            });
        }

        if (!empty($scoreType)) {
            $minPoint = $scoreType == 'promoters'
                ? QuestionnaireEnum::PROMOTER['min']
                : ($scoreType == 'passives'
                    ? QuestionnaireEnum::PASSIVE['min']
                    : ($scoreType == 'detractors' ? QuestionnaireEnum::DETRACTOR['min'] : null));
            $maxPoint = $scoreType == 'promoters'
                ? QuestionnaireEnum::PROMOTER['max']
                : ($scoreType == 'passives'
                    ? QuestionnaireEnum::PASSIVE['max']
                    : ($scoreType == 'detractors' ? QuestionnaireEnum::DETRACTOR['max'] : null));
            if ($minPoint !== null && $maxPoint !== null) {
                $query = $query->where($tableName.'.point', '>=', $minPoint)
                    ->where($tableName.'.point', '<=', $maxPoint);
            }
        }
        return $query;
    }

    /**
     * @param $query
     * @param $data
     * @return mixed
     */
    public function scopeScoring($query, $data)
    {
        $client = $data['client'];
        $questionValue = $data['questionValue'];
        $scoreType = !empty($data['scoreType']) ? $data['scoreType'] : '';

        $query = $query->where('questionnaire_answers.client_id', $client->id)
            ->where('questionnaire_answers.questionnaire_id', '!=', $questionValue->id);

        if (!empty($data['fromDate']) && !empty($data['toDate'])) {
            $fromDate = $data['fromDate'];
            $toDate = $data['toDate'];
            $query = $query->whereNull('questionnaire_answers.skip_question_id')
                ->where(function($q) use ($fromDate, $toDate) {
                    return $q->whereDate('questionnaire_answers.created_at', '>=', $fromDate)
                        ->whereDate('questionnaire_answers.created_at', '<=', $toDate);
                });
        }

        if (!empty($scoreType)) {
            $minPoint = $scoreType == 'promoters'
                ? QuestionnaireEnum::PROMOTER['min']
                : ($scoreType == 'passives'
                    ? QuestionnaireEnum::PASSIVE['min']
                    : ($scoreType == 'detractors' ? QuestionnaireEnum::DETRACTOR['min'] : null));
            $maxPoint = $scoreType == 'promoters'
                ? QuestionnaireEnum::PROMOTER['max']
                : ($scoreType == 'passives'
                    ? QuestionnaireEnum::PASSIVE['max']
                    : ($scoreType == 'detractors' ? QuestionnaireEnum::DETRACTOR['max'] : null));
            if ($minPoint !== null && $maxPoint !== null) {
                $query = $query->where('questionnaire_answers.point', '>=', $minPoint)
                    ->where('questionnaire_answers.point', '<=', $maxPoint);
            }
        }
        return $query;
    }

    /**
     * @param $query
     * @param $questionnaireId
     * @param bool $condition
     * @return mixed
     */
    public function scopeIgnoreValue($query, $questionnaireId, $condition = false)
    {
        if ($condition) {
            return $query->where('questionnaire_answers.questionnaire_id', '!=', $questionnaireId);
        }
        return $query;
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasOne
     */
    public function questionnaireDiscussionAnswer()
    {
        return $this->hasOne(QuestionnaireDiscussionAnswer::class);
    }

    /**
     * @return BelongsTo
     */
    public function user(): BelongsTo
    {
        return $this->belongsTo(User::class);
    }

    /**
     * @return HasMany
     */
    public function conversations(): HasMany
    {
        return $this->hasMany(MessageConversation::class, "comment_id", "id");
    }
}
