<?php

namespace App\Models\Segment;

use App\Contracts\ClientProperty;
use App\Enum\SegmentTypes;
use App\Models\Client\Client;
use App\Models\Event\Event;
use App\Models\User\User;
use App\Traits\HasBusinessKey;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;

class Segment extends Model implements ClientProperty
{
    //
    use HasBusinessKey;
    protected $table = 'segments';

    protected $guarded = ['id'];
    
    protected $with = ['segmentType'];
    
    /**
     * @return BelongsTo
     */
    public function client(): BelongsTo
    {
        return $this->belongsTo(Client::class);
    }

    /**
     * @return BelongsTo
     */
    public function segmentType(): BelongsTo
    {
        return $this->belongsTo(SegmentType::class,'segment_type_id');
    }


    /**
     * @return BelongsToMany
     */
    public function users(): BelongsToMany
    {
        return $this->belongsToMany(User::class,'segment_user');
    }

    /**
     * @return BelongsToMany
     */
    public function activeUsers()
    {
        return $this->users()->active();
    }

    /**
     * @return BelongsToMany
     */
    public function events(): BelongsToMany
    {
        return $this->belongsToMany(Event::class);
    }

    public function scopeByClient($query, $clientId)
    {
        return $query->where('client_id', $clientId);
    }

    /**
     * Return related column name
     * @param $segment
     * @return string
     */
    public function getRelatedColumn()
    {
        $nameSegmentType = $this->segment_type_id;
        switch ($nameSegmentType) {
            case SegmentTypes::LOCATION:
                return 'location_id';
            case SegmentTypes::TEAM:
                return 'team_id';
            case SegmentTypes::DEPARTMENT:
                return 'department_id';
            case SegmentTypes::GENDER:
                return 'gender_id';
        }
    }

    public function getClientId()
    {
        return $this->client_id;
    }
    
    public function scopeAllSegmentType($query) {
        $query->whereHas('segmentType', function ($q) {
           $q->where('id', SegmentTypes::ALL);
        });
    }
    
    public function isAllSegmentType()
    {
        return $this->segmentType->id == SegmentTypes::ALL;
    }

    /**
     * Get included custom segment type users
     * 
     * @return mixed
     */
    public function includedCustomSegmentTypeUsers()
    {
        if ($this->isAllSegmentType()) {
            return $this->client->users()->active();
        } else {
            return $this->activeUsers();
        }
    }
}
