<?php

namespace App\Models\Survey;

use App\Enum\FilterTypes;
use App\Enum\TimeTypes;
use App\Helpers\FilterFluent;
use App\Helpers\ModelHelper;
use App\Helpers\TimeHelper;
use App\Models\Action\Action;
use App\Models\Action\ActionSequenceStep;
use App\Models\Journey\JourneyStep;
use App\Models\Segment\EmploymentPhaseSegment;
use App\Models\User\Employee;
use App\Services\Segment\SegmentType\GetSegmentTypeDetailEmploymentPhaseData;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use App\Enum\QuestionTypes as QuestionTypesEnum;
use Illuminate\Database\Eloquent\Relations\HasManyThrough;
use Illuminate\Support\Facades\DB;

/**
 * Class SurveyQuestion
 * @package App\Models\Survey
 */
class SurveyQuestion extends Model
{
    protected $guarded = ['id'];

    protected $touches = ['survey'];
    
    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function surveyOptions()
    {
        return $this->hasMany(SurveyOption::class);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function survey()
    {
        return $this->belongsTo(Survey::class);
    }

    /**
     * @param $query
     */
    public function scopeOrdered($query)
    {
        $query->orderBy('order', 'asc');
    }

    /**
     * @return HasManyThrough
     */
    public function textResult()
    {
        return $this->hasManyThrough(SurveyTextResult::class, SurveyResult::class, 'question_id', 'survey_result_id', 'id', 'id');
    }

    private function customFilters($query, $filters, $is_text = false)
    {
        $query->leftJoin('segments as s1', 'employees.department_id', 's1.id')
            ->leftJoin('segments as s2', 'employees.team_id', 's2.id')
            ->leftJoin('segments as s3', 'employees.location_id', 's3.id')
            ->leftJoin('segments as s4', 'employees.gender_id', 's4.id');

        /**
         * Filters
         */
        if (!empty($filters['journeys'])) {
            $journeyCode = array_map(function ($item) { return $item['value']; }, $filters['journeys']);
            $query->whereIn('journeys.code', $journeyCode);
        }
        if (!empty($filters['isUsed'])) {
            foreach ($filters['isUsed'] as $filter)
            {
                $value = $filter['value'];
                if ($value === 'Yes') {
                    $query->whereNotNull('journeys.code');
                } else {
                    $query->whereNull('journeys.code');
                }
            }
        }
        if (!empty($filters['sentiment']) && $is_text) {
            foreach ($filters['sentiment'] as $filter)
            {
                $query->where('sentiment', strtoupper($filter['value']));
            }

        }
        if (!empty($filters['score']) && $is_text) {
            foreach ($filters['score'] as $filter)
            {
                $type = $filter['type'];
                FilterFluent::applyBasicFilter($query, $type, 'positive_score', (float)$filter['value'] / 100);
            }

        }
        if (!empty($filters['department'])) {
            foreach ($filters['department'] as $filter)
            {
                $type = $filter['type'];
                FilterFluent::applyBasicFilter($query, $type, 's1.name', $filter['value']);
            }
        }
        if (!empty($filters['location'])) {
            foreach ($filters['location'] as $filter)
            {
                $type = $filter['type'];
                FilterFluent::applyBasicFilter($query, $type, 's3.name', $filter['value']);
            }
        }
        if (!empty($filters['gender'])) {
            foreach ($filters['gender'] as $filter)
            {
                $type = $filter['type'];
                FilterFluent::applyBasicFilter($query, $type, 's4.name', $filter['value']);
            }
        }
        if (!empty($filters['employmentPhrase']))
        {
            foreach ($filters['employmentPhrase'] as $filter)
            {
                $type = $filter['type'];
                $employmentPhraseQuery = EmploymentPhaseSegment::query();
                FilterFluent::applyBasicFilter($employmentPhraseQuery, $type, 'name', $filter['value']);
                $segments = $employmentPhraseQuery->get();

                foreach ($segments as $employmentPhrase)
                {
                    $field = $employmentPhrase['type'];
                    $column_name = 'employees.' . $field;

                    $typeMin = strtoupper($employmentPhrase['type_min']);
                    $typeMax = strtoupper($employmentPhrase['type_max']);
                    $query->whereRaw('TIMESTAMPDIFF(' . $typeMin . ', ' . $column_name . ', NOW()) >= ?', [$employmentPhrase['min']]);
                    $query->whereRaw('TIMESTAMPDIFF(' . $typeMax . ', ' . $column_name . ', NOW()) <= ?', [$employmentPhrase['max']]);
                }
            }
        }
        if (!empty($filters['race'])) {
            foreach ($filters['race'] as $filter)
            {
                $type = $filter['type'];
                FilterFluent::applyBasicFilter($query, $type, 'employees.race', $filter['value']);
            }
        }
        if (!empty($filters['team'])) {
            foreach ($filters['team'] as $filter)
            {
                $type = $filter['type'];
                FilterFluent::applyBasicFilter($query, $type, 's2.name', $filter['value']);
            }
        }

        return $query;
    }

    private function getTimeFromNow($type, $value)
    {
        $now = Carbon::now();
        switch ($type)
        {
            case TimeTypes::DAY:
                return $value < 0 ? $now->subDays($value) : $now->addDay($value);
            case TimeTypes::MONTH:
                return $value < 0 ? $now->subMonth($value) : $now->addMonth($value);
            case TimeTypes::YEAR:
                return $value < 0 ? $now->subYear($value) : $now->addYear($value);
            default:
                return $now;
        }
    }

    private function joinToJourneyAction($query)
    {
        $query = $query->join('action_requests', 'action_requests.id', 'action_request_submissions.action_request_id')
            ->join('users', 'users.id', 'action_request_submissions.user_id')
            ->join('employees', 'employees.user_id', 'users.id');

        $query = $query->leftJoin('journey_steps', function ($join) {
            $join->on('journey_steps.id', '=', 'action_requests.actionable_id')
                ->where('action_requests.actionable_type', JourneyStep::class);
        })->leftJoin('journeys', 'journeys.id', 'journey_steps.journey_id');

        $query = $query->leftJoin('actions', function ($join) {
            $join->on('actions.id', '=', 'action_requests.actionable_id')
                ->where('action_requests.actionable_type', Action::class);
        });

        $query = $query->leftJoin('action_sequence_steps', function ($join) {
            $join->on('action_sequence_steps.id', '=', 'action_requests.actionable_id')
                ->where('action_requests.actionable_type', ActionSequenceStep::class);
        })->leftJoin('action_sequences', 'action_sequences.id', 'action_sequence_steps.action_sequence_id');
        return $query;
    }

    public function textResultByUser($from = '', $to = '', $filters = []) {
        $query = $this->textResult()->join('action_request_submissions', 'action_request_submissions.id', 'survey_results.submission_id');
        if (!empty($from)) {
            $query->whereDate('action_request_submissions.created_at', '>=', $from);
        }
        if (!empty($to)) {
            $query->whereDate('action_request_submissions.created_at', '<=', $to);
        }
        $query = $this->joinToJourneyAction($query);
        $query = $this->customFilters($query, $filters, true);

        return $query->whereNotNull('text')
            ->where('text', '!=', '')
            ->select(DB::raw("users.first_name, users.surname, journeys.name as journey_name, action_requests.actionable_type, action_requests.contentable_type, 
            action_sequences.name as action_sequences_name, text, mixed_score, positive_score, neutral_score, negative_score, sentiment, action_request_submissions.created_at"))
            ->groupBy('action_request_submissions.id')
            ->orderBy('action_request_submissions.created_at');
    }

    public function textResultCount($from = '', $to = '', $filters = [])
    {
        $query = $this->textResult()->join('action_request_submissions', 'action_request_submissions.id', 'survey_results.submission_id');
        if (!empty($from)) {
            $query->whereDate('action_request_submissions.created_at', '>=', $from);
        }
        if (!empty($to)) {
            $query->whereDate('action_request_submissions.created_at', '<=', $to);
        }
        $query = $this->joinToJourneyAction($query);
        $query = $this->customFilters($query, $filters, true);
        return $query->select(DB::raw("text, sentiment, COUNT(text) as total"))->groupBy('text')->orderBy('text');
    }

    /**
     * @return HasManyThrough
     */
    public function ratingResult()
    {
        return $this->hasManyThrough(SurveyRatingResult::class, SurveyResult::class, 'question_id', 'survey_result_id', 'id', 'id');
    }

    public function ratingResultByUser($from = '', $to = '', $filters = []) {
        $query = $this->ratingResult()->join('action_request_submissions', 'action_request_submissions.id', 'survey_results.submission_id');
        if (!empty($from)) {
            $query->whereDate('action_request_submissions.created_at', '>=', $from);
        }
        if (!empty($to)) {
            $query->whereDate('action_request_submissions.created_at', '<=', $to);
        }
        $query = $this->joinToJourneyAction($query);
        $query = $this->customFilters($query, $filters);

        return $query->select(DB::raw("users.first_name, users.surname, journeys.name as journey_name, action_requests.actionable_type, action_requests.contentable_type, 
        action_sequences.name as action_sequences_name, rating, action_request_submissions.created_at"))
            ->groupBy('action_request_submissions.id')
            ->orderBy('action_request_submissions.created_at');
    }

    public function ratingResultCount($from = '', $to = '', $filters = [])
    {
        $query = $this->ratingResult()->join('action_request_submissions', 'action_request_submissions.id', 'survey_results.submission_id');
        if (!empty($from)) {
            $query->whereDate('action_request_submissions.created_at', '>=', $from);
        }
        if (!empty($to)) {
            $query->whereDate('action_request_submissions.created_at', '<=', $to);
        }
        $query = $this->joinToJourneyAction($query);
        $query = $this->customFilters($query, $filters);
        return $query->select(DB::raw("rating, COUNT(rating) as total"))->groupBy('rating')->orderBy('rating');
    }

    public function ratingResultSegmentsTypeCount($from = '', $to = '', $filters = [])
    {
        $query = $this->ratingResult()->select(['segments.name', 'rating', DB::raw("COUNT(rating) as total")])
            ->join('action_request_submissions', 'action_request_submissions.id', 'survey_results.submission_id')
            ->join('segment_user', 'action_request_submissions.user_id', 'segment_user.user_id')
            ->join('segments', 'segments.id', 'segment_user.segment_id');
        if (!empty($from)) {
            $query->whereDate('action_request_submissions.created_at', '>=', $from);
        }
        if (!empty($to)) {
            $query->whereDate('action_request_submissions.created_at', '<=', $to);
        }
        $query = $this->joinToJourneyAction($query);
        $query = $this->customFilters($query, $filters);
        return $query->groupBy(['rating', 'segments.id'])->orderBy('rating');
    }

    /**
     * @return HasManyThrough
     */
    public function choiceResult()
    {
        return $this->hasManyThrough(SurveyChoiceResult::class, SurveyResult::class, 'question_id', 'survey_result_id', 'id', 'id');
    }

    public function choiceResultByUser($from = '', $to = '', $filters = []) {
        $query = $this->choiceResult()->join('action_request_submissions', 'action_request_submissions.id', 'survey_results.submission_id')
            ->join('survey_options', 'survey_options.id', 'survey_choice_results.option_id');
        if (!empty($from)) {
            $query->whereDate('action_request_submissions.created_at', '>=', $from);
        }
        if (!empty($to)) {
            $query->whereDate('action_request_submissions.created_at', '<=', $to);
        }
        $query = $this->joinToJourneyAction($query);
        $query = $this->customFilters($query, $filters);
        return $query->select(DB::raw("users.first_name, users.surname, journeys.name as journey_name, action_requests.actionable_type, action_requests.contentable_type, 
        action_sequences.name as action_sequences_name, option_id, survey_options.content, action_request_submissions.created_at"))
            ->groupBy('action_request_submissions.id')
            ->orderBy('action_request_submissions.created_at');
    }

    public function choiceResultCount($from = '', $to = '', $filters = [])
    {
        $query = $this->choiceResult()->join('action_request_submissions', 'action_request_submissions.id', 'survey_results.submission_id');
        if (!empty($from)) {
            $query->whereDate('action_request_submissions.created_at', '>=', $from);
        }
        if (!empty($to)) {
            $query->whereDate('action_request_submissions.created_at', '<=', $to);
        }
        $query = $this->joinToJourneyAction($query);
        $query = $this->customFilters($query, $filters);
        return $query->select(DB::raw("option_id, COUNT(option_id) as total"))->groupBy('option_id')->orderBy('option_id');
    }

    public function choiceResultSegmentsTypeCount($from = '', $to = '', $filters = [])
    {
        $query = $this->choiceResult()->select(['segments.name', 'option_id', DB::raw("COUNT(option_id) as total")])
            ->join('action_request_submissions', 'action_request_submissions.id', 'survey_results.submission_id')
            ->join('segment_user', 'action_request_submissions.user_id', 'segment_user.user_id')
            ->join('segments', 'segments.id', 'segment_user.segment_id');
        if (!empty($from)) {
            $query->whereDate('action_request_submissions.created_at', '>=', $from);
        }
        if (!empty($to)) {
            $query->whereDate('action_request_submissions.created_at', '<=', $to);
        }
        $query = $this->joinToJourneyAction($query);

        $query = $this->customFilters($query, $filters);
        return $query->groupBy(['option_id', 'segments.id'])->orderBy('option_id')->orderBy('option_id');
    }
}
