<?php

namespace App\Models\User;

use App\Enum\EmployeeStatus;
use App\Enum\Roles;
use App\Models\Action\Action;
use App\Models\Action\ActionSequence;
use App\Models\ActionRequest\ActionRequestSubmission;
use App\Models\AlertMessage\AlertMessage;
use App\Models\AppRating\AppRating;
use App\Models\Attachment;
use App\Models\Client\Client;
use App\Models\Form\FormDocument;
use App\Models\Journey\Journey;
use App\Models\Language\Language;
use App\Models\Message\Conversation;
use App\Models\Message\Message;
use App\Models\Message\Messageable;
use App\Models\Segment\Segment;
use App\Notifications\CustomResetPassword;
use App\Notifications\CustomVerifyEmail;
use App\Traits\HasBusinessKey;
use Carbon\Carbon;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use App\Models\Event\Event;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Database\Eloquent\Relations\MorphToMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Passport\HasApiTokens;
use Spatie\Permission\Traits\HasRoles;
use App\Helpers\FileHelper;
/**
 * Class User
 * @property mixed id
 * @package App\Models\User
 * @method static active()
 */
class User extends Authenticatable implements MustVerifyEmail
{
    use Notifiable, HasApiTokens, SoftDeletes, HasRoles, HasBusinessKey;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name', 'email', 'email_change', 'password', 'status', 'created_at', 'first_name', 'surname', 'linkedin_url',
        'is_demo', 'profile_pic', 'external_avatar_url'
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token', 'google2fa_secret',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    protected $dates = [
        'last_login',
        'password_updated_at'
    ];

    /**
     * @return BelongsToMany
     */
    public function segments(): BelongsToMany
    {
        return $this->belongsToMany(Segment::class, 'segment_user');
    }

    /**
     * @return BelongsToMany
     */
    public function userEvent(): BelongsToMany
    {
        return $this->belongsToMany(Event::class);
    }

    /**
     * @return BelongsToMany
     */
    public function userPermissions(): BelongsToMany
    {
        return $this->belongsToMany(UserPermission::class);
    }

    /**
     * TODO fix this to collection through event_user and event_segments
     * @return HasMany
     */
    public function getEventsCollection()
    {
        $events = $this->userEvent;
        $eventBySegment = self::with('segments.events')
            ->whereHas('segments', function($query) {
                $query->has('events');
            })
            ->where('id',$this->id)->first();
        foreach($eventBySegment->segments as $segments)
        {
            $events = $events->concat($segments->events);
        }
        return $events->unique();
    }

    /**
     * @return BelongsToMany
     */
    public function client()
    {
        return $this->belongsToMany(Client::class, Employee::class);
    }

    /**
     * @return HasOne
     */
    public function employee(): HasOne
    {
        return $this->hasOne(Employee::class);
    }

    /**
     * @return mixed
     */
    public function getClient()
    {
        return $this->client->first();
    }

    /**
     * @return mixed
     */
    public function getFriendlyName() {
        return $this->first_name;
    }

    public function isSuperAdmin (): bool
    {
        return $this->hasRole(Roles::SUPER_ADMIN);
    }

    public function isClientAdmin(): bool
    {
        return $this->hasRole(Roles::ADMIN);
    }

    /**
     * @param string $token
     */
    public function sendPasswordResetNotification($token)
    {
        $this->notify(new CustomResetPassword($token));
    }

    /**
     * Send the email verification notification.
     *
     * @return void
     */
    public function sendEmailVerificationNotification()
    {
        $this->notify(new CustomVerifyEmail);
    }

    /**
     * Send the email verification notification.
     *
     * @return void
     */
    public function sendEmailVerificationUpdateEmail()
    {
        $this->notify(new CustomVerifyEmail);
    }

    public function scopeActive($query)
    {
        return $query->join('employees', 'users.id', '=', 'employees.user_id')->where('employees.status', EmployeeStatus::ACTIVE);
        /*return $query->whereHas('employee', function($q) {
            $q->where('status', EmployeeStatus::ACTIVE);
        });*/
    }

    public function scopeByTimezone($query, $timezones)
    {
//        return $query->join('employees', 'users.id', '=', 'employees.user_id')->whereIn('employees.timezone', $timezones);
        return $query->whereHas('employee',function($query) use($timezones){
            $query->whereIn('timezone', $timezones);
        });
    }

    public function scopeManager($query)
    {
        return $query->whereIn('users.id',Employee::groupBy('manager_id')->select('manager_id')->get()->pluck('manager_id'));
    }

    /**
     * @return BelongsToMany
     */
    public function journeys(): BelongsToMany
    {
        return $this->belongsToMany(Journey::class);
    }

    /**
     * @return BelongsToMany
     */
    public function actions(): BelongsToMany
    {
        return $this->belongsToMany(Action::class);
    }

    /**
     * @return BelongsToMany
     */
    public function availableActions(): BelongsToMany
    {
        return $this->belongsToMany(Action::class)->where('send_on', '<=', Carbon::now());
    }

    /**
     * @return BelongsToMany
     */
    public function actionSequences(): BelongsToMany
    {
        return $this->belongsToMany(ActionSequence::class);
    }
    
    public function getFullNameAttribute()
    {
        return $this->surname . ', ' . $this->first_name;
    }

    /**
     * @return BelongsTo
     */
    public function avatar(): BelongsTo
    {
        return $this->belongsTo(Attachment::class, 'profile_pic');
    }

    /**
     * @return BelongsTo
     */
    public function language(): BelongsTo
    {
        return $this->belongsTo(Language::class,'preferred_language');
    }

    /**
     * Get url from s3
     * @return string
     */
    public function getAvatarUrlAttribute(): string
    {
        return $this->avatar ? FileHelper::getPublicUrl($this->avatar->path) : "";
    }

    public function getAvatarUrlMobileAttribute(): string
    {
        return $this->avatar ? FileHelper::getSignedUrl($this->avatar->path) : "";
    }

    /**
     * @return bool
     */
    public function markAsWelcomeMailSent()
    {
        return $this->forceFill([
            'is_welcome_mail_sent' => true,
        ])->save();
    }

    /**
     * @return BelongsToMany
     */
    public function readAlertMessages(): BelongsToMany
    {
        return $this->belongsToMany(AlertMessage::class, 'alert_message_read_user');
    }

    /**
     * @return HasMany
     */
    public function actionRequestSubmissions()
    {
        return $this->hasMany(ActionRequestSubmission::class);
    }

    /**
     * @return HasMany
     */
    public function appRatings()
    {
        return $this->hasMany(AppRating::class);
    }

    /**
     * @return hasMany
     */
    public function conversations(): MorphToMany
    {
        return $this->morphToMany(Conversation::class, 'messageable')->using(Messageable::class);
    }

    /**
     * @return HasMany
     */
    public function messages(): HasMany
    {
        return $this->hasMany(Message::class, 'sender_id');
    }

    /**
     * @return HasMany
     */
    public function documents(): HasMany
    {
        return $this->hasMany(FormDocument::class);
    }

    /**
     * Apply new email change
     * 
     * @return void
     */
    public function applyEmailChange()
    {
        if (!empty($this->email_change)) {
            $this->email = $this->email_change;
            $this->email_change = null;
            $this->save();
        }
    }

    /**
     * @param int $timestamp
     * @return bool
     */
    public function mark2FAAsVerified(int $timestamp)
    {
        return $this->forceFill([
            'google2fa_timestamp' => $timestamp,
        ])->save();
    }

    /**
     * @return bool
     */
    public function isExactFrontUser()
    {
        $roleNames = $this->getRoleNames();

        return ($roleNames->count() === 1 && $roleNames->contains(Roles::FRONT_USER));
    }
}
