<?php

namespace App\Providers;

use App\AppContext\ContextManager;
use App\AppContext\ClientContext;
use App\AppContext\RequestContext;
use App\AppContext\UserContext;
use App\Exceptions\ApiExceptionHandler;
use App\Guards\AuthenticatedConsole;
use App\Guards\AuthorizedClient;
use App\Guards\ClientIsActive;
use App\Guards\ClientSubscriptionIsExpired;
use App\Guards\GuardNames;
use App\Guards\UserEmailVerified;
use App\Guards\UserIsActive;
use App\Guards\UserIsBelongToClient;
use App\Models\Client\Client;
use App\Models\Message\Message;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Observers\ClientObserver;
use App\Observers\MessageObserver;
use App\Observers\QuestionnaireAnswerObserver;
use App\Guards\UserIsClientAdminOrSuperAdmin;
use App\Guards\UserIsSuperAdmin;
use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\ServiceProvider;
use Illuminate\Support\Facades\Validator;
use DateTime;

class AppServiceProvider extends ServiceProvider
{
    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        $this->registerBusinessKey();
        $this->registerApiExceptionHandler();
        $this->registerContextHandler();
        $this->registerGuards();
    }

    /**
     * Register business key for migration
     */
    public function registerBusinessKey()
    {
        $businessKey = config('app.bussiness_key_name') ?? 'code';
        Blueprint::macro('hasBusinessKey', function () use ($businessKey) {
            if (!Schema::hasColumn($this->table, $businessKey)) {
                return $this->string($businessKey, 31);
            }
        });
        Blueprint::macro('dropBusinessKey', function () use ($businessKey) {
            if (Schema::hasColumn($this->table, $businessKey)) {
                return $this->dropColumn($businessKey);
            }
        });
    }

    /**
     * Register the exception handler - extends the Dingo one
     *
     * @return void
     */
    protected function registerApiExceptionHandler()
    {
        $this->app->singleton('api.exception', function ($app) {
            return new ApiExceptionHandler(
                $app['Illuminate\Contracts\Debug\ExceptionHandler'],
                Config('api.errorFormat'),
                Config('api.debug')
            );
        });
    }

    protected function registerContextHandler()
    {
        $this->app->singleton(ClientContext::class, function () {
            return new ClientContext;
        });

        $this->app->singleton(UserContext::class, function () {
            return new UserContext;
        });

        $this->app->singleton(RequestContext::class, function () {
            return new RequestContext;
        });

        $this->app->singleton('context', function () {
            return new ContextManager([
                'client' => ClientContext::class,
                'user' => UserContext::class,
                'request' => RequestContext::class
            ]);
        });
    }

    protected function registerGuards()
    {
        $this->app->bind(GuardNames::ClientIsActive, ClientIsActive::class);
        $this->app->bind(GuardNames::ClientSubscriptionIsExpired, ClientSubscriptionIsExpired::class);
        $this->app->bind(GuardNames::UserIsActive, UserIsActive::class);
        $this->app->bind(GuardNames::UserEmailVerified, UserEmailVerified::class);
        $this->app->bind(GuardNames::UserIsBelongToClient, UserIsBelongToClient::class);
        $this->app->bind(GuardNames::AuthorizedClient, AuthorizedClient::class);
        $this->app->bind(GuardNames::UserIsClientAdminOrSuperAdmin, UserIsClientAdminOrSuperAdmin::class);
        $this->app->bind(GuardNames::UserIsSuperAdmin, UserIsSuperAdmin::class);
        $this->app->bind(GuardNames::AuthenticatedConsole, AuthenticatedConsole::class);
    }

    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        Validator::extend('date_utc', function ($attribute, $value, $parameters, $validator) {
            return !!DateTime::createFromFormat(DateTime::ISO8601, $value);
        });

        Validator::replacer('date_utc', function ($message, $attribute, $rule, $parameters) {
            return  __('validation.date_utc', ['attribute' => $attribute]);
        });

        // Register Questionnaire answer observer
        QuestionnaireAnswer::observe(QuestionnaireAnswerObserver::class);

        Message::observe(MessageObserver::class);

        Client::observe(ClientObserver::class);
    }
}
