<?php

namespace App\Services\Action\ActionSequence;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\ActionTypes;
use App\Enum\EmailNotifications;
use App\Enum\ErrorCode;
use App\Enum\NotificationTypes;
use App\Enum\Roles;
use App\Events\ActionSequenceUpdated;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ModelHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\UserHelper;
use App\Jobs\SendNotification;
use App\Jobs\SendNotificationConsole;
use App\Jobs\SendNotificationEmail;
use App\Models\Action\ActionSequence;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\UserAccess\GetAccessByRole;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;

class AddEmployees extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($args)
    {
        return [
            'code' => 'required',
            'users' => 'array',
            'segments' => 'array',
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Add employees to journey
     * 
     * @param array $data
     * @return mixed
     */
    public function performAction(array $data)
    {
        $actionSequence = ActionSequence::findByBusinessKey($data['code']);
        PermissionHelper::hasAccessOn($actionSequence);

        // Prevent add employee into action sequence which has no steps
        if ($actionSequence->steps()->count() === 0) {
            throw new InvalidRequestException(
                'Cannot add employee into action sequence which has no steps',
                ErrorCode::EMPTY_ACTION_SEQUENCE_STEPS
            );
        }

        // Add users
        $userIds = [];

        if (isset($data['users']) && count($data['users']) > 0) {
            foreach ($data['users'] as $userCode) {
                if (!empty($userCode)) {
                    $user = User::findByBusinessKey($userCode);
                    PermissionHelper::hasAccessOn($user->employee);
                    $userIds[] = $user->getKey();
                }
            }
        }

        // Add users of segments
        if (isset($data['segments'])) {
            $segmentUserIds = UserHelper::getUserIdBySegment($data['segments']);
            $userIds = array_merge($userIds, $segmentUserIds);
        }
        $userIds = array_unique($userIds);

        // Remove existed users
        $actionSequenceUserIds = $actionSequence->users()->pluck('id')->toArray();
        foreach ($userIds as $key => $userId) {
            if (in_array($userId, $actionSequenceUserIds)) {
                unset($userIds[$key]);
            }
        }

        $actionSequenceUserData = [];
        foreach ($userIds as $key => $value) {
            $actionSequenceUserData[$value]['created_at'] = Carbon::now();
        }

        $actionSequence->users()->attach($actionSequenceUserData);

        event(new ActionSequenceUpdated($actionSequence));

        if (!empty($userIds)) {
            // Push notification to employees
            $arguments = array(
                'sender' => Auth::user()->id,
                "userIds" => ModelHelper::convertIdToBusinessKey(User::class, $userIds),
                "headings" => __("New action assigned"),
                "contents" => __("Tap to open action"),
                "data" => [
                    "type" => NotificationTypes::ACTION_ADDED,
                    "id" => $actionSequence->id,
                    "action_type" => ActionTypes::ACTION_SEQUENCE
                ],
                'send_after' => null
            );
            SendNotification::dispatch($arguments);
        }

        // Send Invite Notifications to employees
        $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
        $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

        $users = User::whereIntegerInRaw('id', $userIds)->get();
        if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_CONSOLE_EMPLOYEE_ACTION]) {
            foreach ($users as $user) {
                SendNotificationConsole::dispatch($user, $user, 'Action Sequence Invitation');
            }
        }
        if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_ACTION]) {
            foreach ($users as $user) {
                SendNotificationEmail::dispatch($user, EmailNotifications::ACTION_SEQUENCE_INVITATION, ['sequence' => $actionSequence->name]);
            }
        }

        return $users;
    }
}