<?php

namespace App\Services\Action\ActionSequence;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\DateHelper;
use App\Helpers\ZipHelper;
use App\Models\Action\ActionSequence;
use App\Models\ContentFile\ContentFile;
use App\Models\ContentPhoto\ContentPhoto;
use App\Models\Form\FormTemplate;
use App\Models\IDVerification\IDVerification;
use App\Models\Quiz\Quiz;
use App\Models\Survey\Survey;
use App\Models\User\User;
use App\Services\ActionRequest\Submission\Export\SaveFormDocument;
use App\Services\ActionRequest\Submission\Export\SaveHeadshotPhotoImage;
use App\Services\ActionRequest\Submission\Export\SaveIDVerificationImage;
use App\Services\ActionRequest\Submission\Export\SaveQuizResultCSV;
use App\Services\ActionRequest\Submission\Export\SaveSurveyResultCSV;
use App\Services\ActionRequest\Submission\Export\SaveUploadedContent;
use App\Services\BaseService;
use Illuminate\Support\Facades\Storage;
use Symfony\Component\HttpFoundation\BinaryFileResponse;

class DownloadEmployeeSubmission extends BaseService implements Validatable, Performable
{
    const FOLDER_NAME_FORMAT = '[:employeeName]_[:actionName]';
    const HEADSHOT_PHOTO_IMAGE_NAME = 'headshot_photo';
    const ID_VERIFICATION_FRONT_IMAGE_NAME = 'front_ID_image';
    const ID_VERIFICATION_BACK_IMAGE_NAME = 'back_ID_image';
    const IMAGE_EXTENSION = '.png';
    
    private $actionSequence;
    private $employee;
    private $client;
    
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($args)
    {
        return [
            'code' => 'required', 
            'employee_code' => 'required'
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Download employee submission
     * 
     * @param array $data
     * @return bool|BinaryFileResponse
     */
    public function performAction(array $data)
    {
        $submissions = app(GetEmployeeSubmission::class)->execute($data);
        
        if (!empty($submissions)) {
            $this->actionSequence = ActionSequence::findByBusinessKey($data['code']);
            $this->employee = User::findByBusinessKey($data['employee_code']);
            $this->client = Context::client()->get();
            
            $folderName = str_replace(
                [':employeeName', ':actionName'],
                [$this->employee->full_name, $this->actionSequence->name],
                self::FOLDER_NAME_FORMAT);
            $folderName = $this->makeFolderName($folderName);
            Storage::disk('local')->makeDirectory($folderName);
            
            foreach ($submissions as $index => $submission) {
                $step = $submission['step'];
                $submission =  $submission['submission'];
                // Only save file when user submitted results
                if (!empty($submission)) {
                    $stepActionRequest = $step->actionRequest;

                    $stepFolderName = $folderName . '/' . $this->makeFolderName($step->name);
                    Storage::disk('local')->makeDirectory($stepFolderName);
                    
                    $folderPath = config('filesystems.disks.local.root') . '/' . $stepFolderName;
                    
                    switch ($stepActionRequest->contentable_type) {
                        case Quiz::class:
                            $quiz = $stepActionRequest->contentable;
                            $quizResults = $submission->quizResults;
                    
                            $data = [
                                'date' => DateHelper::formatYearMonthDay($submission->created_at),
                                'client_name' => $this->client->name,
                                'action_name' => $this->actionSequence->name,
                                'employee_name' => $this->employee->full_name,
                                'employee_city' => $this->employee->city,
                                'employee_job_title' => $this->employee->employee->job_title,
                            ];

                            $header = ['Date', 'Company name', 'Action name', 'Employee name', 'Employee city',
                                'Employee role title'];

                            app(SaveQuizResultCSV::class)->execute(
                                $quiz, $quizResults, $data, $header, $folderPath
                            );
                            
                            break;
                        case Survey::class:
                            $survey = $stepActionRequest->contentable;
                            $surveyResults = $submission->surveyResults;
                        
                            $data = [
                                'date' => DateHelper::formatYearMonthDay($submission->created_at),
                                'client_name' => $this->client->name,
                                'action_name' => $this->actionSequence->name,
                                'employee_name' => $this->employee->full_name,
                                'employee_city' => $this->employee->city,
                                'employee_job_title' => $this->employee->employee->job_title,
                            ];

                            $header = ['Date', 'Company name', 'Action name', 'Employee name', 'Employee city',
                                'Employee role title'];

                            app(SaveSurveyResultCSV::class)->execute(
                                $survey, $surveyResults, $data, $header, $folderPath
                            );
                            
                            break;
                        case IDVerification::class:
                            $idVerificationResults = $submission->idVerificationResults;
                            $i = 0;
                            foreach ($idVerificationResults as $result) {
                                // TODO: Add result csv for verifai data
                                $i++;
                                $frontImageName = self::ID_VERIFICATION_FRONT_IMAGE_NAME . $i . self::IMAGE_EXTENSION;
                                $backImageName = self::ID_VERIFICATION_BACK_IMAGE_NAME . $i . self::IMAGE_EXTENSION;
                                app(SaveIDVerificationImage::class)->execute(
                                    $result, $stepFolderName, $frontImageName, $backImageName
                                );
                            }
                            break;
                        case ContentPhoto::class:
                            $photoResults = $submission->contentPhotoResults;
                            $i = 0;
                            foreach ($photoResults as $result) {
                                $i++;
                                $headshotPhotoName = self::HEADSHOT_PHOTO_IMAGE_NAME . $i . self::IMAGE_EXTENSION;
                                app(SaveHeadshotPhotoImage::class)->execute(
                                    $result, $stepFolderName, $headshotPhotoName
                                );
                            }
                            break;
                        case FormTemplate::class:
                            $formResults = $submission->formResults;
                            app(SaveFormDocument::class)->execute($formResults, $this->client, $stepFolderName);

                            break;
                        case ContentFile::class:
                            $contentResult = $submission->contentFileResults()->first();
                            if ($contentResult) {
                                app(SaveUploadedContent::class)->execute($contentResult, $stepFolderName);
                            }

                            break;
                    }
                }
            }
            
            // Zip and download saved folder
            $folderToZip = config('filesystems.disks.local.root') . '/' . $folderName;
            $zippedFile = ZipHelper::zipFolder($folderToZip);
            // Delete saved folder 
            Storage::disk('local')->deleteDirectory($folderName);

            if ($zippedFile) {
                return response()->download($zippedFile)->deleteFileAfterSend();
            }
        }
        
        return false;
    }

    /**
     * Make valid folder name
     * 
     * @param $name
     * @return string
     */
    public function makeFolderName($name)
    {
        $folderName = preg_replace('/\s+/', '_', $name);
        $folderName = preg_replace('/[\\/:*?"<>|]/', '_', $folderName);
        
        return $folderName;
    }
}
