<?php

namespace App\Services\Action\ActionSequence;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\ActionRequestTypes;
use App\Enum\ErrorCode;
use App\Events\ActionSequenceUpdated;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\PermissionHelper;
use App\Models\Action\ActionSequence;
use App\Services\ActionRequest\CheckPlanAvailableForStepType;
use App\Services\ActionRequest\CreateActionRequest;
use App\Services\ActionRequest\UpdateActionRequest;
use App\Services\BaseService;
use App\Services\Tool\Form\CheckValidTemplate;
use Illuminate\Support\Arr;
use Illuminate\Validation\Rule;


class SaveActionSequenceStep extends BaseService implements Validatable, Performable
{
    private $data;

    /**
     * Get the validation rules that apply to the service.
     *
     * @param $args
     * @return array
     */
    public function getRules($args)
    {
        $rules = [
            'code' => 'required',
            'steps' => 'array'
        ];
        
        if (!empty($args['steps'])) {
            foreach ($args['steps'] as $key => $value) {
                $rules["steps.$key.type"] = ['required', Rule::in(ActionRequestTypes::ACTION_TYPES)];
                $rules["steps.$key.name"] = ['required', 'string', 'max:255'];
                $rules["steps.$key.order"] = ['integer'];

                if (isset($value['type'])) {
                    if (in_array($value['type'], ActionRequestTypes::CONFIGURABLE_TYPES)) {
                        $rules["steps.$key.header_image_id"] = ['nullable', 'integer'];
                        $rules["steps.$key.message"] = ['required', 'string'];
                    }
                    
                    if (in_array($value['type'], ActionRequestTypes::CONTENTABLE_TYPES)) {
                        $rules["steps.$key.content_id"] = ['required'];

                        if ($value['type'] == ActionRequestTypes::ID_VERIFICATION) {
                            array_push($rules["steps.$key.content_id"], Rule::in([
                                ActionRequestTypes::ID_VERIFICATION_PASSPORT,
                                ActionRequestTypes::ID_VERIFICATION_DRIVER_LICENSE,
                                ActionRequestTypes::ID_VERIFICATION_NATIONAL_ID
                            ]));
                        }
                    }
                }
            }
        }
        
        return $rules;
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Save action sequence step
     * 
     * @param array $data
     * @return ActionSequence
     */
    public function performAction(array $data) : ActionSequence
    {
        $actionSequence = ActionSequence::findByBusinessKey($data['code']);
        PermissionHelper::hasAccessOn($actionSequence);

        $inputQuestions = $data['steps'] ?? [];

        // Prevent update action sequence which has employee with empty steps
        if ($actionSequence->users()->count() > 0 && empty($inputQuestions)) {
            throw new InvalidRequestException(
                'Cannot update action sequence without steps.',
                ErrorCode::EMPTY_ACTION_SEQUENCE_STEPS
            );
        }

        // Prevent editing action sequence when having submissions from employees
        $steps = $actionSequence->steps->load('actionRequest.submissions');
        foreach ($steps as $step) {
            if ($step->hasSubmission()) {
                throw new InvalidRequestException(
                    "This action sequence has submissions from employees. Cannot Edit anymore.",
                    ErrorCode::ACTION_SEQUENCE_HAS_SUBMISSION
                );
            }
        }
        
        // Delete disappeared steps of given action sequence
        $stepIds = Arr::pluck($inputQuestions, 'id');
        foreach ($actionSequence->steps as $step) {
            if (!in_array($step->id, $stepIds)) {
                $step->actionRequest()->delete();
                $step->delete();
            }
        }
        $inputQuestions = Arr::sort($inputQuestions,function($value,$index){
            return $value['order'] ?? $index;
        });
        $inputQuestions = array_values($inputQuestions);
        foreach ($inputQuestions as $index => $step) {
            $this->checkPlanAvailableForStepType($step['type']);
            $stepData = [
                'name' => $step['name'],
                'message' => in_array($step['type'], ActionRequestTypes::CONFIGURABLE_TYPES) ?
                    $step['message'] : null,
                'header_image_id' => $step['header_image_id'] ?? null,
                'order' => $index
            ];

            $contentableModel = ActionRequestHelper::getContentableModel($step['type'], $step['content_id'] ?? null);

            // Prevent adding contents which is not published
            if (in_array($step['type'], ActionRequestTypes::PUBLISHABLE_TYPES) &&
                !$contentableModel->is_published) {
                throw new InvalidRequestException("Invalid publishable content", ErrorCode::INVALID_PUBLISHABLE_CONTENT, ['step' => $stepData['name']]);
            }

            if ($step['type'] == ActionRequestTypes::FORM) {
                app(CheckValidTemplate::class)->execute([
                    'template_id' => $contentableModel->template_id,
                    'replacements' => ['step' => $stepData['name']]
                ]);
            }

            // Save action sequence step
            $createdStep = $actionSequence->steps()->updateOrCreate(
                ['id' => $step['id'] ?? null],
                $stepData
            );

            $actionRequest = $createdStep->actionRequest;
            if (in_array($createdStep->id, $stepIds) && !empty($actionRequest)) {
                // Update action request
                app(UpdateActionRequest::class)->execute([], $createdStep, $contentableModel, $actionRequest);
            } else {
                // Create action request
                app(CreateActionRequest::class)->execute([], $createdStep, $contentableModel);
            }
        }

        event(new ActionSequenceUpdated($actionSequence));

        return $actionSequence->refresh();
    }

    /**
     * @param $type
     */
    public function checkPlanAvailableForStepType($type)
    {
        app(CheckPlanAvailableForStepType::class)->execute($type);
    }
}
