<?php

namespace App\Services\Action;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\ActionRequestTypes;
use App\Enum\EmailNotifications;
use App\Enum\ErrorCode;
use App\Enum\NotificationTypes;
use App\Enum\Roles;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\DateHelper;
use App\Helpers\ObjectHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\UserHelper;
use App\Jobs\CreateSingleActionDocument;
use App\Jobs\SendNotification;
use App\Jobs\SendNotificationConsole;
use App\Jobs\SendNotificationEmail;
use App\Models\Action\Action;
use App\Models\Library\LibraryItem;
use App\Models\Library\Link;
use App\Models\User\User;
use App\Services\ActionRequest\CheckPlanAvailableForStepType;
use App\Services\ActionRequest\CreateActionRequest;
use App\Services\BaseService;
use App\Services\Tool\Form\CheckValidTemplate;
use App\Services\UserAccess\GetAccessByRole;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\Rule;

class CreateAction extends BaseService implements Validatable, Performable
{
    private $data;

    /**
     * Get the validation rules that apply to the service.
     *
     * @param $data
     * @return array
     */
    public function getRules($data)
    {
        $rules = [
            'type' => ['required', Rule::in(ActionRequestTypes::ACTION_TYPES)],
            'users' => 'array|required_without_all:segments',
            'segments' => 'array|required_without_all:users',
            'is_send_immediately' => 'required|boolean',
            'due_by' => 'required|date_utc|after:now'
        ];

        if (isset($data['type'])) {
            if ($data['type'] === ActionRequestTypes::HEADSHOT_PHOTO) {
                $rules["approvers"] = 'array|required';
            }
            if (in_array($data['type'], ActionRequestTypes::CONFIGURABLE_TYPES)) {
                $rules["header_image_id"] = ['nullable', 'integer'];
                $rules["message"] = ['string', 'nullable'];
            }

            if (in_array($data['type'], ActionRequestTypes::CONTENTABLE_TYPES)) {
                $rules['content_id'] = ['required'];

                if ($data['type'] == ActionRequestTypes::ID_VERIFICATION) {
                    array_push($rules['content_id'], Rule::in([
                        ActionRequestTypes::ID_VERIFICATION_PASSPORT,
                        ActionRequestTypes::ID_VERIFICATION_DRIVER_LICENSE,
                        ActionRequestTypes::ID_VERIFICATION_NATIONAL_ID
                    ]));
                }
            }
        }

        if (isset($data['is_send_immediately']) && !$data['is_send_immediately']) {
            $rules['send_on'] = 'required|date_utc|after:now';
            $rules['due_by'] = 'required|date_utc|after:send_on';
        }

        return $rules;
    }

    /**
     * Create action
     * 
     * @param array $data
     * @return Action
     */
    public function performAction(array $data): Action
    {
        $this->checkPlanAvailableForStepType($data['type']);

        $contentableModel = ActionRequestHelper::getContentableModel(
            $data['type'],
            ObjectHelper::nullOrValue($data, 'content_id')
        );

        // Prevent adding contents which is not published
        if (
            in_array($data['type'], ActionRequestTypes::PUBLISHABLE_TYPES) &&
            !$contentableModel->is_published
        ) {
            throw new InvalidRequestException("Invalid publishable content", ErrorCode::INVALID_PUBLISHABLE_CONTENT);
        }

        if ($data['type'] === ActionRequestTypes::FORM) {
            app(CheckValidTemplate::class)->execute(['template_id' => $contentableModel->template_id]);
        }

        $user = Context::user()->get();
        $client = Context::client()->get();

        $actionData = [
            'client_id' => $client->getKey(),
            'is_send_immediately' => $data['is_send_immediately'],
            'send_on' => $data['is_send_immediately'] ? null : ObjectHelper::nullOrValue($data, 'send_on'),
            'due_by' => $data['due_by'],
            'created_by' => $user->getKey(),
            'header_image_id' => ObjectHelper::nullOrValue($data, 'header_image_id'),
            'message' => ObjectHelper::nullOrValue($data, 'message')
        ];

        if (!in_array($data['type'], ActionRequestTypes::CONFIGURABLE_TYPES)) {
            $actionData = Arr::except($actionData, ['header_image_id', 'message']);
        }

        // Create action
        $createdAction = Action::create($actionData);

        app(CreateActionRequest::class)->execute([], $createdAction, $contentableModel);

        // Add users
        $userIds = [];
        $userCodes = [];
        if (isset($data['users']) && count($data['users']) > 0) {
            foreach ($data['users'] as $code) {
                if (!empty($code)) {
                    $user = User::findByBusinessKey($code);
                    PermissionHelper::hasAccessOn($user->employee);
                    $userCodes[] = $code;
                    $userIds[] = $user->getKey();
                }
            }
        }

        // Add users of segments
        if (isset($data['segments'])) {
            $segmentUserIds = UserHelper::getUserIdBySegment($data['segments']);
            $userIds = array_merge($userIds, $segmentUserIds);
        }
        $createdAction->users()->attach(array_unique($userIds));

        // Add approvers
        $approverIds = [];
        $approverCodes = ObjectHelper::nullOrValue($data, 'approvers');
        if (!empty($approverCodes)) {
            foreach ($approverCodes as $code) {
                if (!empty($code)) {
                    $user = User::findByBusinessKey($code);
                    if (!$user->hasRole(Roles::ADMIN)) {
                        throw new InvalidRequestException();
                    }
                    PermissionHelper::hasAccessOn(UserHelper::getClientByUserCode($code));
                    $approverIds[] = $user->getKey();
                }
            }
        }
        $createdAction->approvers()->attach($approverIds);

        // Create Panda documents for form type action
        if ($data['type'] === ActionRequestTypes::FORM) {
            CreateSingleActionDocument::dispatch($createdAction);
        }

        // Push notification to employees
        if (!empty($userCodes)) {
            $fileType = null;
            $actionRequest = $createdAction->actionRequest;
            if (
                isset($actionRequest->contentable->linkable->assets) &&
                isset($actionRequest->contentable->linkable->assets->first()->type)
            ) {
                $fileType = $actionRequest->contentable->linkable->assets->first()->type;
            }
            if (is_a($actionRequest->contentable, LibraryItem::class)) {
                if (is_a($actionRequest->contentable->linkable, Link::class)) {
                    $fileType = 'Link';
                } else {
                    $fileType = isset($actionRequest->contentable->linkable->original_name) ?
                        ucfirst(pathinfo($actionRequest->contentable->linkable->original_name, PATHINFO_EXTENSION)) :
                        null;
                }
            }


            $arguments = array(
                'sender' => Auth::user()->id,
                "userIds" => $userCodes,
                "headings" => __("New action assigned"),
                "contents" => __("Tap to open action"),
                "data" => [
                    "type" => NotificationTypes::ACTION_ADDED,
                    "id" => $createdAction->actionRequest->id,
                    "action_type" => $data['type'],
                    "content_type" => empty($data['content_id']) ? '' : $data['content_id'],
                    "file_type" => $fileType,
                ],
                'send_after' => is_null($createdAction->send_on) ? null : DateHelper::format($createdAction->send_on)
            );
            SendNotification::dispatch($arguments);



            // Send Invite Notifications to employees
            $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
            $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

            $users = User::whereIntegerInRaw('id', $userIds)->get();
            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_ACTION]) {
                foreach ($users as $iuser) {
                    SendNotificationEmail::dispatch($iuser, EmailNotifications::ACTION_SINGLE_INVITATION, ['action' => $createdAction->getNameAttribute()]);
                }
            }
        }

        return $createdAction;
    }

    /**
     * @param $type
     */
    public function checkPlanAvailableForStepType($type)
    {
        app(CheckPlanAvailableForStepType::class)->execute($type);
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }
}
