<?php

namespace App\Services\Action;

use App\Enum\ActionTypes;
use App\Enum\SubmissionStatus;
use App\Helpers\FilterFluent;
use App\Helpers\ModelHelper;
use App\Helpers\SortFluent;
use App\Models\Action\Action;
use App\Models\Action\ActionSequence;
use App\Models\Action\ActionSequenceUser;
use App\Models\Action\ActionUser;
use App\Models\User\User;
use App\Services\Action\ActionSequence\GetEmployeeSubmission;
use App\Services\ActionRequest\GetUserSubmission;
use App\Services\BaseService;
use App\Traits\Pageable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;


class GetActionEmployees extends BaseService
{
    use Pageable;


    /**
     * Get action submissions
     *
     * @param array $options
     * @return Builder[]|Collection
     * @throws ValidationException
     */
    public function performAction(array $options)
    {
        $actionSequenceUserQuery = ActionSequenceUser::leftJoin(
            'action_sequences as as',
            'as.id',
            'action_sequence_user.action_sequence_id'
        )
            ->select('user_id')
            ->addSelect(DB::raw("action_sequence_id as action_id"))
            ->addSelect(DB::raw("'" . ActionTypes::ACTION_SEQUENCE . "' as action_type"))
            ->addSelect('as.created_at');
        ModelHelper::setConditionFindByClient($actionSequenceUserQuery, "as");

        $actionUserQuery = ActionUser::leftJoin('action_requests as ar', function($join) {
            $join->on('ar.actionable_id', 'action_user.action_id')
                ->where('ar.actionable_type', Action::class);
        })
            ->leftJoin('action_request_submissions as ars', function($join) {
                $join->on('ars.action_request_id', 'ar.id')
                    ->on('ars.user_id', 'action_user.user_id');
            })
            ->leftJoin('actions', 'actions.id', 'action_user.action_id')
            ->where(function($query) {
                $query->where('ars.status', '!=', SubmissionStatus::PENDING)
                    ->orWhereNull('ars.status');
            })
            ->select('action_user.user_id')
            ->addSelect('action_user.action_id')
            ->addSelect(DB::raw("'" . ActionTypes::SINGLE_ACTION . "' as action_type"))
            ->addSelect('actions.created_at')
            ->union($actionSequenceUserQuery);

        ModelHelper::setConditionFindByClient($actionUserQuery, "actions");

        $combinedActionUserQuery = User::with(['employee.client', 'avatar'])
            ->select('users.*', 'combined_action_user.*')
            ->joinSub($actionUserQuery, 'combined_action_user', function($join) {
                $join->on('users.id', 'combined_action_user.user_id');
            });
        
        if (empty($options['sort'])) {
            $combinedActionUserQuery->orderBy('combined_action_user.created_at', 'desc')
                ->orderBy('combined_action_user.user_id');
        }

        FilterFluent::init()
            ->setQuery($combinedActionUserQuery)
            ->setFilterOptions($options)
            ->setAllowedFilters(['name', 'job_title'])
            ->validate()
            ->removeFilter('name')
            ->removeFilter('job_title')
            ->setCustomFilter('name', function($query, $type, $col, $value) {
                FilterFluent::applyCombinedColumnFilter($query, $type, 'surname', 'first_name', $value, 'users');
            })
            ->setCustomFilter('job_title', function($query, $type, $col, $value) {
                $query->whereHas('employee', function($q) use ($type, $col, $value) {
                    FilterFluent::applyBasicFilter($q, $type, 'job_title', $value);
                });
            })
            ->applyFilter();

        SortFluent::init()
            ->setQuery($combinedActionUserQuery)
            ->setSortOptions($options)
            ->setAllowedSorts(['name', 'job_title'])
            ->validate()
            ->removeSort(['name', 'job_title'])
            ->setCustomSort('name', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'surname', $direction, 'users');
            })
            ->setCustomSort('job_title', function($query, $col, $direction) {
                $query->leftJoin('employees', 'employees.user_id', 'users.id');
                SortFluent::applyBasicSort($query, 'job_title', $direction, 'employees');
            })
            ->applySort();

        $employees = $this->applyPaging($options, $combinedActionUserQuery);

        // Attach employee action submissions
        foreach ($employees as $employee) {
            $this->attachEmployeeSubmission($employee);
        }

        return $employees;
    }

    /**
     * Attach employee submission
     *
     * @param $employee
     */
    function attachEmployeeSubmission($employee)
    {
        $submissions = [];
        $actionName = null;
        switch ($employee->action_type) {
            case ActionTypes::SINGLE_ACTION:
                $action = Action::find($employee->action_id);
                $actionRequest = $action->actionRequest;
                $submission = app(GetUserSubmission::class)->execute($actionRequest, $employee->id);
                $employee->single_action_submission = $submission;
                break;
            case ActionTypes::ACTION_SEQUENCE:
                $action = ActionSequence::find($employee->action_id);

                $submissions = app(GetEmployeeSubmission::class)->execute(
                    [
                        'code' => $action->code,
                        'employee_code' => $employee->code
                    ]
                );
                
                $employee->action_sequence_submission = $submissions;
                break;
            default:
                $action = null;
        }

        $employee->action = $action;
    }
}