<?php

namespace App\Services\Action;

use App\Enum\SubmissionStatus;
use App\Facades\Context;
use App\Helpers\SortFluent;
use App\Models\Action\Action;
use App\Models\Action\ActionSequenceStep;
use App\Models\ActionRequest\ActionRequestSubmission;
use App\Models\Journey\JourneyStep;
use App\Services\BaseService;
use App\Traits\Pageable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Validation\ValidationException;


class GetPendingSubmission extends BaseService
{
    use Pageable;
    
    /**
     * Get pending reviews
     *
     * @param array $options
     * @return Builder[]|Collection
     * @throws ValidationException
     */
    public function performAction(array $options)
    {
        $currentClient = Context::client()->get();
        $currentUserId =  Context::user()->get()->getKey();
        
        $pendingSubmissionQuery = ActionRequestSubmission::query()->with('user.employee.client')->join(
            'action_requests as ar',
            'ar.id',
            'action_request_submissions.action_request_id'
        )
            ->where(function ($query) use ($currentUserId) {
                $query->whereIn('ar.actionable_type', [JourneyStep::class, ActionSequenceStep::class])
                    ->orWhere(function ($query) use ($currentUserId) {
                        $query->where('ar.actionable_type', Action::class)
                            ->whereExists(function ($query) use ($currentUserId) {
                                $query->from('action_approver as ap')
                                    ->whereRaw('ap.action_id = ar.actionable_id')
                                    ->where('ap.user_id', $currentUserId);
                            });
                    });
            })
            ->where('action_request_submissions.status', SubmissionStatus::PENDING)
            ->select('action_request_submissions.*');
        if($currentClient){
            $userIds = $currentClient->users()->pluck('id')->toArray();
            $pendingSubmissionQuery->whereIntegerInRaw('action_request_submissions.user_id', $userIds);
        }
        SortFluent::init()
            ->setQuery($pendingSubmissionQuery)
            ->setSortOptions($options)
            ->setAllowedSorts(['name', 'job_title', 'start_date'])
            ->validate()
            ->removeSort(['name', 'job_title', 'start_date'])
            ->setCustomSort('name', function ($query, $col, $direction) {
                $query->leftJoin('users as u1', 'u1.id', 'action_request_submissions.user_id');
                SortFluent::applyBasicSort($query, 'surname', $direction, 'u1');
            })
            ->setCustomSort('job_title', function ($query, $col, $direction) {
                $query->leftJoin('users as u2', 'u2.id', 'action_request_submissions.user_id')
                    ->leftJoin('employees as e1', 'e1.user_id', 'u2.id');
                SortFluent::applyBasicSort($query, 'job_title', $direction, 'e1');
            })
            ->setCustomSort('start_date', function ($query, $col, $direction) {
                $query->leftJoin('users as u3', 'u3.id', 'action_request_submissions.user_id')
                    ->leftJoin('employees as e2', 'e2.user_id', 'u3.id');
                SortFluent::applyBasicSort($query, 'start_date', $direction, 'e2');
            })
            ->applySort();
        
        return $this->applyPaging($options, $pendingSubmissionQuery);
    }
}