<?php


namespace App\Services\ActionRequest;


use App\Contracts\Performable;
use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\QuestionTypes as QuestionTypesEnum;
use App\Facades\Context;
use App\Helpers\PandaDocHelper;
use App\Models\ActionRequest\ActionRequestSubmission;
use App\Models\Attachment;
use App\Models\ContentPhoto\ContentPhoto;
use App\Models\Form\FormTemplate;
use App\Models\IDVerification\IDVerification;
use App\Models\Quiz\Quiz;
use App\Models\Survey\Survey;
use App\Services\Attachment\DeleteAttachment;
use App\Services\BaseService;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Collection;

/**
 * Class ClearActionRequestSubmission
 * @package App\Services\ActionRequest
 */
class ClearActionRequestSubmission extends BaseService implements Validatable, Performable, Transactionable
{
    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * @var
     */
    protected $pandaDocHelper;

    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($args)
    {
        return [
            'action_request' => 'required'
        ];
    }


    /**
     * Get employee submission
     *
     * @param array $data
     * @return array
     */
    public function performAction(array $data)
    {
        $actionRequest = $data['action_request'];
        $query = ActionRequestSubmission::where('action_request_id', $actionRequest->id);
        if (isset($data['user_id'])) {
            $query->where('user_id', $data['user_id']);
        }
        $submissions = $query->get();
        foreach ($submissions as $submission) {
            $this->deleteSubmissionChild($actionRequest->contentable_type, $submission);
            $submission->delete();
        }
    }

    /**
     * Get panda doc helper
     */
    protected function setPandaDocHelper()
    {
        $currentClient = Context::client()->get();
        $formAuthenticate = $currentClient->formAuthenticate;
        if (!empty($formAuthenticate)) {
            $this->pandaDocHelper = new PandaDocHelper();
            $this->pandaDocHelper->setToken($formAuthenticate->access_token);
        }
    }

    /**
     * Delete all submission child
     * @param string $content
     * @param Model $model
     */
    public function deleteSubmissionChild(string $content, Model $model)
    {
        switch ($content) {
            case Quiz::class:
                $model->quizResults()->delete();
                break;
            case Survey::class:
                $this->deleteSurveyResultDetail($model->surveyResults);
                $model->surveyResults()->delete();
                break;
            case FormTemplate::class:
                $this->setPandaDocHelper();
                foreach ($model->formResults as $formResult) {
                    if ($this->pandaDocHelper instanceof PandaDocHelper && $formResult->document && $formResult->document->document_id) {
                        $this->pandaDocHelper->deleteDocument($formResult->document->document_id);
                    }
                    $formResult->document()->delete();
                }
                $model->formResults()->delete();
                break;
            case IDVerification::class:
                foreach ($model->idVerificationResults as $idVerificationResult) {
                    if ($idVerificationResult->frontAttachment) {
                        $this->deleteAttachment($idVerificationResult->frontAttachment);
                    }
                    if ($idVerificationResult->backAttachment) {
                        $this->deleteAttachment($idVerificationResult->backAttachment);
                    }
                }
                //Delete subbmission details
                $model->idVerificationResults()->delete();
                break;
            case ContentPhoto::class:
                foreach ($model->contentPhotoResults as $contentPhotoResult) {
                    if ($contentPhotoResult->attachment) {
                        $this->deleteAttachment($contentPhotoResult->attachment);
                    }
                }
                $model->contentPhotoResults()->delete();
                break;
        }
    }

    /**
     * @param Collection $surveyResults
     */
    public function deleteSurveyResultDetail(Collection $surveyResults)
    {
        foreach ($surveyResults as $surveyResult) {
            if(!$surveyResult->question){
                continue;
            }
            $questionType = $surveyResult->question->type;
            switch ($questionType) {
                case QuestionTypesEnum::TYPE_MULTI:
                    if ($surveyResult->surveyChoiceResults) {
                        $surveyResult->surveyChoiceResults()->delete();
                    }
                    break;
                case QuestionTypesEnum::TYPE_RATING:
                    if ($surveyResult->surveyRatingResult) {
                        $surveyResult->surveyRatingResult()->delete();
                    }
                    break;
                case QuestionTypesEnum::TYPE_TEXT:
                    if ($surveyResult->surveyTextResult) {
                        $surveyResult->surveyTextResult()->delete();
                    }
                    break;

            }
        }
    }

    /**
     * @param Attachment $attachment
     */
    public function deleteAttachment($attachment)
    {
        app(DeleteAttachment::class)->execute($attachment);
    }
}