<?php

namespace App\Services\ActionRequest;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\EmailNotifications;
use App\Enum\ErrorCode;
use App\Enum\NotificationTypes;
use App\Enum\Roles;
use App\Enum\SubmissionStatus;
use App\Events\SubmissionUpdated;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Jobs\SendNotification;
use App\Jobs\SendNotificationConsole;
use App\Jobs\SendNotificationEmail;
use App\Models\ActionRequest\ActionRequestSubmission;
use App\Services\BaseService;
use App\Services\UserAccess\GetAccessByRole;
use Illuminate\Support\Facades\Auth;

class RejectSubmission extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($args)
    {
        return [
            'submission_id' => 'required|integer',
            'reason' => 'required|string|max:1023'
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     *  Reject submission
     * 
     * @param array $data
     * @return ActionRequestSubmission
     */
    public function performAction(array $data): ActionRequestSubmission
    {
        $submission = ActionRequestSubmission::findOrFail($data['submission_id']);
        if (!$submission->isPending()) {
            throw new InvalidRequestException('Invalid request', ErrorCode::INVALID_REQUEST);
        }

        $actionRequest = $submission->actionRequest;
        app(CheckUserAuthorizedActionRequest::class)->execute(Context::user()->get(), $actionRequest);

        $submission->status = SubmissionStatus::REJECTED;
        $submission->reason = $data['reason'];
        $submission->save();

        event(new SubmissionUpdated($submission));
        $userSubmission = $submission->user;
        if ($userSubmission) {
            $arguments = array(
                "userIds" => $userSubmission->code,
                "headings" => __("Headshot photo rejected"),
                "contents" => __("Tap to upload a new one"),
                "data" => [
                    "type" => NotificationTypes::JOURNEY_REJECTED,
                    "id" => $submission->actionRequest->id
                ],
                'send_after' => null
            );
            SendNotification::dispatch($arguments);
        }



        // Send Invite Notifications to employees
        $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
        $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

        if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_CONSOLE_EMPLOYEE_JOURNEY]) {
            SendNotificationConsole::dispatch($userSubmission, $userSubmission, 'Submitted content rejected');
        }
        if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_JOURNEY]) {
            SendNotificationEmail::dispatch($userSubmission, EmailNotifications::JOURNEY_REJECTION, ['reason' => $data['reason'], 'step' => 'Anonymous step']);
        }
        return $submission;
    }
}