<?php

namespace App\Services\AlertMessage;

use App\Enum\AccessPermissions;
use App\Enum\Roles;
use App\Facades\Context;
use App\Helpers\UserAccessHelper;
use App\Models\AlertMessage\AlertMessage;
use App\Models\Event\Event;
use App\Services\BaseService;
use App\Enum\AlertMessage as AlertMessageEnum;
use App\Services\QuestionnaireAnswer\CheckExistSensitiveComment;
use App\Services\QuestionnaireAnswer\GetSensitiveComment;
use App\Services\User\Expiring\Check2FAIsExpiring;
use App\Services\User\Expiring\CheckCardIsExpiring;
use App\Services\User\Expiring\CheckPasswordIsExpiring;
use Carbon\Carbon;
use Illuminate\Support\Collection;

class GetAlertMessage extends BaseService
{

    /**
     * Get alert messages
     * 
     * @param array $args
     * @return Collection
     */
    public function performAction(array $args)
    {
        $user = Context::user()->get();
        $availableMessages = collect();
        if (!$user->hasRole(Roles::ADMIN)) {
            return $availableMessages;
        }
        $readMessages = $user->readAlertMessages()->pluck('alert_message_id')->toArray();
        $alertMessages = AlertMessage::whereIntegerNotInRaw('id', $readMessages)
            ->where(function($sub) {
                $sub->where('expire_on', '>=', Carbon::now());
                $sub->orWhereNull('expire_on');
            })
            ->orderBy('priority')
            ->orderBy('id', 'desc')
            ->get();

        foreach ($alertMessages as $key => $alertMessage) {
            if ($alertMessage->type == AlertMessageEnum::AUTOMATIC_TYPE) {
                switch ($alertMessage->message) {
                    case AlertMessageEnum::EXPIRING_PASSWORD_MESSAGE:
                        $expiryDate = app(CheckPasswordIsExpiring::class)->execute([
                            'user' => $user,
                            'day_interval' => config('interval.expiring_password_day')
                        ]);
                        if ($expiryDate) {
                            $alertMessage->expiry_date = $expiryDate;
                            $availableMessages->push($alertMessage);
                        }
                        break;
                        
                    case AlertMessageEnum::DISABLING_2FA_MESSAGE:
                        $expiryDate = app(Check2FAIsExpiring::class)->execute([
                            'user' => $user,
                            'day_interval' => config('interval.disabling_2fa_day')
                        ]);
                        if ($expiryDate) {
                            $alertMessage->expiry_date = $expiryDate;
                            $availableMessages->push($alertMessage);
                        }
                        break;
                        
                    case AlertMessageEnum::EXPIRING_CARD_MESSAGE:
                        $isExpiring = app(CheckCardIsExpiring::class)->execute([
                            'user' => $user,
                            'day_interval' => config('interval.expiring_card_day')
                        ]);
                        if ($isExpiring) {
                            $alertMessage->expiry_date = $isExpiring['expiry_date'];
                            $alertMessage->card = $isExpiring['card'];
                            $availableMessages->push($alertMessage);
                        }
                        break;

                    case AlertMessageEnum::SENSITIVE_COMMENT:
                        if (UserAccessHelper::hasAccess(AccessPermissions::ACCESS_SENSITIVE_COMMENT) &&
                            $existSensitiveComment = app(CheckExistSensitiveComment::class)->execute()) {
                            $sensitiveComments = app(GetSensitiveComment::class)->execute();
                            $alertMessage->from_date = $sensitiveComments['from_date'];
                            $alertMessage->to_date = $sensitiveComments['to_date'];
                            $alertMessage->expiry_date = Carbon::now()->addDay();
                            $availableMessages->push($alertMessage);
                        }
                        break;
                    case AlertMessageEnum::UPCOMING_EVENT:
                        $alertMessage->expiry_date = $alertMessage['expire_on'];
                        $payload = json_decode($alertMessage->payload);
                        $event = Event::setEagerLoads([])->find($payload->id);
                        $alertMessage->payload = [
                            'id' => $event->getKey(),
                            'name' => $event->name,
                            'start_date' => $event->start_date
                        ];
                        $availableMessages->push($alertMessage);
                        break;
                }
            } else if ($alertMessage->isActive()) {
                $availableMessages->push($alertMessage);
            }
        }

        return $availableMessages;
    }
}
