<?php

namespace App\Services\Asset;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\AllowedFileSizes;
use App\Enum\AssetType;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ResourceLocationBuilder;
use App\Models\Asset;
use App\Models\Client\Client;
use App\Services\Attachment\CreateAttachment;
use App\Services\AuthBaseService;
use App\Helpers\FileHelper;
use App\Services\Asset\GetAssets;
use App\Http\Resources\Library\Attachment;

class CreateAsset extends AuthBaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($data)
    {
        $max_size = isset($data['is_editor']) && filter_var($data['is_editor'], FILTER_VALIDATE_BOOLEAN) ? AllowedFileSizes::IMAGE_SIZE_EDITOR : AllowedFileSizes::COMMON_SIZE;
        return [
            'title' => 'string',
            'client_id' => 'string',
            'file' => 'required|max:' . $max_size,
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Create asset
     *
     * @param array $data
     * @return Asset
     */
    public function performAction(array $data): Asset
    {

        $clientId = Context::client()->get()->getKey();
        $data['client_id'] = $clientId;
        $data['created_by'] = $this->getCurrentUser()->id;

        $mimeType = $data['file']->getMimeType();

        $type = null;

        switch ($mimeType) {
            case 'application/pdf':
                $type = AssetType::PDF;
                break;
            case 'audio/mpeg':
            case 'audio/mp3':
                $type = AssetType::MP3;
                break;
            case 'video/mp4':
            case 'audio/mp4':
            case 'application/mp4':
                $type = AssetType::MP4;
                break;
            case 'image/jpeg':
            case 'image/png':
            case 'image/gif':
                $type = AssetType::IMAGE;
                break;
        }

        if (empty($type)) {
            throw new InvalidRequestException('Invalid file type. Only support pdf, jpg, png, gif.');
        }

//        $fileExisted = $this->checkExistFileName($data['file']);
//        if ($fileExisted) {
//            throw new InvalidRequestException('Invalid file name. Filename already exists.');
//        }

        $client = Client::findorFail($clientId);
        $fileDir = (new ResourceLocationBuilder())->setClient($client->code)->getAssetPath();
        $attachment = app(CreateAttachment::class)->execute($data['file'], $fileDir, $this->getCurrentUser()->id);

        $data['type'] = $type;

        $asset = new Asset($data);
        $asset->attachment()->associate($attachment);
        $asset->save();

        return $asset->refresh();
    }

//    private function checkExistFileName($file)
//    {
//        $assets =  app(GetAssets::class)->execute();
//        $originalName = FileHelper::getOriginalName($file);
//        foreach ($assets as $asset) {
//            $attachment = new Attachment($asset->attachment);
//
//            if ($attachment->original_name === $originalName) {
//                return true;
//            }
//        }
//
//        return false;
//    }
}
