<?php

namespace App\Services\Attachment;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\ErrorCode;
use App\Enum\OCRAllowedMimeTypes;
use App\Enum\OCRJobStatus;
use App\Exceptions\InvalidRequestException;
use App\Exceptions\NotFoundHttpException;
use App\Exceptions\UnauthorizedHttpException;
use App\Facades\Context;
use App\Models\Attachment;
use App\Models\Attachment\AttachmentOCR;
use App\Services\BaseService;
use Illuminate\Contracts\Filesystem\FileNotFoundException;
use Illuminate\Support\Facades\Storage;

class GetDocumentOCR extends BaseService implements Validatable, Performable
{

    public function getRules(array $args)
    {
        return [
            'attachment_id' => 'required'
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Get document OCR
     *
     * @param array $args
     * @return mixed
     * @throws FileNotFoundException
     */
    public function performAction(array $args)
    {
        $client = Context::client()->get();

        $attachment = Attachment::find($args['attachment_id']);
        if (!$attachment) {
            throw new NotFoundHttpException();
        }
        if (!in_array($attachment->mime_type, OCRAllowedMimeTypes::AllowedMimeTypes, true)) {
            throw new InvalidRequestException(
                'The format of the document is not supported',
                ErrorCode::UNSUPPORTED_DOCUMENT_FORMAT
            );
        }

        $attachmentOCR = AttachmentOCR::where('attachment_id', $args['attachment_id'])->latest()->first();
        if (!$attachmentOCR) {
            throw new NotFoundHttpException();
        }

        $attachmentPath = $attachmentOCR->attachment->path;
        $isMatchClientPath = preg_match('/clients\/(.*?)\//', $attachmentPath, $matches);
        if (!$isMatchClientPath || $client->getBusinessKey() !== $matches[1]) {
            throw new UnauthorizedHttpException();
        }

        $lines = [];
        $documentMetadata = null;
        $totalConfidenceScore = 0;
        $count = 0;
        $isGetAllResult = config('ocr.get_all_result');
        $pageNo = 1;
        $path = $this->getOCRResultPath($attachmentPath, $attachmentOCR->ocr_job_id, $pageNo);
        while (Storage::disk('s3OCR')->exists($path)) {
            $contents = Storage::disk('s3OCR')->get($path);
            $decodedContents = json_decode($contents, true);
            foreach ($decodedContents['Blocks'] as $block) {
                if ($block['BlockType'] === 'LINE') {
                    $lines[] = [
                        'text' => $block['Text'],
                        'confidence_score' => $block['Confidence']
                    ];
                    $totalConfidenceScore += $block['Confidence'];
                    $count++;
                }
            }
            $documentMetadata = ['pages' => $decodedContents['DocumentMetadata']['Pages']];
            $attachmentOCR->ocr_job_status = $decodedContents['JobStatus'];

            if (!$isGetAllResult) {
                break;
            }

            $pageNo++;
            $path = $this->getOCRResultPath($attachmentPath, $attachmentOCR->ocr_job_id, $pageNo);
        }
        $attachmentOCR->save();

        if ($attachmentOCR->ocr_job_status === OCRJobStatus::IN_PROGRESS ||
            $attachmentOCR->ocr_job_status === OCRJobStatus::PARTIAL_SUCCESS) {
            throw new InvalidRequestException(
                'Document OCR in progress. Please try again later.',
                ErrorCode::DOCUMENT_OCR_IN_PROGRESS
            );
        }

        $attachmentOCR->lines = $lines;
        $attachmentOCR->average_confidence_score = ($count > 0) ? (float)($totalConfidenceScore / $count) : $count;
        $attachmentOCR->document_metadata = $documentMetadata;

        return $attachmentOCR;
    }

    public function getOCRResultPath($attachmentPath, $jobId, $pageNo)
    {
        return config('ocr.ocr_env') . '/' . dirname($attachmentPath) . '/' . $jobId . '/' . $pageNo;
    }
}
