<?php


namespace App\Services;

use App\Enum\Roles;
use App\Models\Client\Client;
use Illuminate\Support\Facades\Auth;
use App\Exceptions\InvalidRequestException;
use App\Exceptions\UnauthorizedHttpException;

abstract class AuthBaseService extends BaseService
{
    protected $currentUser;
    public function __construct()
    {
        $this->currentUser = Auth::user();
    }

    /**
     * @return \Illuminate\Contracts\Auth\Authenticatable|null
     */
    protected function getCurrentUser() {
        return $this->currentUser;
    }

    protected function isSuperAdmin () {
        return $this->getCurrentUser()->hasRole(Roles::SUPER_ADMIN);
    }

    protected function isClientAdmin() {
        return $this->getCurrentUser()->hasRole(Roles::ADMIN);
    }

    /**
     * If client admin login, return related client otherwise if SA logged in,
     * return client_id is converted by code from request
     * 
     * @param string|null $clientCode
     * @return int|null
     */
    protected function getClientId(string $clientCode = null) {
        if ($this->isClientAdmin()) {
            return $this->getCurrentUser()->client->first()->id;
        } else if($this->isSuperAdmin() && isset($clientCode)){
            $client = Client::findByBusinessKey($clientCode);
            return $client->getKey();
        }
        return null;
    }

    /**
     * Force to get client_id, if empty throw exception
     *
     * @param string $clientCode
     * @return int
     */
    protected function forceGetClientId(string $clientCode = null): int {
        $clientId = $this->getClientId($clientCode);
        if (empty($clientId)) {
            throw new InvalidRequestException('Invalid client info.');
        }
        return $clientId;
    }

    /**
     * Ensure user's client is match with resource's client.
     */
    protected function ensureAuthorizedClient($clientId): bool {
        if ($this->isClientAdmin()) {
            $userClientId =  $this->getCurrentUser()->client->first()->id;
            if ($userClientId === $clientId) {
                return true;
            }
        } else if($this->isSuperAdmin()){
            return true;
        }
        throw new UnauthorizedHttpException();
    }

    /**
     * Ensure current user is super admin
     */
    protected function ensureSuperAdmin(): void
    {
        if (!$this->isSuperAdmin()) {
            throw new UnauthorizedHttpException();
        }
    }

    /**
     * Ensure current user is client admin
     */
    protected function ensureClientAdmin(): void
    {
        if (!$this->isClientAdmin()) {
            throw new UnauthorizedHttpException();
        }
    }
}
