<?php


namespace App\Services;

use App\Contracts\Performable;
use App\Contracts\Permissionable;
use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\ErrorCode;
use App\Exceptions\InvalidRequestException;
use App\Exceptions\UnauthorizedHttpException;
use App\Services\Tool\Quiz\SaveQuizQuestion;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

abstract class BaseService
{
    const HAS_ALL = 'ALL';
    const HAS_ANY = 'ANY';

    /**
     * Validate all datas to execute the service.
     *
     * @param array $args
     * @return bool
     */
    protected function performValidation(array $args): array
    {
        if ($this instanceof Validatable) {
            return Validator::make($args, $this->getRules($args))
                ->validate();
        }
        return array();
    }

    /**
     * Validate all data to execute the service.
     *
     * @param array $args
     * @param string $checkType
     * @return bool
     */
    protected function performPermissionCheck(array $args, string $checkType = self::HAS_ALL): bool
    {
        if ($this instanceof Permissionable) {
            $permissions = $this->getPermissions($args);
            if (!is_array($permissions) || sizeof($permissions) === 0)
                return true;
            $user = $this->getUser($args);
            if (!$user) {
                throw new InvalidRequestException('User not found for permission check');
            }

            $isSuccessOnce = false;
            foreach ($this->getPermissions($args) as $permission) {
                $isSuccessOnce = $user->can($permission);
                if ($isSuccessOnce && $checkType === self::HAS_ANY) {
                    break;
                }
                if (!$isSuccessOnce && $checkType === self::HAS_ALL) {
                    throw new UnauthorizedHttpException('User do not have permission to access the service', ErrorCode::PERMISSION_INVALID);
                }
            }

            if (!$isSuccessOnce && $checkType === self::HAS_ANY) {
                throw new UnauthorizedHttpException('User do not have permission to access the service', ErrorCode::PERMISSION_INVALID);
            }
            return true;
        }
        return true;
    }

    protected function performableCheck()
    {
        if ($this instanceof Performable && !$this->canPerform()) {
            throw new InvalidRequestException("Current user is not allowed to perform service", ErrorCode::NOT_PERFORMABLE);
        }
    }

    protected function transaction($callback)
    {
        return DB::transaction(function () use ($callback) {
            return call_user_func($callback);
        });
    }

    function execute(array $args = [])
    {
        $this->performableCheck();
        $this->performPermissionCheck($args);

        $this->performValidation($args);
        if ($this instanceof Transactionable) {
            return DB::transaction(function () use ($args) {
                return $this->performAction($args);
            });
        } else {
            return $this->performAction($args);
        }
    }

    abstract function performAction(array $args);
}
