<?php


namespace App\Services\Client;

use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\AllowedFileSizes;
use App\Enum\AllowedFileTypes;
use App\Facades\Context;
use App\Helpers\ModelHelper;
use App\Helpers\ResourceLocationBuilder;
use App\Models\Client\Client;
use App\Models\Client\ClientConfiguration;
use App\Models\Phone;
use App\Models\Subscription\SubscriptionPlan;
use App\Services\Attachment\CreateAttachment;
use App\Services\BaseService;
use App\Services\Client\Traits\CheckStripe;
use App\Services\Segment\CreateDefaultGenderSegment;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Arr;

/**
 * Class CreateClient
 * @package App\Services\Client
 */
class CreateClient extends BaseService implements Validatable, Transactionable
{
    use CheckStripe;

    /**
     * Get the validation rules that apply to the service.
     *
     * @param $args
     * @return array
     */
    public function getRules($args): array
    {
        return [
            'name' => 'required|string',
            'logo' => 'nullable|mimes:' . AllowedFileTypes::IMAGE . '|max:' . AllowedFileSizes::IMAGE_SIZE,
            'plan_id' => 'required',
            'industry_id' => 'required',
            'stripe_cus_id' => 'required|string',
            'phone_number' => 'required|string',
            'phone_country' => 'required|string',
        ];
    }

    /**
     * Create Client
     *
     * @param  array  $args
     * @return Client
     * @throws Exception
     */
    public function performAction(array $args): Client
    {
        $user = Context::user()->get();
        $subscription = $this->checkValidStripeCustomer($args['stripe_cus_id']);

        $phone =  Phone::create(['country_code' => $args['phone_country'], 'phone_number' => $args['phone_number']]);
        $args = Arr::except($args, ['phone_number','phone_country']);
        $args['plan_id'] = ModelHelper::convertBusinessKeyToId(SubscriptionPlan::class, $args['plan_id']);
        $client = new Client($args);
        $client->phone()->associate($phone);
        $client->subscription_period_start = $subscription->current_period_start;
        $client->subscription_period_end = $subscription->current_period_end;
        $client->save();
        $client->refresh();

        if (empty($args['logo'])) {
            $logoFile = new UploadedFile(
                public_path("images/Qualee-Logo-White.png"),
                'Qualee-Logo-White.png',
                'image/png',
                null,
                true
            );
        } else {
            $logoFile =  $args['logo'];
        }
        $fileDir = (new ResourceLocationBuilder())->setClient($client->code)->getLogoPath();
        $logo = app(CreateAttachment::class)->execute($logoFile, $fileDir, $user->id);
        $client->logo()->associate($logo);
        $client->save();

        $clientConfigurations = ClientConfiguration::all();

        foreach ($clientConfigurations as $clientConfiguration) {
            $client->clientConfigurations()->attach($clientConfiguration->getKey(), [
                'value' => $clientConfiguration->default_value,
                'applied_date' => Carbon::now()
            ]);
        }

        //Create default gender segment
        app(CreateDefaultGenderSegment::class)->execute(['client_id' => $client->getKey()]);

        $client->refresh();


        return $client;
    }

}
