<?php


namespace App\Services\Client;

use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Models\Action\Action;
use App\Models\Asset;
use App\Models\Attachment;
use App\Models\Client\Client;
use App\Models\Client\ClientValue;
use App\Models\DataControl\DataControl;
use App\Models\Event\Event;
use App\Models\Initiative\Initiative;
use App\Models\Journey\Journey;
use App\Models\Library\LibraryItem;
use App\Models\Library\Link;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Questionnaire\QuestionnaireClientSetting;
use App\Models\Questionnaire\QuestionnaireUserCompleted;
use App\Models\PowerSearch\SearchableObject;
use App\Models\Quiz\Quiz;
use App\Models\Schedule\ScheduleSurvey;
use App\Models\Segment\Segment;
use App\Models\Survey\Survey;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\Client\ClientValue\DeleteClientValue;
use App\Services\DataControl\DeleteDataControl;
use App\Services\Schedule\ForceDeleteSchedule;
use App\Services\Tool\Journey\DeleteJourney;
use App\Services\Tool\Quiz\DeleteQuiz;
use App\Services\Tool\Survey\DeleteSurvey;
use App\Services\SuperAdmin\Actionable\DeleteActionSequence;
use App\Services\SuperAdmin\Actionable\ClearActionRequestSubmission;
use App\Services\User\RevokeUserToken;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use App\Models\Action\ActionSequence;

/**
 * Class DeleteClient
 * @package App\Services\Client
 */
class DeleteClient extends BaseService implements Validatable, Transactionable
{
    /**
     * @var
     */
    private $client;
    /**
     * @var
     */
    private $userIdArray;

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args)
    {
        return [
            'code' => 'required|string|exists:clients,code',
        ];
    }

    /**
     * Delete client
     *
     * @param array $data
     * @return bool
     */
    public function performAction(array $data): bool
    {
        $this->client = Client::findByBusinessKey($data['code']);
        $this->userIdArray = $this->client->employees()->withTrashed()->get()->pluck('user_id')->toArray();

        // Revoke all user tokens
        $users = $this->client->users()->with('tokens')->get();
        foreach ($users as $user) {
            app(RevokeUserToken::class)->execute($user);
        }

        // User, Employee, Segment, Library, Event, Journey ,Quiz, Survey, Asset, Phone, Logo
        $this->deleteSegment();
        $this->deleteLibrary();
        $this->deleteEvent();
        $this->deleteJourney();
        $this->deleteActionSequence();
        $this->deleteQuiz();
        $this->deleteSurvey();
        $this->deleteAsset();
        $this->deletePowerSearch();

        // Engage Module
        $this->deleteEngageClientValues();
        $this->deleteEngageDataControl();
        $this->deleteEngageInitiative();
        $this->deleteEngageQuestionnaire();
        $this->deleteEngageSchedule();


        $this->deleteUser();
        $this->deleteClient();


        return true;
    }

    /**
     * Delete client
     */
    private function deleteClient()
    {
        $this->client->phone()->delete();
        $this->client->logo()->delete();
        $this->client->clientConfigurations()->detach();
        $this->client->delete();
    }

    /**
     * Delete user and employee
     */
    private function deleteUser()
    {
        // Force delete client users and employees
        //Revoker role and permissions
        DB::table(config('permission.table_names.model_has_roles'))->where('model_type', User::class)
            ->whereIn('model_id', $this->userIdArray)->delete();
        DB::table(config('permission.table_names.model_has_permissions'))->where('model_type', User::class)
            ->whereIn('model_id', $this->userIdArray)->delete();
        User::whereIn('id', $this->userIdArray)->forceDelete();
        $this->client->employees()->withTrashed()->forceDelete();
    }

    /**
     * Delete all segment and related info
     */
    private function deleteSegment()
    {
        DB::table('segment_user')->whereIn('segment_id', Segment::where('client_id', $this->client->getKey())->get()->pluck('id'))->delete();
        Segment::where('client_id', $this->client->getKey())->delete();
    }

    /**
     * Delete all library of client
     */
    private function deleteLibrary()
    {
        $paths = Attachment::whereHas('libraryItem', function($item) {
            $item->where('client_id', $this->client->getKey());
        })->pluck('path');
        //Delete file in s3
        Storage::disk('s3')->delete($paths);
        Attachment::whereHas('libraryItem', function($item) {
            $item->where('client_id', $this->client->getKey());
        })->delete();

        Link::whereHas('libraryItem', function($item) {
            $item->where('client_id', $this->client->getKey());
        })->delete();

        LibraryItem::where('client_id', $this->client->getKey())->delete();
    }

    /**
     * Delete all library of event
     */
    private function deleteEvent()
    {
        $eventIdArray = Event::where('client_id', $this->client->getKey())->get()->pluck('id');
        DB::table('event_user')->whereIn('event_id', $eventIdArray)->delete();
        DB::table('event_segment')->whereIn('event_id', $eventIdArray)->delete();
        Event::where('client_id', $this->client->getKey())->delete();
    }

    /**
     * Delete all library of event
     */
    private function deleteJourney()
    {
        $journeys = Journey::where('client_id', $this->client->getKey());
        foreach ($journeys as $journey) {
            app(DeleteJourney::class)->execute(['code' => $journey->getBusinessKey()]);
        }
    }

    /**
     * Delete all library of event
     */
    private function deleteActionSequence()
    {
        $actionSequences = ActionSequence::where('client_id', $this->client->getKey());
        foreach ($actionSequences as $actionSequence) {
            app(DeleteActionSequence::class)->execute(['code' => $actionSequence->getBusinessKey()]);
        }
    }

    /**
     * Delete all library of event
     */
    private function deleteSingleAction()
    {
        $actions = Action::where('client_id', $this->client->getKey());
        foreach ($actions as $action) {
            app(ClearActionRequestSubmission::class)->execute(
                ['action_request' => $action->actionRequest]
            );
            $action->users()->detach();
        }
        Action::where('client_id', $this->client->getKey())->delete();
    }
    /**
     * Delete all library of event
     */
    private function deleteQuiz()
    {
        $quizzes = Quiz::where('client_id', $this->client->getKey());
        foreach ($quizzes as $quiz) {
            app(DeleteQuiz::class)->execute(['code' => $quiz->getBusinessKey()]);
        }
    }

    /**
     * Delete all library of event
     */
    private function deleteSurvey()
    {
        $surveys = Survey::where('client_id', $this->client->getKey());
        foreach ($surveys as $survey) {
            app(DeleteSurvey::class)->execute(['code' => $survey->getBusinessKey()]);
        }
    }

    /**
     * Delete all library of event
     */
    private function deleteAsset()
    {
        $attamentIdArray = Asset::where('client_id', $this->client->getKey())->get()->pluck('attachment_id');
        Asset::where('client_id', $this->client->getKey())->delete();
        $paths = Attachment::whereIn('id', $attamentIdArray)->get()->pluck('path');
        Storage::disk('s3')->delete($paths);
        Attachment::whereIn('id', $attamentIdArray)->delete();
    }

    /**
     * @return bool
     */
    private function deleteEngageClientValues()
    {
        $clientValues = ClientValue::where('client_id', $this->client->getKey())->get();
        foreach ($clientValues as $clientValue) {
            app(DeleteClientValue::class)->execute(['id' => $clientValue->getKey()]);
        }
        return true;
    }

    /**
     * @return mixed
     */
    private function deleteEngageDataControl()
    {
        return DataControl::where('client_id', $this->client->getKey())->delete();
    }

    /**
     * @return mixed
     */
    private function deleteEngageInitiative()
    {
        return Initiative::where('client_id', $this->client->getKey())->delete();
    }

    /**
     * @return bool
     */
    private function deleteEngageQuestionnaire()
    {
        // Delete Questionnaire Answer
        QuestionnaireAnswer::whereIn('user_id', $this->userIdArray)->delete();
        // Delete completed round
        QuestionnaireUserCompleted::whereIn('user_id', $this->userIdArray)->delete();

        // Delete Questionnaire client setting (published/unpublished)
        QuestionnaireClientSetting::where('client_id', $this->client->getKey())->delete();

        return true;
    }

    /**
     * @return bool
     */
    private function deleteEngageSchedule()
    {

        $survey = ScheduleSurvey::where('client_id', $this->client->getKey())->withTrashed()->get();

        foreach ($survey as $item) {
            app(ForceDeleteSchedule::class)->execute(['id' => $item->id]);
        }
        return true;
    }

    public function deletePowerSearch()
    {
        SearchableObject::where('client_id', $this->client->getKey())->delete();
    }
}
