<?php


namespace App\Services\Client;


use App\Contracts\Validatable;
use App\Enum\PlanConfigurations;
use App\Enum\Roles;
use App\Events\UserAccessGranted;
use App\Exceptions\UnauthorizedHttpException;
use App\Jobs\SendConsoleAccessEmail;
use App\Jobs\SendNewAdminEmail;
use App\Jobs\SendVerifyEmail;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;
use Illuminate\Auth\Events\Verified;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;

class GrantAdminAccess extends BaseService implements Validatable
{
    public function getRules(array $args)
    {
        return [
            'users' => 'array|required'
        ];
    }

    public function performAction(array $data)
    {
        $company = $this->getCurrentCompany();
        $users = array();
        foreach ($data['users'] as $userCode) {
            $user = User::findByBusinessKey($userCode);
            if ($user->employee->client_id !== $company->getKey()) {
                throw new UnauthorizedHttpException();
            }
            if($user->hasRole(Roles::ADMIN)){
                continue;
            }
            //Check limit plan admin quantity
            app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_ADMINISTRATOR);

            $user->assignRole(Roles::ADMIN);

            if ($user->markEmailAsVerified()) {
//                SendVerifyEmail::dispatch($user);
                event(new Verified($user));
                if (!$user->is_welcome_mail_sent) {
                    // Generate a random password
                    $password = Str::random(8);
                    $this->setUserPassword($user, $password);

                    SendNewAdminEmail::dispatch($user, $password, true);
                }
            }
            
            SendConsoleAccessEmail::dispatch($user);

            event(new UserAccessGranted($user, Roles::ADMIN));

            $users[] = $user;
        }
        return $users;
    }


    /**
     * @return mixed
     */
    public function getCurrentCompany()
    {
        return app(GetCompany::class)->execute();
    }

    /**
     * Set user password
     *
     * @param User $user
     * @param string $password
     */
    public function setUserPassword(User $user, string $password)
    {
        $user->password = Hash::make($password);
        $user->save();
    }
}