<?php


namespace App\Services\Client;


use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\Roles;
use App\Events\UserAccessGranted;
use App\Exceptions\UnauthorizedHttpException;
use App\Helpers\PermissionHelper;
use App\Jobs\SendConsoleAccessEmail;
use App\Jobs\SendNewAdminEmail;
use App\Jobs\SendVerifyEmail;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\UserAccess\GetClientRoleAccessNames;
use Illuminate\Auth\Events\Verified;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;

class GrantManagerAccess extends BaseService implements Validatable, Transactionable
{
    public function getRules(array $args)
    {
        return [
            'users' => 'array|required',
            'users.*' => 'required',
        ];
    }

    /**
     * Grant user manager access
     *
     * @param array $data
     * @return array
     */
    public function performAction(array $data)
    {
        $users = array();
        foreach ($data['users'] as $userCode) {
            $user = User::findByBusinessKey($userCode);
            PermissionHelper::hasAccessOn($user->employee);

            if($user->hasRole(Roles::MANAGER)){
                continue;
            }

//            if($user->hasRole(Roles::ADMIN)) {
//                throw new UnauthorizedHttpException();
//            }

            if ($user->isExactFrontUser()) {
                SendConsoleAccessEmail::dispatch($user);
            }

            $user->assignRole(Roles::MANAGER);
            // Give manager access permissions
            $managerAccessNames = app(GetClientRoleAccessNames::class)->execute(['role' => Roles::MANAGER]);
            $user->givePermissionTo($managerAccessNames);

            // Send verification email for console login
            if ($user->markEmailAsVerified()) {
//                SendVerifyEmail::dispatch($user);
                event(new Verified($user));
                if (!$user->is_welcome_mail_sent) {
                    // Generate a random password
                    $password = Str::random(8);
                    $this->setUserPassword($user, $password);

                    SendNewAdminEmail::dispatch($user, $password, true);
                }
            }

            event(new UserAccessGranted($user, Roles::MANAGER));

            $users[] = $user;
        }

        return $users;
    }

    /**
     * Set user password
     *
     * @param User $user
     * @param string $password
     */
    public function setUserPassword(User $user, string $password)
    {
        $user->password = Hash::make($password);
        $user->save();
    }
}