<?php


namespace App\Services\Client;

use App\Contracts\Validatable;
use App\Contracts\Transactionable;
use App\Enum\AllowedFileSizes;
use App\Enum\AllowedFileTypes;
use App\Facades\Context;
use App\Helpers\ModelHelper;
use App\Helpers\ResourceLocationBuilder;
use App\Models\Client\Client;
use App\Models\Phone;
use App\Models\Subscription\SubscriptionPlan;
use App\Services\Attachment\CreateAttachment;
use App\Services\Attachment\UpdateAttachment;
use App\Services\BaseService;
use Illuminate\Support\Arr;
use App\Services\Client\Traits\CheckStripe;

class UpdateClient extends BaseService implements Validatable, Transactionable
{
    use CheckStripe;

    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($args)
    {
        return [
            'code' => 'required|string|exists:clients,code',
            'name' => 'string',
            'plan_id' => 'required',
            'industry_id' => 'required',
            'stripe_cus_id' => 'required|string',
            'phone_id' => 'string',
            'phone_number' => 'exclude_if:phone_id,null|string',
            'phone_country' => 'exclude_if:phone_id,null|string',
            'logo' => 'nullable|mimes:' . AllowedFileTypes::IMAGE . '|max:' . AllowedFileSizes::IMAGE_SIZE
        ];
    }

    /**
     * Update client
     * 
     * @param array $data
     * @return Client
     * @throws \Exception
     */
    public function performAction(array $data): Client
    {
        $subscription = $this->checkValidStripeCustomer($data['stripe_cus_id']);

        $client = Client::findByBusinessKey($data['code']);
        $currentUser = Context::user()->get();

        if (isset($data['logo'])) {
            $fileDir = (new ResourceLocationBuilder())->setClient($client->code)->getLogoPath();
            if ($client->logo) {
                app(UpdateAttachment::class)->execute($client->logo, $data['logo'], $fileDir, $currentUser->getKey());
            } else {
                $logo = app(CreateAttachment::class)->execute($data['logo'], $fileDir, $currentUser->getKey());
                $client->logo()->associate($logo);
            }
        }

        if (empty($client->phone)) {
            $phone = Phone::create(['country_code' => $data['phone_country'], 'phone_number' => $data['phone_number']]);
            $client->phone()->associate($phone);
        } else {
            $client->phone->fill(['country_code' => $data['phone_country'], 'phone_number' => $data['phone_number']]);
            $client->phone->save();
        }

        $data = Arr::except($data, ['phone_number','phone_country', 'phone_id']);
        $data['plan_id'] = ModelHelper::convertBusinessKeyToId(SubscriptionPlan::class, $data['plan_id']);
        $client->fill($data);
        $client->subscription_period_start = $subscription->current_period_start;
        $client->subscription_period_end = $subscription->current_period_end;
        $client->save();

        return $client->refresh();
    }
}
