<?php


namespace App\Services\Client;

use App\Contracts\Validatable;
use App\Enum\AllowedFileSizes;
use App\Enum\AllowedFileTypes;
use App\Enum\RequireSSOOptions;
use App\Facades\Context;
use App\Helpers\ResourceLocationBuilder;
use App\Models\Client\Client;
use App\Models\Client\ClientConfiguration;
use App\Services\Attachment\CreateAttachment;
use App\Services\Attachment\UpdateAttachment;
use App\Services\BaseService;
use Carbon\Carbon;
use Illuminate\Validation\Rule;

class UpdateCompany extends BaseService implements Validatable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @param $args
     * @return array
     */
    public function getRules($args)
    {
        $rules = [
            'logo' => 'nullable|mimes:' . AllowedFileTypes::IMAGE . '|max:' . AllowedFileSizes::IMAGE_SIZE,
            'company_size' => 'string',
            'config.request_two_factor' => 'boolean',
            'config.admin_password_reset' => 'string',
            'config.delete_former_employees_after' => 'string',
            'config.console_login_timeout' => 'string',
            'config.enable_app_document_download_and_printing' => 'boolean',
            'config.config_primary_colour' => 'string',
            'config.config_secondary_colour' => 'string',
            'config.is_default_colour' => 'boolean',
            'config.is_enable_sso' => 'boolean',
            'config.show_enps_chart_for_managers' => 'boolean',
            'config.show_enps_chart_for_employees' => 'boolean',
            'config.auto_reply_messages' => 'string',
            'config.period_case_visibility' => 'string'
        ];
        if (isset($args['config']['is_enable_sso']) && $args['config']['is_enable_sso']) {
            $rules = array_merge($rules, [
                'config.sso_log_in_url' => 'required|string',
                'config.sso_log_out_url' => 'string|nullable',
                'config.sso_entityid_url' => 'required|string',
                'config.require_single_sign_on' => ['string', 'required', Rule::in(RequireSSOOptions::OPTIONS)],
                'config.sso_certificate' => 'required|string',
            ]);
        }
        return $rules;
    }

    /**
     * Update client
     *
     * @param array $args
     * @return Client
     */
    public function performAction(array $args): Client
    {
        $user = Context::user()->get();
        $client = Context::client()->get();
        if (isset($args['logo'])) {
            $fileDir = (new ResourceLocationBuilder())->setClient($client->code)->getLogoPath();
            if ($client->logo) {
                app(UpdateAttachment::class)->execute($client->logo, $args['logo'], $fileDir, $user->id);
            } else {
                $logo = app(CreateAttachment::class)->execute($args['logo'], $fileDir, $user->id);
                $client->logo()->associate($logo);
            }
        }
        $args = collect($args);
        $client->update($args->only(['company_size'])->toArray());
        if (isset($args['config'])) {
            foreach ($args['config'] as $key => $config) {
                $clientConfiguration = ClientConfiguration::where('name', $key)->first();
                if ($clientConfiguration) {
                    $configurationDetail = $client->clientConfigurations()
                        ->where('configuration_id', $clientConfiguration->getKey())
                        ->first();
                    // Only update when the config is changed or empty
                    if (empty($configurationDetail) || $configurationDetail->pivot->value != $config) {
                        $client->clientConfigurations()->sync([
                            $clientConfiguration->getKey() => [
                                'value' => $config,
                                'applied_date' => Carbon::now()
                            ]
                        ], false);
                    }
                }
            }
        }
        return $client->refresh();
    }
}