<?php


namespace App\Services\Dashboard\Message;


use App\Enum\ConversationStatues;
use App\Enum\PlanConfigurations;
use App\Enum\SegmentTypes;
use App\Facades\Context;
use App\Models\Message\AnonymousConversationCategory;
use App\Models\Message\Conversation;
use App\Models\Questionnaire\Questionnaire;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Questionnaire\QuestionnaireDiscussionAnswer;
use App\Models\Segment\Segment;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;
use Carbon\Carbon;
use stdClass;

/**
 * Class GetWhistleblowingDashboardData
 * @package App\Services\Dashboard\Message
 */
class GetWhistleblowingDashboardData extends BaseService
{
    /**
     * @param  array  $args
     * @return array
     */
    public function performAction(array $args): array
    {
        app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_WHISTLEBLOWING_CAPABILITIES);
        $currentClient = Context::client()->get();
        $qdaTable = QuestionnaireDiscussionAnswer::getModel()->getTable();
        $answerTable = QuestionnaireAnswer::getModel()->getTable();
        $whistleBlowings = Conversation::query()
            ->select("conversations.*")
            ->whistleblowing()
            ->ofClient($currentClient->id)
            ->leftJoin($qdaTable, "$qdaTable.conversation_id", "conversations.id")
            ->leftJoin($answerTable, "$answerTable.id", "$qdaTable.questionnaire_answer_id")
            ->where('conversations.status', '!=', ConversationStatues::DELETED)
            ->where(function($subQuery) use ($answerTable) {
                $subQuery->orWhere("$answerTable.is_sensitive", 0);
                $subQuery->orWhereNull("$answerTable.is_sensitive");
            })
            ->with([
                'anonymousConversationStatusLogs', 'anonymousConversationCategories', 'users.employee.location'
            ])->get();
        $totalCase = $whistleBlowings->count();
        $reviewingCase = $whistleBlowings->where('status', ConversationStatues::REVIEWING)->count();
        $escalatedCase = $whistleBlowings->where('status', ConversationStatues::ESCALATED)->count();
        $resolvingCase = $whistleBlowings->where('status', ConversationStatues::RESOLVING)->count();
        $inProgressCase = $whistleBlowings->where('status', ConversationStatues::ACTIVE)->count();
        $closedCase = $whistleBlowings->where('status', ConversationStatues::CLOSE)->count();
        $caseCloseRate = $totalCase === 0 ? 0 : round($closedCase / $totalCase * 100);
        $reopenTimes = 0;

        $anonymousConversationCategories = $this->getListAnonymousConversationCategories();
        $locationSegments = $this->getLocationSegments($currentClient->id);

        $whistleBlowings->each(function ($item) use (
            &$reopenTimes,
            &$anonymousConversationCategories,
            &$locationSegments
        ) {
            $anonymousUser = $item->users->where('pivot.is_anonymous', true)->first()->employee ?? null;

            if (isset($anonymousUser->location)
                && isset($locationSegments[$anonymousUser->location->getKey()])) {
                array_push($locationSegments[$anonymousUser->location->getKey()]->all_cases, $item);
                $locationSegments[$anonymousUser->location->getKey()]->total_cases++;
            }

            $openTimes = $item->anonymousConversationStatusLogs->where('status', ConversationStatues::ACTIVE)->count();
            //Conversation will mark as reopen conversation 2 more than 1 time change status to active.
            if ($openTimes > 1 && $item->status === ConversationStatues::ACTIVE) {
//                $reopenTimes += $openTimes - 1;
                /**
                 * We need to count reopened cases, not total reopens. And the current status is Active
                 */
                $reopenTimes++;
            }
            $currentAnonymousConversationStatusLogs = $item->anonymousConversationStatusLogs;
            $currentAnonymousConversationCategories = $item->anonymousConversationCategories;
            //Count anonymous category
            $currentAnonymousConversationCategories->each(function ($item) use (&$anonymousConversationCategories) {
                $anonymousConversationCategories[$item->id]->total_cases++;
            });

            //Count case closed time
            //Count case opened time
            $currentAnonymousConversationStatusLogs->each(function ($log, $index) use (
                $item,
                &$anonymousConversationCategories,
                $currentAnonymousConversationStatusLogs,
                $currentAnonymousConversationCategories
            ) {
                //If current is active and next item is closed, mean this case has been closed once
                if ($log->status === ConversationStatues::ACTIVE
                    && isset($currentAnonymousConversationStatusLogs[$index + 1])
                    && $currentAnonymousConversationStatusLogs[$index + 1]->status === ConversationStatues::CLOSE) {
                    $currentAnonymousConversationCategories->each(function ($category) use (
                        $anonymousConversationCategories,
                        $log,
                        $currentAnonymousConversationStatusLogs,
                        $index
                    ) {
                        $anonymousConversationCategories[$category->id]->day_closed += Carbon::now()->diffIndays($currentAnonymousConversationStatusLogs[$index + 1]->created_at);
                        $anonymousConversationCategories[$category->id]->case_closed++;
                    });
                }

                $latestLogIndex = count($currentAnonymousConversationStatusLogs) - 1;
                if ($latestLogIndex > -1 && $currentAnonymousConversationStatusLogs[$latestLogIndex]->status === ConversationStatues::ACTIVE) {
                    $currentAnonymousConversationCategories->each(function ($category) use (
                        $item,
                        $anonymousConversationCategories,
                        $log,
                        $currentAnonymousConversationStatusLogs
                    ) {
                        $diffDay = Carbon::now()->diffIndays($currentAnonymousConversationStatusLogs[0]->created_at);
                        $anonymousConversationCategories[$category->id]->day_opened += $diffDay;
                        $anonymousConversationCategories[$category->id]->case_opened++;
                        if (empty($anonymousConversationCategories[$category->id]->longest_case) || $anonymousConversationCategories[$category->id]->longest_case_days < $diffDay) {
                            $anonymousConversationCategories[$category->id]->longest_case_days = $diffDay;
                            $anonymousConversationCategories[$category->id]->longest_case = $item;
                        }

                        if (empty($anonymousConversationCategories[$category->id]->shortest_case) || $anonymousConversationCategories[$category->id]->shortest_case_days > $diffDay) {
                            $anonymousConversationCategories[$category->id]->shortest_case_days = $diffDay;
                            $anonymousConversationCategories[$category->id]->shortest_case = $item;
                        }
                    });
                }
            });
        });
        $anonymousConversationCategories->map(function ($item) {
            $item->closed_case_avg_time = $item->case_closed === 0 ? 0 : round($item->day_closed / $item->case_closed);
            $item->opened_case_avg_time = $item->case_opened === 0 ? 0 : round($item->day_opened / $item->case_opened);
            return $item;
        });
        $anonymousConversationCategories = array_values($anonymousConversationCategories->toArray());

        $locationSegments = array_values($locationSegments
            ->sortByDesc('total_cases')
            ->take(10)
            ->toArray());

        array_map(function ($item) {
            $item->case_closed = 0;
            $item->day_closed = 0;
            $item->day_opened = 0;
            $item->case_opened = 0;
            $item->longest_case = null;
            $item->longest_case_days = 0;
            $item->shortest_case = null;
            $item->shortest_case_days = 0;
            foreach ($item->all_cases as $one_case) {
                if ($one_case->status === ConversationStatues::CLOSE) {
                    $item->case_closed++;
                    $closed = array_values(array_filter($one_case->anonymousConversationStatusLogs->toArray(), function ($log) { return $log['status'] === ConversationStatues::CLOSE; }));
                    if (count($closed) > 0) {
                        $item->day_closed += Carbon::now()->diffIndays($closed[count($closed) - 1]['created_at']);
                    }
                }

                if ($one_case->status === ConversationStatues::ACTIVE) {
                    $item->case_opened++;
                    $opened = array_values(array_filter($one_case->anonymousConversationStatusLogs->toArray(), function ($log) { return $log['status'] === ConversationStatues::ACTIVE; }));
                    if (count($opened) > 0) {
                        $diffDay = Carbon::now()->diffIndays($opened[0]['created_at']);
                        $item->day_opened += $diffDay;
                        if (empty($item->longest_case) || $item->longest_case_days < $diffDay) {
                            $item->longest_case_days = $diffDay;
                            $item->longest_case = $one_case;
                        }

                        if (empty($item->shortest_case) || $item->shortest_case_days > $diffDay) {
                            $item->shortest_case_days = $diffDay;
                            $item->shortest_case = $one_case;
                        }
                    }
                }
            }
            $item->closed_case_avg_time = $item->case_closed === 0 ? 0 : round($item->day_closed / $item->case_closed);
            $item->opened_case_avg_time = $item->case_opened === 0 ? 0 : round($item->day_opened / $item->case_opened);
            return $item;
        }, $locationSegments);

        return [
            'total_cases'                       => $totalCase,
            'in_review_case'                    => $reviewingCase,
            'escalated_case'                    => $escalatedCase,
            'resolving_case'                    => $resolvingCase,
            'in_progress_case'                  => $inProgressCase,
            'closed_case'                       => $closedCase,
            'reopened'                          => $reopenTimes,
            'case_close_rate'                   => $caseCloseRate,
            'anonymous_conversation_categories' => $anonymousConversationCategories,
            'locations'                         => $locationSegments,
        ];
    }

    /**
     * @return AnonymousConversationCategory[]|\Illuminate\Database\Eloquent\Collection|\Illuminate\Support\Collection
     */
    public function getListAnonymousConversationCategories()
    {
        return AnonymousConversationCategory::all()
            ->map(function ($item) {
                $category = new stdClass();
                $category->id = $item->id;
                $category->name = $item->name;
                $category->total_cases = 0;

                $category->day_closed = 0;
                $category->case_closed = 0;

                $category->day_opened = 0;
                $category->case_opened = 0;

                $category->longest_case = null;
                $category->longest_case_days = 0;

                $category->shortest_case = null;
                $category->shortest_case_days = 0;

                return $category;
            })
            ->mapWithKeys(function ($item) {
                return [$item->id => $item];
            });
    }

    /**
     * @param $clientId
     * @return \Illuminate\Database\Eloquent\Builder[]|\Illuminate\Database\Eloquent\Collection|\Illuminate\Support\Collection
     */
    public function getLocationSegments($clientId)
    {
        return Segment::query()
            ->where('client_id', $clientId)
            ->where('segment_type_id', SegmentTypes::LOCATION)
            ->get()
            ->map(function ($item) {
                $location = new stdClass();
                $location->id = $item->id;
                $location->name = $item->name;
                $location->total_cases = 0;
                $location->all_cases = [];

                $location->day_closed = 0;
                $location->case_closed = 0;

                $location->day_opened = 0;
                $location->case_opened = 0;

                $location->longest_case = null;
                $location->longest_case_days = 0;

                $location->shortest_case = null;
                $location->shortest_case_days = 0;
                return $location;
            })
            ->mapWithKeys(function ($item) {
                return [$item->id => $item];
            });
    }

}