<?php

namespace App\Services\Employee;

use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\EmailNotifications;
use App\Enum\ErrorCode;
use App\Enum\FilterTypes;
use App\Enum\Roles;
use App\Facades\Context;
use App\Helpers\FilterFluent;
use App\Helpers\ObjectHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\SortFluent;
use App\Http\Resources\User\EmployeeExported as EmployeeResource;
use App\Jobs\SendNotificationEmail;
use App\Models\Library\DocumentProfile;
use App\Models\User\User;
use App\Rules\ThrowableUniqueRule;
use App\Services\BaseService;
use App\Services\UserAccess\GetAccessByRole;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\DB;

class BulkUpdateContentProfiles extends BaseService implements Validatable
{

    private $updateCacheExpire;
    private $updateCacheKey;

    function performAction(array $data)
    {
        ini_set('max_execution_time', 900);
        $options = Context::request()->getOptions();
        $client = Context::client()->get();
        $codes = $data['codes'] ?? 'all';

        $this->updateCacheExpire = \Carbon\Carbon::now()->addMinutes(30);
        $this->updateCacheKey = "update_" . $client->getKey() . "_" . $data['id'];
        $processKey = $this->updateCacheKey . "_processing";
        $doneKey = $this->updateCacheKey . "_done";

        $documentProfiles = DB::table('document_profiles')
            ->join('document_profile_employee', 'document_profiles.id', 'document_profile_employee.document_profile_id')
            ->select('document_profile_employee.employee_id', 'document_profiles.name', 'document_profiles.id')
            ->where('document_profiles.client_id', $client->getKey());

        $query = User::with('employee.client')
            ->join('employees', 'users.id', 'employees.user_id')
            ->leftJoin('segments as s1', 'employees.department_id', 's1.id')
            ->leftJoin('segments as s2', 'employees.team_id', 's2.id')
            ->leftJoin('segments as s3', 'employees.location_id', 's3.id')
            ->leftJoin('segments as s4', 'employees.gender_id', 's4.id')
            ->leftJoinSub($documentProfiles, 'dpl', function ($join) {
                $join->on('employees.id', '=', 'dpl.employee_id');
            })
            ->groupBy('users.id');
        if ($codes === 'all' && isset($data['excludeIds']) && $data['excludeIds']) {
            $query->whereNotIn('users.code', $data['excludeIds']);
        } else if (is_array($codes)) {
            $query->whereIn('users.code', $data['codes']);
        }
        $query->where('employees.client_id', $client->getKey());
        $query->select('users.*', 's1.name as department_name', 's2.name as team_name', 's3.name as location_name', DB::raw('group_concat(dpl.id) as document_profiles'));

        FilterFluent::init()
            ->setQuery($query)
            ->setTable('users')
            ->setFilterOptions($options)
            ->setAllowedFilters([
                'name',
                'department',
                'team',
                'location',
                'status',
                'gender',
                'job_title',
                'document_profiles',
                'social_security_number',
                'race',
                'employee_client_id',
                'phone_number',
                'last_login',
                'created_at'
            ])
            ->validate()
            ->removeFilter([
                'name',
                'department',
                'team',
                'location',
                'status',
                'gender',
                'job_title',
                'document_profiles',
                'social_security_number',
                'race',
                'employee_client_id',
                'phone_number',
                'last_login',
                'created_at'
            ])
            ->setCustomFilter('name', function ($query, $type, $col, $value) {
                FilterFluent::applyCombinedColumnFilter($query, $type, 'users.surname', 'users.first_name', $value);
            })
            ->setCustomFilter('last_login', function ($query, $type, $col, $value) {
                if ($value == "Yes") {
                    $query->whereNotNull('last_login');
                } else {
                    $query->whereNull('last_login');
                }
            })
            ->setCustomFilter('created_at', function ($query, $type, $col, $value) {
                $dateValue = explode(";", $value);
                $fromDate = $dateValue[0];
                $toDate = Carbon::parse($dateValue[1])->addDay(1)->toDateTimeString();
                FilterFluent::applyBasicFilter($query, FilterTypes::GREATER_THAN_OR_EQUAL, 'users.created_at', $fromDate);
                FilterFluent::applyBasicFilter($query, FilterTypes::LESS_THAN_OR_EQUAL, 'users.created_at', $toDate);
            })
            ->setCustomFilter('social_security_number', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'social_security_number', $value, 'employees');
            })
            ->setCustomFilter('race', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'race', $value, 'employees');
            })
            ->setCustomFilter('job_title', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'job_title', $value, 'employees');
            })
            ->setCustomFilter('employee_client_id', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'employee_client_id', $value, 'employees');
            })
            ->setCustomFilter('phone_number', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'phone_number', $value, 'employees');
            })
            ->setCustomFilter('department', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's1');
            })
            ->setCustomFilter('team', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's2');
            })
            ->setCustomFilter('location', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's3');
            })
            ->setCustomFilter('gender', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's4');
            })
            ->setCustomFilter('status', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'status', $value, 'employees');
            })
            ->setCustomFilter('document_profiles', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 'dpl');
            })
            ->applyFilter();

        SortFluent::init()
            ->setQuery($query)
            ->setTable('users')
            ->setSortOptions($options)
            ->setAllowedSorts([
                'name',
                'team',
                'department',
                'location',
                'job_title',
                'gender',
                'email',
                'status',
                'document_profiles',
                'social_security_number',
                'race',
                'employee_client_id',
                'phone_number',
                'last_login',
                'created_at'
            ])
            ->validate()
            ->removeSort(['name', 'team', 'department', 'location', 'job_title', 'gender', 'status', 'document_profiles', 'social_security_number', 'race', 'employee_client_id', 'phone_number', 'last_login', 'created_at'])
            ->setCustomSort('name', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'surname', $direction, 'users');
                SortFluent::applyBasicSort($query, 'first_name', $direction, 'users');
            })
            ->setCustomSort('last_login', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'last_login', $direction, 'users');
            })
            ->setCustomSort('created_at', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'created_at', $direction, 'users');
            })
            ->setCustomSort('department', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'name', $direction, 's1');
            })
            ->setCustomSort('team', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'name', $direction, 's2');
            })
            ->setCustomSort('location', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'name', $direction, 's3');
            })
            ->setCustomSort('gender', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'name', $direction, 's4');
            })
            ->setCustomSort('status', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'status', $direction, 'employees');
            })
            ->setCustomSort('social_security_number', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'social_security_number', $direction, 'employees');
            })
            ->setCustomSort('job_title', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'job_title', $direction, 'employees');
            })
            ->setCustomSort('race', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'race', $direction, 'employees');
            })
            ->setCustomSort('employee_client_id', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'employee_client_id', $direction, 'employees');
            })
            ->setCustomSort('phone_number', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'phone_number', $direction, 'employees');
            })
            ->setCustomSort('document_profiles', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'name', $direction, 'dpl');
            })
            ->applySort();

        $users = $query->get();

        $employees = EmployeeResource::collection($users)->jsonSerialize();

        $documentProfiles = array_filter(ObjectHelper::nullOrValue($data, 'document_profiles') ?? []);
        $documentProfileIds = CreateEmployee::getValidDocumentProfileIds($documentProfiles);

        foreach ($employees as $employee) {
            $prevDocuments = $employee->documentProfiles->pluck('id');

            $code = $employee["code"];
            $document_profiles = isset($employee["document_profiles"]) ? explode(",", $employee["document_profiles"]) : [];
            $document_profiles = array_unique(array_merge($documentProfileIds, array_map('intval', $document_profiles)));
            $user = User::findByBusinessKey($code);
            $employee = $user->employee;
            PermissionHelper::hasAccessOn($employee);
            $employee->documentProfiles()->sync($document_profiles);
            $user->refresh();
            Cache::put($processKey, true, $this->updateCacheExpire);


            // Send content profile access update notification to employees
            $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
            $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];
            $updatedProfileIds = array_diff($document_profiles, $prevDocuments);
            $updatedProfiles = DocumentProfile::whereIn('id', $updatedProfileIds)->get();
            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_FILE]) {
                foreach ($users as $user) {
                    foreach ($updatedProfiles as $profile) {
                        SendNotificationEmail::dispatch($user, EmailNotifications::PROFILE_ASSIGNMENT, ['profile' => $profile->name]);
                    }
                }
            }
        }
        Cache::put($processKey, false, $this->updateCacheExpire);
        Cache::put($doneKey, true, $this->updateCacheExpire);
    }

    public function getUpdateStatus($code)
    {
        $clientId = Context::client()->get()->id;
        $key = "update_" . $clientId . "_" . $code;
        return [
            'inProcessing' => Cache::get($key . "_processing") ?? true,
            'isCompleted' => Cache::get($key . "_done") ?? false,
        ];
    }

    public function getRules(array $args)
    {
        // TODO: Implement getRules() method.
        return [
            'document_profiles' => ['array'],
        ];
    }
}