<?php

namespace App\Services\Employee;

use App\Contracts\Validatable;
use App\Enum\EmployeeStatus;
use App\Enum\ErrorCode;
use App\Enum\PlanConfigurations;
use App\Exceptions\InvalidRequestException;
use App\Exceptions\UnauthorizedHttpException;
use App\Facades\Context;
use App\Helpers\PermissionHelper;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;
use App\Services\User\RevokeUserToken;
use Illuminate\Validation\Rule;

class ChangeEmployeeStatus extends BaseService implements Validatable
{

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args)
    {
        return [
            'code' => 'required',
            'status' => ['required', Rule::in([EmployeeStatus::ACTIVE, EmployeeStatus::INACTIVE])]
        ];
    }

    /**
     * Change employee status
     *
     * @param array $data
     * @param string $status
     * @return User
     */
    public function performAction(array $data): User
    {
        $user = User::findByBusinessKey($data['code']);
        if($user->getKey() === Context::user()->get()->getKey()){
            if (isset($data['is_multiple']) && $data['is_multiple']) {
                return $user->refresh();
            }
            throw new InvalidRequestException('Your request is invalid.', ErrorCode::UNAUTHORIZED);
        }
        $employee = $user->employee;
        PermissionHelper::hasAccessOn($employee);
        // If status as same as current status, do nothing
        if ($data['status'] === $employee->status) {
            return $user;
        }
        if ($employee->status === EmployeeStatus::INCOMPLETE) {
            if (isset($data['is_multiple']) && $data['is_multiple']) {
                return $user->refresh();
            }
            throw new InvalidRequestException('Cannot enable incomplete employee.');
        }
        if ($data['status'] === EmployeeStatus::ACTIVE) {
            // Default create is active
            app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_EMPLOYEE);
        }
        $employee->status = $data['status'];
        $employee->save();
        
        if ($data['status'] === EmployeeStatus::INACTIVE) {
            app(RevokeUserToken::class)->execute($user);
        }
        
        return $user->refresh();
    }
}
