<?php

namespace App\Services\Employee;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\AllowedFileSizes;
use App\Enum\AllowedFileTypes;
use App\Enum\BooleanStatus;
use App\Enum\EmployeeStatus;
use App\Enum\EmploymentTypes;
use App\Enum\ErrorCode;
use App\Enum\PlanConfigurations;
use App\Enum\PowerSearchActions;
use App\Enum\PowerSearchTypes;
use App\Enum\Roles;
use App\Facades\Context;
use App\Helpers\ObjectHelper;
use App\Helpers\PermissionHelper;
use App\Jobs\IndexPowerSearch;
use App\Jobs\SendNewFrontUserEmail;
use App\Jobs\SendVerifyEmail;
use App\Models\Library\DocumentProfile;
use App\Models\PowerSearch\SearchableObject;
use App\Models\Segment\Segment;
use App\Models\User\User;
use App\Rules\ThrowableUniqueRule;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;
use Illuminate\Validation\Rule;

class CreateEmployee extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @param $data
     * @return array
     */
    public function getRules($data)
    {
        $rules = [
            'first_name'          => ['required', 'string', 'max:255'],
            'surname'             => ['required', 'string', 'max:255'],
            'email'               => ['required', 'email', 'max:255', new ThrowableUniqueRule(ErrorCode::EMAIL_ALREADY_USED, 'users', 'email')],
            'date_of_birth'       => ['date'],
            'start_date'          => ['date'],
            'document_profiles'   => ['array'],
            'job_title'           => ['nullable', 'string'],
            'manager'             => ['nullable', 'exists:users,code'],
            'secondary_manager'   => ['nullable', 'exists:users,code'],
            'manager_notifications'=> ['nullable', 'string'],
            'social_security_number'          => ['nullable', 'string', 'max:255'],
            'race'          => ['nullable', 'string', 'max:255'],
            'employee_client_id'   => ['nullable', 'string', 'max:255'],
            'phone_number' => ['nullable', 'string', 'max:255'],
            'employment_type'     => [Rule::in([
                EmploymentTypes::FULL_TIME,
                EmploymentTypes::PART_TIME,
                EmploymentTypes::CONTRACTOR,
                EmploymentTypes::PARTNER,
                EmploymentTypes::VENDOR,
                EmploymentTypes::CONTACT,
                EmploymentTypes::CANDIDATE,
                EmploymentTypes::CUSTOMER,
                EmploymentTypes::AGENT,
                EmploymentTypes::INTERN,
                EmploymentTypes::OTHERS,
            ])],
            'role'                => ['required', Rule::in([
                Roles::ADMIN,
                Roles::FRONT_USER,
            ])],
            'avatar'              => 'nullable|mimes:'.AllowedFileTypes::IMAGE.'|max:'.AllowedFileSizes::IMAGE_SIZE,
            'external_avatar_url' => 'nullable',
            'status'              => ['nullable', Rule::in([
                EmployeeStatus::ACTIVE,
                EmployeeStatus::INACTIVE,
                EmployeeStatus::INCOMPLETE,
            ])]
        ];

        $role = $data['role'];

        if ($role === Roles::FRONT_USER) {
            $additionalFieldRule = (isset($data['is_additional_field_required']) &&
                $data['is_additional_field_required']) ? 'required' : 'nullable';
            $rules['date_of_birth'][] = $additionalFieldRule;
            $rules['start_date'][] = $additionalFieldRule;
            $rules['employment_type'][] = $additionalFieldRule;
            $rules['document_profiles'][] = $additionalFieldRule;
            $rules['document_profiles.*'][] = $additionalFieldRule;
        }

        return $rules;
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Create employee
     *
     * @param  array  $data
     * @return User
     * @throws \Exception
     */
    public function performAction(array $data): User
    {
        $role = $data['role'];

        //Default create is active
        app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_EMPLOYEE);
        if ($role === Roles::ADMIN) {
            app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_ADMINISTRATOR);
        }

        $clientId = Context::client()->get()->getKey();
        $data['client_id'] = $clientId;

        // Check client has access on document profiles
        $documentProfiles = array_filter(ObjectHelper::nullOrValue($data, 'document_profiles') ?? []);
        $documentProfileIds = self::getValidDocumentProfileIds($documentProfiles);

        $password = Str::random(8);

        $userData = [
            'first_name' => $data['first_name'],
            'surname'    => $data['surname'],
            'email'      => $data['email'],
            'password'   => Hash::make($password),
        ];

        $externalAvatarUrl = ObjectHelper::nullOrValue($data, 'external_avatar_url');
        if ($externalAvatarUrl) {
            $userData[] = $externalAvatarUrl;
        }

        $createdUser = User::create($userData);

        // Add avatar
        if (isset($data['avatar'])) {
            app(UpdateAvatar::class)->execute(['avatar' => $data['avatar'], 'user_id' => $createdUser->getKey()]);
        }

        $genderSegmentId = ObjectHelper::nullOrValue($data, 'gender_id');
        $departmentSegmentId = ObjectHelper::nullOrValue($data, 'department_id');
        $teamSegmentId = ObjectHelper::nullOrValue($data, 'team_id');
        $locationSegmentId = ObjectHelper::nullOrValue($data, 'location_id');

        $manager = null;
        $secondaryManager = null;
        $managerCode = ObjectHelper::nullOrValue($data, 'manager');
        $secondaryManagerCode = ObjectHelper::nullOrValue($data, 'secondary_manager');
        if ($managerCode) {
            $manager = User::whereHas('employee', function ($q) use ($clientId) {
                $q->where('client_id', $clientId);
            })->whereHas('roles', function ($q) {
                $q->where('name', Roles::MANAGER);
            })->where('code', $managerCode)->first();
        }

        if ($secondaryManagerCode) {
            $secondaryManager = User::whereHas('employee', function ($q) use ($clientId) {
                $q->where('client_id', $clientId);
            })->whereHas('roles', function ($q) {
                $q->where('name', Roles::MANAGER);
            })->where('code', $secondaryManagerCode)->first();
        }

        $managerNotificationIds = [];
        $managerNotificationCodes = ObjectHelper::nullOrValue($data, 'manager_notifications');
        if ($managerNotificationCodes) {
            $managerNotificationCodeArr = explode(';', $managerNotificationCodes);
            $managerNotifications = User::whereHas('employee', function ($q) use ($clientId) {
                $q->where('client_id', $clientId);
            })->whereIn('code', $managerNotificationCodeArr)->get();

            foreach ($managerNotifications as $managerNotification) {
                array_push($managerNotificationIds, $managerNotification->id);
            }
        }

        $createdEmployee = $createdUser->employee()->create([
            'date_of_birth'       => ObjectHelper::nullOrDate($data, 'date_of_birth'),
            'start_date'          => ObjectHelper::nullOrDate($data, 'start_date'),
            'job_title'           => ObjectHelper::nullOrValue($data, 'job_title'),
            'employment_type'     => ObjectHelper::nullOrValue($data, 'employment_type'),
            'client_id'           => $data['client_id'],
            'gender_id'           => $genderSegmentId,
            'department_id'       => $departmentSegmentId,
            'team_id'             => $teamSegmentId,
            'location_id'         => $locationSegmentId,
            'manager_id'          => $manager ? $manager->getKey() : null,
            'secondary_manager_id'=> $secondaryManager ? $secondaryManager->getKey() : null,
            'manager_notification_id' => sizeof($managerNotificationIds) > 0 ? implode(";", $managerNotificationIds) : null,
            'is_default_password' => BooleanStatus::YES,
            'profile_pic'         => $createdUser->profile_pic,
            'status'              => ObjectHelper::nullOrValue($data, 'status') ?? EmployeeStatus::ACTIVE,
            'social_security_number' => ObjectHelper::nullOrValue($data, 'social_security_number'),
            'race' => ObjectHelper::nullOrValue($data, 'race'),
            'employee_client_id' => ObjectHelper::nullOrValue($data, 'employee_client_id'),
            'phone_number' => ObjectHelper::nullOrValue($data, 'phone_number'),
        ]);

        $createdEmployee->documentProfiles()->attach($documentProfileIds);

        $createdUser->assignRole($role);

        // Add employee to segment
        $this->addEmployeeToSegment($genderSegmentId, $createdUser);
        $this->addEmployeeToSegment($departmentSegmentId, $createdUser);
        $this->addEmployeeToSegment($teamSegmentId, $createdUser);
        $this->addEmployeeToSegment($locationSegmentId, $createdUser);

        if ($data['action'] === 'invite') {
            // Dispatch send mail job
            switch ($role) {
                case Roles::FRONT_USER:
                    SendNewFrontUserEmail::dispatch($createdUser, $password, true);
                    break;
                case Roles::ADMIN:
                    SendVerifyEmail::dispatch($createdUser);
            }
        }
        

        IndexPowerSearch::dispatch($createdUser->code, PowerSearchTypes::EMPLOYEE, PowerSearchActions::ADD, $createdUser->employee->client_id);
        $newText = $createdUser->employee->social_security_number;
        $clientId = $createdUser->employee->client_id;
        $code = $createdUser->code;
        $this->savePowerSearch($newText, $code, $clientId, PowerSearchTypes::SOCIAL_SECURITY_NUMBER);
        $this->savePowerSearch($createdUser->employee->employee_client_id, $code, $clientId, PowerSearchTypes::EMPLOYEE_CLIENT_ID);
        $this->savePowerSearch($createdUser->employee->phone_number, $code, $clientId, PowerSearchTypes::PHONE_NUMBER);
        $this->savePowerSearch($createdUser->email, $code, $clientId, PowerSearchTypes::EMPLOYEE_EMAIL);
        return $createdUser->refresh();
    }

    private function savePowerSearch($newText, $code, $clientId, $type) {
        if ($newText && !empty($clientId)) {
            $searchableObj = SearchableObject::where('client_id', $clientId)
                ->where('object_type', $type)
                ->where('object_id', $code)->first();
            if ($searchableObj) {
                $searchableObj->object_content = $newText;
            } else {
                $searchableObj = new SearchableObject();
                $searchableObj->object_content = $newText;
                $searchableObj->client_id = $clientId;
                $searchableObj->object_type = $type;
                $searchableObj->object_id = $code;
            }
            $searchableObj->save();
        }
    }

    public function addEmployeeToSegment($segmentId, User $user)
    {
        $clientId = Context::client()->get()->getKey();
        $segment = Segment::find($segmentId);
        if ($segment && $segment->getClientId() === $clientId) {
            $segment->users()->attach($user->getKey());
        }
    }

    public static function getValidDocumentProfileIds(array $documentProfiles)
    {
        $documentProfileIds = [];
        foreach ($documentProfiles as $documentProfileId) {
            $documentProfile = DocumentProfile::findOrFail($documentProfileId);
            PermissionHelper::hasAccessOn($documentProfile);
            $documentProfileIds[] = $documentProfile->getKey();
        }

        return $documentProfileIds;
    }
}
