<?php

namespace App\Services\Employee;

use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\ErrorCode;
use App\Enum\PowerSearchActions;
use App\Enum\PowerSearchTypes;
use App\Enum\QualeeStatus;
use App\Enum\Roles;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\DebugHelper;
use App\Helpers\PandaDocHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\ResourceLocationBuilder;
use App\Jobs\IndexPowerSearch;
use App\Models\PowerSearch\SearchableObject;
use App\Models\User\User;
use App\Services\Action\DeleteActionIfNoUser;
use App\Services\ActionRequest\ClearActionRequestSubmission;
use App\Services\Attachment\DeleteAttachment;
use App\Services\BaseService;
use App\Services\Client\AssignWBConversationToAnotherAdmin;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;

class DeleteEmployee extends BaseService implements Validatable, Transactionable
{

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args)
    {
        return [
            'code' => 'required'
        ];
    }

    /**
     * Delete employee
     *
     * @param array $data
     * @return bool
     */
    public function performAction(array $data): bool
    {

        $user = User::findByBusinessKey($data['code']);
        if ($user->getKey() === Context::user()->get()->getKey()) {
            if (isset($data['is_multiple']) && $data['is_multiple']) {
                return false;
            }
            throw new InvalidRequestException('Your request is invalid.', ErrorCode::UNAUTHORIZED);
        }
        $employee = $user->employee;
        PermissionHelper::hasAccessOn($employee);

        IndexPowerSearch::dispatch($user->code, PowerSearchTypes::EMPLOYEE, PowerSearchActions::DELETE, $employee->client_id);
        SearchableObject::where('client_id', $employee->client_id)
            ->where('object_type', PowerSearchTypes::SOCIAL_SECURITY_NUMBER)
            ->where('object_id', $user->code)->delete();
        SearchableObject::where('client_id', $employee->client_id)
            ->where('object_type', PowerSearchTypes::EMPLOYEE_CLIENT_ID)
            ->where('object_id', $user->code)->delete();
        SearchableObject::where('client_id', $employee->client_id)
            ->where('object_type', PowerSearchTypes::PHONE_NUMBER)
            ->where('object_id', $user->code)->delete();

        // Delete user's data on S3
        $userPath = (new ResourceLocationBuilder())
            ->setClient($user->getClient()->getBusinessKey())
            ->setUser($user->getBusinessKey())
            ->getUserPath();
        Storage::disk('s3')->deleteDirectory($userPath);

        $this->deleteUserData($user);
        $user->employee()->forceDelete();
        $user->forceDelete();

        return true;
    }

    /**
     * @param User $user
     * @return void
     */
    public function deleteUserData(User $user)
    {
        $actions = $user->actions;
        $user->segments()->detach();
        $user->journeys()->detach();
        $user->actions()->detach();
        $user->actionSequences()->detach();
        $user->userEvent()->detach();
        $user->tokens()->delete();
        $user->appRatings()->delete();
        $user->readAlertMessages()->detach();

        $submissions = $user->actionRequestSubmissions()->with('actionRequest')->get();
        foreach ($submissions as $submission) {
            app(ClearActionRequestSubmission::class)->execute([
                'action_request' => $submission->actionRequest,
                'user_id' => $user->getKey()
            ]);
        }
        foreach ($actions as $action) {
            app(DeleteActionIfNoUser::class)->execute(['action' => $action]);
        }

        $this->deleteFormDocuments($user);
        $this->deleteUserMessageData($user);
    }

    public function deleteUserMessageData($user)
    {
        //assign WB conversations to another admin
        if ($user->hasRole(Roles::WHISTLEBLOWER_ADMIN)) {
            app(AssignWBConversationToAnotherAdmin::class)->execute(['user' => $user]);
        }
        $attachments = collect([]);
        $user->conversations()->wherePivot('status', QualeeStatus::ACTIVE)->get()->each(
            function ($conversation) use ($attachments) {
                Log::info($conversation);
                $attachments->push(
                    $conversation->rawMessages->pluck('attachment')->unique()->filter(
                        function ($value) {
                            return !!$value;
                        }
                    )
                );
                $conversation->rawMessages()->delete();
                $conversation->users()->detach();
            }
        );
        $user->conversations()->delete();

        $attachments->push($user->messages->pluck('attachment')->unique()->filter(function($value) {
            return !!$value;
        }));

        $user->messages()->delete();

        $attachments->collapse()->each(function($item) {
            app(DeleteAttachment::class)->execute($item);
        });
    }

    public function deleteFormDocuments(User $user)
    {
        $client = $user->getClient();
        if (!empty($client->formAuthenticate)) {
            $pandaDocHelper = new PandaDocHelper();
            $pandaDocHelper->setToken($client->formAuthenticate->access_token);

            foreach ($user->documents as $document) {
                $pandaDocHelper->deleteDocument($document->document_id);
            }
        }

        $user->documents()->delete();
    }
}
