<?php

namespace App\Services\Employee;

use App\Enum\EmploymentTypes;
use App\Enum\Roles;
use App\Enum\SegmentTypes;
use App\Exceptions\InvalidRequestException;
use App\Models\Library\DocumentProfile;
use App\Models\Segment\Segment;
use App\Models\User\User;
use Brick\Math\Exception\NumberFormatException;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use Maatwebsite\Excel\Concerns\Importable;
use Maatwebsite\Excel\Concerns\SkipsErrors;
use Maatwebsite\Excel\Concerns\SkipsFailures;
use Maatwebsite\Excel\Concerns\SkipsOnError;
use Maatwebsite\Excel\Concerns\SkipsOnFailure;
use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\WithChunkReading;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use Maatwebsite\Excel\Concerns\WithValidation;
use Maatwebsite\Excel\Events\BeforeImport;
use Maatwebsite\Excel\Validators\Failure;
use Safe\DateTime;
use Safe\Exceptions\DatetimeException;
use App\Models\Journey\Journey;
use App\Models\Action\ActionSequence;
use App\Services\Tool\Journey\AddEmployees as AddEmployeeToJourney;
use App\Services\Action\ActionSequence\AddEmployees as AddEmployeeToActionSequence;
use Throwable;

class EmployeeImport implements ToCollection, WithHeadingRow, WithChunkReading, WithValidation, SkipsOnFailure, WithEvents, SkipsOnError
{
    use Importable, SkipsFailures, SkipsErrors;

    const GLOBAL_DATE_FORMAT = 'Y-m-d';

    private $size     = 0;
    private $success  = 0;
    private $skip = 0;
    private $skipRows = [];
    private $clientId = null;
    private $genderSegment;
    private $departmentSegment;
    private $teamSegment;
    private $locationSegment;
    private $documentProfile;
    private $importCacheKey;
    private $importCacheExpire;

    public function importWithClientId($clientId, $filePath = null, string $disk = null, string $readerType = null, $id = null)
    {
        $this->clientId = $clientId;
        $this->importCacheExpire = Carbon::now()->addMinutes(30);
        $this->importCacheKey = "import_" . $this->clientId . "_" . $id;
        $this->genderSegment = $this->getSegmentNameWithIdWithByType(SegmentTypes::GENDER);
        $this->departmentSegment = $this->getSegmentNameWithIdWithByType(SegmentTypes::DEPARTMENT);
        $this->teamSegment = $this->getSegmentNameWithIdWithByType(SegmentTypes::TEAM);
        $this->locationSegment = $this->getSegmentNameWithIdWithByType(SegmentTypes::LOCATION);
        $this->documentProfile = $this->getDocumentProfiles();
        return $this->import($filePath, $disk, $readerType);
    }

    public function prepareForValidation($data)
    {
        $data['employment_type'] = $this->getValueIgnoreCase($data['employment_type'], [
            EmploymentTypes::FULL_TIME,
            EmploymentTypes::PART_TIME,
            EmploymentTypes::CONTRACTOR,
            EmploymentTypes::PARTNER,
            EmploymentTypes::VENDOR,
            EmploymentTypes::CONTACT,
            EmploymentTypes::CANDIDATE,
            EmploymentTypes::CUSTOMER,
            EmploymentTypes::AGENT,
            EmploymentTypes::INTERN,
            EmploymentTypes::OTHERS,
        ]);
        return $data;
    }

    public function registerEvents(): array
    {
        return [
            BeforeImport::class => function (BeforeImport $event) {
                $totalRows = $event->getReader()->getTotalRows();
                if (!empty($totalRows)) {
                    Cache::put($this->importCacheKey . '_total', $totalRows['Worksheet'] - 1, $this->importCacheExpire);
                }
            }
        ];
    }

    public function collection(Collection $rows)
    {
        $successKey = $this->importCacheKey .  "_success";
        Cache::put($this->importCacheKey . '_failures', $this->failures(), $this->importCacheExpire);
        $skipKey = $this->importCacheKey .  "_skip";
        $skipRowsKey = $this->importCacheKey .  "_skipErrors";

        $this->success = 0;
        $this->size    = $rows->count();
        $failedKeys = array_column(array_column($this->failures()->jsonSerialize(), "values"), "email");
        $emails = [];
        $index = 1;

        foreach ($rows as $row) {
            $index++;
            $email = $row['email'];
            // Skip importing current row if have validation failure
            if (in_array($row['email'], $failedKeys)) {
                $this->skip++;
                $skip = Cache::get($skipKey) ?? 0;
                Cache::put($skipKey, $skip + 1, $this->importCacheExpire);
                $skipErrors = Cache::get($skipRowsKey) ?? [];
                $fields = [];
                if (in_array($email, $emails)) {
                    $fields[] = array("fieldName" => "Email", "field" => 'email', "errorCode" => ["EMAIL_DUPLICATED"]);
                } else {
                    $emails[] = $email;
                }
                $data = array("rowNum" => $index, "fields" => $fields);
                $skipErrors[] = $data;
                Cache::put($skipRowsKey, $skipErrors, $this->importCacheExpire);
                continue;
            }
            if (in_array($email, $emails)) {
                // Handle duplicate email found
                // You can throw an exception, log an error, or perform any other action
                $this->skip++;
                $skip = Cache::get($skipKey) ?? 0;
                Cache::put($skipKey, $skip + 1, $this->importCacheExpire);
                $skipErrors = Cache::get($skipRowsKey) ?? [];
                $fields = [];
                $fields[] = array("fieldName" => "Email", "field" => 'email', "errorCode" => ["EMAIL_DUPLICATED"]);
                $data = array("rowNum" => $index, "fields" => $fields);
                $skipErrors[] = $data;
                Cache::put($skipRowsKey, $skipErrors, $this->importCacheExpire);
                continue;
            } else {
                $emails[] = $email;
            }
            $data = [
                'first_name'          => (string)$row['first_name'],
                'surname'             => (string)$row['surname'],
                'email'               => trim($row['email']),
                'date_of_birth'       => $row['date_of_birth'],
                'start_date'          => $row['start_date'],
                'job_title'           => (string)$row['job_title'],
                'employment_type'     => $row['employment_type'],
                'role'                => Roles::FRONT_USER,
                'document_profiles'   => [$this->documentProfile[$row['document_profile']] ?? null],

                // convert email to code
                'manager'             => !empty($row['manager']) ? $this->getUserCodeFromEmail($row['manager']) : null,
                "secondary_manager"   => !empty($row['secondary_manager']) ? $this->getUserCodeFromEmail($row['secondary_manager']) : null,

                // convert to segment id
                'gender_id'           => $this->genderSegment[$row['gender']] ?? null,
                'department_id'       => $this->departmentSegment[$row['department']] ?? null,
                'team_id'             => $this->teamSegment[$row['team']] ?? null,
                'location_id'         => $this->locationSegment[$row['location']] ?? null,
                'social_security_number' => (string)$row['national_id'],
                'race'                => (string)$row['race'],
                'employee_client_id'  => (string)$row['employee_id'],
                'phone_number' => (string)$row['phone_number'],
                'action'         => strtolower((string)$row['app_status']),
            ];
            try {
                $createdUser = app(CreateEmployee::class)->execute($data);


                if (!empty($row['journey'])) {
                    $journeyCode = $this->getJourneyCode($row['journey'], $this->clientId);
                    $journeyData = [
                        'code' => $journeyCode,
                        'users' => [$createdUser->code],
                        'segments' => [],
                        'exclude_users' => [],
                        'exclude_segments' => []
                    ];

                    if (!empty($journeyCode)) {
                        app(AddEmployeeToJourney::class)->execute($journeyData);
                    }
                }

                if (!empty($row['action_sequence'])) {

                    $actionSequenceCode = $this->getActionSequenceCode($row['action_sequence'], $this->clientId);
                    $actionSequenceData = [
                        'code' => $actionSequenceCode,
                        'users' => [$createdUser->code],
                        'segments' => []
                    ];

                    if (!empty($actionSequenceCode)) {
                        app(AddEmployeeToActionSequence::class)->execute($actionSequenceData);
                    }
                }
                $this->success++;
                $success = Cache::get($successKey) ?? 0;
                Cache::put($successKey, $success + 1, $this->importCacheExpire);
            } catch (ValidationException | InvalidRequestException $e) {
                $this->skip++;
                $skip = Cache::get($skipKey) ?? 0;
                Cache::put($skipKey, $skip + 1, $this->importCacheExpire);
                continue;
            }
        }
    }

    public function onError(Throwable $e)
    {
        // TODO: Implement onError() method.
        $this->onFailure(new Failure(0, '', [$e->getMessage()]));
    }

    private function getUserCodeFromEmail($email)
    {
        $user = User::where('email', trim($email))->first();
        if ($user) {
            return $user->getBusinessKey();
        }
        return null;
    }

    private function getJourneyCode($name, $clientId)
    {
        $value = Journey::where('name', 'like', $name)
            ->where('client_id', $clientId)
            ->first();
        return $value ? $value->code : null;
    }

    private function getActionSequenceCode($name, $clientId)
    {
        $value = ActionSequence::where('name', 'like', $name)
            ->where('client_id', $clientId)
            ->first();
        return $value ? $value->code : null;
    }

    public function uniqueBy()
    {
        return 'email';
    }

    // using for WithBatchInserts
    public function batchSize(): int
    {
        return 100;
    }

    // using for WithChunkReading
    public function chunkSize(): int
    {
        return 100;
    }

    public function rules(): array
    {
        return [
            'first_name'       => function ($attribute, $value, $onFailure) {
                $this->hasValue($attribute, $value, $onFailure);
            },
            'surname'          => function ($attribute, $value, $onFailure) {
                $this->hasValue($attribute, $value, $onFailure);
            },
            'email'            => function ($attribute, $value, $onFailure) {
                $this->hasValue($attribute, $value, $onFailure);
                $this->validateEmail($attribute, $value, $onFailure);
                $this->checkEmailExists($attribute, $value, $onFailure);
            },
            'manager'          => function ($attribute, $value, $onFailure) {
                if (!empty($value)) {
                    $this->checkEmailNotExist($attribute, $value, $onFailure);
                    $this->validateEmail($attribute, $value, $onFailure);
                }
            },
            'secondary_manager' => function ($attribute, $value, $onFailure) {
                if (!empty($value)) {
                    $this->checkEmailNotExist($attribute, $value, $onFailure);
                    $this->validateEmail($attribute, $value, $onFailure);
                }
            },
            'date_of_birth'    => function ($attribute, $value, $onFailure) {
                $this->hasValue($attribute, $value, $onFailure);
                $this->hasRightFormat($attribute, $value, $onFailure);
            },
            'start_date'       => function ($attribute, $value, $onFailure) {
                $this->hasValue($attribute, $value, $onFailure);
                $this->hasRightFormat($attribute, $value, $onFailure);
            },
            'job_title'        => function ($attribute, $value, $onFailure) {
                $this->hasValue($attribute, $value, $onFailure);
            },
            'phone_number'     => function ($attribute, $value, $onFailure) {
                $this->formatPhoneNumber($attribute, $value, $onFailure);
            },
            'employment_type'  => function ($attribute, $value, $onFailure) {
                $this->hasValue($attribute, $value, $onFailure);
                $this->checkWrongValue($attribute, $value, $onFailure, [
                    EmploymentTypes::FULL_TIME,
                    EmploymentTypes::PART_TIME,
                    EmploymentTypes::CONTRACTOR,
                    EmploymentTypes::PARTNER,
                    EmploymentTypes::VENDOR,
                    EmploymentTypes::CONTACT,
                    EmploymentTypes::CANDIDATE,
                    EmploymentTypes::CUSTOMER,
                    EmploymentTypes::AGENT,
                    EmploymentTypes::INTERN,
                    EmploymentTypes::OTHERS,
                ]);
            },
            'gender'           => function ($attribute, $value, $onFailure) {
                $this->hasValue($attribute, $value, $onFailure);
                // $this->checkWrongValue($attribute, $value, $onFailure, [
                //     'female',
                //     'male',
                // ]);
            },
            'department'       => function ($attribute, $value, $onFailure) {
                $this->hasValue($attribute, $value, $onFailure);
            },
//            'team'             => function ($attribute, $value, $onFailure) {
//                $this->hasValue($attribute, $value, $onFailure);
//            },
            'location'         => function ($attribute, $value, $onFailure) {
                $this->hasValue($attribute, $value, $onFailure);
            },
            'document_profile' => function ($attribute, $value, $onFailure) {
                $this->hasValue($attribute, $value, $onFailure);
                $documentProfileId = $this->documentProfile[$value] ?? null;
                if (!$documentProfileId) {
                    $onFailure('WRONG_VALUE');
                }
            },
        ];
    }

    private function hasValue($attribute, $value, $onFailure)
    {
        if ($value === '' || $value == null) {
            $onFailure('EMPTY_VALUE');
        }
    }

    private function validateEmail($attribute, $value, $onFailure)
    {
        if (!filter_var(trim($value), FILTER_VALIDATE_EMAIL)) {
            $onFailure('WRONG_FORMAT');
        }
    }

    private function checkEmailExists($attribute, $value, $onFailure)
    {
        if (User::where('email', trim($value))->withTrashed()->exists()) {
            $onFailure('EMAIL_EXISTS');
        }
    }

    private function checkEmailNotExist($attribute, $value, $onFailure)
    {
        if (!User::where('email', trim($value))->withTrashed()->exists()) {
            $onFailure('EMAIL_NOT_EXIST');
        }
    }

    private function checkWrongValue($attribute, $value, $onFailure, array $validValues)
    {
        if (!in_array($value, $validValues)) {
            $onFailure('WRONG_VALUE');
        }
    }

    private function hasRightFormat($attribute, $value, $onFailure)
    {
        try {
            $d = DateTime::createFromFormat(self::GLOBAL_DATE_FORMAT, $value);
        } catch (DatetimeException $e) {
            $onFailure('WRONG_FORMAT');
        }
    }

    private function formatPhoneNumber($attribute, $value, $onFailure)
    {
        try {
            $d = is_numeric($value);
        } catch (NumberFormatException $e) {
            $onFailure('WRONG_FORMAT');
        }
    }

    public function getRowCount(): int
    {
        return $this->size;
    }

    public function getSuccessCount(): int
    {
        return $this->success;
    }

    public function getSkipCount(): int
    {
        return $this->skip;
    }

    private function getValueIgnoreCase($value, $compareValues)
    {
        foreach ($compareValues as $compareValue) {
            $ignoreCharacters = ['-', ' ', '_'];
            $value = str_replace($ignoreCharacters, '', $value);
            $ignoredCaseValue = str_replace($ignoreCharacters, '', $compareValue);
            if (Str::lower($ignoredCaseValue) === Str::lower($value)) {
                return $compareValue;
            }
        }
        return $value;
    }

    private function getSegmentNameWithIdWithByType($type)
    {
        return Segment::where('segment_type_id', $type)
            ->where('client_id', $this->clientId)->pluck('id', 'name');
    }

    private function getDocumentProfiles()
    {
        return DocumentProfile::where('client_id', $this->clientId)->pluck('id', 'name');
    }
}