<?php


namespace App\Services\Employee;

use App\Enum\FilterTypes;
use App\Facades\Context;
use App\Helpers\FilterFluent;
use App\Helpers\SortFluent;
use App\Models\User\User;
use App\Services\BaseService;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use App\Helpers\CsvHelper;
use App\Http\Resources\User\EmployeeExported as EmployeeResource;
use Illuminate\Support\Str;

class ExportEmployees extends BaseService
{
    /**
     * Download employee's activities
     * 
     * @param array $data
     * @return string
     */
    public function performAction(array $data)
    {
        $options = Context::request()->getOptions();
        $client = Context::client()->get();
        $codes = $data['codes'] ?? 'all';

        $documentProfiles = DB::table('document_profiles')
            ->join('document_profile_employee', 'document_profiles.id', 'document_profile_employee.document_profile_id')
            ->select('document_profile_employee.employee_id', 'document_profiles.name')
            ->where('document_profiles.client_id', $client->getKey());

        $query = User::with('employee.client')
            ->join('employees', 'users.id', 'employees.user_id')
            ->leftJoin('segments as s1', 'employees.department_id', 's1.id')
            ->leftJoin('segments as s2', 'employees.team_id', 's2.id')
            ->leftJoin('segments as s3', 'employees.location_id', 's3.id')
            ->leftJoin('segments as s4', 'employees.gender_id', 's4.id')
            ->leftJoinSub($documentProfiles, 'dpl', function ($join) {
                $join->on('employees.id', '=', 'dpl.employee_id');
            })
            ->groupBy('users.id');
        if ($codes === 'all' && isset($data['excludeIds']) && $data['excludeIds']) {
            $query->whereNotIn('users.code', $data['excludeIds']);
        } else if (is_array($codes)) {
            $query->whereIn('users.code', $data['codes']);
        }
        $query->where('employees.client_id', $client->getKey());
        $query->select('users.*', 's1.name as department_name', 's2.name as team_name', 's3.name as location_name', 's4.name as gender_name', DB::raw('group_concat(dpl.name) as document_profiles'));

        FilterFluent::init()
            ->setQuery($query)
            ->setTable('users')
            ->setFilterOptions($options)
            ->setAllowedFilters(['name', 'department', 'team', 'location', 'status', 'document_profiles', 'job_title', 'gender',
                'social_security_number', 'race', 'employee_client_id', 'phone_number', 'last_login', 'created_at'])
            ->validate()
            ->removeFilter(['name', 'department', 'team', 'location', 'status', 'document_profiles', 'job_title', 'gender',
                'social_security_number', 'race', 'employee_client_id', 'phone_number', 'last_login', 'created_at'])
            ->setCustomFilter('name', function ($query, $type, $col, $value) {
                FilterFluent::applyCombinedColumnFilter($query, $type, 'users.surname','users.first_name', $value);
            })
            ->setCustomFilter('last_login', function($query, $type, $col, $value) {
                if ($value == "Yes") {
                    $query->whereNotNull('last_login');
                } else {
                    $query->whereNull('last_login');
                }
            })
            ->setCustomFilter('created_at', function($query, $type, $col, $value) {
                $dateValue = explode(";",$value);
                $fromDate = $dateValue[0];
                $toDate = Carbon::parse($dateValue[1])->addDay(1)->toDateTimeString();
                FilterFluent::applyBasicFilter($query, FilterTypes::GREATER_THAN_OR_EQUAL, 'users.created_at', $fromDate);
                FilterFluent::applyBasicFilter($query, FilterTypes::LESS_THAN_OR_EQUAL, 'users.created_at', $toDate);
            })
            ->setCustomFilter('social_security_number', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'social_security_number', $value, 'employees');
            })
            ->setCustomFilter('race', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'race', $value, 'employees');
            })
            ->setCustomFilter('job_title', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'job_title', $value, 'employees');
            })
            ->setCustomFilter('employee_client_id', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'employee_client_id', $value, 'employees');
            })
            ->setCustomFilter('phone_number', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'phone_number', $value, 'employees');
            })
            ->setCustomFilter('department', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's1');
            })
            ->setCustomFilter('team', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's2');
            })
            ->setCustomFilter('location', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's3');
            })
            ->setCustomFilter('gender', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's4');
            })
            ->setCustomFilter('status', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'status', $value, 'employees');
            })
            ->setCustomFilter('document_profiles', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 'dpl');
            })
            ->applyFilter();

        SortFluent::init()
            ->setQuery($query)
            ->setTable('users')
            ->setSortOptions($options)
            ->setAllowedSorts(['name', 'team', 'department', 'location', 'email', 'status', 'job_title', 'gender', 'document_profiles', 'social_security_number', 'race', 'employee_client_id', 'phone_number', 'last_login', 'created_at'])
            ->validate()
            ->removeSort(['name', 'team', 'department', 'location', 'status', 'job_title', 'gender', 'document_profiles', 'social_security_number', 'race', 'employee_client_id', 'phone_number', 'last_login', 'created_at'])
            ->setCustomSort('name', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'surname', $direction, 'users');
                SortFluent::applyBasicSort($query, 'first_name', $direction, 'users');
            })
            ->setCustomSort('last_login', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'last_login', $direction, 'users');
            })
            ->setCustomSort('created_at', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'created_at', $direction, 'users');
            })
            ->setCustomSort('department', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'name', $direction, 's1');
            })
            ->setCustomSort('team', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'name', $direction, 's2');
            })
            ->setCustomSort('location', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'name', $direction, 's3');
            })
            ->setCustomSort('gender', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'name', $direction, 's4');
            })
            ->setCustomSort('status', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'status', $direction, 'employees');
            })
            ->setCustomSort('social_security_number', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'social_security_number', $direction, 'employees');
            })
            ->setCustomSort('race', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'race', $direction, 'employees');
            })
            ->setCustomSort('job_title', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'job_title', $direction, 'employees');
            })
            ->setCustomSort('employee_client_id', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'employee_client_id', $direction, 'employees');
            })
            ->setCustomSort('phone_number', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'phone_number', $direction, 'employees');
            })
            ->setCustomSort('document_profiles', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'name', $direction, 'dpl');
            })
            ->applySort();

        $users = $query->get();

        $employees = EmployeeResource::collection($users)->jsonSerialize();

        $header = explode("-", $data['columns']['labels']);
        $columnIds = explode("-", $data['columns']['ids']);


        // TODO Make it common
        $folderName = Str::uuid();
        Storage::disk('local')->makeDirectory($folderName);
        $folderPath = config('filesystems.disks.local.root') . '/' . $folderName;
        $filePath = $folderPath . '/employees.csv';

        $rows = [];

        foreach ($employees as $employee) {
            $employee['name'] = $employee['surname'] . ',' . $employee['first_name'];

            $row = [];
            foreach ($columnIds as $column) {
                $row[] = !empty($employee[$column]) ? $employee[$column] : '';
            }
            $rows[] = $row;
        }

        CsvHelper::exportCsv($filePath, $rows, $header);
        return response()->download($filePath)->deleteFileAfterSend();
    }
}
