<?php

namespace App\Services\Employee;

use App\Enum\EmployeeStatus;
use App\Enum\FilterTypes;
use App\Facades\Context;
use App\Helpers\FilterFluent;
use App\Jobs\SendGeneratedPasswordsEmail;
use App\Models\User\User;
use App\Services\BaseService;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class GeneratePassword extends BaseService
{
    function performAction(array $data)
    {
        $options = Context::request()->getOptions();
        $client = Context::client()->get();
        $codes = $data['codes'] ?? 'all';

        $documentProfiles = DB::table('document_profiles')
            ->join('document_profile_employee', 'document_profiles.id', 'document_profile_employee.document_profile_id')
            ->select('document_profile_employee.employee_id', 'document_profiles.name')
            ->where('document_profiles.client_id', $client->getKey());

        $query = User::query()->with('employee', 'employee.department', 'employee.location', 'employee.manager', 'employee.documentProfiles')
            ->join('employees', 'employees.user_id', 'users.id')
            ->leftJoin('segments as s1', 'employees.department_id', 's1.id')
            ->leftJoin('segments as s2', 'employees.team_id', 's2.id')
            ->leftJoin('segments as s3', 'employees.location_id', 's3.id')
            ->leftJoin('segments as s4', 'employees.gender_id', 's4.id')
            ->where('users.last_login', null)
            ->where('employees.client_id', $client->getKey())
            ->where('employees.status', EmployeeStatus::ACTIVE)
            ->leftJoinSub($documentProfiles, 'dpl', function ($join) {
                $join->on('employees.id', '=', 'dpl.employee_id');
            })
            ->groupBy('users.id');


        if ($codes === 'all' && isset($data['excludeIds']) && $data['excludeIds']) {
            $query->whereNotIn('users.code', $data['excludeIds']);
        } else if (is_array($codes)) {
            $query->whereIn('users.code', $data['codes']);
        }
        $query->select('users.*', 'employees.date_of_birth', 's1.name as department', 's3.name as location', DB::raw('group_concat(dpl.name) as document_profiles'));

        FilterFluent::init()
            ->setQuery($query)
            ->setTable('users')
            ->setFilterOptions($options)
            ->setAllowedFilters(['name', 'department', 'team', 'location', 'status', 'document_profiles', 'job_title', 'gender',
                'social_security_number', 'race', 'employee_client_id', 'phone_number', 'last_login', 'created_at'])
            ->validate()
            ->removeFilter(['name', 'department', 'team', 'location', 'status', 'document_profiles', 'job_title', 'gender',
                'social_security_number', 'race', 'employee_client_id', 'phone_number', 'last_login', 'created_at'])
            ->setCustomFilter('name', function ($query, $type, $col, $value) {
                FilterFluent::applyCombinedColumnFilter($query, $type, 'users.surname','users.first_name', $value);
            })
            ->setCustomFilter('last_login', function($query, $type, $col, $value) {
                if ($value == "Yes") {
                    $query->whereNotNull('last_login');
                } else {
                    $query->whereNull('last_login');
                }
            })
            ->setCustomFilter('created_at', function($query, $type, $col, $value) {
                $dateValue = explode(";",$value);
                $fromDate = $dateValue[0];
                $toDate = Carbon::parse($dateValue[1])->addDay(1)->toDateTimeString();
                FilterFluent::applyBasicFilter($query, FilterTypes::GREATER_THAN_OR_EQUAL, 'users.created_at', $fromDate);
                FilterFluent::applyBasicFilter($query, FilterTypes::LESS_THAN_OR_EQUAL, 'users.created_at', $toDate);
            })
            ->setCustomFilter('social_security_number', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'social_security_number', $value, 'employees');
            })
            ->setCustomFilter('race', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'race', $value, 'employees');
            })
            ->setCustomFilter('job_title', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'job_title', $value, 'employees');
            })
            ->setCustomFilter('employee_client_id', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'employee_client_id', $value, 'employees');
            })
            ->setCustomFilter('phone_number', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'phone_number', $value, 'employees');
            })
            ->setCustomFilter('department', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's1');
            })
            ->setCustomFilter('team', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's2');
            })
            ->setCustomFilter('location', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's3');
            })
            ->setCustomFilter('gender', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 's4');
            })
            ->setCustomFilter('status', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'status', $value, 'employees');
            })
            ->setCustomFilter('document_profiles', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 'dpl');
            })
            ->applyFilter();

        $users = $query->get();
        SendGeneratedPasswordsEmail::dispatch(Auth::user(), $users);
    }
}