<?php

namespace App\Services\Employee;

use App\Contracts\Validatable;
use App\Enum\EmployeeStatus;
use App\Enum\ErrorCode;
use App\Enum\PlanConfigurations;
use App\Enum\PowerSearchTypes;
use App\Exceptions\InvalidRequestException;
use App\Exceptions\UnauthorizedHttpException;
use App\Facades\Context;
use App\Helpers\PermissionHelper;
use App\Models\PowerSearch\SearchableObject;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;
use App\Services\User\RevokeUserToken;
use Illuminate\Validation\Rule;
use App\Enum\SegmentTypes;
use App\Models\Segment\Segment;
use App\Helpers\ObjectHelper;
use App\Rules\ThrowableUniqueRule;
use Illuminate\Support\Arr;
use App\Jobs\SendVerifyEmailChange;

class QuickUpdateEmployee extends BaseService implements Validatable
{

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args)
    {
        $this->user = User::findByBusinessKey($args['code']);
        $rules = [];
        switch ($args['type']) {
            case 'segment':
                $rules = [
                    'segment_key' => 'required',
                    'segment_code' => 'required',
                    'type' => 'required',
                ];
                break;
            case 'content_profile':
                $rules = [
                    'document_profiles' => ['array'],
                    'type' => 'required',
                ];
                break;
            case 'email':
                $rules = [
                    'email' => ['required', 'email', 'max:255', new ThrowableUniqueRule(ErrorCode::EMAIL_ALREADY_USED, 'users', 'email', $this->user->id)],
                    'type' => 'required',
                ];
                break;
            case 'race':
                $rules = [
                    'race' => 'required',
                    'type' => 'required',
                ];
                break;
            case 'phone_number':
                $rules = [
                    'phone_number' => 'required',
                    'type' => 'required',
                ];
                break;
            case 'social_security_number':
                $rules = [
                    'social_security_number' => 'required',
                    'type' => 'required',
                ];
                break;
            case 'employee_client_id':
                $rules = [
                    'employee_client_id' => 'required',
                    'type' => 'required',
                ];
                break;
            default:
                break;
        }
        return $rules;
    }

    /**
     * Quick update employee
     *
     * @param array $data
     * @param string $status
     * @return User
     */
    public function performAction(array $data): User
    {
        $user = User::findByBusinessKey($data['code']);
        $employee = $user->employee;
        PermissionHelper::hasAccessOn($employee);

        if ($data['type'] === 'segment') {
            $segment = Segment::findByBusinessKey($data['segment_code']);
            $segmentArray = [
                $employee->department_id,
                $employee->location_id,
                $employee->team_id,
            ];
            $segmentType = '';

            switch ($data['segment_key']) {
                case 'department':
                    $employee->department_id = $segment->id;
                    $employee->team_id = null;
                    $segmentType = SegmentTypes::DEPARTMENT;
                    break;
                case 'location':
                    $employee->location_id = $segment->id;
                    $segmentType = SegmentTypes::LOCATION;
                    break;
                case 'team':
                    $employee->team_id = $segment->id;
                    $segmentType = SegmentTypes::TEAM;
                    break;

                default:
                    break;
            }

            // If segment as same as current segment, do nothing
            if (in_array($segment->id, $segmentArray)) {
                return $user;
            }

            $employee->save();
            if ($segmentType) {
                $this->updateEmployeeSegment($segmentType, $segment->id);
            }
        }

        if ($data['type'] === 'content_profile') {
            // Check client has access on document profiles
            $documentProfiles = array_filter(ObjectHelper::nullOrValue($data, 'document_profiles') ?? []);
            $documentProfileIds = CreateEmployee::getValidDocumentProfileIds($documentProfiles);

            $employee->documentProfiles()->sync($documentProfileIds);
        }

        if ($data['type'] === 'email') {
            $userData = [
                'email' => $data['email'],
            ];

            $isChangedEmail = false;
            if ($user->email != $data['email']) {
                $userData['email_change'] = $data['email'];
                $userData = Arr::except($userData, 'email');
                if ($user->email_change != $data['email']) {
                    $isChangedEmail = true;
                }
            }

            $user->update($userData);
            $user->refresh();

            if ($isChangedEmail) {
                SendVerifyEmailChange::dispatch($user, $user->email_change);
            }
        }

        if ($data['type'] === 'race') {
            if ($employee->race != $data['race']) {
                $employee->race = $data['race'];
                $employee->save();
            }
        }
        $clientId = $employee->client_id;
        $code = $this->user->code;
        if ($data['type'] === 'phone_number') {
            if ($employee->phone_number != $data['phone_number']) {
                $employee->phone_number = $data['phone_number'];
                $employee->save();
                $this->savePowerSearch($employee->phone_number, $code, $clientId, PowerSearchTypes::PHONE_NUMBER);
            }
        }
        if ($data['type'] === 'social_security_number') {
            if ($employee->social_security_number != $data['social_security_number']) {
                $employee->social_security_number = $data['social_security_number'];
                $employee->save();
                $newText = $employee->social_security_number;
                $this->savePowerSearch($newText, $code, $clientId, PowerSearchTypes::SOCIAL_SECURITY_NUMBER);
            }
        }
        if ($data['type'] === 'employee_client_id') {
            if ($employee->employee_client_id != $data['employee_client_id']) {
                $employee->employee_client_id = $data['employee_client_id'];
                $employee->save();
                $this->savePowerSearch($employee->employee_client_id, $code, $clientId, PowerSearchTypes::EMPLOYEE_CLIENT_ID);
            }
        }
        return $user->refresh();
    }

    private function savePowerSearch($newText, $code, $clientId, $type) {
        if ($newText && !empty($clientId)) {
            $searchableObj = SearchableObject::where('client_id', $clientId)
                ->where('object_type', $type)
                ->where('object_id', $code)->first();
            if ($searchableObj) {
                $searchableObj->object_content = $newText;
            } else {
                $searchableObj = new SearchableObject();
                $searchableObj->object_content = $newText;
                $searchableObj->client_id = $clientId;
                $searchableObj->object_type = $type;
                $searchableObj->object_id = $code;
            }
            $searchableObj->save();
        }
    }

    public function updateEmployeeSegment($segmentType, $segmentId)
    {
        $user = $this->user;
        // Remove employee from old segment
        $oldSegments = $user->segments()->where('segment_type_id', $segmentType)->get();
        foreach ($oldSegments as $oldSegment) {
            $oldSegment->users()->detach($user->getKey());
        }

        $newSegment = Segment::find($segmentId);
        $clientId = Context::client()->get()->getKey();
        if ($newSegment && $newSegment->getClientId() === $clientId) {
            $newSegment->users()->attach($user->getKey());
        }
    }
}
