<?php

namespace App\Services\Employee;

use App\Contracts\Performable;
use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\DataTransferObject\StreamedFile;
use App\Enum\AllowedFileSizes;
use App\Enum\AllowedFileTypes;
use App\Enum\EmployeeStatus;
use App\Enum\EmploymentTypes;
use App\Enum\ErrorCode;
use App\Enum\ImageDimensions;
use App\Enum\PowerSearchActions;
use App\Enum\PowerSearchTypes;
use App\Enum\RoleLevels;
use App\Enum\Roles;
use App\Enum\SegmentTypes;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ImageHelper;
use App\Helpers\ModelHelper;
use App\Helpers\ObjectHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\ResourceLocationBuilder;
use App\Jobs\SendVerifyEmailChange;
use App\Models\PowerSearch\SearchableObject;
use App\Models\Segment\Segment;
use App\Models\User\User;
use App\Rules\ThrowableUniqueRule;
use App\Services\Attachment\CreateAttachment;
use App\Services\Attachment\UpdateAttachment;
use App\Services\BaseService;
use Illuminate\Support\Arr;
use Illuminate\Validation\Rule;
use Illuminate\Validation\ValidationException;
use App\Jobs\IndexPowerSearch;

class UpdateEmployee extends BaseService implements Validatable, Performable, Transactionable
{
    private $user;

    /**
     * Get the validation rules that apply to the service.
     *
     * @param $data
     * @return array
     */
    public function getRules($data)
    {
        $this->user = User::findByBusinessKey($data['code']);
        $rules = [
            'first_name' => ['required', 'string', 'max:255'],
            'surname' => ['required', 'string', 'max:255'],
            'email' => ['required', 'email', 'max:255', new ThrowableUniqueRule(ErrorCode::EMAIL_ALREADY_USED, 'users', 'email', $this->user->id)],
            'date_of_birth' => ['date'],
            'start_date' => ['date'],
            'document_profiles' => ['array'],
            'job_title' => ['nullable', 'string'],
            'manager' => ['nullable', 'exists:users,code', Rule::notIn([$this->user->code])],
            'secondary_manager' => ['nullable', 'exists:users,code', Rule::notIn([$this->user->code])],
            'manager_notifications'=> ['nullable', 'string'],
            'social_security_number'          => ['nullable', 'string', 'max:255'],
            'race'          => ['nullable', 'string', 'max:255'],
            'employee_client_id'   => ['nullable', 'string', 'max:255'],
            'phone_number' => ['nullable', 'string', 'max:255'],
            'employment_type' => [Rule::in([
                EmploymentTypes::FULL_TIME,
                EmploymentTypes::PART_TIME,
                EmploymentTypes::CONTRACTOR,
                EmploymentTypes::PARTNER,
                EmploymentTypes::VENDOR,
                EmploymentTypes::CONTACT,
                EmploymentTypes::CANDIDATE,
                EmploymentTypes::CUSTOMER,
                EmploymentTypes::AGENT,
                EmploymentTypes::INTERN,
                EmploymentTypes::OTHERS,
            ])],
            'avatar' => 'nullable|mimes:'. AllowedFileTypes::IMAGE . '|max:' . AllowedFileSizes::IMAGE_SIZE,
            'external_avatar_url' => 'nullable',
            'is_avatar_removed' => 'nullable|boolean'
        ];

        if ($this->user->hasRole(Roles::FRONT_USER)) {
            $additionalFieldRule = (isset($data['is_additional_field_required']) &&
                $data['is_additional_field_required']) ? 'required' : 'nullable';
            $rules['date_of_birth'][] = $additionalFieldRule;
            $rules['start_date'][] = $additionalFieldRule;
            $rules['employment_type'][] = $additionalFieldRule;
            $rules['document_profiles'][] = $additionalFieldRule;
            $rules['document_profiles.*'][] = $additionalFieldRule;
        }
        
        return $rules;
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Update employee
     * 
     * @param array $data
     * @return User
     * @throws \Exception
     */
    public function performAction(array $data)
    {
        $client = Context::client()->get();
        $employee = $this->user->employee;
        PermissionHelper::hasAccessOn($employee);

        // Check client has access on document profiles
        $documentProfiles = array_filter(ObjectHelper::nullOrValue($data, 'document_profiles') ?? []);
        $documentProfileIds = CreateEmployee::getValidDocumentProfileIds($documentProfiles);

        $userData = [
            'first_name' => $data['first_name'],
            'surname' => $data['surname'],
            'email' => $data['email'],
        ];

        $externalAvatarUrl = ObjectHelper::nullOrValue($data, 'external_avatar_url');
        if ($externalAvatarUrl) {
            $userData[] = $externalAvatarUrl;
        }

        $isChangedEmail = false;
        if ($this->user->email != $data['email']) {
            $userData['email_change'] = $data['email'];
            $userData = Arr::except($userData, 'email');
            if ($this->user->email_change != $data['email']) {
                $isChangedEmail = true;
            }
        }

        if (ObjectHelper::nullOrValue($data, 'is_avatar_removed')) {
            // Delete avatar
            app(DeleteAvatar::class)->execute(['user_id' => $this->user->getKey()]);
        } else {
            // Update avatar
            $avatar = $data['avatar'] ?? null;
            app(UpdateAvatar::class)->execute(['avatar' => $avatar, 'user_id' => $this->user->getKey()]);
        }

        $this->user->update($userData);
        $this->user->refresh();

        $genderSegmentId = ObjectHelper::nullOrValue($data, 'gender_id');
        $departmentSegmentId = ObjectHelper::nullOrValue($data, 'department_id');
        $teamSegmentId = ObjectHelper::nullOrValue($data, 'team_id');
        $locationSegmentId = ObjectHelper::nullOrValue($data, 'location_id');

        $manager = null;
        $secondaryManager = null;
        $managerCode = ObjectHelper::nullOrValue($data, 'manager');
        $secondaryManagerCode = ObjectHelper::nullOrValue($data, 'secondary_manager');
        if ($managerCode) {
            $manager = User::whereHas('employee', function ($q) use ($client) {
                $q->where('client_id', $client->getKey());
            })->whereHas('roles', function ($q) {
                $q->where('name', Roles::MANAGER);
            })->where('code', $managerCode)->first();
        }

        if ($secondaryManagerCode) {
            $secondaryManager = User::whereHas('employee', function ($q) use ($client) {
                $q->where('client_id', $client->getKey());
            })->whereHas('roles', function ($q) {
                $q->where('name', Roles::MANAGER);
            })->where('code', $secondaryManagerCode)->first();
        }

        $managerNotificationIds = [];
        $managerNotificationCodes = ObjectHelper::nullOrValue($data, 'manager_notifications');
        if ($managerNotificationCodes) {
            $managerNotificationCodeArr = explode(';', $managerNotificationCodes);
            $managerNotifications = User::whereHas('employee', function ($q) use ($client) {
                $q->where('client_id', $client->getKey());
            })->whereIn('code', $managerNotificationCodeArr)->get();

            foreach ($managerNotifications as $managerNotification) {
                array_push($managerNotificationIds, $managerNotification->id);
            }
        }

        $employeeData = [
            'date_of_birth' => ObjectHelper::nullOrDate($data, 'date_of_birth'),
            'start_date' => ObjectHelper::nullOrDate($data, 'start_date'),
            'job_title' => ObjectHelper::nullOrValue($data, 'job_title'),
            'employment_type' => ObjectHelper::nullOrValue($data, 'employment_type'),
            'manager_id' => $manager ? $manager->getKey() : null,
            'secondary_manager_id' => $secondaryManager ? $secondaryManager->getKey() : null,
            'manager_notification_id' => sizeof($managerNotificationIds) > 0 ? implode(";", $managerNotificationIds) : null,
            'profile_pic' => $this->user->profile_pic,
            'gender_id'           => $genderSegmentId,
            'department_id'       => $departmentSegmentId,
            'team_id'             => $teamSegmentId,
            'location_id'         => $locationSegmentId,
            'social_security_number' => ObjectHelper::nullOrValue($data, 'social_security_number'),
            'race' => ObjectHelper::nullOrValue($data, 'race'),
            'employee_client_id' => ObjectHelper::nullOrValue($data, 'employee_client_id'),
            'phone_number' => ObjectHelper::nullOrValue($data, 'phone_number'),
        ];

        $this->updateEmployeeSegment(SegmentTypes::GENDER, $genderSegmentId);
        $this->updateEmployeeSegment(SegmentTypes::DEPARTMENT, $departmentSegmentId);
        $this->updateEmployeeSegment(SegmentTypes::TEAM, $teamSegmentId);
        $this->updateEmployeeSegment(SegmentTypes::LOCATION, $locationSegmentId);

        $employeeData = Arr::where($employeeData, function ($value) {
            return !empty($value);
        });

        // Change status to 'complete' after successfully updated manually
        $isAdditionalFieldRequired = (isset($data['is_additional_field_required']) &&
            $data['is_additional_field_required']);
        if ($employee->status === EmployeeStatus::INCOMPLETE && $isAdditionalFieldRequired) {
            $employeeData['status'] = EmployeeStatus::ACTIVE;
        }

        $employee->update($employeeData);
        $employee->documentProfiles()->sync($documentProfileIds);

        IndexPowerSearch::dispatch($this->user->code, PowerSearchTypes::EMPLOYEE, PowerSearchActions::ADD, $employee->client_id);

        $newText = $employee->social_security_number;
        $clientId = $employee->client_id;
        $code = $this->user->code;
        $this->savePowerSearch($newText, $code, $clientId, PowerSearchTypes::SOCIAL_SECURITY_NUMBER);
        $this->savePowerSearch($employee->employee_client_id, $code, $clientId, PowerSearchTypes::EMPLOYEE_CLIENT_ID);
        $this->savePowerSearch($employee->phone_number, $code, $clientId, PowerSearchTypes::PHONE_NUMBER);
        if ($isChangedEmail) {
            SendVerifyEmailChange::dispatch($this->user, $this->user->email_change);
            $this->savePowerSearch($this->user->email, $code, $clientId, PowerSearchTypes::EMPLOYEE_EMAIL);
        }

        return $this->user->refresh();
    }

    private function savePowerSearch($newText, $code, $clientId, $type) {
        if ($newText && !empty($clientId)) {
            $searchableObj = SearchableObject::where('client_id', $clientId)
                ->where('object_type', $type)
                ->where('object_id', $code)->first();
            if ($searchableObj) {
                $searchableObj->object_content = $newText;
            } else {
                $searchableObj = new SearchableObject();
                $searchableObj->object_content = $newText;
                $searchableObj->client_id = $clientId;
                $searchableObj->object_type = $type;
                $searchableObj->object_id = $code;
            }
            $searchableObj->save();
        }
    }

    public function updateEmployeeSegment($segmentType, $segmentId)
    {
        $user = $this->user;
        // Remove employee from old segment
        $oldSegments = $user->segments()->where('segment_type_id', $segmentType)->get();
        foreach ($oldSegments as $oldSegment) {
            $oldSegment->users()->detach($user->getKey());
        }

        $newSegment = Segment::find($segmentId);
        $clientId = Context::client()->get()->getKey();
        if ($newSegment && $newSegment->getClientId() === $clientId) {
            $newSegment->users()->attach($user->getKey());
        }
    }
}
