<?php

namespace App\Services\Engagement;

use App\Enum\EmployeeStatus;
use App\Helpers\DebugHelper;
use App\Helpers\ModelHelper;
use App\Contracts\Validatable;
use App\Services\BaseService;
use App\Traits\Pageable;
use App\Traits\Sortable;
use Carbon\Carbon;
use App\Models\Questionnaire\Questionnaire;
use App\Enum\QuestionnaireEnum;
use App\Enum\ScheduleEnum;
use Illuminate\Support\Facades\DB;
use App\Models\Schedule\ScheduleSurvey;
use App\Models\Schedule\ScheduleFrequencyRound;
use App\Models\Schedule\ScheduleRoundQuestionnaire;
use App\Models\Schedule\ScheduleSegments;
use App\Models\Segment\SegmentUser;
use App\Services\DataControl\GetDataControl;
use App\Models\Driver\Driver;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Services\Report\GetUserListAvailableOfRound;
use App\Facades\Context;
use App\Enum\SegmentTypes;
use App\Models\Segment\Segment;

class EmployeeNetResponse extends BaseService implements Validatable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules(array $args): array
    {
        return [];
    }

    /**
     *
     * Represents the total percentage of Promoters minus the total percentage of Detractors
     *
     * @param array $options
     * @return mixed
     */
    public function performAction(array $options)
    {
        return $this->geteNPS($options);
    }

    /**
     * calculation scoring average for eNPS
     * @param array $options
     */
    public function geteNPS(array $data)
    {
        $data['start_date'] = isset($data['start_date']) ? Carbon::parse($data['start_date'])->toDateString() : null;
        $data['end_date'] = isset($data['end_date']) ? Carbon::parse($data['end_date'])->toDateString() : null;
        $client = Context::client()->get();
        $query = QuestionnaireAnswer::query();
        $query->select(['questionnaire_id','weight',
            DB::raw("((SELECT 
                COUNT(point)
            FROM
                questionnaire_answers
            WHERE
                point >= ".QuestionnaireEnum::PROMOTER['min']." AND point <= ".QuestionnaireEnum::PROMOTER['max'].
                " AND client_id = ".$client->id.
                " AND questionnaire_answers.driver_id = drivers.id)/count(*))*100 promoters"),

            DB::raw("((SELECT 
                COUNT(point)
            FROM
                questionnaire_answers
            WHERE
                point >= ".QuestionnaireEnum::PASSIVE['min']." AND point <= ".QuestionnaireEnum::PASSIVE['max'].
                " AND client_id = ".$client->id.
                " AND questionnaire_answers.driver_id = drivers.id)/count(*))*100 passive"),

            DB::raw("((SELECT 
                COUNT(point)
            FROM
                questionnaire_answers
            WHERE
                point >= ".QuestionnaireEnum::DETRACTOR['min']." AND point <= ".QuestionnaireEnum::DETRACTOR['max'].
                " AND client_id = ".$client->id.
                " AND questionnaire_answers.driver_id = drivers.id)/count(*))*100 detractor")])

        ->selectRaw("IF(drivers.parent_id != 0, drivers.parent_id, drivers.id) driver")
        ->selectRaw('sum(point)/count(*) as avg_weight_score')
        ->selectRaw('(sum(point)/count(*)) * drivers.weight as driver_score')
        ->selectRaw('count(questionnaire_id) as total_responses')
        ->selectRaw('sum(point) as weighted_score')
        ->from('questionnaire_answers')
        ->join('drivers','drivers.id','questionnaire_answers.driver_id');
        $query->where('questionnaire_answers.client_id', $client->id);

        // check empty state
        $isEmpty = $query->count();
        if ($isEmpty == 0) return null;

        $query->whereNull('questionnaire_answers.skip_question_id');
        if(isset($data['start_date']) && isset($data['end_date'])){
            $query->where(function($q) use ($data) {
                return $q->whereDate('questionnaire_answers.created_at', '>=', $data['start_date'])
                    ->whereDate('questionnaire_answers.created_at', '<=', $data['end_date']);
            });
        }
        $query->groupBy('questionnaire_answers.questionnaire_id','drivers.parent_id', 'drivers.weight', 'drivers.id', 'questionnaire_answers.created_at');
        if(isset($data['driver'])){
            $driverIds = explode(",", $data['driver']);
            $query->havingRaw('driver IN(?)', $driverIds);
        }

        $query->orderBy('driver');
        $query->orderBy('questionnaire_id');

        $enps = $query->get();
        return $enps;
    }

    /**
     * calculation scoring average for eNPS
     * @param array $options
     */
    public function geteNPSProcessing(array $data)
    {
        $client = Context::client()->get();
        $start_date = Carbon::now()->addMonth(-24)->toDateString();
        $end_date = Carbon::now()->toDateString();

        $query = Questionnaire::query();
        $query->select(['questionnaire_id','weight',
            DB::raw("((SELECT 
                COUNT(point)
            FROM
                questionnaire_answers
            WHERE 
                point >= ".QuestionnaireEnum::PROMOTER['min']." AND point <= ".QuestionnaireEnum::PROMOTER['max'].
                " AND questionnaire_answers.driver_id = drivers.id)/count(*))*100 promoters"),

            DB::raw("((SELECT 
                COUNT(point)
            FROM
                questionnaire_answers
            WHERE
                point >= ".QuestionnaireEnum::PASSIVE['min']." AND point <= ".QuestionnaireEnum::PASSIVE['max'].
                " AND questionnaire_answers.driver_id = drivers.id)/count(*))*100 passive"),

            DB::raw("((SELECT 
                COUNT(point)
            FROM
                questionnaire_answers
            WHERE
                point >= ".QuestionnaireEnum::DETRACTOR['min']." AND point <= ".QuestionnaireEnum::DETRACTOR['max'].
                " AND questionnaire_answers.driver_id = drivers.id)/count(*))*100 detractor")])
        ->selectRaw("DATE_FORMAT(questionnaire_answers.created_at, '%Y-%m') month")
        ->selectRaw('sum(point)/count(*) as avg_weight_score')
        ->selectRaw('(sum(point)/count(*)) * drivers.weight as driver_score')
        ->selectRaw('count(questionnaire_id) as total_responses')
        ->selectRaw('sum(point) as weighted_score')
        ->from('questionnaire_answers')
        ->join('drivers','drivers.id','questionnaire_answers.driver_id');

        $query->whereNull('questionnaire_answers.skip_question_id');
        if(isset($start_date) && isset($end_date)){
            $query->where(function($q) use ($start_date, $end_date) {
                return $q->whereDate('questionnaire_answers.created_at', '>=', $start_date)
                    ->whereDate('questionnaire_answers.created_at', '<=', $end_date);
            });
        }
        $query->where('questionnaire_answers.client_id', $client->id);
        $query->groupBy('questionnaire_answers.questionnaire_id','drivers.weight', 'drivers.id', 'questionnaire_answers.created_at');
        $query->orderBy('month');

        $enps = $query->get();
        return $enps;
    }

    /**
     * get participate employees answered
     * @param array $options
     */
    public function getParticipate($options=[])
    {
        $LOCATION_SEGMENT_IDS = [];
        $DEPARTMENT_SEGMENT_IDS = [];
        $TEAM_SEGMENT_IDS = [];
        $MANAGER_SEGMENT_IDS = [];
        $GENDER_SEGMENT_IDS = [];

        $client = Context::client()->get();

        $query = QuestionnaireAnswer::query()
            ->select(DB::raw("COUNT(questionnaire_answers.id) as total_participate"), 'schedule_survey.total_questionnaire')
            ->join('schedule_survey', 'schedule_survey.id', '=','questionnaire_answers.survey_id');
            if (isset($options['segments_filter'])) {
                $query->join('employees', 'employees.user_id', 'questionnaire_answers.user_id');
                foreach ($options['segments_filter'] as $key => $segment) {
                    $segmentItem = Segment::where('code', $segment['value'])->first();
    
                    switch ($segmentItem['segment_type_id']) {
                        case SegmentTypes::LOCATION:
                            array_push($LOCATION_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::DEPARTMENT:
                            array_push($DEPARTMENT_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::TEAM:
                            array_push($TEAM_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::MANAGER:
                            array_push($MANAGER_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::GENDER:
                            array_push($GENDER_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        default:
                            break;
                    }
                }
    
                switch (true) {
                    case count($LOCATION_SEGMENT_IDS) > 0:
                        $query->whereIn('employees.location_id', $LOCATION_SEGMENT_IDS);
                        break;
                    case count($DEPARTMENT_SEGMENT_IDS) > 0:
                        $query->whereIn('employees.department_id', $DEPARTMENT_SEGMENT_IDS);
                        break;
                    case count($TEAM_SEGMENT_IDS) > 0:
                        $query->whereIn('employees.team_id', $TEAM_SEGMENT_IDS);
                        break;
                    case count($MANAGER_SEGMENT_IDS) > 0:
                        $query->whereIn('employees.manager_id', $MANAGER_SEGMENT_IDS);
                        break;
                    case count($GENDER_SEGMENT_IDS) > 0:
                        $query->whereIn('employees.gender_id', $GENDER_SEGMENT_IDS);
                        break;
                    default:
                        break;
                }
            }
            $query->where('schedule_survey.client_id', $client->getKey());
            $query->where('schedule_survey.total_questionnaire', '>', '0');
            $query->whereNull('schedule_survey.deleted_at');
            $query->whereNull('questionnaire_answers.skip_question_id');
            $query->groupBy('questionnaire_answers.user_id', 'questionnaire_answers.survey_id', 'schedule_survey.total_questionnaire');
//            $query->having('total_participate', '>=', DB::raw('schedule_survey.total_questionnaire'));
        return $query->get();
    }

    public function getTotalScheduleSurvey($options = [])
    {
        $LOCATION_SEGMENT_IDS = [];
        $DEPARTMENT_SEGMENT_IDS = [];
        $TEAM_SEGMENT_IDS = [];
        $MANAGER_SEGMENT_IDS = [];
        $GENDER_SEGMENT_IDS = [];

        $currentClient = Context::client()->get();
        $query = ScheduleRoundQuestionnaire::query()
            ->join('schedule_survey',
                'schedule_survey.id',
                '=',
                'schedule_round_questionnaires.schedule_survey_id');

            if (isset($options['segments_filter'])) {
                $query->leftJoin('employees', 'employees.user_id', 'schedule_round_questionnaires.user_id');
                foreach ($options['segments_filter'] as $key => $segment) {
                    $segmentItem = Segment::where('code', $segment['value'])->first();
    
                    switch ($segmentItem['segment_type_id']) {
                        case SegmentTypes::LOCATION:
                            array_push($LOCATION_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::DEPARTMENT:
                            array_push($DEPARTMENT_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::TEAM:
                            array_push($TEAM_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::MANAGER:
                            array_push($MANAGER_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::GENDER:
                            array_push($GENDER_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        default:
                            break;
                    }
                }
    
                switch (true) {
                    case count($LOCATION_SEGMENT_IDS) > 0:
                        $query->whereIn('employees.location_id', $LOCATION_SEGMENT_IDS);
                        break;
                    case count($DEPARTMENT_SEGMENT_IDS) > 0:
                        $query->whereIn('employees.department_id', $DEPARTMENT_SEGMENT_IDS);
                        break;
                    case count($TEAM_SEGMENT_IDS) > 0:
                        $query->whereIn('employees.team_id', $TEAM_SEGMENT_IDS);
                        break;
                    case count($MANAGER_SEGMENT_IDS) > 0:
                        $query->whereIn('employees.manager_id', $MANAGER_SEGMENT_IDS);
                        break;
                    case count($GENDER_SEGMENT_IDS) > 0:
                        $query->whereIn('employees.gender_id', $GENDER_SEGMENT_IDS);
                        break;
                    default:
                        break;
                }
            }
            $query->where('schedule_round_questionnaires.status', ScheduleEnum::ACTIVE);
            $query->whereNull('schedule_survey.deleted_at');
            $query->where('schedule_survey.client_id', $currentClient->getKey());
            $query->where('schedule_survey.status', ScheduleEnum::ACTIVE);

        return $query->count();
    }

}
