<?php


namespace App\Services\Engagement;

use App\Facades\Context;
use App\Services\BaseService;
use App\Traits\Pageable;
use App\Traits\Sortable;
use App\Helpers\FilterFluent;
use Carbon\Carbon;
use App\Models\Questionnaire\Questionnaire;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Enum\QuestionnaireEnum;
use Illuminate\Support\Facades\DB;
use App\Models\Schedule\ScheduleSurvey;
use App\Models\Schedule\ScheduleFrequencyRound;
use App\Models\User\User;

class GetDriverScore extends BaseService
{
    use Sortable, Pageable;

    /**
     * calculation scoring average for eNPS
     * @param array $args
     * @return mixed
     * @throws \Illuminate\Validation\ValidationException
     */
    public function performAction(array $args)
    {
        $user = Context::user()->get();
        $client = Context::client()->get();
        $options = Context::request()->getOptions();
        $trend_from_date = isset($args['from_date']) ? Carbon::parse($args['from_date'])->addDay(-30)->toDateString() : Carbon::now()->addDay(-30)->toDateString();
        $trend_to_date = isset($args['to_date']) ? Carbon::parse($args['to_date'])->addDay(-30)->toDateString() : Carbon::parse($trend_from_date)->addDay(-30)->toDateString();
        if(!isset($args['trend'])){
             $trend_from_date = null;
             $trend_to_date = null;
        }
        $args['from_date'] = isset($args['from_date']) ? Carbon::parse($args['from_date'])->toDateString() : $trend_from_date;
        $args['to_date'] = isset($args['to_date']) ? Carbon::parse($args['to_date'])->toDateString() : $trend_to_date;
        $query = Questionnaire::query();
        $query->select(['questionnaire_id','weight',
            DB::raw("((SELECT 
                COUNT(point)
            FROM
                questionnaire_answers
            WHERE
                point >= ".QuestionnaireEnum::PROMOTER['min']." AND point <= ".QuestionnaireEnum::PROMOTER['max'].
                " AND questionnaire_answers.driver_id = drivers.id)/count(*))*100 promoters"),

            DB::raw("((SELECT 
                COUNT(point)
            FROM
                questionnaire_answers
            WHERE
                point >= ".QuestionnaireEnum::PASSIVE['min']." AND point <= ".QuestionnaireEnum::PASSIVE['max'].
                " AND questionnaire_answers.driver_id = drivers.id)/count(*))*100 passive"),

            DB::raw("((SELECT 
                COUNT(point)
            FROM
                questionnaire_answers
            WHERE
                point >= ".QuestionnaireEnum::DETRACTOR['min']." AND point <= ".QuestionnaireEnum::DETRACTOR['max'].
                " AND questionnaire_answers.driver_id = drivers.id)/count(*))*100 detractor")])
        
        ->selectRaw("IF(drivers.parent_id != 0, drivers.parent_id, drivers.id) driver")
        ->selectRaw('sum(point)/count(*) as avg_weight_score')
        ->selectRaw('(sum(point)/count(*)) * drivers.weight as driver_score')
        ->selectRaw('count(questionnaire_id) as total_responses')
        ->selectRaw('sum(point) as weighted_score')
        ->from('questionnaire_answers')
        ->join('drivers','drivers.id','questionnaire_answers.driver_id');

        $query->whereNull('questionnaire_answers.skip_question_id');
        if(isset($args['from_date']) && isset($args['to_date'])){
            $query->where(function($q) use ($args) {
                return $q->whereDate('questionnaire_answers.created_at', '>=', $args['from_date'])
                    ->whereDate('questionnaire_answers.created_at', '<=', $args['to_date']);
            });
        }
        if(isset($client)){
            $data['client_id'] = $client->id;
        }
        $data['status'] = 1;
        $roundIds = $this->getRoundByClient($data);

        $query->whereIn('questionnaire_answers.round_id', $roundIds);
        $query->groupBy('questionnaire_answers.questionnaire_id','drivers.parent_id', 'drivers.weight', 'drivers.id', 'questionnaire_answers.created_at');
        if(isset($args['driver'])){
            $driverIds = explode(",", $args['driver']);
            $query->havingRaw('driver IN(?)', $driverIds);
        }

        $query->orderBy('driver');
        $query->orderBy('questionnaire_id');
        
        $answers = $query->get();


        // count total employees of company
        $total_employees = User::active()->get()->count();

        // define scoring result
        $eNPScoring = [
            'total_responses' => [],
            'promoters' => [],
            'passive' => [],
            'detractor' => [],
            'driver_avg' => [],
            'driver_enps' => [],
        ];
        
        // processing weighting scoring
        foreach ($answers as $key => $value) {
            $row = $value->toArray();

            $row['driver_enps'] = ((float)$row['promoters'] - (float)$row['detractor']) * (float)$row['weight'];
            $eNPScoring['total_responses'][$row['driver']][] = (int)$row['total_responses'];

            $eNPScoring['promoters'][$row['driver']][] = $row['promoters'];
            $eNPScoring['passive'][$row['driver']][] = $row['passive'];
            $eNPScoring['detractor'][$row['driver']][] = $row['detractor'];

            $eNPScoring['driver_avg'][$row['driver']][] = (float)$row['driver_score'];
            $eNPScoring['driver_enps'][$row['driver']][] = (float)$row['driver_enps'];
        }

        $eNPSDriver = null;
        // processing for one of each driver
        foreach ($eNPScoring['total_responses'] as $key => $value) {
            $eNPSDriver['driver_avg'] = array_sum($eNPScoring['driver_avg'][$key]);
            $eNPSDriver['driver_enps'] = array_sum($eNPScoring['driver_enps'][$key]);
        }
        if(empty($eNPSDriver)){
            $eNPSDriver = [
                'driver_avg' => null,
                'driver_enps' => null
            ];
        }
        return $eNPSDriver;
    }

    public function getRoundByClient(array $data = []): array
    {
        $survey = ScheduleSurvey::where($data)->get()->pluck('id')->toArray();
        $rounds = ScheduleFrequencyRound::whereIn('schedule_survey_id', $survey)->get()->pluck('id')->toArray();
        return $rounds;
    }
}
