<?php


namespace App\Services\Event;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\EmailNotifications;
use App\Enum\NotificationTypes;
use App\Enum\PowerSearchActions;
use App\Enum\PowerSearchTypes;
use App\Enum\Roles;
use App\Facades\Context;
use App\Helpers\ModelHelper;
use App\Helpers\ObjectHelper;
use App\Helpers\PermissionHelper;
use App\Jobs\IndexPowerSearch;
use App\Jobs\SendNotification;
use App\Jobs\SendNotificationConsole;
use App\Jobs\SendNotificationEmail;
use App\Models\Event\Event;
use App\Models\Segment\Segment;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\UserAccess\GetAccessByRole;
use Carbon\Carbon;
use Exception;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Auth;

class CreateEvent extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($data)
    {
        return [
            'name' => 'required|string',
            'location' => 'nullable|string',
            'users' => 'array|exists:users,code',
            'segments' => 'array',
            'start_date' => 'required|date_utc',
            'end_date' => 'required|date_utc',
            'description' => 'nullable|string',
            'client_id' => 'string',
            'is_published' => 'required|boolean',
            'is_full_day' => 'required|boolean'
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Create event
     *
     * @param array $data
     * @return Event
     * @throws Exception
     */
    public function performAction(array $data): Event
    {
        $currentUser = Context::user()->get();
        $currentClient = Context::client()->get();

        $data['client_id'] = $currentClient->getKey();
        $data['created_by'] = $currentUser->getKey();

        $users = ObjectHelper::nullOrValue($data, 'users') ?? [];
        $segments = ObjectHelper::nullOrValue($data, 'segments') ?? [];
        $userCodes = $users;
        $segmentsId = [];

        $hasAllEmployeeSegment = false;

        foreach ($segments as $code) {
            $segment = Segment::findByBusinessKey($code);
            PermissionHelper::hasAccessOn($segment);

            $segmentUserCodes = $segment->includedCustomSegmentTypeUsers()->pluck('code')->toArray();
            $userCodes = array_merge($userCodes, $segmentUserCodes);

            $segmentsId[] = $segment->getKey();

            if ($segment->isAllSegmentType()) {
                $hasAllEmployeeSegment = true;
                $segmentsId = $segment->getKey();
                break;
            }
        }

        $userCodes = array_unique($userCodes);

        $data = Arr::except($data, ['users', 'segments']);
        if ($data['alert']) {
            $parts = explode(" ", $data['alert']);
            $start = Carbon::parse($data['start_date']);
            switch ($parts[1]) {
                case "minutes":
                    $data['alert_at'] = $start->subMinutes($parts[0]);
                    break;
                case "hours":
                    $data['alert_at'] = $start->subHours($parts[0]);
                    break;
                case "days":
                    $data['alert_at'] = $start->subDays($parts[0]);
                    break;
                default:
                    break;
            }
        }

        $event = Event::create($data);

        if (!$hasAllEmployeeSegment) {
            $usersId = !empty($users) ? ModelHelper::convertBusinessKeyToId(User::class, $users) : [];
            $event->users()->attach($usersId);
        }

        $event->segments()->attach($segmentsId);

        IndexPowerSearch::dispatch($event->code, PowerSearchTypes::EVENT, PowerSearchActions::ADD, $event->client_id);

        if ($event->is_published && !empty($userCodes)) {
            $this->sendNotification($userCodes, $event);
        }

        return $event->refresh();
    }

    public function sendNotification($users, $event)
    {
        //Send reminder when new added
        $arguments = array(
            'sender' => Auth::user()->id,
            "userIds" => $users,
            "headings" => __("New event added"),
            "contents" => __("Tap to view all events"),
            "data" => [
                "type" => NotificationTypes::EVENT_ADDED,
                "code" => $event->code,
                'id' => $event->id
            ],
            'send_after' => null
        );
        SendNotification::dispatch($arguments);


        // Send Invite Notifications to employees
        $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
        $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

        $userArr = User::whereIn('code', $users)->get();
        $owner = $event->creator;
        if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_CONSOLE_UPCOMING]) {
            foreach ($userArr as $user) {
                SendNotificationConsole::dispatch($owner, $user, 'Event Invitation');
            }
        }
        if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_UPCOMING]) {
            foreach ($userArr as $user) {
                SendNotificationEmail::dispatch($user, EmailNotifications::EVENT_INVITATION, ['event' => $event->name, 'owner' => $owner->surname . ', ' . $owner->first_name]);
            }
        }
    }
}