<?php


namespace App\Services\Event;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Facades\Context;
use App\Helpers\DateHelper;
use App\Helpers\ObjectHelper;
use App\Models\Event\Event;
use App\Models\User\User;
use App\Services\BaseService;
use Carbon\Carbon;
use Illuminate\Support\Arr;

class SyncEvent extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @param $data
     * @return array
     */
    public function getRules($data)
    {
        return [
            'events' => 'required|array',
            'events.*.name' => 'required|string',
            'events.*.guests' => 'nullable|array',
            'events.*.start_date' => 'required|date_utc',
            'events.*.end_date' => 'required|date_utc',
            'events.*.is_full_day' => 'required|boolean',
            'events.*.location' => 'nullable|string',
            'events.*.description' => 'nullable|string',
            'events.*.source' => 'required',
            'events.*.calendar_id' => 'required',
            'events.*.id' => 'required',
            'events.*.updated_at' => 'nullable|date_utc',
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Sync events
     *
     * @param array $data
     * @return array
     */
    public function performAction(array $data)
    {
        $client = Context::client()->get();
        $synchronizedEvents = [];
        $importedEvents = [];
        $importedEventSourceIds = [];
        $sources = array_unique(Arr::pluck($data['events'], 'source'));
        $calendarIds = array_unique(Arr::pluck($data['events'], 'calendar_id'));

        foreach ($sources as $source) {
            $importSourceEventsWithSourceIdKey = [];
            $importSourceEvents = Event::ofClient($client->getKey())->where('source_type', $source)->get();
            foreach ($importSourceEvents as $importSourceEvent) {
                $importSourceEventsWithSourceIdKey[$importSourceEvent['source_id']] = $importSourceEvent;
            }
            $importedEvents[$source] = $importSourceEventsWithSourceIdKey;
            $importedEventSourceIds[$source] = array_keys($importedEvents[$source]);

            // Delete deleted calendar events from source
            foreach ($calendarIds as $calendarId) {
                $importedCalendarEvents = Event::ofClient($client->getKey())
                    ->where(['source_type' => $source, 'source_calendar_id' => $calendarId])
                    ->whereDate('start_date', '>=', Carbon::today())
                    ->get();

                $inputCalendarEvents = Arr::where($data['events'], function ($event) use ($source, $calendarId) {
                    return $event['source'] === $source && $event['calendar_id'] === $calendarId;
                });
                $inputCalendarEventIds = Arr::pluck($inputCalendarEvents, 'id');

                foreach ($importedCalendarEvents as $importedCalendarEvent) {
                    if (!in_array($importedCalendarEvent->source_id, $inputCalendarEventIds, true)) {
                        app(DeleteEvent::class)->execute(['code' => $importedCalendarEvent->getBusinessKey()]);
                    }
                }
            }
        }

        foreach ($data['events'] as $event) {
            $guests = ObjectHelper::nullOrValue($event, 'guests') ?? [];
            $guests = Arr::where($guests, function ($value) {
                return !empty($value);
            });
            /*$userCodes = User::whereHas('employee', function ($q) use ($client) {
                $q->where('client_id', $client->getKey());
            })->whereIn('email', $guests)->pluck('code')->toArray();*/
            $recurrence = ObjectHelper::nullOrValue($event, 'recurrence') ?? null;
            $recurrenceEndDate = ObjectHelper::nullOrValue($event, 'recurrence_end_date') ?? null;

            $eventData = [
                'name' => $event['name'],
                'users' => [],
                'start_date' => $event['start_date'],
                'end_date' => $event['end_date'],
                'location' => ObjectHelper::nullOrValue($event, 'location'),
                'description' => ObjectHelper::nullOrValue($event, 'description'),
                'is_full_day' => $event['is_full_day'],
                'is_published' => 0,
                'source_type' => $event['source'],
                'source_calendar_id' => $event['calendar_id'],
                'source_id' => $event['id'],
                'source_updated_at' => ObjectHelper::nullOrValue($event, 'updated_at'),
                'recurrence' => $recurrence ? json_encode($recurrence) : null,
                'recurrence_end_date' => $recurrenceEndDate
            ];

            if (in_array($event['id'], $importedEventSourceIds[$event['source']], true)) {
                $importedEvent = $importedEvents[$event['source']][$event['id']];
                $inputEventUpdatedDate = DateHelper::create($event['updated_at']);
                if (empty($importedEvent->source_updated_at) ||
                    $importedEvent->source_updated_at->notEqualTo($inputEventUpdatedDate)) {
                    $synchronizedEvents[] = app(UpdateEvent::class)->execute(
                        array_merge(['code' => $importedEvent->getBusinessKey()], $eventData)
                    );
                }
            } else {
                $synchronizedEvents[] = app(CreateEvent::class)->execute($eventData);
            }
        }

        return $synchronizedEvents;
    }
}
