<?php


namespace App\Services\Event;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\EmailNotifications;
use App\Enum\NotificationTypes;
use App\Enum\PowerSearchActions;
use App\Enum\PowerSearchTypes;
use App\Enum\Roles;
use App\Facades\Context;
use App\Helpers\ModelHelper;
use App\Helpers\ObjectHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\UserAccessHelper;
use App\Jobs\IndexPowerSearch;
use App\Jobs\SendNotification;
use App\Jobs\SendNotificationConsole;
use App\Jobs\SendNotificationEmail;
use App\Models\Event\Event;
use App\Models\Segment\Segment;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\UserAccess\GetAccessByRole;
use Exception;
use Illuminate\Support\Arr;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Auth;

class UpdateEvent extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($data)
    {
        return [
            'code' => 'required|string|exists:events,code',
            'name' => 'required|string',
            'location' => 'nullable|string',
            'users' => 'array|exists:users,code',
            'segments' => 'array',
            'start_date' => 'required|date_utc',
            'end_date' => 'required|date_utc',
            'description' => 'nullable|string',
            'client_id' => 'integer',
            'is_published' => 'required|boolean',
            'is_full_day' => 'required|boolean'
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Update event
     *
     * @param array $data
     * @return Event
     * @throws Exception
     */
    public function performAction(array $data): Event
    {
        $currentClient = Context::client()->get();

        $event = Event::findByBusinessKey($data['code']);
        PermissionHelper::hasAccessOn($event);
        UserAccessHelper::hasAccessAsOwner($event);

        $users = ObjectHelper::nullOrValue($data, 'users') ?? [];
        $segments = ObjectHelper::nullOrValue($data, 'segments');
        $userCodes = $users;
        $segmentsId = [];

        $hasAllEmployeeSegment = false;
        if (!empty($segments)) {
            foreach ($segments as $code) {
                $segment = Segment::findByBusinessKey($code);
                PermissionHelper::hasAccessOn($segment);

                $segmentUserCodes = $segment->includedCustomSegmentTypeUsers()->pluck('code')->toArray();
                $userCodes = array_merge($userCodes, $segmentUserCodes);

                $segmentsId[] = $segment->getKey();

                if ($segment->isAllSegmentType()) {
                    $hasAllEmployeeSegment = true;
                    $segmentsId = $segment->getKey();
                    break;
                }
            }
        }
        $userCodes = array_unique($userCodes);

        $data['client_id'] = $currentClient->getKey();
        $data = Arr::except($data, ['users', 'segments']);

        if ($data['alert']) {
            $parts = explode(" ", $data['alert']);
            $start = \Carbon\Carbon::parse($data['start_date']);
            switch ($parts[1]) {
                case "minutes":
                    $data['alert_at'] = $start->subMinutes($parts[0]);
                    break;
                case "hours":
                    $data['alert_at'] = $start->subHours($parts[0]);
                    break;
                case "days":
                    $data['alert_at'] = $start->subDays($parts[0]);
                    break;
                default:
                    break;
            }
        }

        $event->update($data);

        if ($hasAllEmployeeSegment) {
            $usersId = [];
        } else {
            $usersId = !empty($users) ? ModelHelper::convertBusinessKeyToId(User::class, $users) : [];
        }

        $event->segments()->sync($segmentsId);
        $event->users()->sync($usersId);

        IndexPowerSearch::dispatch($event->code, PowerSearchTypes::EVENT, PowerSearchActions::ADD, $event->client_id);

        if ((bool) $data['is_published'] && !empty($userCodes)) {
            $arguments = array(
                'sender' => Auth::user()->id,
                "userIds" => $userCodes,
                "headings" => __("Event updated"),
                "contents" => __($data['name'] . ", " . Carbon::parse($data['start_date'])->diffForHumans()),
                "data" => [
                    "type" => NotificationTypes::EVENT_UPDATED,
                    "code" => $data['code']
                ],
                'send_after' => null
            );
            SendNotification::dispatch($arguments);


            // Send Invite Notifications to employees
            $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
            $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

            $userArr = User::whereIn('code', $userCodes)->get();
            $owner = $event->creator;
            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_CONSOLE_UPCOMING]) {
                foreach ($userArr as $user) {
                    SendNotificationConsole::dispatch($owner, $user, 'Upcoming Event updated');
                }
            }
            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_UPCOMING]) {
                foreach ($userArr as $user) {
                    SendNotificationEmail::dispatch($user, EmailNotifications::EVENT_REMINDER, ['event' => $event->name]);
                }
            }
        }

        return $event->refresh();
    }
}