<?php

namespace App\Services\Initiative;

use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\EmailNotifications;
use App\Enum\NotificationTypes;
use App\Enum\Roles;
use App\Facades\Context;
use App\Jobs\SendNotification;
use App\Jobs\SendNotificationConsole;
use App\Jobs\SendNotificationEmail;
use App\Models\Initiative\Initiative;
use App\Models\Segment\Segment;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\UserAccess\GetAccessByRole;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\Rule;

class CreateInitiative extends BaseService implements Validatable, Transactionable
{

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args): array
    {
        $client = Context::client()->get();
        return [
            'driver_id' => 'required|exists:drivers,id',
            'responsible' => 'required|exists:users,code',
            'title' => [
                'required',
                Rule::unique('initiatives')->where(function ($query) use ($args, $client) {
                    return $query->where([
                        'client_id' => $client->id,
                        'driver_id' => $args['driver_id'],
                        'title' => $args['title'],
                    ]);
                })
            ],
            'publish_to' => 'required',
            'description' => 'required',
            'due_by' => 'required',
        ];
    }

    /**
     * Create initiative
     *
     * @param array $data
     * @return Initiative
     */
    public function performAction(array $data)
    {
        $user = Context::user()->get();
        $client = Context::client()->get();
        $clientUsersCode = $client->users()
            ->whereNull('deleted_at')
            ->pluck('code')
            ->toArray();

        $responsible = User::findByBusinessKey($data['responsible']);
        $data['responsible_id'] = $responsible->getKey();
        $data['client_id'] = $client->id;
        $data['created_by'] = $user->id;

        $initiative = Initiative::create($data);

        // Distribute initiative segment
        if (!empty($initiative)) {
            app(SyncInitiativeSegment::class)->execute([
                'initiative' => $initiative,
                'segment_code' => $data['publish_to']
            ]);
        }

        if (!empty($clientUsersCode)) {
            $arguments = array(
                "userIds" => $clientUsersCode,
                "headings" => __("New initiative added"),
                "contents" => __("Tap to view the engagement initiative"),
                "data" => [
                    "type" => NotificationTypes::INITIATIVE_ADDED,
                    "id" => $initiative->id,
                ],
                'send_after' => null
            );
            SendNotification::dispatch($arguments);


            // Send Invite Notifications to employees
            $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
            $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

            $userArr = $client->users()->get();
            $owner = $initiative->creator;
            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_INIT]) {
                foreach ($userArr as $iuser) {
                    SendNotificationConsole::dispatch($owner, $iuser, 'Initiative Invitation');
                }
            }
            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_INIT]) {
                foreach ($userArr as $iuser) {
                    SendNotificationEmail::dispatch($iuser, EmailNotifications::INITIATIVE_INVITATION, ['initiative' => $initiative->title, 'owner' => $owner->surname . ', ' . $owner->first_name]);
                }
            }
        }

        return $initiative;
    }
}