<?php


namespace App\Services\Initiative;

use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\EmailNotifications;
use App\Enum\Roles;
use App\Facades\Context;
use App\Jobs\SendNotificationConsole;
use App\Jobs\SendNotificationEmail;
use App\Models\Initiative\Initiative;
use App\Models\Segment\Segment;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\UserAccess\GetAccessByRole;
use Illuminate\Validation\Rule;

class EditInitiative extends BaseService implements Validatable, Transactionable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    protected $rules;

    public function getRules(array $args): array
    {
        $client = Context::client()->get();

        return [
            'driver_id' => 'required|exists:drivers,id',
            'responsible' => 'required|exists:users,code',
            'title' => [
                'required',
                Rule::unique('initiatives')
                    ->ignore($args['id'])
                    ->where(function ($query) use ($args, $client) {
                        return $query->where([
                            'client_id' => $client->id,
                            'driver_id' => $args['driver_id'],
                            'title' => $args['title'],
                        ]);
                    })
            ],
            'description' => 'required',
            'publish_to' => 'required',
            'due_by' => 'required',
        ];
    }

    /**
     * Create driver
     *
     * @param array $data
     * @return Initiative
     */
    public function performAction(array $data)
    {
        $initiative = Initiative::findOrFail($data['id']);
        $user = Context::user()->get();
        $client = Context::client()->get();

        $responsible = User::findByBusinessKey($data['responsible']);
        $data['responsible_id'] = $responsible->getKey();
        $data['client_id'] = $client->id;
        $data['created_by'] = $user->id;

        $status = $initiative->update($data);
        if ($status) {
            app(SyncInitiativeSegment::class)->execute([
                'initiative' => $initiative,
                'segment_code' => $data['publish_to']
            ]);
        }


        // Send Invite Notifications to employees
        $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
        $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

        $userArr = $client->users()->get();
        $owner = $initiative->creator;
        if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_INIT]) {
            foreach ($userArr as $iuser) {
                SendNotificationConsole::dispatch($owner, $iuser, 'Initiative Modified');
            }
        }
        if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_INIT]) {
            foreach ($userArr as $iuser) {
                SendNotificationEmail::dispatch($iuser, EmailNotifications::INITIATIVE_UPDATED, ['initiative' => $initiative->title, 'owner' => $owner->surname . ', ' . $owner->first_name]);
            }
        }
        return $initiative;
    }
}