<?php


namespace App\Services\Library;

use App\Contracts\Performable;
use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\AllowedFileSizes;
use App\Enum\AllowedFileTypes;
use App\Enum\EmailNotifications;
use App\Enum\PlanConfigurations;
use App\Enum\PowerSearchActions;
use App\Enum\PowerSearchTypes;
use App\Enum\Roles;
use App\Exceptions\UnauthorizedHttpException;
use App\Facades\Context;
use App\Models\Library\LibraryItem;
use App\Services\Attachment\CreateAttachment;
use App\Helpers\ResourceLocationBuilder;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;
use App\Services\UserAccess\GetAccessByRole;
use Illuminate\Support\Arr;
use App\Jobs\IndexPowerSearch;
use App\Jobs\SendNotificationEmail;
use Qualee\AiDocSearch\Services\ServerlessService;
use Str;
use Illuminate\Support\Facades\Log;

class CreateLibraryAttachment extends BaseService implements Validatable, Transactionable, Performable
{
    public function getRules(array $args): array
    {
        return [
            'name' => 'required|string',
            'client_id' => 'string',
            'document_profile_id' => 'required',
            'type' => 'string|required',
            'file' => 'required|mimes:' . AllowedFileTypes::COMMON_FILE . '|max:' . AllowedFileSizes::COMMON_SIZE
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Create library item
     *
     * @param array $args
     * @return LibraryItem
     */
    public function performAction(array $args): LibraryItem
    {
        $client = Context::client()->get();
        $user = Context::user()->get();

        app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_DOCUMENT);

        if (!app(hasAccessOnContentProfile::class)->execute($args['document_profile_id'])) {
            throw new UnauthorizedHttpException();
        }

        $clientId = $client->id;
        $args['client_id'] = $clientId;
        $args['created_by'] = $user->id;
        $args['uid'] = (string) Str::uuid();

        $fileDir = (new ResourceLocationBuilder())->setClient($client->getBusinessKey())
            ->setDocumentProfile($args['document_profile_id'])->getDocumentProfilePath();
        $attachment = app(CreateAttachment::class)->execute($args['file'], $fileDir, $user->id);
        $inputFile = $args['file'];


        $args = Arr::except($args, ['file']);
        $libraryItem = new LibraryItem($args);

        $libraryItem->linkable()->associate($attachment);

        $libraryItem->save();

        IndexPowerSearch::dispatch($libraryItem->id, PowerSearchTypes::LIBRARY, PowerSearchActions::ADD, $libraryItem->client_id);

        if (!empty($inputFile)) {
            $args['file'] = $inputFile;
            $args['file_path'] = $attachment->path;
            app(ServerlessService::class)->addDoc($args);
            app(ServerlessService::class)->startDetectTextract($args);
        }


        // Send content profile access update notification to employees
        $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
        $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];
        // print_r($accessEmployeePermissions);
        if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_FILE]) {
            SendNotificationEmail::dispatch($user, EmailNotifications::FILE_ADDED, ['filename' => $attachment->name]);
        }

        return $libraryItem->refresh();
    }
}