<?php


namespace App\Services\Library;

use App\Contracts\Performable;
use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\PlanConfigurations;
use App\Enum\PowerSearchActions;
use App\Enum\PowerSearchTypes;
use App\Exceptions\UnauthorizedHttpException;
use App\Facades\Context;
use App\Models\Library\LibraryItem;
use App\Models\Library\Link;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;
use Illuminate\Support\Arr;
use App\Jobs\IndexPowerSearch;
use Illuminate\Support\Str;

class CreateLibraryLink extends BaseService implements Transactionable, Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args): array
    {
        return [
            'name' => 'required|string',
            'client_id' => 'string',
            'document_profile_id' => 'required',
            'type' => 'string|required',
            'url' => 'string|required',
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Create library item
     *
     * @param array $args
     * @return LibraryItem
     */
    public function performAction(array $args): LibraryItem
    {
        $client = Context::client()->get();
        $user = Context::user()->get();

        app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_DOCUMENT);

        if (!app(hasAccessOnContentProfile::class)->execute($args['document_profile_id'])) {
            throw new UnauthorizedHttpException();
        }

        $clientId = $client->id;
        $args['client_id'] = $clientId;
        $args['created_by'] = $user->id;
        $args['uid'] = (string)Str::uuid();
        
        $link = Link::create(['url' =>$args['url']]);

        $args = Arr::except($args, ['link']);
        $libraryItem = new LibraryItem($args);

        $libraryItem->linkable()->associate($link);

        $libraryItem->save();

        IndexPowerSearch::dispatch($libraryItem->id, PowerSearchTypes::LIBRARY, PowerSearchActions::ADD, $libraryItem->client_id);

        return $libraryItem->refresh();
    }
}
