<?php


namespace App\Services\Library;

use App\Contracts\Performable;
use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\ActionRequestTypes;
use App\Enum\ErrorCode;
use App\Exceptions\InvalidRequestException;
use App\Exceptions\UnauthorizedHttpException;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\UserAccessHelper;
use App\Models\Attachment;
use App\Models\Library\LibraryItem;
use App\Services\Attachment\DeleteAttachment;
use App\Services\BaseService;
use App\Jobs\IndexPowerSearch;
use Qualee\AiDocSearch\Services\ServerlessService;
use App\Enum\LibraryItemType;

class DeleteMultipleLibraryItem extends BaseService implements Validatable, Transactionable, Performable
{
    public function getRules(array $args): array
    {
        return [
            'id' => 'required',
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }
    /**
     * Delete library item
     *
     * @param array $args
     * @return bool
     */
    public function performAction(array $args): bool
    {
        $user = Context::user()->get();
        $id = $args['id'];
        $libraryItem = LibraryItem::where('id', $args['id'])->first();
        
        if ($libraryItem == null) {
            throw new InvalidRequestException('No Library item with id: ' . $id, ErrorCode::INVALID_PARAM);
        }

        PermissionHelper::hasAccessOn($libraryItem);

        if (!app(hasAccessOnContentProfile::class)->execute($libraryItem->document_profile_id)) {
            throw new UnauthorizedHttpException();
        }
        

        $this->checkLibraryItemEligibleToDelete($libraryItem);

        if($libraryItem->linkable instanceof Attachment) {
            app(DeleteAttachment::class)->execute($libraryItem->linkable);
        } else {
            $libraryItem->linkable()->delete();
        }

        $libraryItem->delete();

        // Delete Docs In AI Doc Search service.
        if ($libraryItem->type === LibraryItemType::FILE) {
            app(ServerlessService::class)->deleteDoc($libraryItem);
        }

        IndexPowerSearch::dispatch($libraryItem->id, 'library', 'delete', $libraryItem->client_id);

        return true;
    }

    /**
     * @param LibraryItem $libraryItem
     * @return bool
     * @throws InvalidRequestException
     */
    public function checkLibraryItemEligibleToDelete(LibraryItem $libraryItem)
    {
        // Check if survey is in-used
        if (count($libraryItem->actionRequests) > 0) {
            $firstActionRequest = $libraryItem->actionRequests->first();
            $actionName = ActionRequestHelper::getActionableName(
                $firstActionRequest->actionable_type, $firstActionRequest->actionable
            );
            $actionType = ActionRequestHelper::getActionableType($firstActionRequest->actionable_type);
            
            switch ($actionType) {
                case ActionRequestTypes::ACTION_SEQUENCE:
                    $errorCode = ErrorCode::LIBRARY_ITEM_IN_USED_ACTION_SEQUENCE;
                    break;
                case ActionRequestTypes::ACTION:
                    $errorCode = ErrorCode::LIBRARY_ITEM_IN_USED_ACTION;
                    break;
                case ActionRequestTypes::JOURNEY:
                    $errorCode = ErrorCode::LIBRARY_ITEM_IN_USED_JOURNEY;
                    break;
                default:
                    $errorCode = null;
            }
            
            throw new InvalidRequestException(
                "This file being used in $actionName $actionType. To Delete the file, please remove it from the $actionType first.",
                $errorCode,
                ['action' => $actionName, 'type' => $actionType]
            );
        }
        
        return true;
    }
}
