<?php


namespace App\Services\Library;

use App\Enum\AccessPermissions;
use App\Enum\FilterTypes;
use App\Enum\LibraryItemType;
use App\Facades\Context;
use App\Helpers\ModelHelper;
use App\Helpers\SortFluent;
use App\Helpers\UserAccessHelper;
use App\Models\Journey\JourneyUser;
use App\Models\Library\LibraryItem;
use App\Models\Attachment;
use App\Services\BaseService;
use App\Traits\Pageable;
use App\Traits\Sortable;
use App\Helpers\FilterFluent;
use App\Models\Journey\JourneyStep;

class GetLibraryItems extends BaseService
{
    use Sortable, Pageable;

    /**
     * Get list event
     * @param array $args
     * @return mixed
     * @throws \Illuminate\Validation\ValidationException
     */
    public function performAction(array $args)
    {
        $user = Context::user()->get();
        $options = Context::request()->getOptions();

        $query = LibraryItem::query()->with('client', 'actionRequests');
        $query->select('library_items.*');

        if (!UserAccessHelper::hasAccess(AccessPermissions::ACCESS_COMPANY_LIBRARY) &&
            UserAccessHelper::hasAccess(AccessPermissions::ACCESS_CONTENT_PROFILE_LIBRARY)) {
            $query->whereIn('document_profile_id', $user->employee->getDocumentProfileIds());
        }

        ModelHelper::setConditionFindByClient($query, "library_items");

        FilterFluent::init()
        ->setQuery($query)
        ->setTable('library_items')
        ->setFilterOptions($options)
        ->setAllowedFilters(['id', 'name', 'file_name', 'created_at', 'document_profile', 'uploaded_by', 'extension', 'type', 'journeys', 'isUsed'])
        ->validate()
        ->removeFilter('file_name')
        ->removeFilter('document_profile')
        ->removeFilter('uploaded_by')
        ->removeFilter('created_at')
        ->removeFilter('extension')
        ->removeFilter('journeys')
        ->removeFilter('isUsed')
        ->setCustomFilter('file_name', function ($query, $type, $col, $value) {
            $query->whereHasMorph('linkable', Attachment::class, function ($q) use ($type, $col, $value) {
                FilterFluent::applyBasicFilter($q, $type, 'name', $value);
            });
        })
        ->setCustomFilter('document_profile', function ($query, $type, $col, $value) {
            $query->whereHas('documentProfile', function ($q) use ($type, $col, $value) {
                FilterFluent::applyBasicFilter($q, $type, 'name', $value);
            });
        })
        ->setCustomFilter('created_at', function ($query, $type, $col, $value) {
            $dateValue = explode(";",$value);
            $fromDate = $dateValue[0];
            $toDate = $dateValue[1];
            FilterFluent::applyBasicFilter($query, FilterTypes::GREATER_THAN_OR_EQUAL, 'created_at', $fromDate);
            FilterFluent::applyBasicFilter($query, FilterTypes::LESS_THAN_OR_EQUAL, 'created_at', $toDate);
        })
        ->setCustomFilter('journeys', function ($query, $type, $col, $value) {
            $query->whereHas('actionRequests', function ($q2) use ($type, $col, $value) {
                $q2->whereHasMorph('actionable', JourneyStep::class, function ($q3) use ($type, $col, $value) {
                    $q3->whereHas('journey', function ($q4) use ($type, $col, $value) {
                        FilterFluent::applyBasicFilter($q4, $type, 'code', $value);
                    });
                });
            });
        })
        ->setCustomFilter('isUsed', function ($query, $type, $col, $value) {
            if ($value == "Yes") {
                $query->whereHas('actionRequests');
            } else {
                $query->whereDoesntHave('actionRequests');
            }
        })
        ->setCustomFilter('uploaded_by', function ($query, $type, $col, $value) {
            $query->whereHas('creator', function ($q) use ($type, $col, $value) {
                FilterFluent::applyBasicFilter($q, $type, 'code', $value);
            });
        })
        ->setCustomFilter('extension', function ($query, $type, $col, $value) {
            if (in_array(strtolower(LibraryItemType::LINK), explode(',', $value))) {
                $query->where(function ($subQuery) use ($type, $col, $value) {
                    $subQuery->where('type', LibraryItemType::LINK)
                        ->orWhereHasMorph('linkable', Attachment::class, function ($q) use ($type, $col, $value) {
                            FilterFluent::applyFileExtensionFilter($q, $type, 'original_name', $value);
                        });
                });
            } else {
                $query->whereHasMorph('linkable', Attachment::class, function ($q) use ($type, $col, $value) {
                    FilterFluent::applyFileExtensionFilter($q, $type, 'original_name', $value);
                });
            }
        })->applyFilter();

        SortFluent::init()
        ->setQuery($query)
        ->setTable('library_items')
        ->setSortOptions($options)
        ->setAllowedSorts(['name', 'file_name', 'created_at', 'document_profile', 'uploaded_by', 'extension', 'size'])
        ->validate()
        ->removeSort(['uploaded_by','file_name','document_profile', 'extension', 'size'])
        ->setCustomSort('uploaded_by', function ($query, $col,$direction) {
            $query->leftJoin('users','created_by','users.id');
            SortFluent::applyBasicSort($query, 'name',$direction,'users');
        })
        ->setCustomSort('file_name', function ($query, $col, $direction) {
            $query->leftJoin('attachment', 'linkable_id',  'attachment.id');
            SortFluent::applyBasicSort($query,  'name', $direction, 'attachment');
        })
        ->setCustomSort('document_profile', function ($query, $col, $direction) {
            $query->leftJoin('document_profiles', 'document_profile_id',  'document_profiles.id');
            SortFluent::applyBasicSort($query,  'name', $direction, 'document_profiles');
        })
        ->setCustomSort('extension', function ($query, $col, $direction) {
            $query->leftJoin('attachment as at1', function ($join) {
                $join->on('linkable_id', 'at1.id')
                    ->where('linkable_type', Attachment::class);
            });
            SortFluent::applyFileExtensionSort($query,  'original_name', $direction, 'at1');
        })
        ->setCustomSort('size', function ($query, $col, $direction) {
            $query->leftJoin('attachment as at2', function ($join) {
                $join->on('linkable_id', 'at2.id')
                    ->where('linkable_type', Attachment::class);
            });
            SortFluent::applyBasicSort($query,  'size', $direction, 'at2');
        })
        ->applySort();
        
        $query->latest();

        if (isset($data['isNotPaging']) && $data['isNotPaging']) {
            return $query->get();
        } else {
            return $this->applyPaging($options, $query);
        }
    }
}
