<?php


namespace App\Services\Library;

use App\Contracts\Performable;
use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\AllowedFileSizes;
use App\Enum\AllowedFileTypes;
use App\Enum\EmailNotifications;
use App\Enum\PowerSearchActions;
use App\Enum\PowerSearchTypes;
use App\Exceptions\UnauthorizedHttpException;
use App\Facades\Context;
use App\Helpers\PermissionHelper;
use App\Helpers\UserAccessHelper;
use App\Models\Library\LibraryItem;
use App\Services\BaseService;
use App\Services\UserAccess\GetAccessByRole;
use Illuminate\Support\Arr;
use App\Exceptions\InvalidRequestException;
use App\Enum\LibraryItemType;
use App\Enum\Roles;
use App\Helpers\ResourceLocationBuilder;
use App\Services\Attachment\UpdateAttachment;
use App\Jobs\IndexPowerSearch;
use App\Jobs\SendNotificationEmail;
use Qualee\AiDocSearch\Services\ServerlessService;
use Str;

class UpdateLibraryItem extends BaseService implements Validatable, Transactionable, Performable
{
    public function getRules(array $args): array
    {
        return [
            'id' => 'required',
            'name' => 'required|string',
            'file' => 'nullable|mimes:' . AllowedFileTypes::COMMON_FILE . '|max:' . AllowedFileSizes::COMMON_SIZE
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Update library item
     *
     * @param array $args
     * @return LibraryItem
     */
    public function performAction(array $args): LibraryItem
    {
        $user = Context::user()->get();
        $client = Context::client()->get();

        $clientId = $client->id;
        $args['client_id'] = $clientId;
        $libraryItem = LibraryItem::findOrFail($args['id']);

        if (!app(hasAccessOnContentProfile::class)->execute($libraryItem->document_profile_id)) {
            throw new UnauthorizedHttpException();
        }

        PermissionHelper::hasAccessOn($libraryItem);
        if ($libraryItem->type === LibraryItemType::FILE) {
            if (!empty($args['file'])) {
                $fileDir = (new ResourceLocationBuilder())->setClient($client->getBusinessKey())
                    ->setDocumentProfile($libraryItem->document_profile_id)->getDocumentProfilePath();
                $oldFileName = $libraryItem->linkable->path;
                $attachment = app(UpdateAttachment::class)->execute($libraryItem->linkable, $args['file'], $fileDir, $user->id);
                $inputFile = $args['file'];
                $args = Arr::except($args, ['file', 'url']);


                // Send content profile access update notification to employees
                $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
                $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];
                if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_FILE]) {
                    SendNotificationEmail::dispatch($user, EmailNotifications::FILE_UPDATED, ['profile' => $libraryItem->documentProfile->name, 'filename' => $attachment->name]);
                }
            }
        } else if ($libraryItem->type === LibraryItemType::LINK) {
            if (empty($args['url'])) {
                throw new InvalidRequestException('Missing url.');
            }
            $args = Arr::except($args, ['file']);
            $link = $libraryItem->linkable;
            $link->update(['url' => $args['url']]);
        } else {
            throw new InvalidRequestException('Invalid library item type.');
        }

        $libraryItem->update($args);

        if ($libraryItem->type === LibraryItemType::FILE) {
            // Update document to AI Doc Search service
            if (!empty($inputFile)) {
                $args['document_profile_id'] = $libraryItem->document_profile_id;
                $args['file_path'] = $attachment->path;
                $args['old_file_path'] = $oldFileName;
                $args['file'] = $inputFile;
                $args['uid'] = $libraryItem->uid;
                $args['env'] = config('qualee-ai-doc-search.ads_env');
                app(ServerlessService::class)->updateDoc($args);
                app(ServerlessService::class)->startDetectTextract($args);
            }
            app(ServerlessService::class)->updateESDoc($libraryItem);
        }

        IndexPowerSearch::dispatch($libraryItem->id, PowerSearchTypes::LIBRARY, PowerSearchActions::ADD, $libraryItem->client_id);

        return $libraryItem->refresh();
    }
}