<?php


namespace App\Services\Message;


use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\ConversationStatues;
use App\Enum\EmailNotifications;
use App\Enum\MessageNotifyTypes;
use App\Enum\NotificationTypes;
use App\Enum\PlanConfigurations;
use App\Enum\Roles;
use App\Events\NotifyMessageCreated;
use App\Http\Resources\Message\ChangeStatusLog;
use App\Jobs\SendNotification;
use App\Jobs\SendNotificationEmail;
use App\Models\Message\AnonymousConversationStatusLog;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;
use App\Services\UserAccess\GetAccessByRole;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\Rule;

/**
 * Class CloseAnonymousConversation
 * @package App\Services\Message
 */
class ChangeAnonymousConversationStatus extends BaseService implements Validatable, Transactionable
{
    /**
     * @param  array  $args
     * @return array
     */
    public function getRules(array $args): array
    {
        return [
            'id' => 'required',
            'status' => ['required', Rule::in([ConversationStatues::ACTIVE, ConversationStatues::CLOSE, ConversationStatues::ESCALATED, ConversationStatues::RESOLVING, ConversationStatues::REVIEWING])]
        ];
    }

    /**
     * @param  array  $args
     */
    public function performAction(array $args): void
    {
        app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_MESSAGE);
        $status = $args['status'];
        $conversation = app(CheckPermissionOnManageAnonymousConversation::class)->execute(['id' => $args['id']]);
        if ($conversation->status !== $status) {
            $conversation->status = $status;
            $conversation->save();
            $latestChangeStatusLog = AnonymousConversationStatusLog::query()
                ->where('conversation_id', $conversation->id)
                ->latest()->first();
            $changeStatusLog = app(CreateAnonymousConversationStatusLogs::class)->execute([
                'conversation_id' => $conversation->id,
                'status' => $status
            ]);
            $this->broadcastChangeStatusToAnonymous($changeStatusLog, $latestChangeStatusLog, $conversation);
        }
    }

    /**
     * @param $changeStatusLog
     * @param $latestChangeStatusLog
     * @param $conversation
     */
    public function broadcastChangeStatusToAnonymous($changeStatusLog, $latestChangeStatusLog, $conversation)
    {
        $anonymousUser = $conversation->users()->wherePivot('is_anonymous', true)->first();
        if ($anonymousUser) {
            $changeStatusLog->statusFrom = $latestChangeStatusLog->status;
            $changeStatusLog->statusTo = $changeStatusLog->status;
            $changeStatusResource = (new ChangeStatusLog($changeStatusLog))->resolve();
            $arguments = array(
                "userIds" => $anonymousUser->code,
                "headings" => __("Your whistleblowing case changed status"),
                "contents" => __(
                    'notify.change_status',
                    ['status' => $changeStatusLog->status]
                ),
                "data" => [
                    "type" => NotificationTypes::MESSAGE_RECEIVED,
                    "conversation_id" => $conversation->id
                ],
                'send_after' => null
            );
            SendNotification::dispatch($arguments);
            broadcast(
                new NotifyMessageCreated(
                    MessageNotifyTypes::CHANGE_STATUS,
                    $changeStatusResource,
                    $anonymousUser->code,
                    $conversation->id
                )
            )->toOthers();


            // Send Invite Notifications to employees
            $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
            $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_MENTION]) {
                SendNotificationEmail::dispatch($anonymousUser, EmailNotifications::MESSAGE_CASE_UPDATE, ['prev_status' => $changeStatusLog->statusFrom, 'new_status' => $changeStatusLog->statusTo, 'case' => $conversation->id]);
            }
        }
    }
}