<?php


namespace App\Services\Message;


use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\ConversationStatues;
use App\Enum\ConversationTypes;
use App\Enum\ErrorCode;
use App\Enum\PlanConfigurations;
use App\Enum\QualeeStatus;
use App\Exceptions\InvalidRequestException;
use App\Exceptions\NotFoundHttpException;
use App\Facades\Context;
use App\Models\Message\Conversation;
use App\Models\User\User;
use App\Models\User\UserPermission;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;

class CheckPermissionOnManageAnonymousConversation extends BaseService implements Validatable
{
    public function getRules(array $args)
    {
        return [
            'id' => 'required'
        ];
    }

    public function performAction(array $args)
    {
        app(CheckRemainPlanAction::class)->execute(
            PlanConfigurations::PACKAGE_WHISTLEBLOWING_CAPABILITIES
        );
        $currentUser = Context::user()->get();
        $conversation = Conversation::query()
            ->whereIn('status', ConversationStatues::ALL_LIST)
            ->find($args['id']);
        if (!$conversation) {
            throw new NotFoundHttpException('The conversation has been deleted', ErrorCode::CONVERSATION_NOT_FOUND);
        }
        // Need to check permission first
        $list = [AccessPermissions::ACCESS_WB_MASTER];
        $permission = UserPermission::query()
            ->join('permissions', 'model_has_permissions.permission_id', 'permissions.id')
            ->where('model_id', $currentUser->id)
            ->where('model_type', User::class)
            ->whereIn('permissions.name', $list)
            ->select('permissions.*')
            ->get()->pluck('name');
        $isMaster = count($permission) > 0;
        if (!app(IsUserBelongToConversation::class)->execute($conversation) && !$isMaster) {
            throw new NotFoundHttpException('The conversation not found', ErrorCode::CONVERSATION_NOT_FOUND);
        }

        if ($conversation->type !== ConversationTypes::ANONYMOUS_MESSAGE) {
            throw new InvalidRequestException(
                "Wrong conversation type",
                ErrorCode::MESSAGE_ERROR
            );
        }
        $isCurrentUserInChargeConversation = $conversation->users()
            ->wherePivot('messageable_id', $currentUser->getKey())
            ->wherePivot('is_anonymous', false)
            ->wherePivot('status', QualeeStatus::ACTIVE)->exists();
        if ((!$currentUser->can(AccessPermissions::ACCESS_WB_MESSAGE) || !$isCurrentUserInChargeConversation) && !$isMaster) {
            throw new InvalidRequestException(
                "Wrong user role",
                ErrorCode::WRONG_USER_ROLE
            );
        }
        return $conversation;
    }
}