<?php


namespace App\Services\Message;


use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\ConversationStatues;
use App\Enum\ErrorCode;
use App\Enum\MessageableTypes;
use App\Enum\PlanConfigurations;
use App\Enum\Roles;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Models\Message\Conversation;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;

/**
 * Class CreateAnonymousConversation
 * @package App\Services\Message
 */
class CreateAnonymousConversation extends BaseService implements Validatable
{
    public function getRules(array $args)
    {
        return [
            'users'                => 'required|array',
            'users.public_user'    => 'required|string',
            'users.anonymous_user' => 'required|string',
        ];
    }

    /**
     * @param  array  $data
     * @return mixed
     */
    public function performAction(array $data)
    {
        $client = Context::client()->get();
        //Only create anonymous when client have plan
        app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_WHISTLEBLOWING_CAPABILITIES);

        $publicUser = User::where('code', $data['users']['public_user'])->whereHas('employee', function ($query) use ($client) {
            $query->where('client_id', $client->getKey());
        })->firstOrFail();

        $anonymousUser = User::where('code', $data['users']['anonymous_user'])->whereHas('employee', function ($query) use ($client) {
            $query->where('client_id', $client->getKey());
        })->firstOrFail();

        if (!$publicUser->can(AccessPermissions::ACCESS_WB_MESSAGE) || $anonymousUser->hasRole(Roles::WHISTLEBLOWER_ADMIN)) {
            throw new InvalidRequestException(
                "Wrong user role",
                ErrorCode::WRONG_USER_ROLE
            );
        }
        $userIds = collect([$publicUser->id, $anonymousUser->id])->unique()->toArray();
        $conversation = Conversation::where('type', $data['type'])
            ->whereHas('users', function ($q) use ($userIds, $publicUser, $anonymousUser) {
                $q->where(function ($query) use ($publicUser, $anonymousUser) {
                    $query->where(function ($query) use ($publicUser) {
                        $query->where('messageables.messageable_id', $publicUser->id)
                            ->where('messageables.is_anonymous', 0);
                    })
                        ->orWhere(function ($query) use ($anonymousUser) {
                            $query->where('messageables.messageable_id', $anonymousUser->id)
                                ->where('messageables.is_anonymous', 1);
                        });
                })
                    ->groupBy('conversation_id')
                    ->havingRaw('COUNT(DISTINCT messageable_id) = '.count($userIds));
            })->doesntHave('rawMessages')->get()->first();

        if (!$conversation || (isset($data['force_create']) && $data['force_create'])) {
            $conversation = app(CreateRawConversation::class)->execute($data);
            $conversation->users()->sync([
                $publicUser->id    => ['type' => MessageableTypes::USER],
                $anonymousUser->id => ['type' => MessageableTypes::USER, 'is_anonymous' => true],
            ], false);
            app(CreateAnonymousConversationStatusLogs::class)->execute([
                'conversation_id' => $conversation->id, 'status' => ConversationStatues::ACTIVE
            ]);
            $content = sprintf("Thank you, your unique case ID is: #%s", $conversation->anonymous_id);
            app(CreateMessage::class)->execute([
                'conversation_id' => $conversation->id, 'content' => $content, 'sender_id' => $publicUser->id
            ]);

            if ($publicUser->can(AccessPermissions::ACCESS_WB_AUTOREPLY)) {
                $config = $client->getConfigurationsAttribute();
                $content = $config['auto_reply_messages'];
                if (!empty($content)) {
                    app(CreateMessage::class)->execute([
                        'conversation_id' => $conversation->id, 'content' => $content, 'sender_id' => $publicUser->id
                    ]);
                }
            }
        }
        return $conversation;
    }
}