<?php


namespace App\Services\Message;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\ConversationTypes;
use App\Enum\ErrorCode;
use App\Enum\MessageableTypes;
use App\Enum\QualeeStatus;
use App\Enum\Roles;
use App\Enum\SegmentTypes;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\SegmentHelper;
use App\Models\Message\Conversation;
use App\Models\Segment\SegmentType;
use App\Models\User\User;
use App\Services\BaseService;
use Illuminate\Validation\Rule;

/**
 * Class CreateConversation
 * @package App\Services\Message
 */
class CreateConversation extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules(array $args)
    {
        return [
            'type' => ['required', Rule::in(ConversationTypes::CREATABLE_TYPES)],
            'receiver_code' => ['string', 'required', function ($attribute, $value, $fail) use ($args) {
                if ($value === Context::user()->get()->code && $args['type'] !== ConversationTypes::NOTIFY_MESSAGE) {
                    $fail(__('validation.invalid', ['attribute' => $attribute]));
                }
            }],
            'segment_type_code' => [Rule::requiredIf(function () use ($args) {
                return $args['type'] === ConversationTypes::NOTIFY_MESSAGE;
            })]
        ];
    }

    /**
     * @return bool
     */
    public function canPerform(): bool
    {
        return Context::client()->check() && !Context::client()->get()->is_demo;
    }

    /**
     * Create document profile
     *
     * @param array $data
     * @return Conversation
     */
    public function performAction(array $data): Conversation
    {
        $type = $data['type'];
        switch ($type) {
            case ConversationTypes::DIRECT_MESSAGE:
            case ConversationTypes::NOTIFY_MESSAGE_EMPLOYEE:
                return $this->createBasicConversation($data);
                break;
            case ConversationTypes::NOTIFY_MESSAGE:
                return $this->createNotifyConversation($data);
                break;
            case ConversationTypes::ANONYMOUS_MESSAGE:
                return $this->createAnonymousConversation($data);
                break;
        }
    }

    /**
     * @param $data
     * @return Conversation
     */
    private function createBasicConversation($data)
    {

        return app(CreateBasicConversation::class)->execute($data);
    }

    /**
     * @param $data
     * @return Conversation
     */
    private function createNotifyConversation($data)
    {
        return app(CreateNotifyConversation::class)->execute($data);
    }

    /**
     * @param $data
     * @return Conversation
     */
    public function createAnonymousConversation($data)
    {
        return app(CreateAnonymousConversation::class)->execute($data);
    }
}
