<?php


namespace App\Services\Message;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\AllowedFileSizes;
use App\Enum\AllowedFileTypes;
use App\Enum\ConversationStatues;
use App\Enum\ConversationTypes;
use App\Enum\ErrorCode;
use App\Enum\PlanConfigurations;
use App\Events\OCRDocumentUploaded;
use App\Exceptions\InvalidRequestException;
use App\Exceptions\NotFoundHttpException;
use App\Facades\Context;
use App\Models\Message\Message;
use App\Services\BaseService;
use App\Services\Attachment\CreateAttachment;
use App\Helpers\ResourceLocationBuilder;
use App\Enum\MessageType;
use App\Services\Subscription\CheckRemainPlanAction;

class CreateMessage extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($data)
    {
        return [
            'content' => 'required_without:file|string',
//            'file' => 'required_without:content|nullable|mimes:' . AllowedFileTypes::COMMON_FILE . '|max:' . AllowedFileSizes::COMMON_SIZE,
            'file' => 'required_without:content|nullable|max:' . AllowedFileSizes::COMMON_SIZE,
            'conversation_id' => 'required|string',
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check() && !Context::client()->get()->is_demo;
    }

    /**
     * Create message
     *
     * @param array $data
     * @return Message
     */
    public function performAction(array $data): Message
    {
        app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_MESSAGE);

        $client = Context::client()->get();

        $currentUserId = Context::user()->get()->getKey();

        $conversation = app(CheckPermissionUserOnConversation::class)->execute($data);

        //Closed conversation can't send message
        if ($conversation->status === ConversationStatues::CLOSE) {
            throw new InvalidRequestException('Can\'t send message to this conversation', ErrorCode::MESSAGE_ERROR);
        }
        //Check client has whistleblowing permission
        if($conversation->type === ConversationTypes::ANONYMOUS_MESSAGE){
            app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_WHISTLEBLOWING_CAPABILITIES);
        }

        $message = new Message();
        $message->sender_id = $data['sender_id'] ?? $currentUserId;
        $message->content = $data['content'] ?? '';
        $message->conversation_id = $conversation->id;

        if (!empty($data['file'])) {
            $fileDir = (new ResourceLocationBuilder())->setClient($client->code)
                ->getMessageAttachmentPath();
            $attachment =  app(CreateAttachment::class)->execute($data['file'], $fileDir, $currentUserId);
            $data['content_type'] = isset($data['content_type']) && $data['content_type'] === MessageType::NOTE ? MessageType::NOTE_FILE : MessageType::FILE;
            $message->attachment()->associate($attachment);

            event(new OCRDocumentUploaded($attachment));
        }

        $message->content_type = $data['content_type'] ?? MessageType::TEXT; // File or text or note
        $message->save();

        return $message->refresh();
    }
}
