<?php


namespace App\Services\Message;


use App\Contracts\Validatable;
use App\Enum\MessageType;
use App\Helpers\ZipHelper;
use App\Models\Message\AnonymousConversationStatusLog;
use App\Models\Message\WhistleblowingTransferLog;
use App\Services\BaseService;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

/**
 * Class DownloadAnonymousConversation
 * @package App\Services\Message
 */
class DownloadAnonymousConversation extends BaseService implements Validatable
{
    public $publicUserIds;
    public $users;
    public $conversation;
    public $folderName;
    public $columns;
    public $languages;
    /**
     * @param  array  $args
     */
    public function getRules(array $args)
    {
        return [
            'conversation_id' => 'required|string'
        ];
    }


    /**
     * @param  array  $args
     */
    public function performAction(array $args)
    {
        $this->conversation = app(CheckPermissionOnManageAnonymousConversation::class)->execute(
            ['id' => $args['conversation_id']]
        );
        $this->users = $this->conversation->users->mapWithKeys(
            function ($item) {
                return [$item['id'] => $item];
            }
        );
        $this->publicUserIds = $this->conversation->users->filter(
            function ($item) {
                return $item->pivot->is_anonymous === 0;
            }
        )->pluck('id')->toArray();

        //Create folder to store messages
        $this->folderName = "messages/".Str::uuid();
        Storage::disk('local')->makeDirectory($this->folderName);
        $folderPath = config('filesystems.disks.local.root').'/'.$this->folderName;
        $fileName = $folderPath.'/messages.csv';

        //Get data export
        $file = fopen($fileName, 'w');
        fprintf($file, chr(0xEF).chr(0xBB).chr(0xBF));

        $this->columns = ['index', 'Date', 'Content', 'Type', 'User'];
        $dataExports = $this->getDataExport();
        $index = 1;
        $data = [];
        foreach ($dataExports as $export) {
            $row = [$index++, $export->date, $export->content, $export->type, $export->user];
            if (isset($export->messageTranslations)) {
                foreach($this->languages as $language){
                    $row[] = "";
                }
                foreach($export->messageTranslations as $messageTranslation){
                    $indexLanguage = array_search($messageTranslation->language->name, $this->columns);
                    $row[$indexLanguage] = $messageTranslation->content;
                }
            }
            $data[] = $row;
        }

        fputcsv($file, $this->columns);
        foreach ($data as $row) {
            fputcsv($file, $row);
        }

        fclose($file);
        $zippedFile = ZipHelper::zipFolder($folderPath);

        Storage::disk('local')->deleteDirectory($this->folderName);

        if ($zippedFile) {
            return response()->download($zippedFile)->deleteFileAfterSend();
        }
    }

    public function getDataExport()
    {
        $dataExports = collect([]);
        $this->transferMessageToDataExport($dataExports);
        $this->transferChangeStatusLogsToDataExport($dataExports);
        $this->transferChangeWBAdminLogsToDataExport($dataExports);
        return $dataExports->sortBy('date');
    }

    public function transferMessageToDataExport(&$dataExports)
    {
        $messages = $this->conversation->rawMessages;
        $messages->load(['messageTranslations']);
        foreach ($messages as $message) {
            $dataExport = (object)[];
            $dataExport->date = $message->created_at;
            $dataExport->type = $message->content_type;
            if ($message->content_type === MessageType::FILE || $message->content_type === MessageType::NOTE_FILE) {
                $attachment = $message->attachment;
                $dataExport->content = $attachment->name ?? "";

                $path = $this->folderName.'/'.$attachment->original_name;
                // Save image from S3 to local
                Storage::disk('local')->put($path, Storage::disk('s3')->get($attachment->path));
            } elseif ($message->content_type === MessageType::TEXT || $message->content_type === MessageType::NOTE) {
                $dataExport->content = $message->content ?? "";
            }
            if (in_array($message->sender_id, $this->publicUserIds, true)) {
                $dataExport->user = $this->users[$message->sender_id]->fullName;
            } else {
                $dataExport->user = "";
            }
            //Migrate data
            if ($message->messageTranslations->count() > 0) {
                foreach ($message->messageTranslations as $messageTranslation) {
                    $this->addLanguageToColumn($messageTranslation->language->name);
                }
                $dataExport->messageTranslations = $message->messageTranslations;
            }

            $dataExports->push($dataExport);
        }
    }

    public function addLanguageToColumn($value)
    {
        if (!in_array($value, $this->columns)) {
            $this->columns[] = $value;
            $this->languages[] = $value;
        }
    }

    public function transferChangeStatusLogsToDataExport(&$dataExports)
    {
        $anonymousConversationStatusLogs = AnonymousConversationStatusLog::query()
            ->where('conversation_id', $this->conversation->id)->with('user')->get();
        foreach ($anonymousConversationStatusLogs as $anonymousConversationStatusLog) {
            $dataExport = (object)[];
            $dataExport->date = $anonymousConversationStatusLog->created_at;
            $dataExport->type = "Change status";
            $dataExport->content = "Change status to ".$anonymousConversationStatusLog->status;
            $logUser = $anonymousConversationStatusLog->user;
            $dataExport->user = in_array($logUser->getKey(), $this->publicUserIds, true) ?
                $logUser->fullName : "";
            $dataExports->push($dataExport);
        }
    }

    public function transferChangeWBAdminLogsToDataExport(&$dataExports)
    {
        $whistleblowingTransferLogs = WhistleblowingTransferLog::query()
            ->where('conversation_id', $this->conversation->id)
            ->with(['transferFrom', 'transferTo'])
            ->get();
        foreach ($whistleblowingTransferLogs as $whistleblowingTransferLog) {
            $transferFromName = $whistleblowingTransferLog->transferFrom->fullName ?? "";
            $transferToName = $whistleblowingTransferLog->transferTo->fullName ?? "";
            $dataExport = (object)[];
            $dataExport->date = $whistleblowingTransferLog->created_at;
            $dataExport->type = "Transfer whistleblower";
            $dataExport->content = "Change whistleblower from ".$transferFromName." to ".$transferToName;
            $dataExport->user = $transferFromName;
            $dataExports->push($dataExport);
        }
    }


}