<?php


namespace App\Services\Message;

use App\Enum\AccessPermissions;
use App\Enum\ConversationListTypes;
use App\Enum\ConversationStatues;
use App\Enum\ConversationTypes;
use App\Enum\FilterTypes;
use App\Enum\MessageableTypes;
use App\Enum\PlanConfigurations;
use App\Enum\QualeeStatus;
use App\Facades\Context;
use App\Helpers\FilterFluent;
use App\Helpers\SegmentHelper;
use App\Models\Message\Conversation;
use App\Models\Message\Message;
use App\Models\Segment\SegmentType;
use App\Models\User\User;
use App\Models\User\UserPermission;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;
use App\Traits\Pageable;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class GetConversations extends BaseService
{
    use Pageable;

    /**
     * Get conversations of user
     *
     * @param  array  $data
     * @return mixed
     */
    public function performAction(array $args)
    {
        $client = Context::client()->get();
        app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_MESSAGE);
        $options = Context::request()->getOptions();
        $currentUser = Context::user()->get();

        /**
         * Types of conversation list.
         * Direct messages - ACTIVE: Get only direct messages
         * Active cases - OPEN: Get anonymous messages, status is ACTIVE
         * Closed cases - CLOSE: Get anonymous messages, status is CLOSE
         */
        $conversationType = $args['status'] ?? ConversationListTypes::DIRECT_MESSAGE;



        $queryMessages = Message::select('conversation_id', DB::raw("max(messages.created_at) as message_created_at"))
            ->groupBy('messages.conversation_id');
        $query = Conversation::query()
            ->join('messageables', 'conversations.id', '=', 'messageables.conversation_id')
            ->joinSub($queryMessages, 'messages', function ($join) {
                $join->on('conversations.id', 'messages.conversation_id');
                $join->on('messages.message_created_at', '>', 'messageables.last_available');
            });

        $list = [AccessPermissions::ACCESS_WB_MASTER];
        $permission = UserPermission::query()
            ->join('permissions', 'model_has_permissions.permission_id', 'permissions.id')
            ->where('model_id', $currentUser->id)
            ->where('model_type', User::class)
            ->whereIn('permissions.name', $list)
            ->select('permissions.*')
            ->get()->pluck('name');

        $query->where('messageables.status', QualeeStatus::ACTIVE)
            ->where('conversations.type', '!=', ConversationTypes::NOTIFY_MESSAGE_EMPLOYEE)
            ->where('conversations.client_id', $client->getKey())
            ->select('conversations.*', 'messages.message_created_at')
            ->orderBy('conversations.updated_at', 'DESC');

        if (isset($args['id'])) {
            Conversation::findOrFail($args['id']);
            $query->where('id', $args['id']);
        } else {
            switch ($conversationType) {
                case ConversationListTypes::DIRECT_MESSAGE:
                    $query->where('conversations.type', '!=', ConversationTypes::ANONYMOUS_MESSAGE);
                    $query->where('conversations.status', ConversationStatues::ACTIVE);
                    $query->where('messageables.messageable_id', $currentUser->id);
                    break;
                case ConversationListTypes::ACTIVE:
                    $query->where('conversations.type', ConversationTypes::ANONYMOUS_MESSAGE);
                    $query->whereIn('conversations.status', ConversationStatues::OPEN_LIST);
                    if (count($permission) === 0) {
                        $query->where('messageables.messageable_id', $currentUser->id);
                    }
                    break;
                case ConversationListTypes::CLOSE:
                    $query->where('conversations.status', ConversationStatues::CLOSE);
                    if (count($permission) === 0) {
                        $query->where('messageables.messageable_id', $currentUser->id);
                    }
                    $config = $client->getConfigurationsAttribute();
                    $days = isset($config['period_case_visibility']) ? (int) $config['period_case_visibility'] : 14;
                    if ($days > 0) {
                        $date = Carbon::now()->subDays($days)->toDateString();
                        $query->whereDate('conversations.updated_at', '>=' , $date);
                    }
                    break;
                default:
                    break;
            }
        }
        if (!app(CheckRemainPlanAction::class)->execute(
            PlanConfigurations::PACKAGE_WHISTLEBLOWING_CAPABILITIES,
            false
        )) {
            $query->where('conversations.type', '!=', ConversationTypes::ANONYMOUS_MESSAGE);
        }
        $query->with(['users', 'owner', 'anonymousConversationStatusLogs', 'messages', 'anonymousConversationCategories']);

        $filter = FilterFluent::init()
            ->setQuery($query)
            ->setFilterOptions($options)
            ->setAllowedFilters(['message_type', 'message_content', 'internal_notes', 'notification_send_to', 'status',
                'ticket_id', 'assigned_to', 'assigned_to_me', 'create_date', 'close_date', 'report_category', 'days_open', 'case_location'])
            ->validate();

        $filter->removeFilter(['message_type', 'message_content', 'internal_notes', 'notification_send_to', 'status',
            'ticket_id', 'assigned_to', 'assigned_to_me', 'create_date', 'close_date', 'report_category', 'days_open', 'case_location'])
            ->setCustomFilter('message_type', function ($query, $type, $col, $value) {
                if ($value === 'employeeEvent') {
                    FilterFluent::applyBasicFilter($query, FilterTypes::IS, 'type', ConversationTypes::DIRECT_MESSAGE, 'conversations');
                    $query1 = Message::query()
                        ->where('content_type', 'text')
                        ->where(function($subQuery) {
                            $subQuery->where('content', 'LIKE', "%year work anniversary today%")
                                ->orWhere('content', 'LIKE', "%birthday today%");
                        })->select('conversation_id');
                    $queryMessages = Message::query()->select('messages.conversation_id')->distinct()->whereIn('messages.conversation_id', $query1);
                    $query->whereIn('conversations.id', $queryMessages);
                } else if ($value === 'notification') {
                    FilterFluent::applyBasicFilter($query, FilterTypes::IS, 'type', ConversationTypes::NOTIFY_MESSAGE, 'conversations');
                } else {
                    FilterFluent::applyBasicFilter($query, FilterTypes::IS, 'type', ConversationTypes::DIRECT_MESSAGE, 'conversations');
                }
            })
            ->setCustomFilter('message_content', function ($query, $type, $col, $value) {
                $query1 = Message::query()
                    ->whereIn('content_type', ['text', 'file'])
                    ->where('content', 'LIKE', "%" . $value . "%")
                    ->select('conversation_id');
                $queryMessages = Message::query()->select('messages.conversation_id')->distinct()->whereIn('messages.conversation_id', $query1);

                if ($type === FilterTypes::CONTAIN) {
                    $query->whereIn('conversations.id', $queryMessages);
                } else if ($type === FilterTypes::NOT_CONTAIN) {
                    $query->whereNotIn('conversations.id', $queryMessages);
                }
            })
            ->setCustomFilter('internal_notes', function ($query, $type, $col, $value) {
                $query1 = Message::query()
                    ->whereIn('content_type', ['note', 'file_note'])
                    ->where('content', 'LIKE', "%" . $value . "%")
                    ->select('conversation_id');
                $queryMessages = Message::query()->select('messages.conversation_id')->distinct()->whereIn('messages.conversation_id', $query1);

                if ($type === FilterTypes::CONTAIN) {
                    $query->whereIn('conversations.id', $queryMessages);
                } else if ($type === FilterTypes::NOT_CONTAIN) {
                    $query->whereNotIn('conversations.id', $queryMessages);
                }
            })
            ->setCustomFilter('notification_send_to', function ($query, $type, $col, $value) {
                $receiverValue = explode(";", $value);
                $code = $receiverValue[0];
                $segmentType = $receiverValue[1];
                $segmentTypes = SegmentType::findByBusinessKey($segmentType);
                $receiver = SegmentHelper::getModelSegmentBySegmentType($segmentTypes, $code);
                $receiverId = (int)$receiver->getKey();

                $conversationIds = Conversation::select('id')
                    ->join('messageables', 'conversations.id', '=', 'messageables.conversation_id')
                    ->where('messageables.messageable_id', $receiverId)
                    ->where('messageables.type', MessageableTypes::SEGMENT)
                    ->get()
                    ->toArray();

                $query->whereIn('conversations.id', $conversationIds);
            })
            ->setCustomFilter('status', function ($query, $type, $col, $value) {
                if ($value !== ConversationStatues::REOPEN) {
                    FilterFluent::applyBasicFilter($query, $type, 'status', $value, 'conversations');
                } else {
                    FilterFluent::applyBasicFilter($query, $type, 'status', ConversationStatues::ACTIVE, 'conversations');
                    $query->whereHas('anonymousConversationStatusLogs', function($q) use ($type, $col, $value) {
                        FilterFluent::applyBasicFilter($q, FilterTypes::IS, 'status', ConversationStatues::CLOSE, 'anonymous_conversation_status_logs');
                    });
                }
            })
            ->setCustomFilter('ticket_id', function ($query, $type, $col, $value) {
                if ($type === FilterTypes::IS_NOT) {
                    $query->where('conversations.anonymous_id', '!=', $value);
                } else {
                    FilterFluent::applyBasicFilter($query, $type, 'anonymous_id', $value, 'conversations');
                }
            })
            ->setCustomFilter('assigned_to', function ($query, $type, $col, $value) {
                $assignedUser = User::findByBusinessKey($value);
                $query->where('messageables.messageable_id', $assignedUser->id);
            })
            ->setCustomFilter('assigned_to_me', function ($query, $type, $col, $value) use ($currentUser) {
                $query->where('messageables.messageable_id', $currentUser->id);
            })
            ->setCustomFilter('create_date', function ($query, $type, $col, $value) {
                $dateValue = explode(";",$value);
                $fromDate = Carbon::parse($dateValue[0])->startOfDay()->toDateTimeString();
                $toDate = Carbon::parse($dateValue[1])->addDay(1)->toDateTimeString();
                FilterFluent::applyBasicFilter($query, FilterTypes::GREATER_THAN_OR_EQUAL, 'conversations.created_at', $fromDate);
                FilterFluent::applyBasicFilter($query, FilterTypes::LESS_THAN_OR_EQUAL, 'conversations.created_at', $toDate);
            })
            ->setCustomFilter('close_date', function ($query, $type, $col, $value) {
                $query->whereHas('anonymousConversationStatusLogs', function($q) use ($type, $col, $value) {
                    FilterFluent::applyBasicFilter($q, FilterTypes::IS, 'status', ConversationStatues::CLOSE, 'anonymous_conversation_status_logs');
                    $dateValue = explode(";",$value);
                    $fromDate = Carbon::parse($dateValue[0])->startOfDay()->toDateTimeString();
                    $toDate = Carbon::parse($dateValue[1])->addDay(1)->toDateTimeString();
                    FilterFluent::applyBasicFilter($q, FilterTypes::GREATER_THAN_OR_EQUAL, 'anonymous_conversation_status_logs.created_at', $fromDate);
                    FilterFluent::applyBasicFilter($q, FilterTypes::LESS_THAN_OR_EQUAL, 'anonymous_conversation_status_logs.created_at', $toDate);
                });
            })
            ->setCustomFilter('report_category', function ($query, $type, $col, $value) {
                $receiverValue = array_map(function ($item) { return (int) $item; }, explode(";", $value));
                $query->leftJoin('whistleblowing_category', 'conversations.id', '=', 'whistleblowing_category.conversation_id')
                    ->whereIn('whistleblowing_category.category_id', $receiverValue);
            })
            ->setCustomFilter('case_location', function ($query, $type, $col, $value) {
                $query->whereHas('users', function($q1) use ($type, $col, $value) {
                    $q1->join('employees', 'users.id', 'employees.user_id')
                        ->join('segments', 'employees.location_id', '=', 'segments.id');
                    FilterFluent::applyBasicFilter($q1, $type, 'name', $value, 'segments');
                });
            })
            ->setCustomFilter('days_open', function ($query, $type, $col, $value) {
                $date = date('Y-m-d', strtotime("-$value days"));
                $startOfDate = Carbon::parse($date)->startOfDay()->toDateTimeString();
                $endOfDate = Carbon::parse($date)->endOfDay()->toDateTimeString();
                if ($type === FilterTypes::IS) {
                    $query->whereDate('conversations.created_at', '<=', $endOfDate);
                    $query->whereDate('conversations.created_at', '>=', $startOfDate);
                } else if ($type === FilterTypes::GREATER_THAN) {
                    $query->whereDate('conversations.created_at', '<=', $startOfDate);
                } else if ($type === FilterTypes::LESS_THAN) {
                    $query->whereDate('conversations.created_at', '>=', $endOfDate);
                } else if ($type === FilterTypes::IS_NOT) {
                    $query->whereDate('conversations.created_at', '>=', $endOfDate);
                    $query->whereDate('conversations.created_at', '<=', $startOfDate);
                }
            })
            ->applyFilter();
        $query->distinct('conversations.id');
        $conversations = $this->applyPaging($options, $query);
        $conversations = $this->mapReceiverInfoToJourney($conversations);

        return $conversations;
    }

    /**
     * @param $conversations
     * @return mixed
     */
    public function mapReceiverInfoToJourney($conversations)
    {
        return app(MapReceiverInfoToJourney::class)->execute($conversations);
    }
}
