<?php


namespace App\Services\Message;

use App\Enum\AccessPermissions;
use App\Enum\ConversationTypes;
use App\Enum\PlanConfigurations;
use App\Facades\Context;
use App\Models\Message\AnonymousConversationStatusLog;
use App\Models\Message\WhistleblowingTransferLog;
use App\Models\User\User;
use App\Models\User\UserPermission;
use App\Services\BaseService;
use App\Services\Subscription\CheckRemainPlanAction;
use Carbon\Carbon;
use stdClass;

/**
 * Class GetMessageConversation
 * @package App\Services\Message
 */
class GetMessageConversation extends BaseService
{
    /**
     *
     */
    protected const MILISECOND_FORMAT = 'Y-m-d\TH:i:s.u+';
    /**
     *
     */
    protected const MILISECOND_FORMAT_MYSQL = 'Y-m-d H:i:s.u';

    /**
     * 
     */
    protected const MILISECOND_FORMAT_SHORTMYSQL = 'Y-m-d H:i:s';

    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules(): array
    {
        return [
            'id' => 'required'
        ];
    }

    /**
     * @param  array  $data
     * @return stdClass
     */
    public function performAction(array $data)
    {
        app(CheckRemainPlanAction::class)->execute(PlanConfigurations::PACKAGE_MESSAGE);
        $result = new stdClass();

        $currentUser = Context::user()->get();
        $list = [AccessPermissions::ACCESS_WB_MASTER];
        $permission = UserPermission::query()
            ->join('permissions', 'model_has_permissions.permission_id', 'permissions.id')
            ->where('model_id', $currentUser->id)
            ->where('model_type', User::class)
            ->whereIn('permissions.name', $list)
            ->select('permissions.*')
            ->get()->pluck('name');
        $isMaster = count($permission) > 0;

        $conversation = app(CheckPermissionUserOnConversation::class)->execute(['conversation_id' => $data['id'], 'isMaster' => $isMaster]);
        $conversation->load(['messages.sender']);

        $listUserAnonymous = collect([]);
        $queryMessages = $conversation->messages($isMaster)->with(['conversation', 'messageTranslations']);
        $queryReactionMessages = $conversation->reactionMessages();
        $this->applyCustomPagination($queryMessages, $data);
        $this->applyCustomPagination($queryReactionMessages, $data);
        $queryMessages->orderBy('created_at', 'desc');
        $mergedQuery = $queryMessages->union($queryReactionMessages);
        $messages = $mergedQuery->get();
        $messages = $messages->sortByDesc('created_at')->map(function ($item) use (&$listUserAnonymous, $conversation) {
            //Only display user if this is not anonymous or current sender is not owner of message
            if (!$item->isDisplayUser()) {
                $listUserAnonymous->push($item->sender_id);
            }
            return $item;
        });
        $users = $messages->pluck('sender')->unique()->filter(function ($value) use ($listUserAnonymous) {
            return !$listUserAnonymous->contains($value->getKey());
        });

        if ($conversation->type === ConversationTypes::ANONYMOUS_MESSAGE) {
            $result->transferWBLogs = $this->getTransferWBLogs($messages, $data, $conversation);
            $result->changeStatusLogs = $this->getChangeStatusLogs($messages, $data, $conversation);
        } else if ($conversation->type === ConversationTypes::DIRECT_MESSAGE) {
            if (isset($data['page']) && isset($data['page']['time'])) {
                $time = $data['page']['time'];
                $time = (Carbon::createFromFormat(self::MILISECOND_FORMAT, $time))->format(self::MILISECOND_FORMAT_SHORTMYSQL);
                if (isset($conversation->comment) && isset($conversation->comment->created_at)) {
                    if ($conversation->comment->created_at < $time) {
                        $result->comment = $conversation->comment;
                    }
                }
            } else {
                $result->comment = $conversation->comment;
            }
        }

        $result->users = $users;
        $result->messages = $messages;
        return $result;
    }

    /**
     * @param $query
     * @param $options
     */
    private function applyCustomPagination($query, $options)
    {
        $size = $options['page']['size'] ?? 15;
        $size = $size < 2 ? 2 : $size;
        if (isset($options['page']) && isset($options['page']['time'])) {
            $time = $options['page']['time'];
            $time = (Carbon::createFromFormat(self::MILISECOND_FORMAT, $time))->format(self::MILISECOND_FORMAT_MYSQL);
            $query->where('created_at', '<', $time);
        }
        $query->limit($size);
    }

    /**
     * @param $messages
     * @param $options
     * @param $conversation
     * @return \Illuminate\Database\Eloquent\Builder[]|\Illuminate\Database\Eloquent\Collection
     */
    public function getTransferWBLogs($messages, $options, $conversation)
    {
        $transferWBLogQuery = WhistleblowingTransferLog::query()
            ->with(['transferFrom', 'transferTo'])
            ->where('conversation_id', $conversation->id);
        if (isset($options['page']['time'])) {
            $time = $options['page']['time'];
            $time = (Carbon::createFromFormat(self::MILISECOND_FORMAT, $time))->format(self::MILISECOND_FORMAT_MYSQL);
            $transferWBLogQuery->where('created_at', '<', $time);
        }
        if ($messages->count() > 1) {
            $endTime = $messages->last()->created_at;
            $transferWBLogQuery->where('created_at', '>', $endTime);
        }
        return $transferWBLogQuery->get();
    }

    /**
     * @param $messages
     * @param $options
     * @param $conversation
     * @return \Illuminate\Support\Collection
     */
    public function getChangeStatusLogs($messages, $options, $conversation)
    {
        $changeStatusLogQuery = AnonymousConversationStatusLog::query()
            ->with(['user'])
            ->where('conversation_id', $conversation->id);
        if (isset($options['page']['time'])) {
            $time = $options['page']['time'];
            $time = (Carbon::createFromFormat(self::MILISECOND_FORMAT, $time))->format(self::MILISECOND_FORMAT_MYSQL);
            $changeStatusLogQuery->where('created_at', '<', $time);
        }
        if ($messages->count() > 1) {
            $endTime = $messages->last()->created_at;
            $changeStatusLogQuery->where('created_at', '>', $endTime);
        }
        $changeStatusLogs = $changeStatusLogQuery->get();

        //If in range don't have any log, return empty collection
        if ($changeStatusLogs->count() === 0) {
            return collect([]);
        }

        if ($changeStatusLogs->count() === 1) {
            //If have 1 log, find latest log before this
            //If exist, this is statusfrom
            //If not exist, this is lastest status, return empty collection
            $changeStatusLog = $changeStatusLogs->first();
            $latestChangeStatusLog = AnonymousConversationStatusLog::query()
                ->where('conversation_id', $conversation->id)
                ->where('created_at', '<', $changeStatusLog->created_at)
                ->latest()->first();
            if ($latestChangeStatusLog) {
                $changeStatusLog->statusFrom = $latestChangeStatusLog->status;
                $changeStatusLog->statusTo = $changeStatusLog->status;
                return collect([$changeStatusLog]);
            }

            return collect([]);
        }
        $changeStatusLogsResult = collect([]);
        $changeStatusLogs->each(function ($item, $key) use (&$changeStatusLogsResult, $changeStatusLogs) {
            if ($key > 0) {
                $item->statusFrom = $changeStatusLogs[$key - 1]->status;
                $item->statusTo = $item->status;
                $changeStatusLogsResult->push($item);
            }
        });
        return $changeStatusLogsResult;
    }
}
