<?php


namespace App\Services\Message;


use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\EmailNotifications;
use App\Enum\ErrorCode;
use App\Enum\MessageableTypes;
use App\Enum\MessageNotifyTypes;
use App\Enum\NotificationTypes;
use App\Enum\QualeeStatus;
use App\Enum\Roles;
use App\Events\NotifyMessageCreated;
use App\Exceptions\InvalidRequestException;
use App\Http\Resources\Message\TransferLog;
use App\Jobs\SendNotification;
use App\Jobs\SendNotificationEmail;
use App\Models\Message\Conversation;
use App\Models\Message\Messageable;
use App\Models\Message\WhistleblowingTransferLog;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\UserAccess\GetAccessByRole;
use Illuminate\Support\Facades\Auth;

class TransferWhistleblowerData extends BaseService implements Validatable, Transactionable
{
    /**
     * @param  array  $args
     * @return string[]
     */
    public function getRules(array $args): array
    {
        return [
            'id' => 'required|string',
            'transfer_from' => 'required|integer',
            'transfer_to' => 'required|integer',
        ];
    }

    /**
     * @param array $args
     * @return bool
     */
    public function performAction(array $args): bool
    {
        $conversation = Conversation::findOrFail($args['id']);
        $isThrow = $args['is_throw'] ?? true;

        // Prevent transferring conversation which have only 1 active messageable object
        $activeMessageableCount = Messageable::where('conversation_id', $conversation->getKey())
            ->where('status', QualeeStatus::ACTIVE)
            ->count();
        if ($activeMessageableCount <= 1) {
            if ($isThrow) {
                throw new InvalidRequestException(
                    'Cannot transfer this conversation',
                    ErrorCode::MESSAGE_ERROR
                );
            }
            return false;
        }

        $userTransferFrom = User::active()->findOrFail($args['transfer_from']);
        $userTransferTo = User::active()->findOrFail($args['transfer_to']);
        WhistleblowingTransferLog::create(
            [
                'conversation_id' => $conversation->id,
                'transfer_from' => $userTransferFrom->id,
                'transfer_to' => $userTransferTo->id
            ]
        );

        $conversation->users()->syncWithoutDetaching(
            [
                $userTransferTo->getKey() => [
                    'status' => QualeeStatus::ACTIVE,
                    'is_anonymous' => false,
                    'last_available' => $conversation->created_at,
                    'last_read' => $conversation->created_at,
                ]
            ]
        );

        $conversation->users()->syncWithoutDetaching(
            [$userTransferFrom->getKey() => ['status' => QualeeStatus::INACTIVE]]
        );

        //        $this->broadcastTransferDataToAnonymousUser($userTransferFrom, $userTransferTo, $transferLog, $conversation);
//        $this->broadcastTransferDataToWBAdmin($userTransferFrom, $userTransferTo, $transferLog, $conversation);

        return true;
    }

    /**
     * @param $transferFrom
     * @param $transferTo
     * @param $transferLog
     * @param $conversation
     */
    public function broadcastTransferDataToWBAdmin($transferFrom, $transferTo, $transferLog, $conversation): void
    {
        $transferLogResource = (new TransferLog($transferLog))->resolve();
        $arguments = array(
            'sender' => Auth::user()->id,
            "userIds" => $transferTo->code,
            "headings" => __("A whistleblowing case has been assigned to you"),
            "contents" => __(
                'notify.transfer_conversation_from',
                [
                    'transfer_from_first_name' => $transferFrom->first_name,
                    'transfer_from_surname' => $transferFrom->surname,
                ]
            ),
            "data" => [
                "type" => NotificationTypes::MESSAGE_RECEIVED,
                "conversation_id" => $conversation->id
            ],
            'send_after' => null
        );
        SendNotification::dispatch($arguments);
        broadcast(
            new NotifyMessageCreated(
                MessageNotifyTypes::TRANSFER_WB_ADMIN,
                $transferLogResource,
                $transferTo->code,
                $conversation->id
            )
        )->toOthers();
    }

    /**
     * @param $transferFrom
     * @param $transferTo
     * @param $transferLog
     * @param $conversation
     */
    public function broadcastTransferDataToAnonymousUser($transferFrom, $transferTo, $transferLog, $conversation): void
    {
        $anonymousUser = $conversation->users()->wherePivot('is_anonymous', true)->first();
        if ($anonymousUser) {
            $transferLogResource = (new TransferLog($transferLog))->resolve();
            $arguments = array(
                "userIds" => $anonymousUser->code,
                "headings" => __("Your whistleblowing case has been transferred"),
                "contents" => __(
                    'notify.transfer_conversation_to',
                    ['transfer_to_first_name' => $transferTo->first_name, 'transfer_to_surname' => $transferTo->surname]
                ),
                "data" => [
                    "type" => NotificationTypes::MESSAGE_RECEIVED,
                    "conversation_id" => $conversation->id
                ],
                'send_after' => null
            );
            SendNotification::dispatch($arguments);
            broadcast(
                new NotifyMessageCreated(
                    MessageNotifyTypes::TRANSFER_WB_ADMIN,
                    $transferLogResource,
                    $anonymousUser->code,
                    $conversation->id
                )
            )->toOthers();


            // Send Invite Notifications to employees
            $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
            $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_MENTION]) {
                SendNotificationEmail::dispatch($anonymousUser, EmailNotifications::MESSAGE_CASE_TRANSFER, ['case' => $conversation->id]);
            }
        }
    }
}