<?php

namespace App\Services\Mobile\Action;

use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\ActionRequestTypes;
use App\Enum\SubmissionStatus;
use App\Facades\Context;
use App\Models\Action\ActionUser;
use App\Models\ContentFile\ContentFile;
use App\Models\Form\FormTemplate;
use App\Models\Library\LibraryItem;
use App\Models\Library\Link;
use App\Services\ActionRequest\GetUserSubmission;
use App\Services\BaseService;
use App\Services\Mobile\Journey\IsStepAvailable;
use Auth;
use App\Models\IDVerification\IDVerification;
use App\Models\ContentPhoto\ContentPhoto;
use App\Models\Quiz\Quiz;
use App\Models\Survey\Survey;
use Carbon\Carbon;
use Illuminate\Support\Collection;

class SingleActionPending extends BaseService implements Validatable, Transactionable
{
    public function getRules(array $args): array
    {
        return [];
    }

    /**
     * ActionPendingAction
     * @return mixed
     */
    public function performAction(array $data)
    {
        $user = Context::user()->get();

        $actionsUs = ActionUser::where([
            'user_id' => $user->id,
            'is_completed' => 0,
        ])->get();

        $result = [];
        foreach ($actionsUs as $actionUsItem) {
            $action = $actionUsItem->action()->first();
            $now = Carbon::now();
            if (empty($action) || (!empty($action->send_on) && $action->send_on->gt($now))) continue;

            $actionRequest = $action->actionRequest()->first();
            if (empty($actionRequest)) continue;
            $countQuestion = -1;
            switch ($actionRequest->contentable_type) {
                case IDVerification::class:
                    $actionType = ActionRequestTypes::ID_VERIFICATION;
                    $title = 'Verify your ID';
                    $description = 'Submit photo';
                    break;
                case ContentPhoto::class:
                    $actionType = ActionRequestTypes::HEADSHOT_PHOTO;
                    $title = 'Submit profile photo';
                    $description = 'Take a photo using your phone';
                    break;
                case Quiz::class:
                    $actionType = ActionRequestTypes::QUIZ;
                    $title = 'Take a quiz';
                    $quiz = Quiz::find($actionRequest->contentable_id);
                    $countQuestion = !empty($quiz) ? $quiz->questions()->count() : 0;
                    $description = $countQuestion. ' question'. ($countQuestion > 1 ? 's': '') . ' to complete';
                    break;
                case Survey::class:
                    $actionType = ActionRequestTypes::SURVEY;
                    $title = 'Complete a survey';
                    $survey = Survey::find($actionRequest->contentable_id);
                    $countQuestion = $survey->surveyQuestions()->count();
                    $description = $countQuestion. ' question'. ($countQuestion > 1 ? 's': '') . ' to complete';
                    break;
                case FormTemplate::class:
                    $actionType = ActionRequestTypes::FORM;
                    $formTemplate = FormTemplate::find($actionRequest->contentable_id);
                    $title = $formTemplate->name;
                    $description = 'Submit form';
                    break;
                case ContentFile::class:
                    $actionType = ActionRequestTypes::UPLOAD_CONTENT;
                    $title = 'Upload contract';
                    $description = 'Upload a file';
                    break;
                case LibraryItem::class:
                    $actionType = ActionRequestTypes::FILE;
                    $title = 'Read a file';
                    $description = 'Read a file';
                    break;
                default:
                    $actionType = '';
                    $title = '';
                    $description = '';
                    break;
            }
            if (empty($actionType)) continue;
            $submission = $actionRequest->submissions()->first();
            $isAvailable = app(IsStepAvailable::class)->execute(['step' => $action]);
            $userSubmission = app(GetUserSubmission::class)->execute($actionRequest, Context::user()->get()->id);
            $fileType = null;
            if (isset($actionRequest->contentable->linkable->assets) &&
                isset($actionRequest->contentable->linkable->assets->first()->type)) {
                $fileType = $actionRequest->contentable->linkable->assets->first()->type;
            }
            if (is_a($actionRequest->contentable, LibraryItem::class)) {
                if (is_a($actionRequest->contentable->linkable, Link::class)) {
                    $fileType = 'Link';
                } else {
                    $fileType = isset($actionRequest->contentable->linkable->original_name) ?
                        ucfirst(pathinfo($actionRequest->contentable->linkable->original_name, PATHINFO_EXTENSION)) :
                        null;
                }
            }

            $result[] = [
                'id' => $actionRequest->id,
                'content_id' => $actionRequest->code,
                'content_type' => $actionType,
                'sub_title' => null,
                'name' => null,
                'is_available' => $isAvailable,
                'is_completed' => null,
                'title' => $title,
                'total_questions' => $countQuestion,
                'remaining_questions' => $countQuestion,
                'status' => $userSubmission ? $userSubmission->status : SubmissionStatus::PENDING,
                'description' => $description,
                'expire_date' => $action->due_by,
                'action_type' => $actionType,
                'submission_status' => !empty($submission) ? $submission->status : null,
                'file_type' => $fileType,
                'step_order' => null
            ];
        }

        return new Collection($result);
    }
}
