<?php

namespace App\Services\Mobile\Journey;

use App\Enum\AvailableTerms;
use App\Enum\SubmissionStatus;
use App\Facades\Context;
use App\Helpers\DateHelper;
use App\Models\Action\Action;
use App\Models\Action\ActionSequenceStep;
use App\Models\Journey\JourneyStep;
use App\Services\ActionRequest\GetUserSubmission;
use App\Services\BaseService;
use Illuminate\Support\Carbon;

class IsStepAvailable extends BaseService
{
    public function performAction(array $args)
    {
        $step = $args['step'];
        $today = Carbon::today()->startOfDay();
        $currentUser = Context::user()->get();
        $actionRequest = $step->actionRequest;
        if ($step instanceof ActionSequenceStep || $step instanceof Action) {
            return $this->dialogSuccessMessage($actionRequest->updated_at);
        }


        $isFirstStep = app(IsFirstStep::class)->execute(['step' => $step]);
        $submission = app(GetUserSubmission::class)->execute($actionRequest, $currentUser->id);
        if (!$isFirstStep && !$this->isPrevStepHasValidSubmission($step) && !$submission) {
            return $this->dialogFailMessage(
                null,
                SubmissionStatus::STEP_NOT_AVAILABLE
            );
        }


        if ($step->is_available_immediately) {
            $prevSubmitStep = app(GetPrevStepSubmission::class)->execute($step);
            $prevSubmitDate = $prevSubmitStep ? $prevSubmitStep->updated_at : $actionRequest->updated_at;
            return $this->dialogSuccessMessage($prevSubmitDate);
        }
        $availableDay = $this->getAvailableDay($step->available_term, $step->available_days);
        if ($today->greaterThanOrEqualTo($availableDay)) {
            return $this->dialogSuccessMessage($availableDay);
        } else {
            return $this->dialogFailMessage(
                $step->available_term,
                SubmissionStatus::DATE_NOT_AVAILABLE,
                Carbon::parse($availableDay)->diffInDays($today)
            );
        }


    }

    /**
     * @param null $availableDay
     * @return array
     */
    private function dialogSuccessMessage($availableDay = null): array
    {
        if ($availableDay) {
            $availableDay = DateHelper::formatYearMonthDay($availableDay);
        }
        return [
            'status'       => true,
            'availableDay' => $availableDay
        ];
    }

    /**
     * @param JourneyStep $step
     * @return bool
     */
    public function isPrevStepHasValidSubmission(JourneyStep $step): bool
    {
        return app(IsPrevStepHasValidSubmission::class)->execute($step);
    }

    /**
     * @param $date_available_term
     * @param $message
     * @param null $num
     * @return array
     */
    private function dialogFailMessage($date_available_term, $message, $num = null): array
    {
        return [
            'status'         => false,
            'message'        => $message,
            'available_term' => $date_available_term,
            'days'           => $num
        ];
    }

    public function getAvailableDay($availableTerm, $availableDays)
    {
        $userStartDate = new CarBon($this->getUserStartDate());

        switch ($availableTerm) {
            case AvailableTerms::AVAILABLE_TERM_AFTER_START_DATE:
                $availableDay = $userStartDate->addDays($availableDays);
                break;
            case AvailableTerms::AVAILABLE_TERM_BEFORE_START_DATE:
                $availableDay = $userStartDate->subDays($availableDays);
                break;
        }
        return $availableDay;
    }

    /**
     * Get user start date
     *
     * @return string
     */
    private function getUserStartDate()
    {
        $currentUser = Context::user()->get();
        if ($currentUser->employee->start_date) {
            return DateHelper::format($currentUser->employee->start_date);
        }
        return DateHelper::format(Carbon::today()->startOfDay());
    }
}
