<?php

namespace App\Services\Mobile\Questionnaire;

use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\ScheduleEnum;
use App\Facades\Context;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Schedule\ScheduleFrequencyRound;
use App\Models\Schedule\ScheduleRoundQuestionnaire;
use App\Services\BaseService;
use App\Models\Questionnaire\Questionnaire;
use App\Services\Mobile\Questionnaire\CompletedQuestionnaireAnswer;
use App\Services\SensitiveWord\DetectSensitiveWord;
use Carbon\Carbon;

class HandleAnswerQuestions extends BaseService implements Validatable, Transactionable
{
    public function getRules(array $args): array
    {
        return [
            'answers*.questionnaire_id' => 'required|exists:questionnaires,id',
            'answers.*.point' => 'nullable|numeric|min:0|max:10',
        ];
    }

    /**
     * Store answer
     *
     * @param array $data
     * @return array
     */
    public function performAction(array $data)
    {
        $roundId = $data['round_id'];
        $answers = $data['answers'];

        $user = Context::user()->get();
        $client = Context::client()->get();

        $result = [];
        $round = ScheduleFrequencyRound::findOrFail($roundId);
        $scheduleSurvey = $round->survey;

        $surveyRoundsDetail = ScheduleRoundQuestionnaire::where([
            'schedule_survey_id' => $round->schedule_survey_id,
            'user_id' => $user->id
        ])->first();

        if (empty($surveyRoundsDetail)|| empty($scheduleSurvey) || $scheduleSurvey->status !== ScheduleEnum::ACTIVE) {
            return ['status' => false];
        }

        $questionIds = !empty($surveyRoundsDetail->questionnaire_id[$roundId]) ? $surveyRoundsDetail->questionnaire_id[$roundId] : [];
        $valueIds = !empty($surveyRoundsDetail->client_value[$roundId]) ? $surveyRoundsDetail->client_value[$roundId] : [];
        $answeredQuestion = QuestionnaireAnswer::where(['user_id' => $user->id, 'round_id' => $roundId])->get();

        $isCompleted = app(CompareAnsweredAndRequestAnswerQuestion::class)->execute([
            'answered' => $answeredQuestion,
            'questionsSetting' => $questionIds,
            'valuesSetting' => $valueIds,
        ]);
        if ($isCompleted) {
            return ['status' => false, 'answer' => "You have completed this questionnaire"];
        }
        foreach ($answers as $item) {
            // If request question id in not exist on frequency round question array then stop
            if (!in_array($item['questionnaire_id'], $questionIds)) {
                continue;
            }

            // store/update answer
            $model = new QuestionnaireAnswer();
            $where = [
                'user_id' => $user->id,
                'questionnaire_id' => $item['questionnaire_id'],
                'round_id' => $roundId,
                'client_value_id' => !empty($item['client_value_id']) ? $item['client_value_id'] : null
            ];

            $answer = $model->where($where)->first();
            if (!empty($item['skip_question_id'])) {
                $item['point'] = null;
                $item['comment'] = null;
            } else {
                $item['skip_question_id'] = null;
            }

            // check sensitive word
            $detectSensitiveWords = app(DetectSensitiveWord::class)->execute(['comment' => !empty($item['comment']) ? $item['comment'] : '']);
            $item['is_sensitive'] = !empty($detectSensitiveWords);

            // Update answer if you answered it before
            if (!empty($answer)) {
                $item['analyzed'] = 0;
                $answer->update($item);
            } else {
                $questionnaire = Questionnaire::find($item['questionnaire_id']);
                if (empty($questionnaire)) continue;

                // create new answer
                $item['user_id'] = $user->id;
                $item['round_id'] = $roundId;
                $item['questionnaire_id'] = $questionnaire->id;
                $item['driver_id'] = $questionnaire->driver_id;
                $item['client_id'] = $client->id;
                $item['survey_id'] = $round->schedule_survey_id;

                $answer = $model->create($item);
                $answeredQuestion->push($answer);
            }

            $result[] = $answer;
        }

        $isCompleted = app(CompareAnsweredAndRequestAnswerQuestion::class)->execute([
            'answered' => $answeredQuestion,
            'questionsSetting' => $questionIds,
            'valuesSetting' => $valueIds,
        ]);

        if ($isCompleted) {
            app(CompletedQuestionnaireAnswer::class)->execute([
                'survey_id' => $round->schedule_survey_id,
                'round_id' => $round->id,
                'user_id' => $user->id,
            ]);
        }

        return ['status' => !empty($result)];
    }
}
