<?php
namespace App\Services\Report;

use App\Facades\Context;
use App\Contracts\Validatable;
use App\Helpers\CsvHelper;
use App\Models\DataControl\DataControl;
use App\Models\Segment\SegmentType;
use App\Services\BaseService;
use App\Traits\Pageable;
use App\Traits\Sortable;
use Carbon\Carbon;
use Illuminate\Support\Arr;

class DownloadBoardDrivers extends BaseService implements Validatable
{
    use Sortable, Pageable;

     /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules(array $args): array
    {
        return [
            'from_date' => 'required',
            'to_date' => 'required',
            'segments' => 'required',
        ];
    }

    /**
     * download report board
     * @param array $args
     * @return mixed
     */
    public function performAction(array $args)
    {
        $client = Context::client()->get();
        $segmentType = SegmentType::findByBusinessKey($args['segments']);
        if(empty($segmentType)) return false;

        // Get score type to export

        if (empty($args['score_type'])) {
            $dataControl = DataControl::client($client->id)->first();
            $scoreType = !empty($dataControl) && $dataControl->engagement_score_default !== null
                ? ($dataControl->engagement_score_default == 1 ? 'average' : 'enps')
                : 'enps';
        } else {
            $scoreType = $args['score_type'];
        }

        // Get board data
        $boardData = app(GetBoardDrivers::class)->execute($args);

        // Get driver and client value data
        $drivers = [];
        $clientValues = [];
        foreach ($boardData as $segment) {
            $drivers = array_merge($drivers, Arr::pluck($segment['drivers'], 'name'));
            $clientValues = array_merge($clientValues, Arr::pluck($segment['companyValues'], 'name'));
        }
        $drivers = array_unique($drivers);
        $clientValues = array_unique($clientValues);

        // Header
        $header = array_merge([$segmentType->name, 'Score'], $drivers, $clientValues);
        // Top header
        $now = Carbon::now()->format('Y-m-d');
        $topHeader = ['From', !empty($args['from_date']) ? $args['from_date'] : '', 'To', !empty($args['to_date']) ? $args['to_date'] : '', 'Report Date', $now];

        $rows = [];
        foreach ($boardData as $segment) {
            $segmentScore = 0;
            if (!empty($segment) && !empty($segment['scores'])) {
                $segmentScore = $scoreType === 'enps' ? $segment['scores']['driver_enps'] : $segment['scores']['driver_avg'];
            }

            $rowData = [$segment['name'], round($segmentScore, 1)];

            // Foreach driver column header to map score
            $boardDriver = $segment['drivers'];
            foreach ($drivers as $driver) {
                $findBoardDriver = Arr::where($boardDriver, function ($value) use($driver) {
                    return $value['name'] === $driver;
                });
                $boardDriverScore = !empty($findBoardDriver) ? array_values($findBoardDriver)[0]['scores'] : [];

                $driverScore = $this->getScoreValue($scoreType, $boardDriverScore);;
                $rowData[] = is_null($driverScore) ? 'No data' : round($driverScore, 1);
            }

            // Foreach client value column header to map score
            $boardValues = $segment['companyValues'];
            foreach ($clientValues as $clientValue) {
                $findBoardValue = Arr::where($boardValues, function ($value) use($clientValue) {
                    return $value['name'] === $clientValue;
                });

                $boardValueScore = !empty($findBoardValue) ? array_values($findBoardValue)[0]['scores'] : [];

                $valueScore = $this->getScoreValue($scoreType,$boardValueScore);
                $rowData[] = is_null($valueScore) ? 'No data' : round($valueScore, 1);
            }
            $rows[] = $rowData;
        }

        // Create download file
        $path = config('filesystems.disks.local.root');
        $fileName = 'report-board' . '_' . time() . '.csv';
        $filePath = $path . '/' . $fileName;
        $rows = array_merge([$header], $rows);

        CsvHelper::exportCsv($filePath, $rows, $topHeader);

        return response()->download($filePath)->deleteFileAfterSend();
    }

    public function getScoreValue($scoreType, $boardScores)
    {
        if ($scoreType === 'enps') {
            return $boardScores['driver_enps'] ?? null;
        }

        return $boardScores['driver_avg'] ?? null;
    }
}
