<?php
namespace App\Services\Report;

use App\Contracts\Validatable;
use App\Enum\EmployeeStatus;
use App\Facades\Context;
use App\Models\Driver\Driver;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Segment\Segment;
use App\Models\Segment\SegmentType;
use App\Models\Schedule\ScheduleSegments;
use App\Services\BaseService;
use App\Services\Client\ClientValue\GetClientValue;
use App\Services\DataControl\GetDataControl;
use App\Services\Engagement\BaseDriverScore;
use App\Services\Engagement\GetScoreValues;
use App\Traits\Pageable;
use App\Traits\Sortable;
use Carbon\Carbon;
use App\Enum\SegmentTypes;

class GetBoardDrivers extends BaseService implements Validatable
{
    use Sortable, Pageable;

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args): array
    {
        return [];
    }

    /**
     * calculation scoring average for eNPS
     *
     * @param array $args
     * @return mixed
     */
    public function performAction(array $args)
    {
        $LOCATION_SEGMENT_IDS = [];
        $DEPARTMENT_SEGMENT_IDS = [];
        $TEAM_SEGMENT_IDS = [];
        $MANAGER_SEGMENT_IDS = [];
        $GENDER_SEGMENT_IDS = [];
        $E_NPS_SURVEY_IDS = [];

        $args['from_date'] = isset($args['from_date']) ? Carbon::parse($args['from_date'])->toDateString() : null;
        $args['to_date'] = isset($args['to_date']) ? Carbon::parse($args['to_date'])->toDateString() : null;

        $client = Context::client()->get();
        $dataControl = app(GetDataControl::class)->execute();
        $userAvailableOfRound = app(GetUserListAvailableOfRound::class)->execute([
            'client_id' => $client->id,
            'dataControl' => $dataControl,
        ]);

        $query = QuestionnaireAnswer::query();
        $query->select('segment_user.segment_id')
            ->join('segment_user', 'segment_user.user_id', 'questionnaire_answers.user_id')
            ->join('employees', 'employees.user_id', 'segment_user.user_id');
        $query->where('employees.status', EmployeeStatus::ACTIVE);
        $query->where('questionnaire_answers.client_id', $client->id);

        // check empty state
        $isEmpty = $query->count();
        if ($isEmpty === 0)
            return null;

        $segments_filter = [];
        if (isset($args['segments_filter'])) {
            foreach ($args['segments_filter'] as $key => $segment) {
                if ($segment['is_survey']) {
                    $scheduleSegmentUsers = ScheduleSegments::getUserBySegment($segment['value']);

                    foreach ($scheduleSegmentUsers as $userId) {
                        array_push($E_NPS_SURVEY_IDS, $userId);
                    }
                } else {
                    $segmentItem = Segment::where('code', $segment['value'])->first();

                    $segments_filter = array_merge($segments_filter, [
                        $key => $segment
                    ]);
                    switch ($segmentItem['segment_type_id']) {
                        case SegmentTypes::LOCATION:
                            array_push($LOCATION_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::DEPARTMENT:
                            array_push($DEPARTMENT_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::TEAM:
                            array_push($TEAM_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::MANAGER:
                            array_push($MANAGER_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::GENDER:
                            array_push($GENDER_SEGMENT_IDS, $segmentItem['id']);
                            break;

                        default:
                            break;
                    }
                }
            }

            switch (true) {
                case count($LOCATION_SEGMENT_IDS) > 0:
                    $query->whereIn('employees.location_id', $LOCATION_SEGMENT_IDS);
                    break;
                case count($DEPARTMENT_SEGMENT_IDS) > 0:
                    $query->whereIn('employees.department_id', $DEPARTMENT_SEGMENT_IDS);
                    break;
                case count($TEAM_SEGMENT_IDS) > 0:
                    $query->whereIn('employees.team_id', $TEAM_SEGMENT_IDS);
                    break;
                case count($MANAGER_SEGMENT_IDS) > 0:
                    $query->whereIn('employees.manager_id', $MANAGER_SEGMENT_IDS);
                    break;
                case count($GENDER_SEGMENT_IDS) > 0:
                    $query->whereIn('employees.gender_id', $GENDER_SEGMENT_IDS);
                    break;

                default:
                    break;
            }
        }

        if (!empty($args['from_date']) && !empty($args['to_date'])) {
            $args['from_date'] = !empty($args['from_date']) ? Carbon::parse($args['from_date'])->toDateString() : Carbon::now()->toDateString();
            $args['to_date'] = !empty($args['to_date']) ? Carbon::parse($args['to_date'])->toDateString() : Carbon::now()->toDateString();

            // Get start date based on data referencing
            $now = Carbon::now();
            $availableDate = Carbon::now()->subMonths($dataControl->data_referencing);

            ////////////// Data referencing //////////////
            // If request start date less than available date then start date is available date
            $fromDate = Carbon::parse($args['from_date'])->lessThan($availableDate) ? $availableDate : $args['from_date'];
            $toDate = Carbon::parse($args['to_date'])->greaterThan($now) ? $now : $args['to_date'];

            $query->where(function ($q) use ($fromDate, $toDate) {
                return $q->whereDate('questionnaire_answers.created_at', '>=', $fromDate)
                    ->whereDate('questionnaire_answers.created_at', '<=', $toDate);
            });
        }

        if (isset($args['segments'])) {
            $segment_type_id = SegmentType::findByBusinessKey($args['segments'])->id;
            $segmentIds = Segment::where(['segment_type_id' => $segment_type_id, 'client_id' => $client->id])->pluck('id');
            $query->whereIn('segment_user.segment_id', $segmentIds);
        }

        $query->whereNull('questionnaire_answers.skip_question_id');

        $query->where(function ($q) use ($userAvailableOfRound) {
            foreach ($userAvailableOfRound as $roundId => $usersList) {
                $q->orWhere(function ($q1) use ($usersList, $roundId) {
                    return $q1->where('questionnaire_answers.round_id', $roundId)
                        ->whereIn('questionnaire_answers.user_id', $usersList);
                });
            }
            return $q;
        });

        $query->groupBy('segment_user.segment_id');
        $query->orderBy('segment_id');
        $segmentIds = $query->get()
            ->pluck('segment_id')
            ->toArray();

        $segmentQuery = Segment::query()
            ->with('segmentType')
            ->whereIntegerInRaw('id', $segmentIds);

        $segments = $this->applyPaging($args, $segmentQuery);

        $primaryDrivers = Driver::primary()->pluck('name', 'id')->toArray();

        $companyValues = app(GetClientValue::class)->execute();
        $companyValueIds = $companyValues->pluck('id')->toArray();

        $args['segments_filter'] = $segments_filter;
        $segmentIds = [];
        foreach ($segments as $segment) {
            // Client values
            $valueScores = app(GetScoreValues::class)->execute(
                array_merge([
                    'main_drivers' => $companyValueIds,
                    'segment' => $segment->getKey(),
                    'data_control' => $dataControl
                ], $args)
            );

            $companyValueScores = null;
            foreach ($companyValues as $companyValue) {
                $scores = $valueScores[$companyValue->getKey()] ?? null;
                $companyValueScores[] = array_merge($companyValue->toArray(), ['scores' => $scores]);
            }

            $segment->companyValues = $companyValueScores;

            // Segment driver scores
            $segmentDriverScores = app(BaseDriverScore::class)->execute(
                array_merge([
                    'main_drivers' => array_keys($primaryDrivers),
                    'segment' => $segment->getKey(),
                    'data_control' => $dataControl
                ], $args)
            );

            // Calculate segment average score
            $segmentDriverAvg = null;
            $segmentEnpsAvg = null;
            if (isset($segmentDriverScores)) {
                $driverAvgTotal = 0;
                $driverEnpsTotal = 0;
                $driverCount = count($segmentDriverScores);
                foreach ($segmentDriverScores as $segmentDriverScore) {
                    $driverAvgTotal += (float)$segmentDriverScore['driver_avg'];
                    $driverEnpsTotal += (float)$segmentDriverScore['driver_enps'];
                }
                $segmentDriverAvg = $driverCount > 0 ? ($driverAvgTotal / $driverCount) : 0;
                $segmentEnpsAvg = $driverCount > 0 ? ($driverEnpsTotal / $driverCount) : 0;
            }

            $segment->scores = [
                'driver_avg' => $segmentDriverAvg,
                'driver_enps' => $segmentEnpsAvg,
            ];

            $segmentDrivers = [];
            foreach ($primaryDrivers as $driverId => $driver) {
                $driverScores = $segmentDriverScores[$driverId] ?? null;
                $segmentDrivers[] = ['id' => $driverId, 'name' => $driver, 'scores' => $driverScores];
            }

            $segment->drivers = $segmentDrivers;
            $segmentIds[] = $segment->id;
        }
        return $segments;
    }
}