<?php
namespace App\Services\Report;

use App\Contracts\Validatable;
use App\Enum\EmployeeStatus;
use App\Facades\Context;
use App\Helpers\DateHelper;
use App\Helpers\PermissionHelper;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Segment\Segment;
use App\Services\BaseService;
use App\Services\DataControl\GetDataControl;
use App\Services\Engagement\BaseDriverScore;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;


class GetBoardSegmentProgressOverTime extends BaseService implements Validatable
{

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args): array
    {
        return [
            'segment_code' => 'required',
        ];
    }

    /**
     * get board segment progress over time
     *
     * @param array $args
     * @return mixed
     */
    public function performAction(array $args)
    {
        $client = Context::client()->get();
        $segment = Segment::findByBusinessKey($args['segment_code']);
        PermissionHelper::hasAccessOn($segment);

        $dataControl = app(GetDataControl::class)->execute();

        $now = Carbon::now();
        $availableDate = Carbon::now()->subMonths($dataControl->data_referencing);
        $fromDate = empty($args['from_date']) || Carbon::parse($args['from_date'])->lessThan($availableDate) ? $availableDate : $args['from_date'];
        $toDate = empty($args['to_date']) || Carbon::parse($args['to_date'])->greaterThan($now) ? $now : $args['to_date'];

        $answersByMonth = QuestionnaireAnswer::client($client->getKey())
            ->select(DB::raw('DATE_FORMAT(questionnaire_answers.created_at, \'%Y%m\') as month'))
            ->leftJoin('segment_user', 'segment_user.user_id', 'questionnaire_answers.user_id')
            ->leftJoin('employees', 'employees.user_id', 'questionnaire_answers.user_id')
            ->where('employees.status', EmployeeStatus::ACTIVE)
            ->where('segment_user.segment_id', $segment->getKey())
            ->whereDate('questionnaire_answers.created_at', '>=', $fromDate)
            ->whereDate('questionnaire_answers.created_at', '<', $toDate)
            ->whereNull('questionnaire_answers.skip_question_id')
            ->whereNotNull('questionnaire_answers.point')
            ->groupBy('month')
            ->orderBy('month')
            ->pluck('month')
            ->toArray();

        $segmentScoresOverTime = [];
        foreach ($answersByMonth as $month) {
            $scores = app(BaseDriverScore::class)->execute([
                'by_month' => $month,
                'progress_time' => true,
                'segment' => $segment->getKey(),
                'data_control' => $dataControl
            ]);

            if(isset($scores['driver_avg'])){
                $segmentScoresOverTime[] = [
                    'date' => DateHelper::format(Carbon::parse($month."01")),
                    'scores' => $scores
                ];
            }
        }

        return $segmentScoresOverTime;
    }
}
