<?php

namespace App\Services\Report;

use App\Contracts\Validatable;
use App\Enum\EmployeeStatus;
use App\Facades\Context;
use App\Helpers\TimeHelper;
use App\Models\Driver\Driver;
use App\Models\Questionnaire\Questionnaire;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Schedule\ScheduleSegments;
use App\Models\Segment\EmploymentPhaseSegment;
use App\Models\Segment\Segment;
use App\Services\BaseService;
use App\Services\Client\ClientValue\GetClientValue;
use App\Services\DataControl\GetDataControl;
use App\Services\Engagement\BaseDriverScore;
use App\Services\Engagement\GetScoreValues;
use App\Traits\Pageable;
use App\Traits\Sortable;
use Carbon\Carbon;
use App\Enum\SegmentTypes;
use Illuminate\Support\Facades\DB;

class GetHeatmapSegment extends BaseService implements Validatable
{
    use Sortable, Pageable;

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args): array
    {
        return [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'date_format:Y-m-d|after:from_date'
        ];
    }

    /**
     * calculation scoring average for eNPS
     * @param array $args
     * @return mixed
     */
    public function performAction(array $args)
    {
        $LOCATION_SEGMENT_IDS = [];
        $DEPARTMENT_SEGMENT_IDS = [];
        $TEAM_SEGMENT_IDS = [];
        $MANAGER_SEGMENT_IDS = [];
        $GENDER_SEGMENT_IDS = [];
        $EMPLOYMENT_SEGMENT_IDS = [];
        $AGE_SEGMENT_IDS = [];
        $E_NPS_SEGMENT_IDS = [];

        $args['from_date'] = isset($args['from_date']) ? Carbon::parse($args['from_date'])->toDateString() : null;
        $args['to_date'] = isset($args['to_date']) ? Carbon::parse($args['to_date'])->toDateString() : null;

        $client = Context::client()->get();
        // Check empty data when first use
        $questionValue = Questionnaire::questionValue()->first();
        if (empty($questionValue)) {
            return null;
        }

        $isEmpty = QuestionnaireAnswer::client($client->getKey())
            ->whereNull('skip_question_id')
            ->where('questionnaire_id', '!=', $questionValue->id)
            ->count();
        if ($isEmpty === 0) {
            return null;
        }

        $dataControl = app(GetDataControl::class)->execute();

        $userAvailableOfRound = app(GetUserListAvailableOfRound::class)->execute([
            'client_id' => $client->getKey(),
            'dataControl' => $dataControl,
        ]);


        $query = QuestionnaireAnswer::query();
        $query->select('segment_user.segment_id')
            ->join('segment_user', 'segment_user.user_id', 'questionnaire_answers.user_id')
            ->join('employees', 'employees.user_id', 'questionnaire_answers.user_id');
        $query->where('employees.status', EmployeeStatus::ACTIVE);
        $query->where('questionnaire_answers.client_id', $client->getKey());

        $isEmpty = $query->count();
        if ($isEmpty === 0) {
            return null;
        }

        $segments_filter = [];
        if (isset($args['segments_filter'])) {
            foreach ($args['segments_filter'] as $key => $segment) {
                if ($segment['is_survey'] === 'true') {
                    $scheduleSegments = ScheduleSegments::getSegmentBySchedule($segment['value'])['send_to'];

                    foreach ($scheduleSegments as $segment) {
                        array_push($E_NPS_SEGMENT_IDS, $segment['id']);
                    }
                } else if (isset($segment['is_age']) && $segment['is_age'] === 'true') {
                    array_push($AGE_SEGMENT_IDS, $segment['value']);
                } else if (isset($segment['is_ephase']) && $segment['is_ephase'] === 'true') {
                    array_push($EMPLOYMENT_SEGMENT_IDS, $segment['value']);
                } else if (isset($segment['is_manager']) && $segment['is_manager'] === 'true') {
                    array_push($MANAGER_SEGMENT_IDS, $segment['value']);
                } else {
                    $segmentItem = Segment::where('code', $segment['value'])->first();

                    $segments_filter = array_merge($segments_filter, [
                        $key => $segment
                    ]);

                    switch ($segmentItem['segment_type_id']) {
                        case SegmentTypes::LOCATION:
                            array_push($LOCATION_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::DEPARTMENT:
                            array_push($DEPARTMENT_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::TEAM:
                            array_push($TEAM_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        case SegmentTypes::GENDER:
                            array_push($GENDER_SEGMENT_IDS, $segmentItem['id']);
                            break;
                        default:
                            break;
                    }
                }
            }

            if (count($LOCATION_SEGMENT_IDS) > 0) {
                $query->whereIn('employees.location_id', $LOCATION_SEGMENT_IDS);
            }
            if (count($DEPARTMENT_SEGMENT_IDS) > 0) {
                // $query->whereIn('employees.department_id', $DEPARTMENT_SEGMENT_IDS);
            }
            if (count($TEAM_SEGMENT_IDS) > 0) {
                $query->whereIn('employees.team_id', $TEAM_SEGMENT_IDS);
            }
            if (count($GENDER_SEGMENT_IDS) > 0) {
                $query->whereIn('employees.gender_id', $GENDER_SEGMENT_IDS);
            }
            if (count($EMPLOYMENT_SEGMENT_IDS) > 0) {
                // $query->whereIn('employees.employee_id', $EMPLOYMENT_SEGMENT_IDS);
            }
            if (count($E_NPS_SEGMENT_IDS) > 0) {
                $query->whereIn('segment_user.segment_id', $E_NPS_SEGMENT_IDS);
            }
        }

        $query->whereNotNull('questionnaire_answers.point');
        $query->whereNull('questionnaire_answers.skip_question_id');

        if (!empty($args['from_date']) && !empty($args['to_date'])) {
            $args['from_date'] = !empty($args['from_date']) ? Carbon::parse($args['from_date'])->toDateString() : Carbon::now()->toDateString();
            $args['to_date'] = !empty($args['to_date']) ? Carbon::parse($args['to_date'])->toDateString() : Carbon::now()->toDateString();
            $now = Carbon::now();
            $availableDate = Carbon::now()->subMonths($dataControl->data_referencing);
            $fromDate = Carbon::parse($args['from_date'])->lessThan($availableDate) ? $availableDate : $args['from_date'];
            $toDate = Carbon::parse($args['to_date'])->greaterThan($now) ? $now : $args['to_date'];

            $query->where(function ($q) use ($fromDate, $toDate) {
                return $q->whereDate('questionnaire_answers.created_at', '>=', $fromDate)
                    ->whereDate('questionnaire_answers.created_at', '<=', $toDate);
            });
        }

        if (isset($args['segments'])) {
            $segmentOptions = $args['segments'];
            $conditions = [];
            foreach ($segmentOptions as $options)
            {
                if ($options['name'] === 'Manager') {
                    $codes = explode(",", $options['value']);
                    $idArray = array_map('intval', $codes);
                    $conditions[] = [
                        'column' => 'employees.manager_id',
                        'value' => $idArray,
                    ];
                } else if ($options['name'] === 'Age') {
                    $conditionsAge = [];
                    $codes = explode(",", $options['value']);
                    $idArray = array_map('intval', $codes);
                    foreach ($idArray as $id) {
                        if ($id === 1) {
                            array_push($conditionsAge, [
                                'value' => ['TIMESTAMPDIFF(YEAR, employees.date_of_birth, NOW()) >= 16', 'TIMESTAMPDIFF(YEAR, employees.date_of_birth, NOW()) <= 24'],
                            ]);
                        } else if ($id === 2) {
                            array_push($conditionsAge, [
                                'value' => ['TIMESTAMPDIFF(YEAR, employees.date_of_birth, NOW()) >= 25', 'TIMESTAMPDIFF(YEAR, employees.date_of_birth, NOW()) <= 34'],
                            ]);
                        } else if ($id === 3) {
                            array_push($conditionsAge, [
                                'value' => ['TIMESTAMPDIFF(YEAR, employees.date_of_birth, NOW()) >= 35', 'TIMESTAMPDIFF(YEAR, employees.date_of_birth, NOW()) <= 44'],
                            ]);
                        } else if ($id === 4) {
                            array_push($conditionsAge, [
                                'value' => ['TIMESTAMPDIFF(YEAR, employees.date_of_birth, NOW()) >= 45', 'TIMESTAMPDIFF(YEAR, employees.date_of_birth, NOW()) <= 54'],
                            ]);
                        } else if ($id === 5) {
                            array_push($conditionsAge, [
                                'value' => ['TIMESTAMPDIFF(YEAR, employees.date_of_birth, NOW()) >= 55', 'TIMESTAMPDIFF(YEAR, employees.date_of_birth, NOW()) <= 64'],
                            ]);
                        } else if ($id === 6) {
                            array_push($conditionsAge, [
                                'value' => ['TIMESTAMPDIFF(YEAR, employees.date_of_birth, NOW()) >= 65'],
                            ]);
                        }
                    }
                    $conditions[] = [
                        'column' => null,
                        'value' => $conditionsAge,
                    ];
                } else if ($options['name'] === 'Employment phase') {
                    $conditionsEmployment = [];
                    $codes = explode(",", $options['value']);
                    $segments = EmploymentPhaseSegment::query()->whereIn('code', $codes)->get();
                    foreach ($segments as $employmentPhrase)
                    {
                        $field = $employmentPhrase['type'];
                        $column_name = 'employees.' . $field;

                        $typeMin = strtoupper($employmentPhrase['type_min']);
                        $typeMax = strtoupper($employmentPhrase['type_max']);

                        $subCondition = [];
                        if ($employmentPhrase['min']) {
                            array_push($subCondition, 'TIMESTAMPDIFF(' . $typeMin . ', ' . $column_name . ', NOW()) >= ' . $employmentPhrase['min']);
                        }
                        if ($employmentPhrase['max']) {
                            array_push($subCondition, 'TIMESTAMPDIFF(' . $typeMax . ', ' . $column_name . ', NOW()) <= ' . $employmentPhrase['max']);
                        }

                        array_push($conditionsEmployment, [
                            'value' => $subCondition,
                        ]);
                    }
                    $conditions[] = [
                        'column' => null,
                        'value' => $conditionsEmployment,
                    ];
                } else if ($options['name'] === 'eNPS survey') {
                    $codes = explode(",", $options['value']);
                    $idArray = array_map('intval', $codes);
                    $segmentCodes = [];
                    $excludeCodes = [];
                    foreach ($idArray as $segmentId) {
                        $scheduleSegments = ScheduleSegments::getSegmentIdsBySchedule($segmentId);
                        $segmentCodes = array_merge($segmentCodes, $scheduleSegments['send_to']);
                        $excludeCodes = array_merge($excludeCodes, $scheduleSegments['exclude']);
                    }
                    $conditions[] = [
                        'column' => 'segment_user.segment_id',
                        'value' => $segmentCodes,
                    ];
//                    $query->whereIn('segment_user.segment_id', $segmentCodes)->whereNotIn('segment_user.segment_id', $excludeCodes);
                } else {
                    $codes = explode(",", $options['value']);
                    $segmentIds = Segment::whereIn('code', $codes)->pluck('id')->toArray();
                    $conditions[] = [
                        'column' => 'segment_user.segment_id',
                        'value' => $segmentIds,
                    ];
                }
            }
            $query->where(function ($subQuery) use ($conditions) {
                foreach ($conditions as $condition)
                {
                    if (isset($condition['column'])) {
                        $subQuery->orWhereIn($condition['column'], $condition['value']);
                    } else {
                        $orWhereQuery = $condition['value'];
                        foreach ($orWhereQuery as $oneQuery)
                        {
                            $stringQueries = $oneQuery['value'];
                            $subQuery->orWhere(function ($q) use ($stringQueries) {
                                foreach ($stringQueries as $str) {
                                    $q->whereRaw($str);
                                }
                            });
                        }
                    }
                }
            });
        }

        $query->where(function ($q) use ($userAvailableOfRound) {
            foreach ($userAvailableOfRound as $roundId => $usersList) {
                $q->orWhere(function ($q1) use ($usersList, $roundId) {
                    return $q1->where('questionnaire_answers.round_id', $roundId)
                        ->whereIn('questionnaire_answers.user_id', $usersList);
                });
            }
            return $q;
        });

        $query->groupBy('segment_user.segment_id');
        $query->orderBy('segment_id');

        $segmentIds = $query->get()
            ->pluck('segment_id')
            ->toArray();

        $segmentQuery = Segment::query()
            ->with('segmentType')
            ->whereIntegerInRaw('id', $segmentIds)
            ->orderBy('segment_type_id');

        $segments = $this->applyPaging($args, $segmentQuery);

        $primaryDrivers = Driver::primary()->pluck('name', 'id')->toArray();
        $primaryFullDrivers = Driver::with(['questionnaire'])->primary()->get();
        $companyValues = app(GetClientValue::class)->execute();
        $companyValueIds = $companyValues->pluck('name', 'id')->toArray();

        $args['segments_filter'] = $segments_filter;
        $segmentIds = [];
        foreach ($segments as $segment) {
            $segmentDriverScores = app(BaseDriverScore::class)->execute(
                array_merge([
                    'main_drivers' => array_keys($primaryDrivers),
                    'segment' => $segment->getKey(),
                    'data_control' => $dataControl
                ], $args)
            );

            $valueScores = app(GetScoreValues::class)->execute(
                array_merge([
                    'main_drivers' => array_keys($companyValueIds),
                    'segment' => $segment->getKey(),
                    'data_control' => $dataControl
                ], $args)
            );

            $segmentDrivers = [];
            foreach ($primaryFullDrivers as $driver) {
                $driverScores = $segmentDriverScores[$driver['id']] ?? null;
                $segmentDrivers[] = ['id' => $driver['id'], 'name' => $driver['name'], 'scores' => $driverScores, 'description' => $driver['description'], 'priority' => $driver['priority'], 'questionnaire' => $driver->questionnaire];
            }

            $segmentValues = [];
            foreach ($companyValueIds as $driverId => $driver) {
                $driverScores = $valueScores[$driverId] ?? null;
                $segmentValues[] = ['id' => $driverId, 'name' => $driver, 'scores' => $driverScores, 'description' => '', 'priority' => ''];
            }

            $segment->drivers = array_merge($segmentDrivers, $segmentValues);
            $segmentIds[] = $segment->id;
        }

        if (!empty($segmentIds)) {
            $surveyed_employees = GetParticipateRateForReport::getSurveyedEmployeeDetails(array_merge([
                'segmentIds' => $segmentIds,
                'this_cycle' => true
            ], $args));
            foreach ($segments as $segment) {
                $surveyed_employees_segment = array_filter($surveyed_employees, function ($item) use ($segment) {
                    return $item['segment_id'] === $segment->id;
                });
                $userIds = array_map(function ($item) {
                    return $item['user_id'];
                }, $surveyed_employees_segment);

                // calculate employees count
                $query = $segment->users()->join('employees', 'employees.user_id', 'users.id')->whereIn('users.id', $userIds);
                if (count($LOCATION_SEGMENT_IDS) > 0) {
                    $query->whereIn('employees.location_id', $LOCATION_SEGMENT_IDS);
                }
                if (count($DEPARTMENT_SEGMENT_IDS) > 0) {
                    $query->whereIn('employees.department_id', $DEPARTMENT_SEGMENT_IDS);
                }
                if (count($TEAM_SEGMENT_IDS) > 0) {
                    $query->whereIn('employees.team_id', $TEAM_SEGMENT_IDS);
                }
                if (count($MANAGER_SEGMENT_IDS) > 0) {
                    $query->whereIn('employees.manager_id', $MANAGER_SEGMENT_IDS);
                }
                if (count($GENDER_SEGMENT_IDS) > 0) {
                    $query->whereIn('employees.gender_id', $GENDER_SEGMENT_IDS);
                }
                if (count($AGE_SEGMENT_IDS) > 0) {
                    $query->where(function ($query) use ($AGE_SEGMENT_IDS) {
                        $query->join('employees', 'employees.user_id', 'users.id');
                        foreach ($AGE_SEGMENT_IDS as $age) {
                            $range = [$age * 10 + ($age == 1 ? 6 : 5), ($age < 6 ? (($age + 1) * 10 + 4) : 1000)];
                            $query->orWhereBetween(DB::raw('TIMESTAMPDIFF(YEAR, employees.date_of_birth, CURDATE())'), $range)->get();
                        }
                    });
                }
                $segType = $segment->segment_type_id;
                switch ($segType) {
                    case SegmentTypes::LOCATION:
                        $query->where('employees.location_id', $segment->id);
                        break;
                    case SegmentTypes::DEPARTMENT:
                        $query->where('employees.department_id', $segment->id);
                        break;
                    case SegmentTypes::TEAM:
                        $query->where('employees.team_id', $segment->id);
                        break;
                    case SegmentTypes::MANAGER:
                        $query->where('employees.manager_id', $segment->id);
                        break;
                    case SegmentTypes::GENDER:
                        $query->where('employees.gender_id', $segment->id);
                        break;
                }
                if (count($EMPLOYMENT_SEGMENT_IDS) > 0) {
                    $employees = $query->get();
                    if (count($EMPLOYMENT_SEGMENT_IDS) > 0) {
                        $employmentSegments = EmploymentPhaseSegment::whereIn('code', $EMPLOYMENT_SEGMENT_IDS)->get();
                        foreach ($employmentSegments as $employmentSegment) {
                            $employees = $employees->filter(function ($item) use ($employmentSegment) {
                                $dateString = $item->employee->{$employmentSegment->type};
                                if (!$dateString) {
                                    return false;
                                }
                                $date = new Carbon($dateString);
                                $isInSegment = TimeHelper::isTimeOnRange($date, Carbon::today(), $employmentSegment->min, $employmentSegment->type_min, $employmentSegment->max, $employmentSegment->type_max);
                                return $isInSegment;
                            });
                        }
                    }
                    $segment->employees = $employees->count();
                } else {
                    $segment->employees = $query->count();
                }
            }
        }

        return $segments;
    }
}
