<?php

namespace App\Services\Report;

use App\Contracts\Validatable;
use App\Enum\EmployeeStatus;
use App\Enum\ScheduleEnum;
use App\Facades\Context;
use App\Models\Questionnaire\Questionnaire;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Schedule\ScheduleRoundQuestionnaire;
use App\Services\BaseService;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class GetParticipateRateForReport extends BaseService implements Validatable
{

    function performAction(array $args)
    {
        $args['from_date'] = isset($args['from_date']) ? Carbon::parse($args['from_date'])->toDateString() : null;
        $args['to_date'] = isset($args['to_date']) ? Carbon::parse($args['to_date'])->toDateString() : null;

        if (isset($args['this_week']) && $args['this_week'] && isset($args['to_date'])) {
            if (isset($args['from_date']) && Carbon::parse($args['to_date'])->diffInDays(Carbon::parse($args['from_date'])) > 7) {
                $args['from_date'] = Carbon::parse($args['to_date'])->subDays(7)->toDateString();
            }
        }
        $client = Context::client()->get();
        $queryAll = null;

        $segmentIds = $args['segmentIds'];
        foreach ($segmentIds as $segment) {
            $query = Questionnaire::query();
            $query->select([
                'questionnaire_answers.user_id', 'segment_user.segment_id', 'questionnaire_answers.survey_id',
                DB::raw("COUNT(questionnaire_answers.driver_id) as total_participate")
            ])
                ->join('questionnaire_answers', 'questionnaire_answers.questionnaire_id', 'questionnaires.id')
                ->join('segment_user', 'segment_user.user_id', 'questionnaire_answers.user_id')
                ->join('employees', 'employees.user_id', 'segment_user.user_id')
                ->join('schedule_survey', 'schedule_survey.id', '=', 'questionnaire_answers.survey_id')
                ->where('employees.status', EmployeeStatus::ACTIVE)
                ->where(['segment_user.segment_id' => (int) $segment])
                ->where('questionnaire_answers.client_id', $client->id);

            if (isset($args['from_date']) && isset($args['to_date'])) {
                $query->where(function ($q) use ($args) {
                    return $q->whereDate('questionnaire_answers.created_at', '>=', $args['from_date'])
                        ->whereDate('questionnaire_answers.created_at', '<=', $args['to_date']);
                });
            }
            $query->groupBy('questionnaire_answers.user_id', 'questionnaire_answers.survey_id', 'schedule_survey.total_questionnaire');

            if (isset($queryAll)) {
                $queryAll = $queryAll->union($query);
            } else {
                $queryAll = $query;
            }
        }
        return $queryAll->get()->groupBy('segment_id');
    }

    public static function getSurveyedEmployees($options = [])
    {
        $client = Context::client()->get();
        // Check empty data when first use
        $questionValue = Questionnaire::questionValue()->first();
        if (empty($questionValue)) {
            return [];
        }
        $isEmpty = QuestionnaireAnswer::client($client->id)
            ->join('segment_user', 'segment_user.user_id', 'questionnaire_answers.user_id')
            ->join('employees', 'employees.user_id', 'segment_user.user_id')
            ->where('employees.status', EmployeeStatus::ACTIVE)
            ->whereNull('skip_question_id')
            ->where('questionnaire_id', '!=', $questionValue->id)
            ->count();
        if ($isEmpty == 0) {
            return [];
        }

        $segmentIds = $options['segmentIds'];
        $query = ScheduleRoundQuestionnaire::query()
            ->join('schedule_survey', 'schedule_survey.id', 'schedule_round_questionnaires.schedule_survey_id')
            ->join('employees', 'employees.user_id', 'schedule_round_questionnaires.user_id')
            ->join('segment_user', 'segment_user.user_id', 'employees.user_id')
            ->where('schedule_round_questionnaires.status', ScheduleEnum::ACTIVE)
            ->whereNull('schedule_survey.deleted_at')
            ->where('schedule_survey.client_id', $client->id);

        if (isset($options['this_cycle'])) {
            $query->where('schedule_survey.status', ScheduleEnum::ACTIVE);
        }

        return $query->where('employees.status', EmployeeStatus::ACTIVE)
            ->whereIn('segment_user.segment_id', $segmentIds)
            ->select('segment_user.segment_id', DB::raw('count(DISTINCT segment_user.user_id) as total_users'))
            ->groupBy('segment_user.segment_id')
            ->get()->toArray();
    }

    public static function getSurveyedEmployeeDetails($options = [])
    {
        $client = Context::client()->get();
        // Check empty data when first use
        $questionValue = Questionnaire::questionValue()->first();
        if (empty($questionValue)) {
            return [];
        }
        $isEmpty = QuestionnaireAnswer::client($client->id)
            ->join('segment_user', 'segment_user.user_id', 'questionnaire_answers.user_id')
            ->join('employees', 'employees.user_id', 'segment_user.user_id')
            ->where('employees.status', EmployeeStatus::ACTIVE)
            ->whereNull('skip_question_id')
            ->where('questionnaire_id', '!=', $questionValue->id)
            ->count();
        if ($isEmpty == 0) {
            return [];
        }

        $segmentIds = $options['segmentIds'];
        $query = ScheduleRoundQuestionnaire::query()
            ->join('schedule_survey', 'schedule_survey.id', 'schedule_round_questionnaires.schedule_survey_id')
            ->join('employees', 'employees.user_id', 'schedule_round_questionnaires.user_id')
            ->join('segment_user', 'segment_user.user_id', 'employees.user_id')
            ->join('users', 'employees.user_id', 'users.id')
            ->where('schedule_round_questionnaires.status', ScheduleEnum::ACTIVE)
            ->whereNull('schedule_survey.deleted_at')
            ->whereNotNull('users.last_login')
            ->where('schedule_survey.client_id', $client->id);

        if (isset($options['this_cycle'])) {
            $query->where('schedule_survey.status', ScheduleEnum::ACTIVE);
        }

        return $query->where('employees.status', EmployeeStatus::ACTIVE)
            ->whereIn('segment_user.segment_id', $segmentIds)
            ->get()->toArray();
    }

    public function getRules(array $args)
    {
        // TODO: Implement getRules() method.
        return [];
    }
}
