<?php
namespace App\Services\Report;

use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\ConversationTypes;
use App\Enum\FilterTypes;
use App\Enum\MessageType;
use App\Enum\Roles;
use App\Helpers\FilterFluent;
use App\Helpers\UserAccessHelper;
use App\Models\Client\ClientValue;
use App\Models\Driver\Driver;
use App\Models\Questionnaire\Questionnaire;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Segment\SegmentUser;
use App\Models\User\Employee;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\DataControl\GetDataControl;
use App\Services\Segment\SegmentType\GetSegmentEmployee;
use Carbon\Carbon;
use App\Traits\Pageable;
use App\Traits\Sortable;
use App\Traits\Filterable;
use App\Facades\Context;
use Illuminate\Database\Eloquent\Builder;
use App\Enum\SegmentTypes;
use App\Models\Message\Conversation;
use App\Models\Segment\Segment;

class GetReportComment extends BaseService implements Transactionable
{
    use Sortable, Pageable, Filterable;

    /**
     * get report comments
     *
     * @param array $data
     * @return Mix
     */
    public function performAction(array $data)
    {
        $LOCATION_SEGMENT_IDS = [];
        $DEPARTMENT_SEGMENT_IDS = [];
        $TEAM_SEGMENT_IDS = [];
        $MANAGER_SEGMENT_IDS = [];
        $GENDER_SEGMENT_IDS = [];

        $client = Context::client()->get();
        $user = Context::user()->get();
        $isYourTeam = isset($data['your_team']) && $data['your_team'] == 'true';
        if (!$user->hasRole(Roles::ADMIN)) {
            if (!UserAccessHelper::hasAccess(AccessPermissions::ACCESS_COMPANY_COMMENT) &&
                UserAccessHelper::hasAccess(AccessPermissions::ACCESS_TEAM_COMMENT)) {
                $isYourTeam = true;
            }
        }

        // Check empty data when first use
        $questionValue = Questionnaire::questionValue()->first();
        if (empty($questionValue)) return null;

        $isEmpty = QuestionnaireAnswer::client($client->id)
            ->whereNotNull('comment')
            ->where('questionnaire_id', '!=', $questionValue->id)
            ->count();
        if ($isEmpty == 0) return null;

        $dataControl = app(GetDataControl::class)->execute();
        $queryWhere = [
            'client' => $client,
            'questionValue' => $questionValue,
            'scoreType' => !empty($data['score_type']) ? $data['score_type'] : ''
        ];
        if (!empty($data['from_date']) && !empty($data['to_date'])) {
            $getFromToDate = app(DataControlReferencing::class)->execute([
                'from_date' => $data['from_date'],
                'to_date' => $data['to_date'],
                'data_referencing' => $dataControl->data_referencing
            ]);
            $queryWhere['fromDate'] = $getFromToDate['from_date'];
            $queryWhere['toDate'] = $getFromToDate['to_date'];
        }

        $userAvailableOfRound = app(GetUserListAvailableOfRound::class)->execute([
            'client_id' => $client->id,
            'dataControl' => $dataControl,
            'ignore_value' => true
        ]);

        $query = QuestionnaireAnswer::select('questionnaire_answers.*')->report($queryWhere)
            ->leftJoin('drivers', 'drivers.id', 'questionnaire_answers.driver_id')
            ->leftJoin('client_values', 'client_values.id', 'questionnaire_answers.client_value_id')
            ->where(function($q) use ($userAvailableOfRound) {
                foreach ($userAvailableOfRound as $roundId => $usersList) {
                    $q->orWhere(function($q1) use ($usersList, $roundId) {
                        return $q1->where('questionnaire_answers.round_id', $roundId)
                            ->whereIn('questionnaire_answers.user_id', $usersList);
                    });
                }
                return $q;
            })
            ->groupBy('questionnaire_answers.id');

        if($isYourTeam) {
            $userIds = app(GetSegmentEmployee::class)->getEmployeeSegmentManager($user->code);
            $query->whereIntegerInRaw('questionnaire_answers.user_id', $userIds);
        }

        if ($data['comment_type'] == 'sensitive') {
            $query->join('questionnaire_sensitive_comments', 'questionnaire_sensitive_comments.questionnaire_answer_id', '=', 'questionnaire_answers.id')
                ->where('questionnaire_answers.is_sensitive', 1)
                ->whereNull('questionnaire_answers.reaction')
                ->where('questionnaire_sensitive_comments.is_read', 0);
        }

        if (isset($data['segments_filter']) && is_array($data['segments_filter'])) {
            $query->leftJoin('employees', 'employees.user_id', 'questionnaire_answers.user_id');
            foreach ($data['segments_filter'] as $key => $segment) {
                $segmentItem = Segment::where('code', $segment['value'])->first();

                switch ($segmentItem['segment_type_id']) {
                    case SegmentTypes::LOCATION:
                        array_push($LOCATION_SEGMENT_IDS, $segmentItem['id']);
                        break;
                    case SegmentTypes::DEPARTMENT:
                        array_push($DEPARTMENT_SEGMENT_IDS, $segmentItem['id']);
                        break;
                    case SegmentTypes::TEAM:
                        array_push($TEAM_SEGMENT_IDS, $segmentItem['id']);
                        break;
                    case SegmentTypes::MANAGER:
                        array_push($MANAGER_SEGMENT_IDS, $segmentItem['id']);
                        break;
                    case SegmentTypes::GENDER:
                        array_push($GENDER_SEGMENT_IDS, $segmentItem['id']);
                        break;

                    default:
                        break;
                }
            }

            switch (true) {
                case count($LOCATION_SEGMENT_IDS) > 0:
                    $query->whereIn('employees.location_id', $LOCATION_SEGMENT_IDS);
                    break;
                case count($DEPARTMENT_SEGMENT_IDS) > 0:
                    $query->whereIn('employees.department_id', $DEPARTMENT_SEGMENT_IDS);
                    break;
                case count($TEAM_SEGMENT_IDS) > 0:
                    $query->whereIn('employees.team_id', $TEAM_SEGMENT_IDS);
                    break;
                case count($MANAGER_SEGMENT_IDS) > 0:
                    $query->whereIn('employees.manager_id', $MANAGER_SEGMENT_IDS);
                    break;
                case count($GENDER_SEGMENT_IDS) > 0:
                    $query->whereIn('employees.gender_id', $GENDER_SEGMENT_IDS);
                    break;

                default:
                    break;
            }
        }

        $options = Context::request()->getOptions();
        FilterFluent::init()
            ->setQuery($query)
            ->setTable('users')
            ->setFilterOptions($options)
            ->setAllowedFilters(['comment', 'main_driver', 'point', 'is_sensitive', 'is_sensitive_acknowledged', 'reaction', 'acknowledged_by'])
            ->validate()
            ->removeFilter(['comment', 'main_driver', 'point', 'is_sensitive', 'is_sensitive_acknowledged', 'reaction', 'acknowledged_by'])
            ->setCustomFilter('comment', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'comment', $value, 'questionnaire_answers');
            })
            ->setCustomFilter('reaction', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'reaction', $value, 'questionnaire_answers');
            })
            ->setCustomFilter('point', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'point', $value, 'questionnaire_answers');
            })
            ->setCustomFilter('is_sensitive', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'is_sensitive', $value === 'Yes' ? 1 : 0, 'questionnaire_answers');
            })
            ->setCustomFilter('is_sensitive_acknowledged', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'is_sensitive', 1, 'questionnaire_answers');
                if ($value === 'Yes') {
                    FilterFluent::applyBasicFilter($query, FilterTypes::IS, 'reaction', 'acknowledge', 'questionnaire_answers');
                } else {
                    FilterFluent::applyBasicFilter($query, FilterTypes::IS_NOT, 'reaction', 'acknowledge', 'questionnaire_answers');
                }
            })
            ->setCustomFilter('acknowledged_by', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'is_sensitive', 1, 'questionnaire_answers');
                FilterFluent::applyBasicFilter($query, FilterTypes::IS, 'reaction', 'acknowledge', 'questionnaire_answers');
                $user = User::findByBusinessKey($value);
                FilterFluent::applyBasicFilter($query, FilterTypes::IS, 'reacted_by', $user->id, 'questionnaire_answers');

            })
            ->setCustomFilter('main_driver', function($query, $type, $col, $value) {
                switch ($type) {
                    case FilterTypes::IS:
                    case FilterTypes::IS_NOT:
                        $resultFilter = Driver::select('id')->where('drivers.name', $value)->get();
                        break;
                    case FilterTypes::CONTAIN:
                        $resultFilter = Driver::select('id')->where('drivers.name', 'like', '%'. $value. '%')->get();
                        break;
                    default: //Default filter custom.
                        $resultFilter = [];
                        break;
                }
                if (!empty($resultFilter)) {
                    $mainAndSubDriver = Driver::select('id')->whereIn('id', $resultFilter)->orWhereIn('parent_id', $resultFilter)->get();
                    $valueDriver = ClientValue::select('id')->whereIn('id', $resultFilter)->get();

                    if ($type == FilterTypes::IS || $type == FilterTypes::CONTAIN) {
                        $query->whereIn('questionnaire_answers.driver_id', $mainAndSubDriver)->orWhereIn('questionnaire_answers.client_value_id', $valueDriver);
                    } else if($type == FilterTypes::IS_NOT) {
                        $query->whereNotIn('questionnaire_answers.driver_id', $mainAndSubDriver)->whereNotIn('questionnaire_answers.client_value_id', $valueDriver);
                    }
                }
            })
            ->applyFilter();

        if (!empty($data['sort'])) {
            $data['sort'] = $this->refactorSortingParams($data['sort']);
        }

        $this->applySorting($data, $query);
        $query->with(['questionnaireDiscussionAnswer.user']);
        $query->with(['conversations' => function ($q) {
            $q->where(['type' => ConversationTypes::NOTIFY_MESSAGE_EMPLOYEE]);
        }]);

        return $this->applyPaging($data, $query);
    }

    private function refactorSortingParams($sorts)
    {
        foreach ($sorts as &$itemSort) {
            switch ($itemSort['column']) {
                case 'date' :
                    $itemSort['column'] = 'questionnaire_answers.created_at';
                    break;
                case 'score' :
                    $itemSort['column'] = 'questionnaire_answers.point';
                    break;
            }
        }
        return $sorts;
    }
}
