<?php
namespace App\Services\Report;

use App\Contracts\Validatable;
use App\Enum\EngagementScoreType;
use App\Enum\FilterTypes;
use App\Models\Driver\Driver;
use App\Models\Questionnaire\Questionnaire;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Questionnaire\QuestionnaireTopic;
use App\Services\BaseService;
use App\Services\DataControl\GetDataControl;
use App\Services\Engagement\BaseDriverScore;
use App\Traits\Pageable;
use App\Facades\Context;
use Illuminate\Validation\Rule;

class GetTopicCommentAnalyze extends BaseService implements Validatable
{
    use Pageable;

    public function getRules(array $args)
    {
        return [
            'driver_id' => 'required',
            'from_date' => 'date',
            'to_date' => 'date',
            'score_type' => Rule::in([EngagementScoreType::AVERAGE, EngagementScoreType::ENPS])
        ];
    }

    /**
     * Get topic comment analyze
     *
     * @param array $data
     * @return mixed|null
     */
    public function performAction(array $data)
    {
        $client = Context::client()->get();

        $primaryDriver = Driver::primary()->findOrFail($data['driver_id']);
        $driverIds = array_merge([$primaryDriver->getKey()], $primaryDriver->subDrivers()->pluck('id')->toArray());

        $questionValue = Questionnaire::questionValue()->first();
        if (empty($questionValue)) {
            return null;
        }

        $dataControl = app(GetDataControl::class)->execute();
        $scoreType = $data['score_type'] ?? ($dataControl->engagement_score_default === 0 ?
                EngagementScoreType::AVERAGE : EngagementScoreType::ENPS);

        $availableDates = app(DataControlReferencing::class)->execute([
            'from_date' => $data['from_date'] ?? null,
            'to_date' => $data['to_date'] ?? null,
            'data_referencing' => $dataControl->data_referencing
        ]);
        $from = $availableDates['from_date'];
        $to = $availableDates['to_date'];

        $availableUsersOfRound = app(GetUserListAvailableOfRound::class)->execute([
            'client_id' => $client->getKey(),
            'dataControl' => $dataControl,
        ]);

        $topicDriverQuery = app(GetTopicDriver::class)->getTopicDriverQuery(array_merge($data, [
            'client_id' => $client->getKey(),
            'question_value' => $questionValue,
            'from' => $from,
            'to' => $to,
            'available_users_of_round' => $availableUsersOfRound
        ]));
        $topicDriverQuery->whereIntegerInRaw('questionnaire_answers.driver_id', $driverIds);
        $topicDrivers = $topicDriverQuery->get();

        $topicQuery = QuestionnaireTopic::query()
            ->whereIntegerInRaw('id', $topicDrivers->pluck('topic_id'));
        $topics = $this->applyPaging($data, $topicQuery);

        $topicComments =  QuestionnaireAnswer::join(
            'questionnaire_topic_answer',
            'questionnaire_topic_answer.answer_id',
            'questionnaire_answers.id'
        )
            ->whereIntegerInRaw('questionnaire_topic_answer.topic_id', $topics->pluck('id'))
            ->whereIntegerInRaw('questionnaire_answers.driver_id', $driverIds)
            ->where('questionnaire_answers.client_id', $client->getKey())
            ->whereNotNull('questionnaire_answers.comment')
            ->where('questionnaire_answers.questionnaire_id', '!=', $questionValue->getKey())
            ->where(function ($q) use ($from, $to) {
                return $q->whereDate('questionnaire_answers.created_at', '>=', $from)
                    ->whereDate('questionnaire_answers.created_at', '<=', $to);
            })
            ->where(function($q) use ($availableUsersOfRound) {
                foreach ($availableUsersOfRound as $roundId => $usersList) {
                    $q->orWhere(function($q1) use ($usersList, $roundId) {
                        return $q1->where('round_id', $roundId)->whereIn('user_id', $usersList);
                    });
                }
                return $q;
            })
            ->get()
            ->groupBy('topic_id');

        foreach ($topics as $topic) {
            $comments = $topicComments[$topic->getKey()] ?? [];
            $scores = null;
            if (!empty($comments)) {
                $scores = app(BaseDriverScore::class)->execute(array_merge([
                    'topics' => $driverIds,
                    'comment_ids' => $comments->pluck('id')->toArray(),
                    'data_control' => $dataControl
                ]));
            }
            $topic->comments = $comments;
            $topic->scores = $scores;
        }

        $scoreFilter = collect($data['filter'] ?? [])->filter(static function ($filter) {
            return $filter['column'] === 'score';
        })->first();

        if (!empty($scoreFilter)) {
            foreach ($topics as $key => $topic) {
                $match = false;
                if (!empty($topic['scores'])) {
                    $score = ($scoreType === EngagementScoreType::ENPS) ?
                        $topic['scores']['driver_enps'] : $topic['scores']['driver_avg'];
                    $score = round($score, 1);
                    $filterScore = round((float)$scoreFilter['value'], 1);

                    $match = $scoreFilter['type'] === FilterTypes::IS_NOT ?
                        $score !== $filterScore : $score === $filterScore;
                }

                if (!$match) {
                    $topics->forget($key);
                }
            }
        }

        return $topics;
    }
}
