<?php
namespace App\Services\Report;

use App\Contracts\Validatable;
use App\Enum\EmployeeStatus;
use App\Enum\FilterTypes;
use App\Enum\ReportDefine;
use App\Helpers\FilterFluent;
use App\Models\Driver\Driver;
use App\Models\Questionnaire\Questionnaire;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Questionnaire\QuestionnaireTopic;
use App\Services\BaseService;
use App\Services\DataControl\GetDataControl;
use App\Traits\Pageable;
use App\Facades\Context;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;

class GetTopicDriver extends BaseService implements Validatable
{
    use Pageable;

    public function getRules(array $args)
    {
        return [
            'from_date' => 'date',
            'to_date' => 'date',
        ];
    }

    /**
     * Get topic drivers
     *
     * @param array $data
     * @return mixed|null
     * @throws ValidationException
     */
    public function performAction(array $data)
    {
        $client = Context::client()->get();

        $questionValue = Questionnaire::questionValue()->first();
        if (empty($questionValue)) return null;

        $countComment = QuestionnaireAnswer::query()
            ->join('drivers','drivers.id','questionnaire_answers.driver_id')
            ->join('employees','employees.user_id','questionnaire_answers.user_id')
            ->where('employees.status', EmployeeStatus::ACTIVE)
            ->where('questionnaire_answers.client_id', $client->getKey())
            ->whereNotNull('comment')
            ->where('questionnaire_id', '!=', $questionValue->id)
            ->count();

//        $countComment += QuestionnaireAnswer::query()
//            ->join('client_values', 'client_values.id', 'questionnaire_answers.client_value_id')
//            ->join('employees','employees.user_id','questionnaire_answers.user_id')
//            ->where('employees.status', EmployeeStatus::ACTIVE)
//            ->where('questionnaire_answers.client_id', $client->getKey())
//            ->whereNotNull('comment')
//            ->where('questionnaire_id', '!=', $questionValue->id)
//            ->count();
        if ($countComment === 0) return null;

        $client = Context::client()->get();

        $dataControl = app(GetDataControl::class)->execute();

        $getFromToDate = app(DataControlReferencing::class)->execute([
            'from_date' => $data['from_date'] ?? null,
            'to_date' => $data['to_date'] ?? null,
            'data_referencing' => $dataControl->data_referencing
        ]);

        $availableUsersOfRound = app(GetUserListAvailableOfRound::class)->execute([
            'client_id' => $client->getKey(),
            'dataControl' => $dataControl,
        ]);

        $topicDriverQuery = $this->getTopicDriverQuery(array_merge($data, [
            'client_id' => $client->getKey(),
            'question_value' => $questionValue,
            'from' => $getFromToDate['from_date'],
            'to' => $getFromToDate['to_date'],
            'available_users_of_round' => $availableUsersOfRound
        ]));
        $topicDrivers = $topicDriverQuery->get();

//        $topicClientValuesQuery = $this->getTopicClientValuesQuery(array_merge($data, [
//            'client_id' => $client->getKey(),
//            'question_value' => $questionValue,
//            'from' => $getFromToDate['from_date'],
//            'to' => $getFromToDate['to_date'],
//            'available_users_of_round' => $availableUsersOfRound
//        ]));
//        $topicClientValues = $topicClientValuesQuery->get();


//        $primaryDriverIds = array_merge($topicDrivers->pluck('primary_driver_id')->toArray(),
//            $topicClientValues->pluck('primary_driver_id')->toArray());

        $primaryDriverIds = $topicDrivers->pluck('primary_driver_id')->toArray();
        $driverQuery = Driver::query()
            ->with(['questionnaire', 'subDrivers.questionnaire'])
            ->primary()
            ->whereIntegerInRaw('id', $primaryDriverIds);

        $driverFilter = collect($data['filter'] ?? [])->filter(static function ($filter) {
            return $filter['column'] === 'driver';
        });
        FilterFluent::init()
            ->setQuery($driverQuery)
            ->setFilterOptions(['filter' => $driverFilter->toArray()])
            ->setAllowedFilters(['driver'])
            ->removeFilter('driver')
            ->validate()
            ->setCustomFilter('driver', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'name', $value);
            })
            ->applyFilter();

        return $this->applyPaging($data, $driverQuery);
    }

    /**
     * Get topic driver query
     *
     * @param array $conditions
     * @return Builder
     * @throws ValidationException
     */
    public function getTopicDriverQuery(array $conditions): Builder
    {
        $topicDriverQuery = QuestionnaireTopic::query()
            ->select(DB::raw(
                'questionnaire_topics.id as topic_id,
            IF(drivers.parent_id != 0, drivers.parent_id, drivers.id) as primary_driver_id,
            COUNT(questionnaire_answers.id) as count'
            ))
            ->leftJoin('questionnaire_topic_answer', 'questionnaire_topic_answer.topic_id', 'questionnaire_topics.id')

            ->leftJoin('questionnaire_answers', 'questionnaire_answers.id', 'questionnaire_topic_answer.answer_id')
            ->leftJoin('drivers', 'drivers.id', 'questionnaire_answers.driver_id')
            ->where('questionnaire_answers.client_id', $conditions['client_id'])
            ->whereNotNull('questionnaire_answers.comment')
            ->where('questionnaire_answers.questionnaire_id', '!=', $conditions['question_value']->getKey())
            ->where(function ($q) use ($conditions) {
                return $q->whereDate('questionnaire_answers.created_at', '>=', $conditions['from'])
                    ->whereDate('questionnaire_answers.created_at', '<=', $conditions['to']);
            })
            ->where(function ($q) use ($conditions) {
                foreach ($conditions['available_users_of_round'] as $roundId => $usersList) {
                    $q->orWhere(function ($q1) use ($usersList, $roundId) {
                        return $q1->where('questionnaire_answers.round_id', $roundId)
                            ->whereIn('questionnaire_answers.user_id', $usersList);
                    });
                }
                return $q;
            })
            ->groupBy('questionnaire_topics.id', 'primary_driver_id')
            ->having('count', '>=', ReportDefine::TOPIC['MINIMUM_DRIVER_RESULT']);

        $commentFilter = collect($conditions['filter'] ?? [])->filter(static function ($filter) {
            return $filter['column'] === 'comment' || $filter['column'] === 'comment_count';
        });
        FilterFluent::init()
            ->setQuery($topicDriverQuery)
            ->setFilterOptions(['filter' => $commentFilter->toArray()])
            ->setAllowedFilters(['comment', 'comment_count'])
            ->removeFilter('comment')
            ->removeFilter('comment_count')
            ->validate()
            ->setCustomFilter('comment', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'comment', $value, 'questionnaire_answers');
            })
            ->setCustomFilter('comment_count', function($query, $type, $col, $value) {
                if ($type === FilterTypes::IS_NOT) {
                    $query->having('count', '!=', $value);
                } else if ($type === FilterTypes::IS || $type === FilterTypes::CONTAIN) {
                    $query->having('count', '=', $value);
                }
            })
            ->applyFilter();

        return $topicDriverQuery;
    }

    /**
     * Get topic driver query
     *
     * @param array $conditions
     * @return Builder
     * @throws ValidationException
     */
    public function getTopicClientValuesQuery(array $conditions): Builder
    {
        $topicDriverQuery = QuestionnaireTopic::query()
            ->select(DB::raw(
                'questionnaire_topics.id as topic_id,
            client_values.id as primary_driver_id,
            COUNT(questionnaire_answers.id) as count'
            ))
            ->leftJoin('questionnaire_topic_answer', 'questionnaire_topic_answer.topic_id', 'questionnaire_topics.id')

            ->leftJoin('questionnaire_answers', 'questionnaire_answers.id', 'questionnaire_topic_answer.answer_id')
            ->leftJoin('client_values', 'client_values.id', 'questionnaire_answers.client_value_id')
            ->where('questionnaire_answers.client_id', $conditions['client_id'])
            ->whereNotNull('questionnaire_answers.comment')
            ->where('questionnaire_answers.questionnaire_id', '!=', $conditions['question_value']->getKey())
            ->where(function ($q) use ($conditions) {
                return $q->whereDate('questionnaire_answers.created_at', '>=', $conditions['from'])
                    ->whereDate('questionnaire_answers.created_at', '<=', $conditions['to']);
            })
            ->where(function ($q) use ($conditions) {
                foreach ($conditions['available_users_of_round'] as $roundId => $usersList) {
                    $q->orWhere(function ($q1) use ($usersList, $roundId) {
                        return $q1->where('questionnaire_answers.round_id', $roundId)
                            ->whereIn('questionnaire_answers.user_id', $usersList);
                    });
                }
                return $q;
            })
            ->groupBy('questionnaire_topics.id', 'primary_driver_id')
            ->having('count', '>=', ReportDefine::TOPIC['MINIMUM_DRIVER_RESULT']);

        $commentFilter = collect($conditions['filter'] ?? [])->filter(static function ($filter) {
            return $filter['column'] === 'comment' || $filter['column'] === 'comment_count';
        });
        FilterFluent::init()
            ->setQuery($topicDriverQuery)
            ->setFilterOptions(['filter' => $commentFilter->toArray()])
            ->setAllowedFilters(['comment', 'comment_count'])
            ->removeFilter('comment')
            ->removeFilter('comment_count')
            ->validate()
            ->setCustomFilter('comment', function($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'comment', $value, 'questionnaire_answers');
            })
            ->setCustomFilter('comment_count', function($query, $type, $col, $value) {
                if ($type === FilterTypes::IS_NOT) {
                    $query->having('count', '!=', $value);
                } else if ($type === FilterTypes::IS || $type === FilterTypes::CONTAIN) {
                    $query->having('count', '=', $value);
                }
            })
            ->applyFilter();

        return $topicDriverQuery;
    }
}
