<?php
namespace App\Services\Report;

use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Models\Questionnaire\Questionnaire;
use App\Models\Questionnaire\QuestionnaireAnswer;
use App\Models\Schedule\ScheduleSegmentUser;
use App\Models\User\Employee;
use App\Services\BaseService;
use App\Services\DataControl\GetDataControl;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class GetUserListAvailableOfRound extends BaseService implements Validatable, Transactionable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules(array $args): array
    {
        return [
            'client_id' => 'required'
        ];
    }

    /**
     * get user list availble of round based on data control
     *
     * @param array $data
     * @return array
     */
    public function performAction(array $data)
    {
        $clientId = $data['client_id'];
        $dataControl = !empty($data['dataControl']) ? $data['dataControl'] : app(GetDataControl::class)->execute();

        // get all round survey ids (unique)
        $answerRounds = QuestionnaireAnswer::select('round_id', 'survey_id')
            ->distinct()
            ->client($clientId)
            ->with('surveySegment')
            ->get();
        $questionValue = Questionnaire::questionValue()->first();
        $questionValueIds = $questionValue->id ?? null;

        // Convert send_to and exclude_id from string to array
        $answerRounds = $answerRounds->map(function ($item) {
            if (!empty($item->surveySegment) && !empty($item->surveySegment->send_to) && !is_array($item->surveySegment->send_to)) {
                $item->surveySegment->send_to = explode(",", $item->surveySegment->send_to);
            }
            if (!empty($item->surveySegment) && !empty($item->surveySegment->exclude_id) && !is_array($item->surveySegment->exclude_id)) {
                $item->surveySegment->exclude_id = explode(",", $item->surveySegment->exclude_id);
            }
            return $item;
        });

        // Get all Segment
        $allSurveyIds =  $answerRounds->pluck('survey_id')->unique()->toArray();
        $segmentUsGroup = ScheduleSegmentUser::whereIn('schedule_id', $allSurveyIds)->get()->groupBy('schedule_id');

        $roundIds = $answerRounds->pluck('round_id')->toArray();
        $allResponseCountByRound = QuestionnaireAnswer::client($clientId)
            ->select('round_id', DB::raw('count(*) as response_count'), DB::raw('count(DISTINCT user_id) as responder_count'))
            ->whereIn('round_id', $roundIds)
            ->ignoreValue($questionValueIds, !empty($data['ignore_value']))
            ->groupBy('round_id')
            ->get()
            ->keyBy('round_id')
            ->toArray();

        $expiryDate = Carbon::now()->subDays($dataControl->data_expiry);
        $canScoreEmployeeIds = Employee::ofClient($clientId)
            ->where(function ($q) use ($expiryDate) {
                return $q->whereNull('end_date')->orWhereDate('end_date', '>=', $expiryDate);
            })
            ->pluck('user_id')
            ->toArray();

        $userAvailableOfRound = [];
        //Check round able to pass data control
        foreach ($answerRounds as $answerRound) {
            $surveySegment = $answerRound->surveySegment;
            $segmentSendTo = $surveySegment->send_to;
            $segmentExclude = $surveySegment->exclude_id;
            $scheduleSegmentUser = $segmentUsGroup[$answerRound->survey_id] ?? collect([]);

            ////////////// Data availability //////////////
            // Count response of this round
            $roundResponseCount = $allResponseCountByRound[$answerRound->round_id]['response_count'] ?? 0;
            if ($roundResponseCount < $dataControl->data_availability) continue;

            // The number of responses required from a survey before managers can view comments.
            $roundResponderCount = $allResponseCountByRound[$answerRound->round_id]['responder_count'] ?? 0;

            // Get segment available by check data control
            $availableUser = [];
            if (count($segmentSendTo) > 1) {
                $querySql = null;
                $listOfSegmentUserIds = [];
                foreach ($segmentSendTo as $segmentId) {
                    $scheduleSegmentUserItem = $scheduleSegmentUser->where('segment_id', $segmentId)->first();
                    $segmentUserIds = !empty($scheduleSegmentUserItem) ? json_decode($scheduleSegmentUserItem->user_id, true) : [];
                    $oneQuerySql = QuestionnaireAnswer::query()->client($clientId)
                        ->select([
                            DB::raw("count(*) as segment_response_count"),
                            DB::raw("count(DISTINCT `questionnaire_answers`.`user_id`) as segment_responder_count"),
                        ])
                        ->where('round_id', $answerRound->round_id)
                        ->ignoreValue($questionValueIds, !empty($data['ignore_value']))
                        ->whereIn('user_id', array_unique($segmentUserIds));
                    $listOfSegmentUserIds[] = $segmentUserIds;
                    if (isset($querySql)) {
                        $querySql = $querySql->union($oneQuerySql);
                    } else {
                        $querySql = $oneQuerySql;
                    }
                }
                $results = $querySql->get()->toArray();
                $index = 0;
                foreach ($results as $result) {
                    $segmentResponseCount = $result['segment_response_count'];
                    $segmentResponderCount = $result['segment_responder_count'];
                    $segmentUserIds = $listOfSegmentUserIds[$index];
                    ////////////// Segment Identification //////////////
                    // Minimum number of responses required to identify a unique segment for comparison.
                    // Higher values maximise anonymity, however lower values may be required for smaller teams.

                    ////////////// Anonymity Difference //////////////
                    // Defines the visibility of segments versus the team size.
                    // For example, a setting of 3 would prevent the reviewing of a segment of 5 responses in a team of 6.
                    // minimum segment size <= the number of responder of segment <= (total responder - Anonymity Difference)
                    if ($segmentResponseCount >= $dataControl->segment_identification
                        && $segmentResponderCount <= ($roundResponderCount - $dataControl->anonymity_difference)
                    ) {
                        $availableUser = array_merge($availableUser, $segmentUserIds);
                    }
                    $index++;
                }
            } else {
                if ($roundResponseCount >= $dataControl->segment_identification) {
                    $scheduleSegmentUserItem = $scheduleSegmentUser->where('segment_id', $segmentSendTo[0])->first();
                    $segmentUserIds = !empty($scheduleSegmentUserItem) ? json_decode($scheduleSegmentUserItem->user_id, true) : [];

                    $availableUser = array_merge($availableUser, $segmentUserIds);
                }
            }


            ////////////// Data expiry //////////////
            // The maximum time in which data from former employees will be used in scoring.
            $availableUser = array_intersect($availableUser, $canScoreEmployeeIds);

            //Exclude segment
            $excludeSegmentUs = [];
            if (!empty($segmentExclude) && !empty($scheduleSegmentUser)) {
                foreach ($segmentExclude as $excludeSegmentId) {
                    $scheduleExcludeSegmentUserItem = $scheduleSegmentUser->where('segment_id', $excludeSegmentId)->first();
                    $excludeUsItem = !empty($scheduleExcludeSegmentUserItem) ? json_decode($scheduleExcludeSegmentUserItem->user_id, true) : [];

                    $excludeSegmentUs = array_merge($excludeSegmentUs, $excludeUsItem);
                }
            }

            $availableUser = array_diff($availableUser, $excludeSegmentUs);
            $userAvailableOfRound[$answerRound->round_id] = $availableUser;
        }

        return $userAvailableOfRound;
    }
}
