<?php


namespace App\Services\Schedule;

use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\EmailNotifications;
use App\Enum\Roles;
use App\Enum\ScheduleEnum;
use App\Facades\Context;
use App\Jobs\SendNotificationConsole;
use App\Jobs\SendNotificationEmail;
use App\Models\Client\ClientValue;
use App\Models\Questionnaire\Questionnaire;
use App\Models\Questionnaire\QuestionnaireClientSetting;
use App\Models\Schedule\ScheduleFrequencySetting;
use App\Models\Schedule\ScheduleSegments;
use App\Models\Schedule\ScheduleSurvey;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\UserAccess\GetAccessByRole;
use Carbon\Carbon;

class CreateSchedule extends BaseService implements Validatable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @param  array  $args
     * @return array
     */
    public function getRules(array $args): array
    {
        $expireDate = app(GetValidExpireDate::class)
            ->execute([
                'start_date' => $args['start_date'],
                'schedule_frequency_id' => $args['schedule_frequency_id']
            ]);
        return [
            'name' => 'required|string|max:255',
            'description' => 'string|max:255',
            'schedule_frequency_id' => 'integer|exists:schedule_frequency,id',
            'client_id' => 'integer|exists:clients,id',
            'start_date' => 'required|date_format:Y-m-d|after_or_equal:' . Carbon::now()->toDateString(),
            'created_by' => 'integer|exists:users,id',
            'renewal_date' => 'required|date_format:Y-m-d|after:' . $expireDate->toDateString()
        ];
    }


    /**
     * Create schedule item
     *
     * @param  array  $args
     * @return ScheduleSurvey
     */
    public function performAction(array $args): ScheduleSurvey
    {
        $user = Context::user()->get();
        $client = Context::client()->get();

        $args['renewal_date'] = Carbon::parse($args['renewal_date']);
        $args['status'] = ScheduleEnum::ACTIVE;
        $args['client_id'] = $client->id;
        $args['created_by'] = $user->id;
        $args['description'] = $args['description'] ?? "";
        $survey = ScheduleSurvey::create($args);
        $this->setTotalQuestionnaire($survey);
        $survey = $survey->refresh();



        // Send Invite Notifications to employees
        $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
        $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

        $segIds = $survey->scheduleSegments;
        foreach ($segIds as $seg) {
            $userArr = ScheduleSegments::getUserBySegment($seg->id);
            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_ENPS]) {
                foreach ($userArr as $iuser) {
                    SendNotificationConsole::dispatch($user, $iuser, 'eNPS Invitations');
                }
            }
            if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_ENPS]) {
                foreach ($userArr as $iuser) {
                    SendNotificationEmail::dispatch($iuser, EmailNotifications::ENPS_INVITATION);
                }
            }
        }

        return $survey;
    }

    /**
     * @param $scheduleSurvey
     */
    public function setTotalQuestionnaire($scheduleSurvey)
    {
        $currentClient = Context::client()->get();
        $totalDefaultQuestionnaire = Questionnaire::query()
            ->whereNotNull('driver_id')
            ->whereNotIn('id', QuestionnaireClientSetting::query()
                ->where('client_id', $currentClient->getKey())
                ->where('status', '0')->pluck('questionnaire_id'))->count();

        $totalClientValue = ClientValue::query()
            ->where('client_id', $currentClient->getKey())->count();

        $totalQuestionnaire = $totalDefaultQuestionnaire + $totalClientValue;
        $scheduleSurvey->total_questionnaire = $totalQuestionnaire;
        $scheduleSurvey->save();
    }
}