<?php

namespace App\Services\Segment\SegmentType;

use App\Contracts\Validatable;
use App\Enum\SegmentTypes;
use App\Enum\TimeTypes;
use App\Helpers\ModelHelper;
use App\Helpers\SortFluent;
use App\Helpers\TimeHelper;
use App\Models\Segment\AgeSegment;
use App\Models\Segment\Segment;
use App\Models\Segment\SegmentType;
use App\Models\User\Employee;
use App\Models\User\User;
use App\Services\AuthBaseService;
use App\Traits\Pageable;
use Carbon\Carbon;
use App\Enum\AccessPermissions;

/**
 * Class GetSegmentEmployee
 * @package App\Services\Segment\SegmentType
 */
class GetSegmentEmployee extends AuthBaseService implements Validatable
{
    use Pageable;

    /**
     * @var array
     */
    protected $existedEmployee = array();

    public function getRules(array $args)
    {
        return [
            'code' => 'string|required',
            'segment_code'=> 'string|required',
        ];
    }

    /**
     * Get list quiz
     *
     * @param array $options
     * @return mixed
     */
    public function performAction(array $options)
    {
        $userIds = $this->getListEmployeeBySegmentType($options['code'], $options['segment_code']);

        $query = User::with('employee.client');
        $query->join('employees', 'users.id', 'employees.user_id');
        $query->whereIn('users.id', $userIds);
        $query->select('users.*');

        if (isset($options['filter_on_mobile']) && $options['filter_on_mobile']) {
            ModelHelper::setConditionFindByEmployee($query, AccessPermissions::ACCESS_GROUP_EMPLOYEE_DIRECTORY);
        }
        
        SortFluent::init()
            ->setQuery($query)
            ->setTable('users')
            ->setSortOptions($options)
            ->setAllowedSorts(['name', 'role', 'start_date', 'email'])
            ->validate()
            ->removeSort(['name', 'role', 'start_date'])
            ->setCustomSort('name', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'first_name', $direction, 'users');
                SortFluent::applyBasicSort($query, 'surname', $direction, 'users');
            })
            ->setCustomSort('role', function($query, $col, $direction) {
                $query->leftJoin('employees as e1', 'e1.user_id', 'users.id');
                SortFluent::applyBasicSort($query, 'job_title', $direction, 'e1');
            })
            ->setCustomSort('start_date', function($query, $col, $direction) {
                $query->leftJoin('employees as e2', 'e2.user_id', 'users.id');
                SortFluent::applyBasicSort($query, 'start_date', $direction, 'e2');
            })
            ->applySort();
        return $this->applyPaging($options, $query);
    }

    /**
     * @param $segmentTypeCode
     * @param $segmentCode
     * @return \Illuminate\Support\Collection
     */
    public function getListEmployeeBySegmentType($segmentTypeCode, $segmentCode)
    {
        $segmentType = SegmentType::findByBusinessKey($segmentTypeCode);
        if (in_array($segmentType->id, SegmentTypes::SEGMENT_TYPES_BASIC) ) {
            $employee = $this->getEmployeeSegmentBasic($segmentCode);
        } else if ($segmentType->id === SegmentTypes::MANAGER) {
            $employee = $this->getEmployeeSegmentManager($segmentCode);
        } else if ($segmentType->id === SegmentTypes::AGE) {
            $employee = $this->getEmployeeSegmentAge($segmentCode);
        } else if ($segmentType->id === SegmentTypes::EMPLOYMENT) {
            $employee = $this->getEmploymentSegmentEmploymentPhase($segmentCode);
        }
        return $employee;
    }

    /**
     * @param $segmentCode
     * @return mixed
     */
    public function getEmployeeSegmentBasic($segmentCode)
    {
        return Segment::findByBusinessKey($segmentCode)->includedCustomSegmentTypeUsers()
            ->pluck('users.id')->toArray();
    }

    /**
     * @param $segmentCode
     * @return \Illuminate\Support\Collection
     */
    public function getEmployeeSegmentManager($segmentCode)
    {
        $managerId = ModelHelper::convertBusinessKeyToId(User::class, $segmentCode);
        $employeeQuery = Employee::query()->has('user')
            ->where('manager_id', $managerId)
            ->select('user_id');
        ModelHelper::setConditionFindByClient($employeeQuery);
        return $employeeQuery->get()->pluck('user_id');
    }

    /**
     * @param $segmentCode
     * @return mixed
     */
    public function getEmployeeSegmentAge($segmentCode)
    {
        $employeesQuery = Employee::query()->has('user');
        ModelHelper::setConditionFindByClient($employeesQuery);
        $employees = $employeesQuery->get();

        $segment = AgeSegment::findByBusinessKey($segmentCode);
        $min = $segment->min;
        $max = $segment->max;
        return $employees->filter(function($item) use ($min, $max) {
            //Check Age is on range
            if (is_null($item['date_of_birth'])) {
                return false;
            }
            return TimeHelper::isTimeOnRange($item['date_of_birth'], Carbon::today(), $min, TimeTypes::YEAR, $max, TimeTypes::YEAR);
        })->pluck('user_id');

    }

    /**
     * @param $segmentCode
     * @return mixed
     */
    public function getEmploymentSegmentEmploymentPhase($segmentCode)
    {
        //TODO: add order priority for this segment
        $segments = app(GetSegmentTypeDetailEmploymentPhaseData::class)->execute();
        foreach ($segments as $segment) {
            if ($segment->getBusinessKey() === $segmentCode) {
                return $segment->employees->pluck('user_id');
            }
        }

    }

}
