<?php


namespace App\Services\Segment\SegmentType;


use App\Enum\EmployeeStatus;
use App\Helpers\FilterFluent;
use App\Helpers\ModelHelper;
use App\Helpers\SortFluent;
use App\Models\Segment\Segment;
use App\Models\Segment\SegmentType;
use App\Services\BaseService;
use App\Traits\Pageable;
use Illuminate\Support\Facades\DB;
use App\Enum\AccessPermissions;

/**
 * Class GetSegmentTypeDetail
 * @package App\Services\Segment
 */
class GetSegmentTypeDetailBasic extends BaseService
{
    use Pageable;


    /**
     * @param array $options
     * @return mixed
     * @throws \Illuminate\Validation\ValidationException
     */
    public function performAction(array $options)
    {
        $query = SegmentType::findByBusinessKey($options['code'])->segments();


        $query->select('segments.name', 'segments.code');

        ModelHelper::setConditionFindByClient($query,'segments');

        //count for total user
        $queryTotalEmployee = $this->getQueryTotalEmployee();

        $query->leftjoin(
            DB::raw("(" . $queryTotalEmployee->toSql() . ") as total_employee"),
            function($join) use ($queryTotalEmployee) {
                $join->on('segments.id', '=', 'total_employee.id')
                    ->addBinding($queryTotalEmployee->getBindings());
                // bindings for sub-query WHERE added
            }
        );
        $query->addSelect(DB::raw('coalesce(total_employee.count, 0) as total_employee'));

        //count for total user active
        $queryTotalEmployeeActive = $this->getQueryTotalEmployee(EmployeeStatus::ACTIVE);

        $query->leftjoin(
            DB::raw("(" . $queryTotalEmployeeActive->toSql() . ") as active_employee"),
            function($join) use ($queryTotalEmployeeActive) {
                $join->on('segments.id', '=', 'active_employee.id')
                    ->addBinding($queryTotalEmployeeActive->getBindings());
                // bindings for sub-query WHERE added
            }
        );
        $query->addSelect(DB::raw('coalesce(active_employee.count, 0) as active_employee'));

        FilterFluent::init()
            ->setQuery($query)
            ->setTable('segments')
            ->setFilterOptions($options)
            ->setAllowedFilters(['parent_code'])
            ->removeFilter('parent_code')
            ->validate()
            ->setCustomFilter('parent_code', function ($query, $type, $col, $value) {
                $parentId = ModelHelper::convertBusinessKeyToId(Segment::class, $value);
                FilterFluent::applyBasicFilter($query, $type, 'parent_id', $parentId);
            })
            ->applyFilter();

        SortFluent::init()
            ->setQuery($query)
            ->setTable('segments')
            ->setSortOptions($options)
            ->setAllowedSorts(['name', 'active_employee', 'total_employee'])
            ->validate()
            ->removeSort(['active_employee', 'total_employee'])
            ->setCustomSort('total_employee', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'total_employee', $direction);
            })
            ->setCustomSort('active_employee', function($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'active_employee', $direction);
            })
            ->applySort();

        return $this->applyPaging($options, $query);
    }


    /**
     * @param int $status
     * @return \Illuminate\Database\Eloquent\Builder|mixed
     */
    public function getQueryTotalEmployee(string $status = "")
    {
        $query = Segment::query()
            ->leftJoin('segment_user', 'segment_user.segment_id', 'segments.id')
            ->join('users', 'users.id', 'segment_user.user_id')
            ->join('employees', 'users.id', 'employees.user_id')
            ->groupBy('segments.id')
            ->select(DB::raw('count(users.id) as count, segments.id'));
        if ($status) {
            $query->where('employees.status', $status);
        }
        ModelHelper::setConditionFindByEmployee($query, AccessPermissions::ACCESS_GROUP_EMPLOYEE_DIRECTORY);
        ModelHelper::setConditionFindByClient($query,'segments');
        return $query;
    }
}