<?php

namespace App\Services\Subscription;

use App\Contracts\Validatable;
use App\Enum\EmployeeStatus;
use App\Enum\ErrorCode;
use App\Enum\PlanConfigurations;
use App\Enum\Roles;
use App\Facades\Context;
use App\Helpers\ObjectHelper;
use App\Helpers\StripeHelper;
use App\Models\Client\Client;
use App\Models\Subscription\SubscriptionActivity;
use App\Models\Subscription\SubscriptionPlan;
use App\Models\User\Employee;
use App\Services\BaseService;
use App\Services\Employee\ChangeEmployeeStatus;
use Stripe\Exception\ApiErrorException;
use App\Exceptions\InvalidRequestException;

class ChangePlan extends BaseService implements Validatable
{

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $ags
     * @return array
     */
    public function getRules(array $ags)
    {
        return [
            'plan_id' => 'required',
            'reason' => 'string',
            'password' => 'required|password:api'
        ];
    }

    /**
     * Change Plan
     * 
     * @param array $data
     * @return mixed
     * @throws ApiErrorException
     */
    public function performAction(array $data)
    {
        $newPlan = SubscriptionPlan::findByBusinessKey($data['plan_id']);
        $currentClient = Context::client()->get();

        if ($newPlan->getKey() != $currentClient->plan_id)
        {
            $stripeCustomerId = $currentClient->stripe_cus_id;

            $stripeHelper = new StripeHelper();
            $subscription = $stripeHelper->getCustomerSubscription($stripeCustomerId);
            if ($subscription) {
                $subscriptionItem = $subscription->items->first();
                $newPrice = $newPlan->stripe_price;

                $params = [
                    'cancel_at_period_end' => false,
                    'items' => [
                        [
                            'id' => $subscriptionItem->id,
                            'price' => $newPrice,
                        ]
                    ],
                ];
                
                $stripeHelper->updateSubscription($subscription->id, $params);
                
                // Update client plan
                $currentClient->plan_id = $newPlan->getKey();
                $currentClient->save();

                $currentPlan = $currentClient->subscriptionPlan;
                // Add subscription activity
                SubscriptionActivity::create([
                    'old_plan_id' => $currentPlan->getKey(),
                    'new_plan_id' => $newPlan->getKey(),
                    'reason' => ObjectHelper::nullOrValue($data, 'reason'),
                    'client_id' => $currentClient->getKey()
                ]);
                
                $this->disableExceedingClientEmployeeByPlanLimit($currentClient, $newPlan, Roles::ADMIN);
                $this->disableExceedingClientEmployeeByPlanLimit($currentClient, $newPlan, Roles::FRONT_USER);
            } else {
                throw new InvalidRequestException('Client has not any Stripe subscription. Please create new one', ErrorCode::CLIENT_HAS_NO_STRIPE_SUBSCRIPTION);
            }
        }
        
        return $newPlan;
    }

    /**
     * Disable exceeding client employees by plan limit 
     * 
     * @param Client $client
     * @param SubscriptionPlan $plan
     * @param string $role
     * @return bool|void
     */
    public function disableExceedingClientEmployeeByPlanLimit(Client $client, SubscriptionPlan $plan, string $role)
    {
        switch ($role) {
            case Roles::FRONT_USER:
                $employeeConfigName = PlanConfigurations::PACKAGE_EMPLOYEE;
                break;
            case Roles::ADMIN:
                $employeeConfigName = PlanConfigurations::PACKAGE_ADMINISTRATOR;
                break;
            default:
                return false;
        }
        
        $newPlanEmployeeConfig = $plan->planConfigurations()->where('name', $employeeConfigName)->first();
        if (!empty($newPlanEmployeeConfig)) {
            $employees = Employee::ofClient($client->getKey())
                ->whereHas('user', function ($q) use ($role) {
                    $q->role($role);
                })
                ->active()
                ->oldest()
                ->get();
            $exceededEmployeeCount = count($employees) - intval($newPlanEmployeeConfig->pivot->value);
            if ($exceededEmployeeCount > 0) {
                for ($i = 0; $i < $exceededEmployeeCount; $i++) {
                    $user = $employees[$i]->user;
                    app(ChangeEmployeeStatus::class)->execute([
                        'code' => $user->getBusinessKey(), 
                        'status' => EmployeeStatus::INACTIVE
                    ]);
                }
            }
        }
    }
}
