<?php

namespace App\Services\Tool\Journey;

use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\EmailNotifications;
use App\Enum\NotificationTypes;
use App\Enum\Roles;
use App\Events\JourneyUpdated;
use App\Helpers\ModelHelper;
use App\Helpers\ObjectHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\UserHelper;
use App\Jobs\SendNotification;
use App\Jobs\SendNotificationConsole;
use App\Jobs\SendNotificationEmail;
use App\Models\Journey\Journey;
use App\Models\Segment\Segment;
use App\Models\User\Employee;
use App\Models\User\User;
use App\Services\BaseService;
use App\Services\UserAccess\GetAccessByRole;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;

class AddEmployees extends BaseService implements Validatable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args)
    {
        return [
            'code' => 'required',
            'users' => 'array',
            'segments' => 'array',
            'exclude_users' => 'array',
            'exclude_segments' => 'array',
        ];
    }


    /**
     * Add employees to journey
     * 
     * @param array $data
     * @return mixed
     * @throws \Exception
     */
    public function performAction(array $data)
    {
        $journey = Journey::findByBusinessKey($data['code']);
        PermissionHelper::hasAccessOn($journey);

        $userCodes = ObjectHelper::nullOrValue($data, 'users');
        $userIds = ModelHelper::convertBusinessKeyToId(User::class, $userCodes);
        $userIdArr = $userIds ?? [];

        $userExcludesCodes = ObjectHelper::nullOrValue($data, 'exclude_users');
        $userExcludesIdArr = [];
        if (!empty($userExcludesCodes)) {
            $userExcludesIdArr = ModelHelper::convertBusinessKeyToId(User::class, $userExcludesCodes);
        }

        $segmentCodes = ObjectHelper::nullOrValue($data, 'segments');
        $segmentExcludesCodes = ObjectHelper::nullOrValue($data, 'exclude_segments');

        // Add users of segments
        if (!empty($segmentCodes)) {
            $segmentUserIds = UserHelper::getUserIdBySegment($segmentCodes);
            $userIdArr = array_merge($userIdArr, $segmentUserIds);
        }

        $userIdArr = array_unique($userIdArr);

        // Get users of segments excludes
        if (!empty($segmentExcludesCodes)) {
            $segmentExcludesUserIds = UserHelper::getUserIdBySegment($segmentExcludesCodes);
            $userExcludesIdArr = array_merge($userExcludesIdArr, $segmentExcludesUserIds);
        }
        $userExcludesIdArr = array_unique($userExcludesIdArr);

        // Filter users of segments excludes
        $userIdArr = array_diff($userIdArr, $userExcludesIdArr);

        // Remove existed users
        $journeyUserIds = $journey->users()->pluck('users.id')->toArray();
        $userIdArr = array_diff($userIdArr, $journeyUserIds);

        // Ensure user must belong to current client
        $userQuery = Employee::join('users', 'users.id', 'employees.user_id')
            ->whereIn("employees.user_id", $userIdArr);
        $userCodeArr = $userQuery->pluck('users.code', 'employees.user_id')->toArray();

        $journeyUserData = array_fill_keys(array_keys($userCodeArr), ["created_at" => Carbon::now()]);

        $journey->users()->attach($journeyUserData);

        event(new JourneyUpdated($journey));

        if (!empty($userCodeArr)) {
            $arguments = array(
                'sender' => Auth::user()->id,
                "userIds" => array_values($userCodeArr),
                "headings" => __("New journey assigned"),
                "contents" => __("Tap to get started"),
                "data" => [
                    "type" => NotificationTypes::JOURNEY_ADDED,
                    "id" => $journey->id
                ],
                'send_after' => null
            );
            SendNotification::dispatch($arguments);
        }

        $result = $userQuery->select([
            "users.*",
            "employees.job_title as job_title",
            "employees.start_date as start_date"
        ])->get();


        // Send Invite Notifications to employees
        $users = User::whereIn('id', $userIdArr)->get();
        $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
        $accessEmployeePermissions = $employeeAccess[AccessPermissions::ACCESS_GROUP_NOTIFICATIONS];

        if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_CONSOLE_EMPLOYEE_JOURNEY]) {
            foreach ($users as $user) {
                SendNotificationConsole::dispatch($user, $user, 'Invited you to a journey');
            }
        }
        if ($accessEmployeePermissions[AccessPermissions::NOTIFICATION_EMAIL_EMPLOYEE_JOURNEY]) {
            foreach ($users as $user) {
                SendNotificationEmail::dispatch($user, EmailNotifications::JOURNEY_INVITATION, ['journey' => $journey->name]);
            }
        }

        return $result;
    }
}
