<?php

namespace App\Services\Tool\Journey;

use App\Contracts\Validatable;
use App\Enum\FilterTypes;
use App\Enum\SubmissionStatus;
use App\Facades\Context;
use App\Helpers\FilterFluent;
use App\Helpers\PermissionHelper;
use App\Helpers\SortFluent;
use App\Models\ActionRequest\ActionRequestSubmission;
use App\Models\Journey\Journey;
use App\Services\BaseService;
use App\Traits\Pageable;
use Illuminate\Validation\ValidationException;

class GetJourneyEmployees extends BaseService implements Validatable
{
    use Pageable;

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args)
    {
        return [
            'code' => 'required',
        ];
    }



    /**
     * Get journey employees
     * 
     * @param array $data
     * @return mixed
     * @throws ValidationException
     */
    public function performAction(array $data)
    {
        $options = Context::request()->getOptions();
        $journey = Journey::findByBusinessKey($data['code']);
        $steps = $journey->steps()->with('actionRequest')->get();

        PermissionHelper::hasAccessOn($journey);
        $query = $journey->users()->join('employees', 'users.id', '=', 'employees.user_id')
            ->with(['avatar', 'employee', 'actionRequestSubmissions']);

        $filter = FilterFluent::init()
            ->setQuery($query)
            ->setFilterOptions($options)
            ->setAllowedFilters(['code', 'name', 'job_title', 'start_date', 'isCompleted', 'department', 'team', 'gender',
                'location', 'completedDate', 'last_accessed_date', 'manager'])
            ->validate();

        $filter->removeFilter('name')
            ->removeFilter('job_title')
            ->removeFilter('start_date')
            ->removeFilter('isCompleted')
            ->removeFilter(['department', 'team', 'location', 'gender', 'completedDate', 'last_accessed_date', 'manager'])
            ->setCustomFilter('name', function ($query, $type, $col, $value) {
                FilterFluent::applyCombinedColumnFilter($query, $type, 'surname','first_name', $value, 'users');
            })
            ->setCustomFilter('job_title', function ($query, $type, $col, $value) {
                FilterFluent::applyBasicFilter($query, $type, 'job_title', $value, 'employees');
            })
            ->setCustomFilter('start_date', function ($query, $type, $col, $value) {
                $dateValue = explode(";",$value);
                $fromDate = $dateValue[0];
                $toDate = $dateValue[1];
                FilterFluent::applyBasicFilter($query, FilterTypes::GREATER_THAN_OR_EQUAL, 'start_date', $fromDate, 'employees');
                FilterFluent::applyBasicFilter($query, FilterTypes::LESS_THAN_OR_EQUAL, 'start_date', $toDate, 'employees');
            })
            ->setCustomFilter('department', function($query, $type, $col, $value) {
                $query->join('segments', 'employees.department_id', '=', 'segments.id');
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 'segments');
            })
            ->setCustomFilter('team', function($query, $type, $col, $value) {
                $query->join('segments', 'employees.team_id', '=', 'segments.id');
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 'segments');
            })
            ->setCustomFilter('location', function($query, $type, $col, $value) {
                $query->join('segments', 'employees.location_id', '=', 'segments.id');
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 'segments');
            })
            ->setCustomFilter('gender', function($query, $type, $col, $value) {
                $query->join('segments', 'employees.gender_id', '=', 'segments.id');
                FilterFluent::applyBasicFilter($query, $type, 'name', $value, 'segments');
            })
            ->setCustomFilter('manager', function($query, $type, $col, $value) {
                $query->whereHas('employee', function($q) use ($type, $col, $value) {
                    $q->whereHas('manager', function($qu) use ($type, $col, $value) {
                        FilterFluent::applyCombinedColumnFilter($qu, $type, 'surname','first_name', $value, 'users');
                    });
                });
            })
            ->setCustomFilter('last_accessed_date', function ($query, $type, $col, $value) {
                $query->whereHas('actionRequestSubmissions', function($q) use ($type, $col, $value) {
                    $qu = $q->latest()->limit(1);

                    $dateValue = explode(";",$value);
                    $fromDate = $dateValue[0];
                    $toDate = $dateValue[1];

                    FilterFluent::applyBasicFilter($qu, FilterTypes::GREATER_THAN_OR_EQUAL, 'updated_at', $fromDate);
                    FilterFluent::applyBasicFilter($qu, FilterTypes::LESS_THAN_OR_EQUAL, 'updated_at', $toDate);

                });
            })
            ->setCustomFilter('completedDate', function ($query, $type, $col, $value) use ($journey) {
                $dateValue = explode(";",$value);
                $fromDate = $dateValue[0];
                $toDate = $dateValue[1];
                $query->where('journey_user.is_completed', 1)
                    ->whereDate('journey_user.updated_at', '>=', $fromDate)
                    ->whereDate('journey_user.updated_at', '<=', $toDate);
            })
            ->setCustomFilter('isCompleted', function($query, $type, $col, $value) use ($steps, $journey) {
                if ($value == "Not started") {
                    $query->whereDoesntHave('actionRequestSubmissions');
                } else if ($value == "In Progress") {
                    $query->whereHas('actionRequestSubmissions', function($q) use ($type, $col, $value, $steps) {
                        $q->groupBy('user_id')->havingRaw('COUNT(*) < ?', [$steps->count()]);

                    });
                } else {
                    $query->whereHas('actionRequestSubmissions', function($q) use ($type, $col, $value, $steps) {
                        $q->groupBy('user_id')->havingRaw('COUNT(*) >= ?', [$steps->count()]);
                    });
                }
            })
            ->applyFilter();

        $sorter = SortFluent::init()
            ->setQuery($query)
            ->setTable('users')
            ->setSortOptions($options)
            ->setAllowedSorts(['name', 'job_title', 'start_date'])
            ->validate();

        $sorter->removeSort(['name', 'job_title', 'start_date'])
            ->setCustomSort('name', function ($query, $col, $direction) {
                SortFluent::applyBasicSort($query, 'surname', $direction);
            })
            ->setCustomSort('job_title', function ($query, $col, $direction) {
                $query->leftJoin('employees as e1', 'users.id', 'e1.user_id');
                SortFluent::applyBasicSort($query, 'job_title', $direction, 'e1');
            })
            ->setCustomSort('start_date', function ($query, $col, $direction) {
                $query->leftJoin('employees as e2', 'users.id', 'e2.user_id');
                SortFluent::applyBasicSort($query, 'start_date', $direction, 'e2');
            })
            ->applySort();

        if (isset($data['isNotPaging']) && $data['isNotPaging']) {
            $employees = $query->get();
        } else {
            $employees = $this->applyPaging($options, $query);
        }


        // Attach employee submissions
        foreach($employees as $employee){
            $submissions = app(GetEmployeeSubmission::class)->execute(
                [
                    'code' => $journey->getBusinessKey(),
                    'employee_code' => $employee->getBusinessKey()
                ]
            );
            $employee->stepSubmissions = $submissions;
            $submission = end($submissions)['submission'];
            if (!empty($submission)) {
                $employee->isCompleted = true;
            } else {
                $employee->isCompleted = false;
            }
        }

        return $employees;
    }

    public function getCompletedSubmissionCount(array $actionRequestIds, $userId)
    {
        return ActionRequestSubmission::whereIn('action_request_id', $actionRequestIds)
        ->whereIn('status', SubmissionStatus::COMPLETED_STATUS)
        ->where('user_id', $userId)
        ->select('action_request_id')
        ->groupBy('action_request_id')
        ->get()
        ->count();
    }
}
 