<?php

namespace App\Services\Tool\Journey;

use App\Contracts\Validatable;
use App\Enum\ActionRequestTypes;
use App\Enum\AvailableTerms;
use App\Enum\ErrorCode;
use App\Events\JourneyUpdated;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\PandaDocHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\UserAccessHelper;
use App\Models\Journey\Journey;
use App\Services\ActionRequest\CheckPlanAvailableForStepType;
use App\Services\ActionRequest\CreateActionRequest;
use App\Services\ActionRequest\UpdateActionRequest;
use App\Services\BaseService;
use App\Services\Tool\Form\CheckValidTemplate;
use Illuminate\Support\Arr;
use Illuminate\Validation\Rule;

class SaveJourneyStep extends BaseService implements Validatable
{
    /**
     * @var array
     */
    public $data;

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args)
    {
        $rules = [
            'code' => 'required',
            'steps' => 'array'
        ];

        if (!empty($args['steps'])) {
            foreach ($args['steps'] as $key => $value) {
                $rules["steps.$key.type"] = ['required', Rule::in(ActionRequestTypes::ACTIVE_TYPES)];
                $rules["steps.$key.name"] = ['required', 'string', 'max:255'];
                $rules["steps.$key.step_order"] = ['integer'];

                if (isset($value['step_is_optional']) && !$value['step_is_optional']) {
                    $rules["steps.$key.is_available_immediately"] = ['required', 'boolean'];

                    if (isset($value['is_available_immediately']) && !$value['is_available_immediately']) {
                        $rules["steps.$key.available_days"] = ['required', 'integer'];
                        $rules["steps.$key.available_term"] = [
                            'required', Rule::in([
                                AvailableTerms::AVAILABLE_TERM_BEFORE_START_DATE,
                                AvailableTerms::AVAILABLE_TERM_AFTER_START_DATE
                            ])
                        ];
                    }

                    $rules["steps.$key.is_notify"] = ['required', 'boolean'];
                    if (isset($value['is_notify']) && $value['is_notify']) {
                        // $rules["steps.$key.emails_notify"] = ['required', 'string'];
                    }

                    if (isset($value['type'])) {
                        if (in_array($value['type'], ActionRequestTypes::CONFIGURABLE_TYPES)) {
                            $rules["steps.$key.header_image_id"] = ['nullable', 'integer'];
                            $rules["steps.$key.message"] = ['required', 'string'];
                        }

                        if (in_array($value['type'], ActionRequestTypes::CONTENTABLE_TYPES)) {
                            $rules["steps.$key.content_id"] = ['required'];

                            if ($value['type'] == ActionRequestTypes::ID_VERIFICATION) {
                                array_push($rules["steps.$key.content_id"], Rule::in([
                                    ActionRequestTypes::ID_VERIFICATION_PASSPORT,
                                    ActionRequestTypes::ID_VERIFICATION_DRIVER_LICENSE,
                                    ActionRequestTypes::ID_VERIFICATION_NATIONAL_ID
                                ]));
                            }
                        }
                    }
                }
            }
        }

        return $rules;
    }



    /**
     * Save journey step
     *
     * @param array $data
     * @return Journey 
     */
    public function performAction(array $data): Journey
    {
        $journey = Journey::findByBusinessKey($data['code']);
        PermissionHelper::hasAccessOn($journey);
        UserAccessHelper::hasAccessAsOwner($journey);

        // Prevent editing journey when having submissions from employees
        $steps = $journey->steps->load('actionRequest.submissions');
        //        foreach ($steps as $step) {
//            if ($step->hasSubmission()) {
//                throw new InvalidRequestException(
//                    "This journey has submissions from employees. To Edit the journey, please unpublish first.",
//                    ErrorCode::JOURNEY_HAS_SUBMISSION
//                );
//            }
//        }

        $inputSteps = $data['steps'] ?? [];

        // Delete removed steps of given journey
        $stepIds = Arr::pluck($inputSteps, 'id');
        foreach ($journey->steps as $step) {
            if (!in_array($step->id, $stepIds)) {
                $step->actionRequest()->delete();
                $step->delete();
            }
        }

        $inputSteps = Arr::sort($inputSteps, function ($value, $index) {
            return $value['step_order'] ?? $index;
        });
        $inputSteps = array_values($inputSteps);

        foreach ($inputSteps as $index => $step) {
            $this->checkPlanAvailableForStepType($step['type']);

            $stepData = [
                'name' => $step['name'] ?? null,
                'is_available_immediately' => $step['is_available_immediately'] ?? null,
                'step_is_optional' => $step['step_is_optional'] ?? false,
                'header_image_id' => $step['header_image_id'] ?? null,
                'message' => $step['message'] ?? null,
                'step_order' => $index,
                'is_notify' => $step['is_notify'] ?? null,
                'type' => $step['type'] ?? null,
            ];

            if (!$step['is_available_immediately']) {
                $stepData = Arr::add($stepData, 'available_days', $step['available_days']);
                $stepData = Arr::add($stepData, 'available_term', $step['available_term']);
            }

            if ($step['is_notify']) {
                $stepData = Arr::add($stepData, 'emails_notify', $step['emails_notify']);
            }

            if (!in_array($step['type'], ActionRequestTypes::CONFIGURABLE_TYPES)) {
                Arr::except($stepData, ['header_image_id', 'message']);
            }

            $contentableModel = ActionRequestHelper::getContentableModel($step['type'], $step['content_id'] ?? null);

            PermissionHelper::hasAccessOn($contentableModel);

            if (!is_null($contentableModel) && $step['type'] == ActionRequestTypes::FORM) {
                app(CheckValidTemplate::class)->execute([
                    'template_id' => $contentableModel->template_id ?? null,
                    'replacements' => ['step' => $stepData['name']]
                ]);
            }

            // Prevent adding contents which is not published
            if (
                in_array($step['type'], ActionRequestTypes::PUBLISHABLE_TYPES) &&
                (!is_null($contentableModel) && !$contentableModel->is_published)
            ) {
                throw new InvalidRequestException("Invalid publishable content", ErrorCode::INVALID_PUBLISHABLE_CONTENT, ['step' => $stepData['name']]);
            }

            // Save journey step
            $createdStep = $journey->steps()->updateOrCreate(
                ['id' => $step['id'] ?? null],
                $stepData
            );

            $actionRequest = $createdStep->actionRequest;

            $actionRequestData = [];

            if (in_array($createdStep->id, $stepIds) && !empty($actionRequest)) {
                // Update action request
                app(UpdateActionRequest::class)->execute(
                    $actionRequestData,
                    $createdStep,
                    $contentableModel,
                    $actionRequest
                );
            } else {
                // Create action request
                app(CreateActionRequest::class)->execute(
                    $actionRequestData,
                    $createdStep,
                    $contentableModel
                );
            }
        }

        event(new JourneyUpdated($journey));

        return $journey->refresh();
    }

    /**
     * @param $type
     */
    public function checkPlanAvailableForStepType($type)
    {
        app(CheckPlanAvailableForStepType::class)->execute($type);
    }
}