<?php

namespace App\Services\Tool\Quiz;

use App\Contracts\Performable;
use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\ActionRequestTypes;
use App\Enum\ErrorCode;
use App\Enum\PowerSearchActions;
use App\Enum\PowerSearchTypes;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\UserAccessHelper;
use App\Jobs\IndexPowerSearch;
use App\Models\Quiz\Quiz;
use App\Services\BaseService;

class DeleteQuiz extends BaseService implements Validatable, Transactionable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($args)
    {
        return [
            'code' => 'required',
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Delete quiz
     *
     * @param array $data
     * @return bool
     */
    public function performAction(array $data): bool
    {
        $quiz = Quiz::findByBusinessKey($data['code']);
        PermissionHelper::hasAccessOn($quiz);
        UserAccessHelper::hasAccessAsOwner($quiz);
        
        $this->checkQuizEligibleToDelete($quiz);

        foreach ($quiz->questions as $question) {
            $question->options()->delete();
            $question->delete();
        }
        IndexPowerSearch::dispatch($quiz->getBusinessKey(), PowerSearchTypes::QUIZ, PowerSearchActions::DELETE, $quiz->getClientId());
        $quiz->forceDelete();

        return true;
    }

    /**
     * @param Quiz $quiz
     * @return bool
     * @throws InvalidRequestException
     */
    public function checkQuizEligibleToDelete(Quiz $quiz)
    {
        if (count($quiz->actionRequests) > 0) {
            $firstActionRequest = $quiz->actionRequests->first();
            $actionName = ActionRequestHelper::getActionableName(
                $firstActionRequest->actionable_type, $firstActionRequest->actionable
            );
            $actionType = ActionRequestHelper::getActionableType($firstActionRequest->actionable_type);

            switch ($actionType) {
                case ActionRequestTypes::ACTION_SEQUENCE:
                    $errorCode = ErrorCode::DELETE_QUIZ_IN_USED_ACTION_SEQUENCE;
                    break;
                case ActionRequestTypes::ACTION:
                    $errorCode = ErrorCode::DELETE_QUIZ_IN_USED_ACTION;
                    break;
                case ActionRequestTypes::JOURNEY:
                    $errorCode = ErrorCode::DELETE_QUIZ_IN_USED_JOURNEY;
                    break;
                default:
                    $errorCode = null;
            }
            
            throw new InvalidRequestException(
                "This quiz being used in $actionName $actionType. To Delete the quiz, please remove it from the $actionType first.",
                $errorCode,
                ['action' => $actionName, 'type' => $actionType]
            );
        }
        
        return true;
    }
}
