<?php

namespace App\Services\Tool\Quiz;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\ActionRequestTypes;
use App\Enum\ErrorCode;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\UserAccessHelper;
use App\Models\Quiz\Quiz;
use App\Services\BaseService;
use Illuminate\Support\Arr;
use Illuminate\Validation\ValidationException;

class SaveQuizQuestion extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args)
    {
        return [
            'code' => 'required',
            'questions' => 'array',
            'questions.*.title' => 'required|string',
            'questions.*.options' => 'required|array',
            'questions.*.options.*.content' => 'required|string|max:255',
            'questions.*.correct_option' => 'required|integer',
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Save quiz question
     *
     * @param array $data
     * @return Quiz
     */
    public function performAction(array $data): Quiz
    {
        $quiz = Quiz::findByBusinessKey($data['code']);
        PermissionHelper::hasAccessOn($quiz);
        UserAccessHelper::hasAccessAsOwner($quiz);

        $inputQuestions = $data['questions'] ?? [];

        if ($quiz->is_published && empty($inputQuestions)) {
            throw new InvalidRequestException(
                'Cannot update published quiz without steps.',
                ErrorCode::EMPTY_PUBLISHED_QUIZ_QUESTIONS
            );
        }
        
//        $this->checkQuizEligibleToEdit($quiz);

        // Delete old questions of given quiz
        $questionIds = Arr::pluck($inputQuestions, 'id');
        foreach ($quiz->questions as $question) {
            if (!in_array($question->id, $questionIds)) {
                $question->options()->delete();
                $question->delete();
            }
        }

        foreach ($inputQuestions as $key => $question) {
            $order = $question['order'] ?? $key;
            // Save question
            $createdQuestion = $quiz->questions()->updateOrCreate(
                ['id' => $question['id'] ?? null],
                [
                    'title' => $question['title'],
                    'order' => $order
                ]
            );

            // Delete old options of current question
            $optionIds = Arr::pluck($question['options'], 'id');
            foreach ($createdQuestion->options as $option) {
                if (!in_array($option->id, $optionIds)) {
                    $option->delete();
                }
            }

            foreach ($question['options'] as $key => $option) {
                // Save option
                $createdQuestion->options()->updateOrCreate(
                    ['id' => $option['id'] ?? null],
                    [
                        'content' => $option['content'],
                        'is_correct' => ($question['correct_option'] == $key)
                    ]
                );
            }
        }

        return $quiz->refresh();
    }

    /**
     * @param array $data
     * @return array
     * @throws ValidationException
     */
    public function performValidation(array $data): array
    {
        $data = parent::performValidation($data);

        if (!empty($data['questions'])) {
            foreach ($data['questions'] as $key => $question) {
                if (!in_array($question['correct_option'], array_keys($question['options']))) {
                    throw ValidationException::withMessages([
                        "questions.$key.correct_option" =>
                            __('The correct option value does not exist in option indexes.')
                    ]);
                }
            }
        }

        return $data;
    }

    /**
     * @param Quiz $quiz
     * @return bool
     * @throws InvalidRequestException
     */
    public function checkQuizEligibleToEdit(Quiz $quiz)
    {
        // Check if quiz is in-used in actions which have submission
        $actionRequests = $quiz->actionRequests->load('submissions');
        foreach ($actionRequests as $actionRequest) {
            if ($actionRequest->submissions()->count() > 0) {
                $actionName = ActionRequestHelper::getActionableName(
                    $actionRequest->actionable_type, $actionRequest->actionable
                );
                $actionType = ActionRequestHelper::getActionableType($actionRequest->actionable_type);

                switch ($actionType) {
                    case ActionRequestTypes::ACTION_SEQUENCE:
                        $errorCode = ErrorCode::QUIZ_HAS_ACTION_SEQUENCE_SUBMISSION;
                        break;
                    case ActionRequestTypes::ACTION:
                        $errorCode = ErrorCode::QUIZ_HAS_ACTION_SUBMISSION;
                        break;
                    case ActionRequestTypes::JOURNEY:
                        $errorCode = ErrorCode::QUIZ_HAS_JOURNEY_SUBMISSION;
                        break;
                    default:
                        $errorCode = null;
                }
                
                throw new InvalidRequestException(
                    "This quiz has submission in $actionName $actionType.",
                    $errorCode,
                    ['action' => $actionName, 'type' => $actionType]
                );
            }
        }
        
        return true;
    }
}
