<?php

namespace App\Services\Tool\Quiz;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\ActionRequestTypes;
use App\Enum\ErrorCode;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\PermissionHelper;
use App\Models\Quiz\Quiz;
use App\Services\BaseService;

class UnpublishQuiz extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args)
    {
        return [
            'code' => 'required'
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Publish quiz
     *
     * @param array $data
     * @return Quiz
     */
    public function performAction(array $data): Quiz
    {
        $quiz = Quiz::findByBusinessKey($data['code']);
        PermissionHelper::hasAccessOn($quiz);

        $this->checkQuizEligibleToUnPublish($quiz);
        
        $quiz->is_published = false;
        $quiz->save();
        
        return $quiz->refresh();
    }

    /**
     * @param Quiz $quiz
     * @return bool
     * @throws InvalidRequestException
     */
    public function checkQuizEligibleToUnPublish(Quiz $quiz)
    {
        // Check if quiz is in-used
        if (count($quiz->actionRequests) > 0) {
            $firstActionRequest = $quiz->actionRequests->first();
            $actionName = ActionRequestHelper::getActionableName(
                $firstActionRequest->actionable_type, $firstActionRequest->actionable
            );
            $actionType = ActionRequestHelper::getActionableType($firstActionRequest->actionable_type);

            switch ($actionType) {
                case ActionRequestTypes::ACTION_SEQUENCE:
                    $errorCode = ErrorCode::UN_PUBLISH_QUIZ_IN_USED_ACTION_SEQUENCE;
                    break;
                case ActionRequestTypes::ACTION:
                    $errorCode = ErrorCode::UN_PUBLISH_QUIZ_IN_USED_ACTION;
                    break;
                case ActionRequestTypes::JOURNEY:
                    $errorCode = ErrorCode::UN_PUBLISH_QUIZ_IN_USED_JOURNEY;
                    break;
                default:
                    $errorCode = null;
            }
            
            throw new InvalidRequestException(
                "This quiz being used in $actionName $actionType. To Unpublish the quiz, please remove it from the $actionType first.",
                $errorCode,
                ['action' => $actionName, 'type' => $actionType]
            );
        }

        return true;
    }
}
