<?php


namespace App\Services\Tool\Survey;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\QuestionTypes;
use App\Enum\ActionRequestTypes;
use App\Enum\ErrorCode;
use App\Enum\QuestionTypes as QuestionTypesEnum;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\UserAccessHelper;
use App\Models\Survey\Survey;
use App\Models\Survey\SurveyQuestion;
use App\Services\BaseService;
use Illuminate\Support\Arr;

/**
 * Class UpdateSurvey
 * @package App\Services\Tool\Survey
 */
class AttachQuestion extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $args
     * @return array
     */
    public function getRules(array $args): array
    {
        return [
            'code' => 'required|string',
            'questions' => 'array',
            'questions.*.type' => [
                function($attribute, $value, $fail) {
                    if (!in_array($value, QuestionTypesEnum::LIST_TYPE)) {
                        $fail(__('validation.invalid', ['attribute' => $attribute]));
                    }
                },
            ],
            'questions.*.id' => 'integer',
            'questions.*.title' => 'string',
            'questions.*.options' => [
                "required_if:questions.*.type," . QuestionTypesEnum::TYPE_MULTI,
                "array",
            ],
            'questions.*.options.*.content' => [
                "required_if:questions.*.type," . QuestionTypesEnum::TYPE_MULTI,
                "string",
                "max:255"
            ]
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Create survey
     *
     * @param array $data
     * @return Survey
     */
    public function performAction(array $data): Survey
    {
        $survey = Survey::findByBusinessKey($data['code']);
        PermissionHelper::hasAccessOn($survey);
        UserAccessHelper::hasAccessAsOwner($survey);
        
        $inputQuestions = $data['questions'] ?? [];
        
        if ($survey->is_published && empty($inputQuestions)) {
            throw new InvalidRequestException(
                'Cannot update published survey without steps.',
                ErrorCode::EMPTY_PUBLISHED_SURVEY_QUESTIONS
            );
        }
        
//        $this->checkSurveyEligibleToEdit($survey);

        foreach ($survey->surveyQuestions as $surveyQuestion) {
            if (!in_array($surveyQuestion->id, Arr::pluck($inputQuestions, 'id'))) {
                $surveyQuestion->surveyOptions()->delete();
                $surveyQuestion->delete();
            }
        }
        foreach ($inputQuestions as $key => $question) {
            $question['order'] = $question['order'] ?? $key;
            $surveyQuestion = $survey->surveyQuestions()->updateOrCreate(
                ['id' => $question['id'] ?? null],
                $question
            );
            if ($question['type'] === QuestionTypesEnum::TYPE_MULTI) {
                $optionIds = Arr::pluck($question['options'], 'id');
                foreach ($surveyQuestion->surveyOptions as $surveyOption) {
                    if (!in_array($surveyOption->id, $optionIds)) {
                        $surveyOption->delete();
                    }
                }
                foreach ($question['options'] as $option) {
                    $surveyQuestion->surveyOptions()->updateOrCreate(
                        ['id' => $option['id'] ?? null],
                        $option
                    );
                }
            }
        }

        return $survey->fresh();
    }

    /**
     * @param Survey $survey
     * @return bool
     * @throws InvalidRequestException
     */
    public function checkSurveyEligibleToEdit(Survey $survey)
    {
        // Check if survey is in-used in actions which have submission
        $actionRequests = $survey->actionRequests->load('submissions');
        foreach ($actionRequests as $actionRequest) {
            if ($actionRequest->submissions()->count() > 0) {
                $actionName = ActionRequestHelper::getActionableName(
                    $actionRequest->actionable_type, $actionRequest->actionable
                );
                $actionType = ActionRequestHelper::getActionableType($actionRequest->actionable_type);

                switch ($actionType) {
                    case ActionRequestTypes::ACTION_SEQUENCE:
                        $errorCode = ErrorCode::SURVEY_HAS_ACTION_SEQUENCE_SUBMISSION;
                        break;
                    case ActionRequestTypes::ACTION:
                        $errorCode = ErrorCode::SURVEY_HAS_ACTION_SUBMISSION;
                        break;
                    case ActionRequestTypes::JOURNEY:
                        $errorCode = ErrorCode::SURVEY_HAS_JOURNEY_SUBMISSION;
                        break;
                    default:
                        $errorCode = null;
                }
                
                throw new InvalidRequestException(
                    "This survey has submission in $actionName $actionType.",
                    $errorCode,
                    ['action' => $actionName, 'type' => $actionType]
                );
            }
        }
        
        return true;
    }
}
