<?php


namespace App\Services\Tool\Survey;

use App\Contracts\Performable;
use App\Contracts\Transactionable;
use App\Contracts\Validatable;
use App\Enum\ActionRequestTypes;
use App\Enum\ErrorCode;
use App\Enum\PowerSearchActions;
use App\Enum\PowerSearchTypes;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\PermissionHelper;
use App\Helpers\UserAccessHelper;
use App\Jobs\IndexPowerSearch;
use App\Models\Survey\Survey;
use App\Services\BaseService;

class DeleteSurvey extends BaseService implements Validatable, Transactionable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($args)
    {
        return [
            'code' => 'required|string',
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }

    /**
     * Delete event
     *
     * @param array $data
     * @return bool
     */
    public function performAction(array $data): bool
    {
        $survey = Survey::findByBusinessKey($data['code']);
        PermissionHelper::hasAccessOn($survey);
        UserAccessHelper::hasAccessAsOwner($survey);
        
        $this->checkSurveyEligibleToDelete($survey);

        foreach ($survey->surveyQuestions as $surveyQuestion) {
            $surveyQuestion->surveyOptions()->delete();
            $surveyQuestion->delete();
        }
        IndexPowerSearch::dispatch($survey->getBusinessKey(), PowerSearchTypes::SURVEY, PowerSearchActions::DELETE, $survey->getClientId());
        $survey->forceDelete();

        return true;
    }

    /**
     * @param Survey $survey
     * @return bool
     * @throws InvalidRequestException
     */
    public function checkSurveyEligibleToDelete(Survey $survey)
    {
        // Check if survey is in-used
        if (count($survey->actionRequests) > 0) {
            $firstActionRequest = $survey->actionRequests->first();
            $actionName = ActionRequestHelper::getActionableName(
                $firstActionRequest->actionable_type, $firstActionRequest->actionable
            );
            $actionType = ActionRequestHelper::getActionableType($firstActionRequest->actionable_type);

            switch ($actionType) {
                case ActionRequestTypes::ACTION_SEQUENCE:
                    $errorCode = ErrorCode::DELETE_SURVEY_IN_USED_ACTION_SEQUENCE;
                    break;
                case ActionRequestTypes::ACTION:
                    $errorCode = ErrorCode::DELETE_SURVEY_IN_USED_ACTION;
                    break;
                case ActionRequestTypes::JOURNEY:
                    $errorCode = ErrorCode::DELETE_SURVEY_IN_USED_JOURNEY;
                    break;
                default:
                    $errorCode = null;
            }
            
            throw new InvalidRequestException(
                "This survey being used in $actionName $actionType. To Delete the survey, please remove it from the $actionType first.",
                $errorCode,
                ['action' => $actionName, 'type' => $actionType]
            );
        }
        
        return true;
    }
}
