<?php


namespace App\Services\Tool\Survey;

use App\Contracts\Performable;
use App\Contracts\Validatable;
use App\Enum\ActionRequestTypes;
use App\Enum\ErrorCode;
use App\Exceptions\InvalidRequestException;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\PermissionHelper;
use App\Models\Survey\Survey;
use App\Services\BaseService;

/**
 * Class UpdateSurvey
 * @package App\Services\Tool\Survey
 */
class UnpublishSurvey extends BaseService implements Validatable, Performable
{
    /**
     * Get the validation rules that apply to the service.
     *
     * @return array
     */
    public function getRules($args)
    {
        return [
            'code' => 'required|string',
        ];
    }

    public function canPerform(): bool
    {
        return Context::client()->check();
    }


    /**
     * Create survey
     *
     * @param array $data
     * @return Survey
     */
    public function performAction(array $data): Survey
    {
        $survey = Survey::findByBusinessKey($data['code']);

        PermissionHelper::hasAccessOn($survey);
        
        $this->checkSurveyEligibleToUnPublish($survey);

        $survey->is_published = 0;
        $survey->save();
        return $survey;
    }

    /**
     * @param Survey $survey
     * @return bool
     * @throws InvalidRequestException
     */
    public function checkSurveyEligibleToUnPublish(Survey $survey)
    {
        // Check if survey is in-used
        if (count($survey->actionRequests) > 0) {
            $firstActionRequest = $survey->actionRequests->first();
            $actionName = ActionRequestHelper::getActionableName(
                $firstActionRequest->actionable_type, $firstActionRequest->actionable
            );
            $actionType = ActionRequestHelper::getActionableType($firstActionRequest->actionable_type);

            switch ($actionType) {
                case ActionRequestTypes::ACTION_SEQUENCE:
                    $errorCode = ErrorCode::UN_PUBLISH_SURVEY_IN_USED_ACTION_SEQUENCE;
                    break;
                case ActionRequestTypes::ACTION:
                    $errorCode = ErrorCode::UN_PUBLISH_SURVEY_IN_USED_ACTION;
                    break;
                case ActionRequestTypes::JOURNEY:
                    $errorCode = ErrorCode::UN_PUBLISH_SURVEY_IN_USED_JOURNEY;
                    break;
                default:
                    $errorCode = null;
            }
            
            throw new InvalidRequestException(
                "This survey being used in $actionName $actionType. To Unpublish the survey, please remove it from the $actionType first.",
                $errorCode,
                ['action' => $actionName, 'type' => $actionType]
            );
        }
        
        return true;
    }
}
