<?php

namespace App\Services\UserAccess;

use App\Contracts\Validatable;
use App\Enum\AccessPermissions;
use App\Enum\NotificationTypes;
use App\Enum\RoleEditablePermissions;
use App\Events\ManagerAccessUpdated;
use App\Events\RoleAccessUpdated;
use App\Facades\Context;
use App\Helpers\ActionRequestHelper;
use App\Helpers\DateHelper;
use App\Helpers\ModelHelper;
use App\Helpers\UserAccessHelper;
use App\Jobs\SendNotification;
use App\Models\User\User;
use App\Services\BaseService;
use App\Models\UserAccess\ClientAccess;
use Carbon\Carbon;
use Illuminate\Validation\Rule;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;
use App\Enum\Roles;
use Illuminate\Support\Facades\Auth;

class UpdateRoleAccess extends BaseService implements Validatable
{

    /**
     * Get the validation rules that apply to the service.
     *
     * @param array $data
     * @return array
     */
    public function getRules(array $data)
    {
        switch ($data['role'])
        {
            case Roles::ADMIN:
                $accessPermissions = RoleEditablePermissions::ADMIN_PERMISSIONS;
                break;
            case Roles::MANAGER:
                $accessPermissions = RoleEditablePermissions::MANAGER_PERMISSIONS;
                break;
            case Roles::FRONT_USER:
                $accessPermissions = RoleEditablePermissions::EMPLOYEE_PERMISSIONS;
                break;
            case Roles::WHISTLEBLOWER_ADMIN:
                $accessPermissions = RoleEditablePermissions::WHISTLEBLOWER_PERMISSIONS;
                break;
        }

        return [
            'access' => 'required|array',
            'role' => ['required', Rule::in([Roles::ADMIN, Roles::MANAGER, Roles::FRONT_USER, Roles::WHISTLEBLOWER_ADMIN])],
            'access.*.name' => ['required', Rule::in($accessPermissions)],
            'access.*.is_enabled' => 'required|boolean'
        ];
    }

    /**
     * Update manager access permissions
     *
     * @param array $args
     * @return void
     */
    public function performAction(array $args)
    {
        $client = Context::client()->get();

        $previousPermissions = app(GetClientRoleAccessNames::class)->execute(['role' => $args['role']]);
        $role = Role::firstWhere('name', $args['role']);

        foreach ($args['access'] as $access) {
            if ($access['name'] === AccessPermissions::CLIENT_NOTIFICATION_CONSOLE_UPCOMING) {
                $client['NOTIFICATION_CONSOLE_UPCOMING'] = $access['data'];
                $client->save();
                continue;
            }
            $accessPermission = Permission::where('name', $access['name'])->firstOrFail();

            $accessPermissionData = [
                'client_id' => $client->getKey(),
                'role_id' => $role->getKey(),
                'permission_id' => $accessPermission->getKey(),
            ];

            if ($access['is_enabled']) {
                ClientAccess::firstOrCreate($accessPermissionData);
            } else {
                ClientAccess::where($accessPermissionData)->delete();
            }
        }
        if ($args['role'] !== Roles::FRONT_USER) {
            event(new RoleAccessUpdated($client, $previousPermissions, $args['role']));
        } else {
            $employeeAccess = app(GetAccessByRole::class)->execute(['role' => Roles::FRONT_USER]);
            $userIds = $client->users()->role(Roles::FRONT_USER)->pluck('code')->toArray();
            if (!empty($userIds)) {
                $arguments = array(
                    "userIds" => $userIds,
                    "headings" => __("Your permissions to messages and directory have been changed"),
                    "contents" => __("Tap to see the changes"),
                    "data" => [
                        "type" => NotificationTypes::PERMISSION_CHANGED,
                        "restrict_employee" => [
                            "employee_messages" => $employeeAccess[AccessPermissions::ACCESS_GROUP_EMPLOYEE_MESSAGES],
                            "employee_directory" => $employeeAccess[AccessPermissions::ACCESS_GROUP_EMPLOYEE_DIRECTORY],
                        ],
                    ],
                    'send_after' => null
                );
                SendNotification::dispatch($arguments);
            }
        }
    }
}
