<?php


namespace App\Traits;


use App\Enum\FilterTypes;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Validation\ValidationException;

trait Filterable
{
    private $filterTypes = [FilterTypes::IS, FilterTypes::IS_NOT, FilterTypes::CONTAIN];

    /**
     * Apply filters
     * @param $options
     * @param Builder $query
     * @param array $allowFilters
     * @throws ValidationException
     */
    public function applyFilters($options, Builder $query, $allowFilters = [])
    {
        if (isset($options['filter'])) {
            foreach ($options['filter'] as $filter) {
                $filter = $this->validateFilter($filter, $allowFilters);
                $this->applyFilter($filter, $query);
            }
        }
    }

    /**
     * Validate filter
     * @param $filter
     * @param $allowFilters
     * @return mixed
     * @throws ValidationException
     */
    public function validateFilter($filter, $allowFilters)
    {
        if (!isset($filter['value'])) {
            throw ValidationException::withMessages(['filter[value]' => __('validation.required', ['attribute' => 'filter[value]'])]);
        }
        if(!empty($allowFilters) && !in_array($filter['column'],$allowFilters)){
            throw ValidationException::withMessages(['filter[column]' => __('validation.invalid', ['attribute' => 'filter[column]'])]);
        }

        if (!isset($filter['column'])) {
            throw ValidationException::withMessages(['filter[column]' => __('validation.required', ['attribute' => 'filter[column]'])]);
        }
        if (isset($filter['type'])) {
            if (!in_array($filter['type'], $this->filterTypes)) {
                throw ValidationException::withMessages(['filter[type]' => __('validation.invalid', ['attribute' => 'filter[type]'])]);
            }
        } else {
            $filter['type'] = FilterTypes::CUSTOM;
        }
        return $filter;
    }

    /**
     * Apply filter to query
     * @param $filter
     * @param $query
     */
    public function applyFilter($filter, $query)
    {
        switch ($filter['type']) {
            case FilterTypes::IS_NOT:
                $query->where($filter['column'], '!=', $filter['value']);
                break;
            case FilterTypes::CONTAIN:
                $query->where($filter['column'], 'LIKE', "%" . $filter['value'] . "%");
                break;
            case FilterTypes::IS:
                $query->where($filter['column'], '=', $filter['value']);
                break;
            default: //Default filter custom.
                
        }
    }

    /**
     * Get column filter
     */
    public function getFilter($options, $column, $allowFilters = []) {
        if (isset($options['filter'])) {
            foreach ($options['filter'] as $filter) {
                $filter = $this->validateFilter($filter, $allowFilters);
                if ($filter['column'] === $column) {
                    return $filter;
                }
            }
        }
        return null;
    }
}