<?php

namespace App\Traits;

use Illuminate\Http\JsonResponse;

trait JsonRespondController
{
    /**
     * @var int
     */
    protected $httpStatusCode = 200;

    /**
     * Get HTTP status code of the response.
     *
     * @return int
     */
    public function getHTTPStatusCode()
    {
        return $this->httpStatusCode;
    }

    /**
     * Set HTTP status code of the response.
     *
     * @param int $statusCode
     * @return self
     */
    public function setHTTPStatusCode($statusCode)
    {
        $this->httpStatusCode = $statusCode;

        return $this;
    }

    /**
     * Sends a JSON to the consumer.
     *
     * @param  array $data
     * @param  array  $headers [description]
     * @return JsonResponse
     */
    public function respond($data, $headers = [])
    {
        return response()->json($data, $this->getHTTPStatusCode(), $headers);
    }

    /**
     * Sends a response not found (404) to the request.
     * Error Code = NOT_FOUND.
     *
     * @param string|null $message
     * @return JsonResponse
     */
    public function respondNotFound(string $message = null)
    {
        return $this->setHTTPStatusCode(404)
                    ->respondWithError($message);
    }

    /**
     * Sends a response unauthorized (401) to the request.
     * Error Code = UNAUTHORIZED.
     *
     * @param string $message
     * @return JsonResponse
     */
    public function respondUnauthorized($message = null)
    {
        return $this->setHTTPStatusCode(401)
                    ->respondWithError($message);
    }

    /**
     * Sends a response with error.
     *
     * @param string|array $message
     * @return JsonResponse
     */
    public function respondWithError($message = null)
    {
        return $this->respond([
            'message' => $message
        ]);
    }

    /**
     * Sends a response that the object has been deleted, and also indicates
     * the id of the object that has been deleted.
     *
     * @param  int $id
     * @return JsonResponse
     */
    public function respondObjectDeleted($id)
    {
        return $this->respond([
            'deleted' => true,
            'id' => $id,
        ]);
    }

    /**
     * Sends a response that the object has been deleted, and also indicates
     * the ids of the object that has been deleted.
     *
     * @param  array $data
     * @return JsonResponse
     */
    public function respondObjectMultipleDeleted($data)
    { 
        return $this->respond($data);
    }
}
