<?php

use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Schema\Blueprint;
use App\Models\User\User;
use Illuminate\Support\Facades\DB;
use App\Database\Migrations\AppMigration;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;

class MigratePermissionData extends AppMigration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        //Get all permission data from db
        //Only run when migrate from old to new database
        $roles = collect();
        if(Schema::hasTable('abilities')){
            $roles = DB::table('roles')->where('name','!=','client')->select('*')->get();

            //Only get valid permission
            $permissions =  DB::table('abilities')
                ->whereNotIn('id',DB::table('abilities')->select('id')
                    ->whereIn('name',['create','update','delete','view','edit'])
                    ->whereNull('entity_type')->pluck('id')->toArray())
                ->whereNotIn('id',DB::table('abilities')->select('id')
                    ->where('name','')
                    ->OrwhereNull('name')
                    ->pluck('id')->toArray())
                ->select('abilities.*')->get();
            $clientsId = DB::table('users')
                ->join('assigned_roles','users.id','=','assigned_roles.entity_id')
                ->join('user_details','users.id','=','user_details.user_id')
                ->where('assigned_roles.entity_type','App\\User')
                ->where('assigned_roles.role_id',6)
                ->select('users.id')->pluck('id')->toArray();
            //Get all user with role. Exclude client
            $users = User::whereNotIn('id',$clientsId)->get();
            $userArray = array();
            foreach($users as $user){
                $userResult['entity'] = $user;
                $userRole = DB::table('assigned_roles')
                    ->join('roles','roles.id','=','assigned_roles.role_id')
                    ->where('assigned_roles.entity_id',$user->id)
                    ->select('roles.*')->first();
                //Not migrate if user don't have role
                if(!$userRole)
                    continue;
                $userResult['role'] =$userRole;
                $userArray[$user->id] = $userResult;
            }


            //Backup table
            $currentDay = date("Ymd",time());
            Schema::rename('roles', 'roles_bck' . $currentDay);
            Schema::rename('abilities', 'abilities_bck' . $currentDay);
            Schema::rename('permissions', 'permissions_bck' . $currentDay);
            Schema::rename('assigned_roles', 'assigned_roles_bck' . $currentDay);

        }




        $tableNames = config('permission.table_names');
        $columnNames = config('permission.column_names');

        if (empty($tableNames)) {
            throw new \Exception('Error: config/permission.php not loaded. Run [php artisan config:clear] and try again.');
        }

        Schema::create($tableNames['permissions'], function (Blueprint $table) {
            $table->bigIncrements('id');
            $table->string('name');
            $table->string('guard_name');
            $table->timestamps();
        });

        Schema::create($tableNames['roles'], function (Blueprint $table) {
            $table->bigIncrements('id');
            $table->string('name');
            $table->string('guard_name');
            $table->timestamps();
        });

        Schema::create($tableNames['model_has_permissions'], function (Blueprint $table) use ($tableNames, $columnNames) {
            $table->unsignedBigInteger('permission_id');

            $table->string('model_type');
            $table->unsignedBigInteger($columnNames['model_morph_key']);
            $table->index([$columnNames['model_morph_key'], 'model_type'], 'model_has_permissions_model_id_model_type_index');

            $table->foreign('permission_id')
                ->references('id')
                ->on($tableNames['permissions'])
                ->onDelete('cascade');

            $table->primary(['permission_id', $columnNames['model_morph_key'], 'model_type'],
                    'model_has_permissions_permission_model_type_primary');
        });

        Schema::create($tableNames['model_has_roles'], function (Blueprint $table) use ($tableNames, $columnNames) {
            $table->unsignedBigInteger('role_id');

            $table->string('model_type');
            $table->unsignedBigInteger($columnNames['model_morph_key']);
            $table->index([$columnNames['model_morph_key'], 'model_type'], 'model_has_roles_model_id_model_type_index');

            $table->foreign('role_id')
                ->references('id')
                ->on($tableNames['roles'])
                ->onDelete('cascade');

            $table->primary(['role_id', $columnNames['model_morph_key'], 'model_type'],
                    'model_has_roles_role_model_type_primary');
        });

        Schema::create($tableNames['role_has_permissions'], function (Blueprint $table) use ($tableNames) {
            $table->unsignedBigInteger('permission_id');
            $table->unsignedBigInteger('role_id');

            $table->foreign('permission_id')
                ->references('id')
                ->on($tableNames['permissions'])
                ->onDelete('cascade');

            $table->foreign('role_id')
                ->references('id')
                ->on($tableNames['roles'])
                ->onDelete('cascade');

            $table->primary(['permission_id', 'role_id'], 'role_has_permissions_permission_id_role_id_primary');
        });

        app('cache')
            ->store(config('permission.cache.store') != 'default' ? config('permission.cache.store') : null)
            ->forget(config('permission.cache.key'));

        //Get all permission data from db
        //Only run when migrate from old to new database
        if($roles->count() > 0) {
            //After create new table, migrate old data to new table
            $adminRole = false;
            foreach ($roles as $role) {
                $newRole = Role::create(['name' => $role->name]);
                if ($role->name === 'admin') {
                    $adminRole = $newRole;
                }
            }

            foreach ($permissions as $permission) {
                $name = $permission->entity_type ?
                    str_replace('App\\', "", $permission->entity_type) . "." . $permission->name
                    : $permission->name;
                $name = strtolower($name);
                $newPermission = Permission::create(['name' => $name]);
                //Currently, only aadmin have permission
                $adminRole->givePermissionTo($newPermission);
            }

            foreach ($userArray as $user) {
                $newUser = User::find($user['entity']->id);
                //Assign user to role
                $newUser->assignRole($user['role']->name);
            }
        }
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {

    }
}
