# How to write a service

## Service 
In general, one service layer does exactly one job. If logic code contains more than one job, separate it into 2 services.

The only required method for service is `performAction`. Input is passed from main function `execute`.

```php
<?php

namespace App\Services\Library;

use App\Facades\Context;
use App\Models\Library\DocumentProfile;
use App\Services\BaseService;

class CreateDocumentProfile extends BaseService
{
    /**
     * Create document profile
     *
     * @param array $args
     * @return DocumentProfile
     */
    function performAction(array $args): DocumentProfile
    {
        $client = Context::client()->get();
        $user = Context::user()->get();

        $args['client_id'] = $client->id;
        $args['created_by'] = $user->id;
        $documentProfile = new DocumentProfile($args);
        $documentProfile->save();

        return $documentProfile->refresh();
    }
} 
```

Call a service

```php
$documentProfile = app(CreateDocumentProfile::class)->execute($args); 
```

### Notes

- Do not use any resource object inside service
- If there are more than one DB actions, wrap them in a transaction
- If error occurred, throw an exception instead of returning custom data.

## Extensions

### Permission
 - In order to restrict execution, implements `Performable` interface
 - Permission rules for service layer are put in `getpermissions()` function required by `Performable`.

 ```php
public function getPermissions(array $args)
{
    return ['events.edit'];
}
``` 
### Validation
 - Validate input data for service by implementing interface `Validatable`
 - function `getRules` is required when implement `Validatable`
```php
  public function getRules(array $args)
  {
          return [
              'id' => 'required|integer',
          ];
  }
```

### Transaction
- Ensure all steps are run inside a transaction, implements interface `Transationable`
```

## Sorting 
 - To enable sorting on service, use trait `Sortable`. For sorting, use  `applySorting`:
 ```php
class GetEvent extends BaseService
{
    use Sortable;
    
    public function execute(array $options){
        $query = Event::query();
        $this->applySorting($options,$query);
    }
}
```
 - Options will have the following structure:
```array
$options = [
    'sort'=> [
        'column'=> 'name' //Column to sort,
        'direction'=> 'asc' //Direction to sort, have asc and desc
    ]   
]
```

### Pagination 
 - To enable pagination on service, use trait `Pageable`. For pagination, use  `applyPaging`:
 ```php
class GetEvent extends BaseService
{
    use Pageable;
    
    public function execute(array $options){
        $query = Event::query();
        return $this->applyPaging($options,$query);
    }
}
```
 - Options will have structure:
```array
$options = [
    'page'=> [
        'size'=> 10 //Element per page
        'number'=> 1 //Page number
    ]   
]
```
 - Important, apply paging will return a collection, so they have to put at the end, after all query are done.
  
### Response
 - Data structure response is written in `Http/Resources` folder under Resource type, for more infomation please click [here](https://laravel.com/docs/8.x/eloquent-resources).