<?php

namespace Qualee\AiDocSearch\Services;

use App\FrontUserDetail as FrontUserDetail;
use App\Models\User\Employee;
use App\Helpers\FileHelper;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;

class SearchService
{
    private $document_path_relative = "app/public/documents";

    /**
     * Do search action
     *
     * @param $request
     * @param $user
     * @return \Illuminate\Http\JsonResponse|string
     */
    public function search($request, $user)
    {
        // Support for searching by multiple content profiles
        $profile_id = Employee::firstWhere('user_id', $user->id)->getDocumentProfileIds();
        $client_id = Employee::firstWhere('user_id', $user->id)->client_id;

        try {
            $client = new Client(['headers' => ['Accept' => 'application/json', 'Content-type' => 'application/json']]);
            $body = [
                'term' => $request->term,
                'clientid' => $client_id,
                'profileid' => $profile_id,
                'env' => config('qualee-ai-doc-search.ads_env')
            ];

            $response = $client->request('POST', config('qualee-ai-doc-search.ads_lambda_search_domain') . '/search', ['body' => json_encode($body)]);

            return response()->json([
                'message' => 'Search successfully.',
                'data' => $this->formatData($response->getBody()->getContents(), $request->term)
            ]);
        } catch (GuzzleException $e) {
            Log::error($e);
            return $e->getMessage();
        }
    }

    /**
     * Format data to readable
     *
     * @param $response
     * @param $keyword
     * @return array
     */
    public function formatData($response, $keyword): array
    {
        $data = [];
        $arr = [];
        $responseArr = json_decode($response, true);

        if(isset($responseArr['status']) && ($responseArr['status'] < 200 || $responseArr['status'] >= 300)) {
            return [];
        }

        foreach ($responseArr['hits']['hits'] as $key => $value) {
            $arr['title'] = $value["_source"]['title'];
            $arr['paragraph_hints'] = $this->getListParagraphHint($value['_source']['body'], $keyword);
            $arr['doc_url'] = $this->generateDocUrl(isset($value["_source"]["docname"]) ? $value["_source"]["docname"] : null);
            array_push($data, $arr);
        }
        return $data;
    }

    /**
     * Get hints
     *
     * @param $data
     * @param $keyword
     * @return array
     */
    public function getListParagraphHint($data, $keyword): array
    {
        $listParagraph = [];
        foreach ($data as $k => $sentence) {
            if (strpos(strtolower($sentence), strtolower($keyword)) !== false) {
                array_push($listParagraph, $sentence);
            }
        }
        return $listParagraph;
    }

    /**
     * Create doc url
     *
     * @param $fileName
     * @return string
     */
    public function generateDocUrl($fileName) : string
    {
        if (empty($fileName)) {
            return "";
        }
        return FileHelper::getSignedUrlForAIDocSearch($fileName);
    }
}
